@@ -150,7 +150,8 @@ const WorkerSandbox = EventEmitter.compose({
       sandboxPrototype: proto,
       wantXrays: true,
       wantGlobalProperties: wantGlobalProperties,
-      sameZoneAs: window
+      sameZoneAs: window,
+      metadata: { SDKContentScript: true }
     });
     // We have to ensure that window.top and window.parent are the exact same
     // object than window object, i.e. the sandbox global object. But not
@@ -201,8 +202,15 @@ const WorkerSandbox = EventEmitter.compose({
           clearInterval: 'r'
         }
       },
+      sandbox: {
+        evaluate: evaluate,
+        __exposedProps__: {
+          evaluate: 'r',
+        }
+      },
       __exposedProps__: {
-        timers: 'r'
+        timers: 'r',
+        sandbox: 'r',
       }
     };
     let onEvent = this._onContentEvent.bind(this);
@@ -232,6 +240,19 @@ const WorkerSandbox = EventEmitter.compose({
         self._addonWorker._onContentScriptEvent.apply(self._addonWorker, arguments);
     });
 
+    // unwrap, recreate and propagate async Errors thrown from content-script
+    this.on("error", function onError({instanceOfError, value}) {
+      if (self._addonWorker) {
+        let error = value;
+        if (instanceOfError) {
+          error = new Error(value.message, value.fileName, value.lineNumber);
+          error.stack = value.stack;
+          error.name = value.name;
+        }
+        self._addonWorker._emit('error', error);
+      }
+    });
+
     // Inject `addon` global into target document if document is trusted,
     // `addon` in document is equivalent to `self` in content script.
     if (worker._injectInDocument) {
@@ -554,6 +575,7 @@ const Worker = EventEmitter.compose({
    */
   destroy: function destroy() {
     this._workerCleanup();
+    this._inited = true;
     this._removeAllListeners();
   },
 
@@ -580,6 +602,7 @@ const Worker = EventEmitter.compose({
       this._earlyEvents.length = 0;
       this._emit("detach");
     }
+    this._inited = false;
   },
 
   /**
