@@ -113,12 +113,12 @@ Sanitizer.prototype = {
     cache: {
       clear: function ()
       {
-        var cacheService = Cc["@mozilla.org/network/cache-service;1"].
-                          getService(Ci.nsICacheService);
+        var cache = Cc["@mozilla.org/netwerk/cache-storage-service;1"].
+                    getService(Ci.nsICacheStorageService);
         try {
           // Cache doesn't consult timespan, nor does it have the
           // facility for timespan-based eviction.  Wipe it.
-          cacheService.evictEntries(Ci.nsICache.STORE_ANYWHERE);
+          cache.clear();
         } catch(er) {}
 
         var imageCache = Cc["@mozilla.org/image/tools;1"].
@@ -227,6 +227,12 @@ Sanitizer.prototype = {
           prefs.clearUserPref("general.open_location.last_url");
         }
         catch (e) { }
+
+        try {
+          var seer = Components.classes["@mozilla.org/network/seer;1"]
+                               .getService(Components.interfaces.nsINetworkSeer);
+          seer.reset();
+        } catch (e) { }
       },
 
       get canClear()
@@ -310,48 +316,25 @@ Sanitizer.prototype = {
     downloads: {
       clear: function ()
       {
-        if (DownloadsCommon.useJSTransfer) {
-          Task.spawn(function () {
-            let filterByTime = null;
-            if (this.range) {
-              // Convert microseconds back to milliseconds for date comparisons.
-              let rangeBeginMs = this.range[0] / 1000;
-              let rangeEndMs = this.range[1] / 1000;
-              filterByTime = download => download.startTime >= rangeBeginMs &&
-                                         download.startTime <= rangeEndMs;
-            }
-
-            // Clear all completed/cancelled downloads
-            let list = yield Downloads.getList(Downloads.ALL);
-            list.removeFinished(filterByTime);
-          }.bind(this)).then(null, Components.utils.reportError);
-        }
-        else {
-          var dlMgr = Components.classes["@mozilla.org/download-manager;1"]
-                                .getService(Components.interfaces.nsIDownloadManager);
-
+        Task.spawn(function () {
+          let filterByTime = null;
           if (this.range) {
-            // First, remove the completed/cancelled downloads
-            dlMgr.removeDownloadsByTimeframe(this.range[0], this.range[1]);
+            // Convert microseconds back to milliseconds for date comparisons.
+            let rangeBeginMs = this.range[0] / 1000;
+            let rangeEndMs = this.range[1] / 1000;
+            filterByTime = download => download.startTime >= rangeBeginMs &&
+                                       download.startTime <= rangeEndMs;
           }
-          else {
-            // Clear all completed/cancelled downloads
-            dlMgr.cleanUp();
-            dlMgr.cleanUpPrivate();
-          }
-        }
+
+          // Clear all completed/cancelled downloads
+          let list = yield Downloads.getList(Downloads.ALL);
+          list.removeFinished(filterByTime);
+        }.bind(this)).then(null, Components.utils.reportError);
       },
 
       canClear : function(aCallback, aArg)
       {
-        if (DownloadsCommon.useJSTransfer) {
-          aCallback("downloads", true, aArg);
-        }
-        else {
-          var dlMgr = Components.classes["@mozilla.org/download-manager;1"]
-                                .getService(Components.interfaces.nsIDownloadManager);
-          aCallback("downloads", dlMgr.canCleanUp || dlMgr.canCleanUpPrivate, aArg);
-        }
+        aCallback("downloads", true, aArg);
         return false;
       }
     },
