@@ -27,6 +27,7 @@
 #include "nsISupportsPrimitives.h"
 #include "nsThreadUtils.h"
 #include "nsXULAppAPI.h"
+#include "mozilla/WindowsVersion.h"
 
 #include "windows.h"
 #include "shellapi.h"
@@ -53,6 +54,8 @@
 
 #define NS_TASKBAR_CONTRACTID "@mozilla.org/windows-taskbar;1"
 
+using mozilla::IsWin8OrLater;
+
 NS_IMPL_ISUPPORTS2(nsWindowsShellService, nsIWindowsShellService, nsIShellService)
 
 static nsresult
@@ -160,8 +163,12 @@ typedef struct {
 // more info. The FTP protocol is not checked so advanced users can set the FTP
 // handler to another application and still have Firefox check if it is the
 // default HTTP and HTTPS handler.
+// *** Do not add additional checks here unless you skip them when aForAllTypes
+// is false below***.
 static SETTING gSettings[] = {
   // File Handler Class
+  // ***keep this as the first entry because when aForAllTypes is not set below
+  // it will skip over this check.***
   { MAKE_KEY_NAME1("FirefoxHTML", SOC), VAL_OPEN, OLD_VAL_OPEN },
 
   // Protocol Handler Class - for Vista and above
@@ -223,8 +230,8 @@ LaunchHelper(nsAutoString& aPath)
   STARTUPINFOW si = {sizeof(si), 0};
   PROCESS_INFORMATION pi = {0};
 
-  if (!CreateProcessW(NULL, (LPWSTR)aPath.get(), NULL, NULL, FALSE, 0, NULL,
-                      NULL, &si, &pi)) {
+  if (!CreateProcessW(nullptr, (LPWSTR)aPath.get(), nullptr, nullptr, FALSE,
+                      0, nullptr, nullptr, &si, &pi)) {
     return NS_ERROR_FAILURE;
   }
 
@@ -310,16 +317,6 @@ nsWindowsShellService::ShortcutMaintenance()
   return LaunchHelper(appHelperPath);
 }
 
-static bool
-IsWin8OrLater()
-{
-  OSVERSIONINFOW osInfo;
-  osInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFOW);
-  GetVersionExW(&osInfo);
-  return osInfo.dwMajorVersion > 6 || 
-         (osInfo.dwMajorVersion >= 6 && osInfo.dwMinorVersion >= 2);
-}
-
 static bool
 IsAARDefaultHTTP(IApplicationAssociationRegistration* pAAR,
                  bool* aIsDefaultBrowser)
@@ -355,13 +352,19 @@ IsAARDefaultHTML(IApplicationAssociationRegistration* pAAR,
   return SUCCEEDED(hr);
 }
 
+/*
+ * Query's the AAR for the default status.
+ * This only checks for FirefoxURL and if aCheckAllTypes is set, then
+ * it also checks for FirefoxHTML.  Note that those ProgIDs are shared
+ * by all Firefox browsers.
+*/
 bool
 nsWindowsShellService::IsDefaultBrowserVista(bool aCheckAllTypes,
                                              bool* aIsDefaultBrowser)
 {
   IApplicationAssociationRegistration* pAAR;
   HRESULT hr = CoCreateInstance(CLSID_ApplicationAssociationRegistration,
-                                NULL,
+                                nullptr,
                                 CLSCTX_INPROC,
                                 IID_IApplicationAssociationRegistration,
                                 (void**)&pAAR);
@@ -403,18 +406,11 @@ nsWindowsShellService::IsDefaultBrowser(bool aStartupCheck,
   if (aStartupCheck)
     mCheckedThisSession = true;
 
-  // Check if we only care about a lightweight check, and make sure this
-  // only has an effect on Win8 and later.
-  if (!aForAllTypes && IsWin8OrLater()) {
-    return IsDefaultBrowserVista(false,
-                                 aIsDefaultBrowser) ? NS_OK : NS_ERROR_FAILURE;
-  }
-
   // Assume we're the default unless one of the several checks below tell us
   // otherwise.
   *aIsDefaultBrowser = true;
 
-  PRUnichar exePath[MAX_BUF];
+  wchar_t exePath[MAX_BUF];
   if (!::GetModuleFileNameW(0, exePath, MAX_BUF))
     return NS_ERROR_FAILURE;
 
@@ -428,12 +424,17 @@ nsWindowsShellService::IsDefaultBrowser(bool aStartupCheck,
   HKEY theKey;
   DWORD res;
   nsresult rv;
-  PRUnichar currValue[MAX_BUF];
+  wchar_t currValue[MAX_BUF];
+
+  SETTING* settings = gSettings;
+  if (!aForAllTypes && IsWin8OrLater()) {
+    // Skip over the file handler check
+    settings++;
+  }
 
-  SETTING* settings;
   SETTING* end = gSettings + sizeof(gSettings) / sizeof(SETTING);
 
-  for (settings = gSettings; settings < end; ++settings) {
+  for (; settings < end; ++settings) {
     NS_ConvertUTF8toUTF16 keyName(settings->keyName);
     NS_ConvertUTF8toUTF16 valueData(settings->valueData);
     int32_t offset = valueData.Find("%APPPATH%");
@@ -447,17 +448,18 @@ nsWindowsShellService::IsDefaultBrowser(bool aStartupCheck,
 
     ::ZeroMemory(currValue, sizeof(currValue));
     DWORD len = sizeof currValue;
-    res = ::RegQueryValueExW(theKey, L"", NULL, NULL, (LPBYTE)currValue, &len);
+    res = ::RegQueryValueExW(theKey, L"", nullptr, nullptr,
+                             (LPBYTE)currValue, &len);
     // Close the key that was opened.
     ::RegCloseKey(theKey);
     if (REG_FAILED(res) ||
-        !valueData.Equals(currValue, CaseInsensitiveCompare)) {
+        _wcsicmp(valueData.get(), currValue)) {
       // Key wasn't set or was set to something other than our registry entry.
       NS_ConvertUTF8toUTF16 oldValueData(settings->oldValueData);
       offset = oldValueData.Find("%APPPATH%");
       oldValueData.Replace(offset, 9, appLongPath);
       // The current registry value doesn't match the current or the old format.
-      if (!oldValueData.Equals(currValue, CaseInsensitiveCompare)) {
+      if (_wcsicmp(oldValueData.get(), currValue)) {
         *aIsDefaultBrowser = false;
         return NS_OK;
       }
@@ -489,7 +491,7 @@ nsWindowsShellService::IsDefaultBrowser(bool aStartupCheck,
   // Only check if Firefox is the default browser on Vista and above if the
   // previous checks show that Firefox is the default browser.
   if (*aIsDefaultBrowser) {
-    IsDefaultBrowserVista(true, aIsDefaultBrowser);
+    IsDefaultBrowserVista(aForAllTypes, aIsDefaultBrowser);
   }
 
   // To handle the case where DDE isn't disabled due for a user because there
@@ -497,7 +499,7 @@ nsWindowsShellService::IsDefaultBrowser(bool aStartupCheck,
   // default browser and if dde is disabled for each handler
   // and if it isn't disable it. When Firefox is not the default browser the
   // helper application will disable dde for each handler.
-  if (*aIsDefaultBrowser) {
+  if (*aIsDefaultBrowser && aForAllTypes) {
     // Check ftp settings
 
     end = gDDESettings + sizeof(gDDESettings) / sizeof(SETTING);
@@ -516,8 +518,8 @@ nsWindowsShellService::IsDefaultBrowser(bool aStartupCheck,
 
       ::ZeroMemory(currValue, sizeof(currValue));
       DWORD len = sizeof currValue;
-      res = ::RegQueryValueExW(theKey, L"", NULL, NULL, (LPBYTE)currValue,
-                               &len);
+      res = ::RegQueryValueExW(theKey, L"", nullptr, nullptr,
+                               (LPBYTE)currValue, &len);
       // Close the key that was opened.
       ::RegCloseKey(theKey);
       if (REG_FAILED(res) || PRUnichar('\0') != *currValue) {
@@ -525,9 +527,9 @@ nsWindowsShellService::IsDefaultBrowser(bool aStartupCheck,
         // Delete the key along with all of its childrean and then recreate it.
         const nsString &flatName = PromiseFlatString(keyName);
         ::SHDeleteKeyW(HKEY_CURRENT_USER, flatName.get());
-        res = ::RegCreateKeyExW(HKEY_CURRENT_USER, flatName.get(), 0, NULL,
-                                REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL,
-                                &theKey, NULL);
+        res = ::RegCreateKeyExW(HKEY_CURRENT_USER, flatName.get(), 0, nullptr,
+                                REG_OPTION_NON_VOLATILE, KEY_SET_VALUE,
+                                nullptr, &theKey, nullptr);
         if (REG_FAILED(res)) {
           // If disabling DDE fails try to disable it using the helper
           // application when setting Firefox as the default browser.
@@ -564,13 +566,13 @@ nsWindowsShellService::IsDefaultBrowser(bool aStartupCheck,
 
     ::ZeroMemory(currValue, sizeof(currValue));
     DWORD len = sizeof currValue;
-    res = ::RegQueryValueExW(theKey, L"", NULL, NULL, (LPBYTE)currValue,
+    res = ::RegQueryValueExW(theKey, L"", nullptr, nullptr, (LPBYTE)currValue,
                              &len);
 
     // Don't update the FTP protocol handler's shell open command when the
     // current registry value doesn't exist or matches the old format.
     if (REG_FAILED(res) ||
-        !oldValueOpen.Equals(currValue, CaseInsensitiveCompare)) {
+        _wcsicmp(oldValueOpen.get(), currValue)) {
       ::RegCloseKey(theKey);
       return NS_OK;
     }
@@ -606,27 +608,27 @@ DynSHOpenWithDialog(HWND hwndParent, const OPENASINFO *poainfo)
 {
   typedef HRESULT (WINAPI * SHOpenWithDialogPtr)(HWND hwndParent,
                                                  const OPENASINFO *poainfo);
-  static SHOpenWithDialogPtr SHOpenWithDialogFn = NULL;
-  if (!SHOpenWithDialogFn) {
-    // shell32.dll is in the knownDLLs list so will always be loaded from the
-    // system32 directory.
-    static const PRUnichar kSehllLibraryName[] =  L"shell32.dll";
-    HMODULE shellDLL = ::LoadLibraryW(kSehllLibraryName);
-    if (!shellDLL) {
-      return NS_ERROR_FAILURE;
-    }
-
-    SHOpenWithDialogFn =
-      (SHOpenWithDialogPtr)GetProcAddress(shellDLL, "SHOpenWithDialog");
-    FreeLibrary(shellDLL);
-
-    if (!SHOpenWithDialogFn) {
-      return NS_ERROR_FAILURE;
-    }
+  
+  // shell32.dll is in the knownDLLs list so will always be loaded from the
+  // system32 directory.
+  static const wchar_t kSehllLibraryName[] =  L"shell32.dll";
+  HMODULE shellDLL = ::LoadLibraryW(kSehllLibraryName);
+  if (!shellDLL) {
+    return NS_ERROR_FAILURE;
   }
 
-  return SUCCEEDED(SHOpenWithDialogFn(hwndParent, poainfo)) ? NS_OK :
-                                                              NS_ERROR_FAILURE;
+  SHOpenWithDialogPtr SHOpenWithDialogFn =
+    (SHOpenWithDialogPtr)GetProcAddress(shellDLL, "SHOpenWithDialog");
+
+  if (!SHOpenWithDialogFn) {
+    return NS_ERROR_FAILURE;
+  }
+
+  nsresult rv = 
+    SUCCEEDED(SHOpenWithDialogFn(hwndParent, poainfo)) ? NS_OK :
+                                                         NS_ERROR_FAILURE;
+  FreeLibrary(shellDLL);
+  return rv;
 }
 
 nsresult
@@ -650,8 +652,8 @@ nsWindowsShellService::LaunchControlPanelDefaultPrograms()
   si.dwFlags = STARTF_USESHOWWINDOW;
   si.wShowWindow = SW_SHOWDEFAULT;
   PROCESS_INFORMATION pi = {0};
-  if (!CreateProcessW(controlEXEPath, params, NULL, NULL, FALSE, 0, NULL,
-                      NULL, &si, &pi)) {
+  if (!CreateProcessW(controlEXEPath, params, nullptr, nullptr, FALSE,
+                      0, nullptr, nullptr, &si, &pi)) {
     return NS_ERROR_FAILURE;
   }
   CloseHandle(pi.hProcess);
@@ -665,11 +667,11 @@ nsWindowsShellService::LaunchHTTPHandlerPane()
 {
   OPENASINFO info;
   info.pcszFile = L"http";
-  info.pcszClass = NULL;
+  info.pcszClass = nullptr;
   info.oaifInFlags = OAIF_FORCE_REGISTRATION | 
                      OAIF_URL_PROTOCOL |
                      OAIF_REGISTER_EXT;
-  return DynSHOpenWithDialog(NULL, &info);
+  return DynSHOpenWithDialog(nullptr, &info);
 }
 
 NS_IMETHODIMP
@@ -748,11 +750,16 @@ nsWindowsShellService::SetShouldCheckDefaultBrowser(bool aShouldCheck)
 static nsresult
 WriteBitmap(nsIFile* aFile, imgIContainer* aImage)
 {
-  nsRefPtr<gfxImageSurface> image;
-  nsresult rv = aImage->CopyFrame(imgIContainer::FRAME_FIRST,
-                                  imgIContainer::FLAG_SYNC_DECODE,
-                                  getter_AddRefs(image));
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsresult rv;
+
+  nsRefPtr<gfxASurface> surface;
+  aImage->GetFrame(imgIContainer::FRAME_FIRST,
+                   imgIContainer::FLAG_SYNC_DECODE,
+                   getter_AddRefs(surface));
+  NS_ENSURE_TRUE(surface, NS_ERROR_FAILURE);
+
+  nsRefPtr<gfxImageSurface> image(surface->GetAsReadableARGB32ImageSurface());
+  NS_ENSURE_TRUE(image, NS_ERROR_FAILURE);
 
   int32_t width = image->Width();
   int32_t height = image->Height();
@@ -960,7 +967,7 @@ nsWindowsShellService::OpenApplication(int32_t aApplication)
   if (NS_FAILED(rv))
     return rv;
 
-  PRUnichar buf[MAX_BUF];
+  wchar_t buf[MAX_BUF];
   DWORD type, len = sizeof buf;
   DWORD res = ::RegQueryValueExW(theKey, EmptyString().get(), 0,
                                  &type, (LPBYTE)&buf, &len);
@@ -1023,8 +1030,8 @@ nsWindowsShellService::OpenApplication(int32_t aApplication)
   ::ZeroMemory(&si, sizeof(STARTUPINFOW));
   ::ZeroMemory(&pi, sizeof(PROCESS_INFORMATION));
 
-  BOOL success = ::CreateProcessW(NULL, (LPWSTR)path.get(), NULL,
-                                  NULL, FALSE, 0, NULL,  NULL,
+  BOOL success = ::CreateProcessW(nullptr, (LPWSTR)path.get(), nullptr,
+                                  nullptr, FALSE, 0, nullptr,  nullptr,
                                   &si, &pi);
   if (!success)
     return NS_ERROR_FAILURE;
@@ -1061,7 +1068,7 @@ nsWindowsShellService::SetDesktopBackgroundColor(uint32_t aColor)
                       nsIWindowsRegKey::ACCESS_SET_VALUE);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRUnichar rgb[12];
+  wchar_t rgb[12];
   _snwprintf(rgb, 12, L"%u %u %u", r, g, b);
 
   rv = regKey->WriteStringValue(NS_LITERAL_STRING("Background"),
