@@ -27,6 +27,10 @@ public:
   {
     rv = GetWholeText(aWholeText);
   }
+
+  static already_AddRefed<Text>
+  Constructor(const GlobalObject& aGlobal,
+              const nsAString& aData, ErrorResult& aRv);
 };
 
 } // namespace dom
