@@ -11,6 +11,10 @@
  * that ContentSecurityPolicy has to do.
  */
 
+// these identifiers are also defined in contentSecurityPolicy.manifest.
+const CSP_CLASS_ID = Components.ID("{d1680bb4-1ac0-4772-9437-1188375e44f2}");
+const CSP_CONTRACT_ID = "@mozilla.org/contentsecuritypolicy;1";
+
 /* :::::::: Constants and Helpers ::::::::::::::: */
 
 const Cc = Components.classes;
@@ -31,6 +35,8 @@ const ERROR_FLAG = Ci.nsIScriptError.ERROR_FLAG;
 const INLINE_STYLE_VIOLATION_OBSERVER_SUBJECT = 'violated base restriction: Inline Stylesheets will not apply';
 const INLINE_SCRIPT_VIOLATION_OBSERVER_SUBJECT = 'violated base restriction: Inline Scripts will not execute';
 const EVAL_VIOLATION_OBSERVER_SUBJECT = 'violated base restriction: Code will not be created from strings';
+const SCRIPT_NONCE_VIOLATION_OBSERVER_SUBJECT = 'Inline Script had invalid nonce'
+const STYLE_NONCE_VIOLATION_OBSERVER_SUBJECT = 'Inline Style had invalid nonce'
 
 // The cutoff length of content location in creating CSP cache key.
 const CSP_CACHE_URI_CUTOFF_SIZE = 512;
@@ -49,9 +55,9 @@ function ContentSecurityPolicy() {
 
   this._request = "";
   this._requestOrigin = "";
-  this._requestPrincipal = "";
+  this._weakRequestPrincipal =  { get : function() { return null; } };
   this._referrer = "";
-  this._docRequest = null;
+  this._weakDocRequest = { get : function() { return null; } };
   CSPdebug("CSP object initialized, no policies to enforce yet");
 
   this._cache = { };
@@ -88,6 +94,7 @@ function ContentSecurityPolicy() {
   csp._MAPPINGS[cp.TYPE_SUBDOCUMENT]       = cspr_sd_new.FRAME_SRC;
   csp._MAPPINGS[cp.TYPE_MEDIA]             = cspr_sd_new.MEDIA_SRC;
   csp._MAPPINGS[cp.TYPE_FONT]              = cspr_sd_new.FONT_SRC;
+  csp._MAPPINGS[cp.TYPE_XSLT]              = cspr_sd_new.SCRIPT_SRC;
 
   /* Our original CSP implementation's mappings for XHR and websocket
    * These should be changed to be = cspr_sd.CONNECT_SRC when we remove
@@ -118,8 +125,17 @@ function ContentSecurityPolicy() {
 }
 
 ContentSecurityPolicy.prototype = {
-  classID:          Components.ID("{d1680bb4-1ac0-4772-9437-1188375e44f2}"),
-  QueryInterface:   XPCOMUtils.generateQI([Ci.nsIContentSecurityPolicy]),
+  classID:          CSP_CLASS_ID,
+  QueryInterface:   XPCOMUtils.generateQI([Ci.nsIContentSecurityPolicy,
+                                           Ci.nsISerializable,
+                                           Ci.nsISupports]),
+
+  // Class info is required to be able to serialize
+  classInfo: XPCOMUtils.generateCI({classID: CSP_CLASS_ID,
+                                    contractID: CSP_CONTRACT_ID,
+                                    interfaces: [Ci.nsIContentSecurityPolicy,
+                                                 Ci.nsISerializable],
+                                    flags: Ci.nsIClassInfo.MAIN_THREAD_ONLY}),
 
   get isInitialized() {
     return this._isInitialized;
@@ -188,6 +204,28 @@ ContentSecurityPolicy.prototype = {
     });
   },
 
+  getAllowsNonce: function(aNonce, aContentType, shouldReportViolation) {
+    if (!CSPPrefObserver.experimentalEnabled)
+      return false;
+
+    if (!(aContentType == Ci.nsIContentPolicy.TYPE_SCRIPT ||
+          aContentType == Ci.nsIContentPolicy.TYPE_STYLESHEET)) {
+      CSPdebug("Nonce check requested for an invalid content type (not script or style): " + aContentType);
+      return false;
+    }
+    let ct = ContentSecurityPolicy._MAPPINGS[aContentType];
+
+    // allow it to execute?
+    let policyAllowsNonce = [ policy.permits(null, ct, aNonce) for (policy of this._policies) ];
+
+    shouldReportViolation.value = policyAllowsNonce.some(function(a) { return !a; });
+
+    // allow it to execute?  (Do all the policies allow it to execute)?
+    return this._policies.every(function(policy, i) {
+      return policy._reportOnlyMode || policyAllowsNonce[i];
+    });
+  },
+
   /**
    * For each policy, log any violation on the Error Console and send a report
    * if a report-uri is present in the policy
@@ -200,9 +238,13 @@ ContentSecurityPolicy.prototype = {
    *     sample of the violating content (to aid debugging)
    * @param aLineNum
    *     source line number of the violation (if available)
+   * @param aNonce
+   *     (optional) If this is a nonce violation, include the nonce should we
+   *     can recheck to determine which policies were violated and send the
+   *     appropriate reports.
    */
   logViolationDetails:
-  function(aViolationType, aSourceFile, aScriptSample, aLineNum, violatedPolicyIndex) {
+  function(aViolationType, aSourceFile, aScriptSample, aLineNum, aNonce) {
     for (let policyIndex=0; policyIndex < this._policies.length; policyIndex++) {
       let policy = this._policies[policyIndex];
 
@@ -236,6 +278,24 @@ ContentSecurityPolicy.prototype = {
                                     aSourceFile, aScriptSample, aLineNum);
         }
         break;
+      case Ci.nsIContentSecurityPolicy.VIOLATION_TYPE_NONCE_SCRIPT:
+        let scriptType = ContentSecurityPolicy._MAPPINGS[Ci.nsIContentPolicy.TYPE_SCRIPT];
+        if (!policy.permits(null, scriptType, aNonce)) {
+          var violatedDirective = this._buildViolatedDirectiveString('SCRIPT_SRC', policy);
+          this._asyncReportViolation('self', null, violatedDirective, policyIndex,
+                                     SCRIPT_NONCE_VIOLATION_OBSERVER_SUBJECT,
+                                     aSourceFile, aScriptSample, aLineNum);
+        }
+        break;
+      case Ci.nsIContentSecurityPolicy.VIOLATION_TYPE_NONCE_STYLE:
+        let styleType = ContentSecurityPolicy._MAPPINGS[Ci.nsIContentPolicy.TYPE_STYLE];
+        if (!policy.permits(null, styleType, aNonce)) {
+          var violatedDirective = this._buildViolatedDirectiveString('STYLE_SRC', policy);
+          this._asyncReportViolation('self', null, violatedDirective, policyIndex,
+                                     STYLE_NONCE_VIOLATION_OBSERVER_SUBJECT,
+                                     aSourceFile, aScriptSample, aLineNum);
+        }
+        break;
       }
     }
   },
@@ -243,27 +303,45 @@ ContentSecurityPolicy.prototype = {
   /**
    * Given an nsIHttpChannel, fill out the appropriate data.
    */
-  scanRequestData:
-  function(aChannel) {
-    if (!aChannel)
+  setRequestContext:
+  function(aSelfURI, aReferrerURI, aPrincipal, aChannel) {
+
+    // this requires either a self URI or a http channel.
+    if (!aSelfURI && !aChannel)
       return;
 
-    // Save the docRequest for fetching a policy-uri
-    this._docRequest = aChannel;
+    if (aChannel) {
+      // Save the docRequest for fetching a policy-uri
+      this._weakDocRequest = Cu.getWeakReference(aChannel);
+    }
 
     // save the document URI (minus <fragment>) and referrer for reporting
-    let uri = aChannel.URI.cloneIgnoringRef();
+    let uri = aSelfURI ? aSelfURI.cloneIgnoringRef() : aChannel.URI.cloneIgnoringRef();
     try { // GetUserPass throws for some protocols without userPass
       uri.userPass = '';
     } catch (ex) {}
     this._request = uri.asciiSpec;
     this._requestOrigin = uri;
 
-    //store a reference to the principal, that can later be used in shouldLoad
-    this._requestPrincipal = Components.classes["@mozilla.org/scriptsecuritymanager;1"].
-    getService(Components.interfaces.nsIScriptSecurityManager).getChannelPrincipal(aChannel);
+    // store a reference to the principal, that can later be used in shouldLoad
+    if (aPrincipal) {
+      this._weakRequestPrincipal = Cu.getWeakReference(aPrincipal);
+    } else if (aChannel) {
+      this._weakRequestPrincipal = Cu.getWeakReference(Cc["@mozilla.org/scriptsecuritymanager;1"]
+                                                         .getService(Ci.nsIScriptSecurityManager)
+                                                         .getChannelPrincipal(aChannel));
+    } else {
+      CSPdebug("No principal or channel for document context; violation reports may not work.");
+    }
 
-    if (aChannel.referrer) {
+    // pick one: referrerURI, channel's referrer, or null (first available)
+    let ref = null;
+    if (aReferrerURI)
+      ref = aReferrerURI;
+    else if (aChannel instanceof Ci.nsIHttpChannel)
+      ref = aChannel.referrer;
+
+    if (ref) {
       let referrer = aChannel.referrer.cloneIgnoringRef();
       try { // GetUserPass throws for some protocols without userPass
         referrer.userPass = '';
@@ -282,7 +360,7 @@ ContentSecurityPolicy.prototype = {
   function csp_appendPolicy(aPolicy, selfURI, aReportOnly, aSpecCompliant) {
 #ifndef MOZ_B2G
     CSPdebug("APPENDING POLICY: " + aPolicy);
-    CSPdebug("            SELF: " + selfURI.asciiSpec);
+    CSPdebug("            SELF: " + (selfURI ? selfURI.asciiSpec : " null"));
     CSPdebug("CSP 1.0 COMPLIANT : " + aSpecCompliant);
 #endif
 
@@ -311,13 +389,13 @@ ContentSecurityPolicy.prototype = {
       newpolicy = CSPRep.fromStringSpecCompliant(aPolicy,
                                                  selfURI,
                                                  aReportOnly,
-                                                 this._docRequest,
+                                                 this._weakDocRequest.get(),
                                                  this);
     } else {
       newpolicy = CSPRep.fromString(aPolicy,
                                     selfURI,
                                     aReportOnly,
-                                    this._docRequest,
+                                    this._weakDocRequest.get(),
                                     this);
     }
 
@@ -434,8 +512,8 @@ ContentSecurityPolicy.prototype = {
           // we need to set an nsIChannelEventSink on the channel object
           // so we can tell it to not follow redirects when posting the reports
           chan.notificationCallbacks = new CSPReportRedirectSink(policy);
-          if (this._docRequest) {
-            chan.loadGroup = this._docRequest.loadGroup;
+          if (this._weakDocRequest.get()) {
+            chan.loadGroup = this._weakDocRequest.get().loadGroup;
           }
 
           chan.QueryInterface(Ci.nsIUploadChannel)
@@ -454,7 +532,7 @@ ContentSecurityPolicy.prototype = {
                                   .getService(Ci.nsIContentPolicy);
             if (contentPolicy.shouldLoad(Ci.nsIContentPolicy.TYPE_CSP_REPORT,
                                          chan.URI, this._requestOrigin,
-                                         null, null, null, this._requestPrincipal)
+                                         null, null, null, this._weakRequestPrincipal.get())
                 != Ci.nsIContentPolicy.ACCEPT) {
               continue; // skip unauthorized URIs
             }
@@ -620,8 +698,6 @@ ContentSecurityPolicy.prototype = {
     CSPdebug("shouldLoad location = " + aContentLocation.asciiSpec);
     CSPdebug("shouldLoad content type = " + aContentType);
 #endif
-    // interpret the context, and then pass off to the decision structure
-    var cspContext = ContentSecurityPolicy._MAPPINGS[aContentType];
 
     // The mapping for XHR and websockets is different between our original
     // implementation and the 1.0 spec, we handle this here.
@@ -629,6 +705,14 @@ ContentSecurityPolicy.prototype = {
 
     let cp = Ci.nsIContentPolicy;
 
+    // Infer if this is a preload for elements that use nonce-source. Since,
+    // for preloads, aContext is the document and not the element associated
+    // with the resource, we cannot determine the nonce. See Bug 612921 and
+    // Bug 855326.
+    var possiblePreloadNonceConflict =
+      (aContentType == cp.TYPE_SCRIPT || aContentType == cp.TYPE_STYLESHEET) &&
+      aContext instanceof Ci.nsIDOMHTMLDocument;
+
     // iterate through all the _policies and send reports where a policy is
     // violated.  After the check, determine the overall effect (blocked or
     // loaded?) and cache it.
@@ -662,15 +746,18 @@ ContentSecurityPolicy.prototype = {
 
       // otherwise, honor the translation
       // var source = aContentLocation.scheme + "://" + aContentLocation.hostPort;
-      var res = policy.permits(aContentLocation, cspContext)
-                ? cp.ACCEPT : cp.REJECT_SERVER;
+      let context = CSPPrefObserver.experimentalEnabled ? aContext : null;
+      var res = policy.permits(aContentLocation, cspContext, context) ?
+                cp.ACCEPT : cp.REJECT_SERVER;
       // record whether the thing should be blocked or just reported.
       policyAllowsLoadArray.push(res == cp.ACCEPT || policy._reportOnlyMode);
 
       // frame-ancestors is taken care of early on (as this document is loaded)
 
       // If the result is *NOT* ACCEPT, then send report
-      if (res != Ci.nsIContentPolicy.ACCEPT) {
+      // Do not send report if this is a nonce-source preload - the decision may
+      // be wrong and will incorrectly fail the unit tests.
+      if (res != Ci.nsIContentPolicy.ACCEPT && !possiblePreloadNonceConflict) {
         CSPdebug("blocking request for " + aContentLocation.asciiSpec);
         try {
           let directive = "unknown directive",
@@ -705,7 +792,8 @@ ContentSecurityPolicy.prototype = {
     let ret = (policyAllowsLoadArray.some(function(a,b) { return !a; }) ?
                cp.REJECT_SERVER : cp.ACCEPT);
 
-    if (key) {
+    // Do not cache the result if this is a nonce-source preload
+    if (key && !possiblePreloadNonceConflict) {
       this._cache[key] = ret;
     }
     return ret;
@@ -784,6 +872,51 @@ ContentSecurityPolicy.prototype = {
 
       }, Ci.nsIThread.DISPATCH_NORMAL);
   },
+
+/* ........ nsISerializable Methods: .............. */
+
+  read:
+  function(aStream) {
+
+    this._requestOrigin = aStream.readObject(true);
+    this._requestOrigin.QueryInterface(Ci.nsIURI);
+
+    for (let pCount = aStream.read32(); pCount > 0; pCount--) {
+      let polStr        = aStream.readString();
+      let reportOnly    = aStream.readBoolean();
+      let specCompliant = aStream.readBoolean();
+      // don't need self info because when the policy is turned back into a
+      // string, 'self' is replaced with the explicit source expression.
+      this.appendPolicy(polStr, null, reportOnly, specCompliant);
+    }
+
+    // NOTE: the document instance that's deserializing this object (via its
+    // principal) should hook itself into this._principal manually.  If they
+    // don't, the CSP reports will likely be blocked by nsMixedContentBlocker.
+  },
+
+  write:
+  function(aStream) {
+    // need to serialize the context: request origin and such. They are
+    // used when sending reports.  Since _request and _requestOrigin are just
+    // different representations of the same thing, only save _requestOrigin
+    // (an nsIURI).
+    aStream.writeCompoundObject(this._requestOrigin, Ci.nsIURI, true);
+
+    // we can't serialize a reference to the principal that triggered this
+    // instance to serialize, so when this is deserialized by the principal the
+    // caller must hook it up manually by calling setRequestContext on this
+    // instance with the appropriate nsIChannel.
+
+    // Finally, serialize all the policies.
+    aStream.write32(this._policies.length);
+
+    for each (var policy in this._policies) {
+      aStream.writeWStringZ(policy.toString());
+      aStream.writeBoolean(policy._reportOnlyMode);
+      aStream.writeBoolean(policy._specCompliant);
+    }
+  },
 };
 
 // The POST of the violation report (if it happens) should not follow
