@@ -236,7 +236,7 @@ public:
 
   void SetParameters(bool aAnon, bool aSystem)
   {
-    mIsAnon = aAnon;
+    mIsAnon = aAnon || aSystem;
     mIsSystem = aSystem;
   }
 
@@ -282,6 +282,13 @@ public:
   uint16_t ReadyState();
 
   // request
+  void Open(const nsACString& aMethod, const nsAString& aUrl, ErrorResult& aRv)
+  {
+    Open(aMethod, aUrl, true,
+         mozilla::dom::Optional<nsAString>(),
+         mozilla::dom::Optional<nsAString>(),
+         aRv);
+  }
   void Open(const nsACString& aMethod, const nsAString& aUrl, bool aAsync,
             const mozilla::dom::Optional<nsAString>& aUser,
             const mozilla::dom::Optional<nsAString>& aPassword,
