@@ -6,8 +6,7 @@
 
 #include "mozilla/dom/ImageData.h"
 
-#include "nsDOMClassInfoID.h"
-#include "nsContentUtils.h"
+#include "mozilla/HoldDropJSObjects.h"
 #include "mozilla/dom/ImageDataBinding.h"
 
 #include "jsapi.h"
@@ -18,6 +17,8 @@ namespace dom {
 NS_IMPL_CYCLE_COLLECTING_ADDREF(ImageData)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(ImageData)
 
+NS_IMPL_CYCLE_COLLECTION_CLASS(ImageData)
+
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(ImageData)
   NS_INTERFACE_MAP_ENTRY(nsISupports)
 NS_INTERFACE_MAP_END
@@ -37,20 +38,20 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 void
 ImageData::HoldData()
 {
-  NS_HOLD_JS_OBJECTS(this, ImageData);
+  mozilla::HoldJSObjects(this);
 }
 
 void
 ImageData::DropData()
 {
   if (mData) {
-    mData = NULL;
-    NS_DROP_JS_OBJECTS(this, ImageData);
+    mData = nullptr;
+    mozilla::DropJSObjects(this);
   }
 }
 
 JSObject*
-ImageData::WrapObject(JSContext* cx, JSObject* scope)
+ImageData::WrapObject(JSContext* cx, JS::Handle<JSObject*> scope)
 {
   return ImageDataBinding::Wrap(cx, scope, this);
 }
