@@ -5,6 +5,8 @@
 
 #include "nsDOMBeforeUnloadEvent.h"
 
+using namespace mozilla;
+
 NS_IMPL_ADDREF_INHERITED(nsDOMBeforeUnloadEvent, nsDOMEvent)
 NS_IMPL_RELEASE_INHERITED(nsDOMBeforeUnloadEvent, nsDOMEvent)
 
@@ -29,7 +31,7 @@ nsDOMBeforeUnloadEvent::GetReturnValue(nsAString& aReturnValue)
 nsresult NS_NewDOMBeforeUnloadEvent(nsIDOMEvent** aInstancePtrResult,
                                     mozilla::dom::EventTarget* aOwner,
                                     nsPresContext* aPresContext,
-                                    nsEvent *aEvent) 
+                                    WidgetEvent* aEvent) 
 {
   nsDOMBeforeUnloadEvent* it =
     new nsDOMBeforeUnloadEvent(aOwner, aPresContext, aEvent);
