@@ -129,14 +129,6 @@ public:
     return NS_ERROR_FAILURE;
   }
 
-#ifdef MOZ_DASH
-  virtual nsresult OpenByteRange(nsIStreamListener** aStreamListener,
-                                 MediaByteRange const &aByteRange)
-  {
-    return NS_ERROR_FAILURE;
-  }
-#endif
-
   virtual nsresult GetCachedRanges(nsTArray<MediaByteRange>& aRanges)
   {
     aRanges.AppendElement(MediaByteRange(0, mLength));
