@@ -269,9 +269,9 @@ MediaDecodeTask::Decode()
 
   mDecoderReader->OnDecodeThreadStart();
 
-  VideoInfo videoInfo;
+  MediaInfo mediaInfo;
   nsAutoPtr<MetadataTags> tags;
-  nsresult rv = mDecoderReader->ReadMetadata(&videoInfo, getter_Transfers(tags));
+  nsresult rv = mDecoderReader->ReadMetadata(&mediaInfo, getter_Transfers(tags));
   if (NS_FAILED(rv)) {
     ReportFailureOnMainThread(WebAudioDecodeJob::InvalidContent);
     return;
@@ -291,8 +291,8 @@ MediaDecodeTask::Decode()
 
   MediaQueue<AudioData>& audioQueue = mDecoderReader->AudioQueue();
   uint32_t frameCount = audioQueue.FrameCount();
-  uint32_t channelCount = videoInfo.mAudioChannels;
-  uint32_t sampleRate = videoInfo.mAudioRate;
+  uint32_t channelCount = mediaInfo.mAudio.mChannels;
+  uint32_t sampleRate = mediaInfo.mAudio.mRate;
 
   if (!frameCount || !channelCount || !sampleRate) {
     ReportFailureOnMainThread(WebAudioDecodeJob::InvalidContent);
@@ -346,15 +346,11 @@ MediaDecodeTask::Decode()
       (audioData->mAudioBuffer->Data());
 
     if (sampleRate != destSampleRate) {
-      const uint32_t expectedOutSamples = static_cast<uint32_t>(
-          static_cast<uint64_t>(destSampleRate) *
-          static_cast<uint64_t>(audioData->mFrames) /
-          static_cast<uint64_t>(sampleRate)
-        );
+      const uint32_t maxOutSamples = resampledFrames - mDecodeJob.mWriteIndex;
 
       for (uint32_t i = 0; i < audioData->mChannels; ++i) {
         uint32_t inSamples = audioData->mFrames;
-        uint32_t outSamples = expectedOutSamples;
+        uint32_t outSamples = maxOutSamples;
 
         WebAudioUtils::SpeexResamplerProcess(
             resampler, i, &bufferData[i * audioData->mFrames], &inSamples,
@@ -364,6 +360,7 @@ MediaDecodeTask::Decode()
         if (i == audioData->mChannels - 1) {
           mDecodeJob.mWriteIndex += outSamples;
           MOZ_ASSERT(mDecodeJob.mWriteIndex <= resampledFrames);
+          MOZ_ASSERT(inSamples == audioData->mFrames);
         }
       }
     } else {
@@ -380,32 +377,23 @@ MediaDecodeTask::Decode()
   }
 
   if (sampleRate != destSampleRate) {
-    int inputLatency = speex_resampler_get_input_latency(resampler);
-    int outputLatency = speex_resampler_get_output_latency(resampler);
-    AudioDataValue* zero = (AudioDataValue*)calloc(inputLatency, sizeof(AudioDataValue));
-
-    if (!zero) {
-      // Out of memory!
-      ReportFailureOnMainThread(WebAudioDecodeJob::UnknownError);
-      return;
-    }
-
+    uint32_t inputLatency = speex_resampler_get_input_latency(resampler);
+    const uint32_t maxOutSamples = resampledFrames - mDecodeJob.mWriteIndex;
     for (uint32_t i = 0; i < channelCount; ++i) {
       uint32_t inSamples = inputLatency;
-      uint32_t outSamples = outputLatency;
+      uint32_t outSamples = maxOutSamples;
 
       WebAudioUtils::SpeexResamplerProcess(
-          resampler, i, zero, &inSamples,
+          resampler, i, (AudioDataValue*)nullptr, &inSamples,
           mDecodeJob.mChannelBuffers[i] + mDecodeJob.mWriteIndex,
           &outSamples);
 
       if (i == channelCount - 1) {
         mDecodeJob.mWriteIndex += outSamples;
         MOZ_ASSERT(mDecodeJob.mWriteIndex <= resampledFrames);
+        MOZ_ASSERT(inSamples == inputLatency);
       }
     }
-
-    free(zero);
   }
 
   mPhase = PhaseEnum::AllocateBuffer;
@@ -539,6 +527,17 @@ MediaBufferDecoder::EnsureThreadPoolInitialized()
   return true;
 }
 
+void
+MediaBufferDecoder::Shutdown() {
+  if (mThreadPool) {
+    // Setting threadLimit to 0 causes threads to exit when all events have
+    // been run, like nsIThreadPool::Shutdown(), but doesn't run a nested event
+    // loop nor wait until this has happened.
+    mThreadPool->SetThreadLimit(0);
+    mThreadPool = nullptr;
+  }
+}
+
 WebAudioDecodeJob::WebAudioDecodeJob(const nsACString& aContentType,
                                      AudioContext* aContext,
                                      const ArrayBuffer& aBuffer,
