@@ -14,22 +14,8 @@
 namespace mozilla {
 namespace dom {
 
-NS_IMPL_CYCLE_COLLECTION_CLASS(OscillatorNode)
-
-NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(OscillatorNode)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK(mPeriodicWave)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK(mFrequency)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK(mDetune)
-  if (tmp->Context()) {
-    tmp->Context()->UnregisterOscillatorNode(tmp);
-  }
-NS_IMPL_CYCLE_COLLECTION_UNLINK_END_INHERITED(AudioNode);
-
-NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(OscillatorNode, AudioNode)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mPeriodicWave)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mFrequency)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mDetune)
-NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
+NS_IMPL_CYCLE_COLLECTION_INHERITED_3(OscillatorNode, AudioNode,
+                                     mPeriodicWave, mFrequency, mDetune)
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION_INHERITED(OscillatorNode)
 NS_INTERFACE_MAP_END_INHERITING(AudioNode)
@@ -390,7 +376,7 @@ public:
     float* lowerWaveData = nullptr;
     float tableInterpolationFactor;
     float rate = 1.0 / mSource->SampleRate();
-
+ 
     for (uint32_t i = aStart; i < aEnd; ++i) {
       UpdateParametersIfNeeded(ticks, i);
       mPeriodicWave->waveDataForFundamentalFrequency(mFinalFrequency,
@@ -436,17 +422,17 @@ public:
       return;
     }
 
-    if (ticks + WEBAUDIO_BLOCK_SIZE < mStart) {
-      // We're not playing yet.
-      ComputeSilence(aOutput);
-      return;
-    }
     if (ticks >= mStop) {
       // We've finished playing.
       ComputeSilence(aOutput);
       *aFinished = true;
       return;
     }
+    if (ticks + WEBAUDIO_BLOCK_SIZE < mStart) {
+      // We're not playing yet.
+      ComputeSilence(aOutput);
+      return;
+    }
 
     AllocateAudioBlock(1, aOutput);
     float* output = static_cast<float*>(
@@ -518,13 +504,11 @@ OscillatorNode::OscillatorNode(AudioContext* aContext)
   OscillatorNodeEngine* engine = new OscillatorNodeEngine(this, aContext->Destination());
   mStream = aContext->Graph()->CreateAudioNodeStream(engine, MediaStreamGraph::SOURCE_STREAM);
   engine->SetSourceStream(static_cast<AudioNodeStream*> (mStream.get()));
+  mStream->AddMainThreadListener(this);
 }
 
 OscillatorNode::~OscillatorNode()
 {
-  if (Context()) {
-    Context()->UnregisterOscillatorNode(this);
-  }
 }
 
 JSObject*
@@ -593,11 +577,9 @@ OscillatorNode::Start(double aWhen, ErrorResult& aRv)
 
   // TODO: Perhaps we need to do more here.
   ns->SetStreamTimeParameter(OscillatorNodeEngine::START,
-                             Context()->DestinationStream(),
-                             aWhen);
+                             Context(), aWhen);
 
-  MOZ_ASSERT(!mPlayingRef, "We can only accept a successful start() call once");
-  mPlayingRef.Take(this);
+  MarkActive();
 }
 
 void
@@ -613,8 +595,6 @@ OscillatorNode::Stop(double aWhen, ErrorResult& aRv)
     return;
   }
 
-  mPlayingRef.Drop(this);
-
   AudioNodeStream* ns = static_cast<AudioNodeStream*>(mStream.get());
   if (!ns || !Context()) {
     // We've already stopped and had our stream shut down
@@ -623,8 +603,7 @@ OscillatorNode::Stop(double aWhen, ErrorResult& aRv)
 
   // TODO: Perhaps we need to do more here.
   ns->SetStreamTimeParameter(OscillatorNodeEngine::STOP,
-                             Context()->DestinationStream(),
-                             std::max(0.0, aWhen));
+                             Context(), std::max(0.0, aWhen));
 }
 
 void
@@ -658,7 +637,7 @@ OscillatorNode::NotifyMainThreadStateChanged()
 
     // Drop the playing reference
     // Warning: The below line might delete this.
-    mPlayingRef.Drop(this);
+    MarkInactive();
   }
 }
 
