@@ -105,6 +105,7 @@ WMFReader::InitializeDXVA()
   if (!Preferences::GetBool("media.windows-media-foundation.use-dxva", false)) {
     return false;
   }
+  MOZ_ASSERT(mDecoder->GetImageContainer());
 
   // Extract the layer manager backend type so that we can determine
   // whether it's worthwhile using DXVA. If we're not running with a D3D
@@ -114,7 +115,7 @@ WMFReader::InitializeDXVA()
   MediaDecoderOwner* owner = mDecoder->GetOwner();
   NS_ENSURE_TRUE(owner, false);
 
-  HTMLMediaElement* element = owner->GetMediaElement();
+  dom::HTMLMediaElement* element = owner->GetMediaElement();
   NS_ENSURE_TRUE(element, false);
 
   nsRefPtr<LayerManager> layerManager =
@@ -131,16 +132,6 @@ WMFReader::InitializeDXVA()
   return mDXVA2Manager != nullptr;
 }
 
-static bool
-IsVideoContentType(const nsCString& aContentType)
-{
-  NS_NAMED_LITERAL_CSTRING(video, "video");
-  if (FindInReadable(video, aContentType)) {
-    return true;
-  }
-  return false;
-}
-
 nsresult
 WMFReader::Init(MediaDecoderReader* aCloneDonor)
 {
@@ -161,7 +152,8 @@ WMFReader::Init(MediaDecoderReader* aCloneDonor)
   rv = mByteStream->Init();
   NS_ENSURE_SUCCESS(rv, rv);
 
-  if (IsVideoContentType(mDecoder->GetResource()->GetContentType())) {
+  if (mDecoder->GetImageContainer() != nullptr &&
+      IsVideoContentType(mDecoder->GetResource()->GetContentType())) {
     mUseHwAccel = InitializeDXVA();
   } else {
     mUseHwAccel = false;
@@ -239,7 +231,7 @@ ConfigureSourceReaderStream(IMFSourceReader *aReader,
 
   // Set the uncompressed format. This can fail if the decoder can't produce
   // that type.
-  return aReader->SetCurrentMediaType(aStreamIndex, NULL, type);
+  return aReader->SetCurrentMediaType(aStreamIndex, nullptr, type);
 }
 
 // Returns the duration of the resource, in microseconds.
@@ -284,93 +276,6 @@ GetSourceReaderCanSeek(IMFSourceReader* aReader, bool& aOutCanSeek)
   return S_OK;
 }
 
-static HRESULT
-GetDefaultStride(IMFMediaType *aType, uint32_t* aOutStride)
-{
-  // Try to get the default stride from the media type.
-  HRESULT hr = aType->GetUINT32(MF_MT_DEFAULT_STRIDE, aOutStride);
-  if (SUCCEEDED(hr)) {
-    return S_OK;
-  }
-
-  // Stride attribute not set, calculate it.
-  GUID subtype = GUID_NULL;
-  uint32_t width = 0;
-  uint32_t height = 0;
-
-  hr = aType->GetGUID(MF_MT_SUBTYPE, &subtype);
-  NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
-
-  hr = MFGetAttributeSize(aType, MF_MT_FRAME_SIZE, &width, &height);
-  NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
-
-  hr = wmf::MFGetStrideForBitmapInfoHeader(subtype.Data1, width, (LONG*)(aOutStride));
-  NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
-
-  return hr;
-}
-
-static int32_t
-MFOffsetToInt32(const MFOffset& aOffset)
-{
-  return int32_t(aOffset.value + (aOffset.fract / 65536.0f));
-}
-
-// Gets the sub-region of the video frame that should be displayed.
-// See: http://msdn.microsoft.com/en-us/library/windows/desktop/bb530115(v=vs.85).aspx
-static HRESULT
-GetPictureRegion(IMFMediaType* aMediaType, nsIntRect& aOutPictureRegion)
-{
-  // Determine if "pan and scan" is enabled for this media. If it is, we
-  // only display a region of the video frame, not the entire frame.
-  BOOL panScan = MFGetAttributeUINT32(aMediaType, MF_MT_PAN_SCAN_ENABLED, FALSE);
-
-  // If pan and scan mode is enabled. Try to get the display region.
-  HRESULT hr = E_FAIL;
-  MFVideoArea videoArea;
-  memset(&videoArea, 0, sizeof(MFVideoArea));
-  if (panScan) {
-    hr = aMediaType->GetBlob(MF_MT_PAN_SCAN_APERTURE,
-                             (UINT8*)&videoArea,
-                             sizeof(MFVideoArea),
-                             NULL);
-  }
-
-  // If we're not in pan-and-scan mode, or the pan-and-scan region is not set,
-  // check for a minimimum display aperture.
-  if (!panScan || hr == MF_E_ATTRIBUTENOTFOUND) {
-    hr = aMediaType->GetBlob(MF_MT_MINIMUM_DISPLAY_APERTURE,
-                             (UINT8*)&videoArea,
-                             sizeof(MFVideoArea),
-                             NULL);
-  }
-
-  if (hr == MF_E_ATTRIBUTENOTFOUND) {
-    // Minimum display aperture is not set, for "backward compatibility with
-    // some components", check for a geometric aperture.
-    hr = aMediaType->GetBlob(MF_MT_GEOMETRIC_APERTURE,
-                             (UINT8*)&videoArea,
-                             sizeof(MFVideoArea),
-                             NULL);
-  }
-
-  if (SUCCEEDED(hr)) {
-    // The media specified a picture region, return it.
-    aOutPictureRegion = nsIntRect(MFOffsetToInt32(videoArea.OffsetX),
-                                  MFOffsetToInt32(videoArea.OffsetY),
-                                  videoArea.Area.cx,
-                                  videoArea.Area.cy);
-    return S_OK;
-  }
-
-  // No picture region defined, fall back to using the entire video area.
-  UINT32 width = 0, height = 0;
-  hr = MFGetAttributeSize(aMediaType, MF_MT_FRAME_SIZE, &width, &height);
-  NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
-  aOutPictureRegion = nsIntRect(0, 0, width, height);
-  return S_OK;
-}
-
 HRESULT
 WMFReader::ConfigureVideoFrameGeometry(IMFMediaType* aMediaType)
 {
@@ -414,7 +319,7 @@ WMFReader::ConfigureVideoFrameGeometry(IMFMediaType* aMediaType)
   }
 
   // Success! Save state.
-  mInfo.mDisplay = displaySize;
+  mInfo.mVideo.mDisplay = displaySize;
   GetDefaultStride(aMediaType, &mVideoStride);
   mVideoWidth = width;
   mVideoHeight = height;
@@ -442,6 +347,11 @@ WMFReader::ConfigureVideoDecoder()
     return S_OK;
   }
 
+  if (!mDecoder->GetImageContainer()) {
+    // We can't display the video, so don't bother to decode; disable the stream.
+    return mSourceReader->SetStreamSelection(MF_SOURCE_READER_FIRST_VIDEO_STREAM, FALSE);
+  }
+
   static const GUID MP4VideoTypes[] = {
     MFVideoFormat_H264
   };
@@ -470,7 +380,7 @@ WMFReader::ConfigureVideoDecoder()
 
   LOG("Successfully configured video stream");
 
-  mHasVideo = mInfo.mHasVideo = true;
+  mHasVideo = mInfo.mVideo.mHasVideo = true;
 
   return S_OK;
 }
@@ -482,9 +392,12 @@ WMFReader::GetSupportedAudioCodecs(const GUID** aCodecs, uint32_t* aNumCodecs)
   MOZ_ASSERT(aNumCodecs);
 
   if (mIsMP3Enabled) {
+    GUID aacOrMp3 = MFMPEG4Format_Base;
+    aacOrMp3.Data1 = 0x6D703461;// FOURCC('m','p','4','a');
     static const GUID codecs[] = {
       MFAudioFormat_AAC,
-      MFAudioFormat_MP3
+      MFAudioFormat_MP3,
+      aacOrMp3
     };
     *aCodecs = codecs;
     *aNumCodecs = NS_ARRAY_LENGTH(codecs);
@@ -534,9 +447,9 @@ WMFReader::ConfigureAudioDecoder()
   mAudioChannels = MFGetAttributeUINT32(mediaType, MF_MT_AUDIO_NUM_CHANNELS, 0);
   mAudioBytesPerSample = MFGetAttributeUINT32(mediaType, MF_MT_AUDIO_BITS_PER_SAMPLE, 16) / 8;
 
-  mInfo.mAudioChannels = mAudioChannels;
-  mInfo.mAudioRate = mAudioRate;
-  mHasAudio = mInfo.mHasAudio = true;
+  mInfo.mAudio.mChannels = mAudioChannels;
+  mInfo.mAudio.mRate = mAudioRate;
+  mHasAudio = mInfo.mAudio.mHasAudio = true;
 
   LOG("Successfully configured audio stream. rate=%u channels=%u bitsPerSample=%u",
       mAudioRate, mAudioChannels, mAudioBytesPerSample);
@@ -545,7 +458,7 @@ WMFReader::ConfigureAudioDecoder()
 }
 
 nsresult
-WMFReader::ReadMetadata(VideoInfo* aInfo,
+WMFReader::ReadMetadata(MediaInfo* aInfo,
                         MetadataTags** aTags)
 {
   NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
@@ -578,7 +491,7 @@ WMFReader::ReadMetadata(VideoInfo* aInfo,
   hr = ConfigureAudioDecoder();
   NS_ENSURE_TRUE(SUCCEEDED(hr), NS_ERROR_FAILURE);
 
-  if (mUseHwAccel && mInfo.mHasVideo) {
+  if (mUseHwAccel && mInfo.mVideo.mHasVideo) {
     RefPtr<IMFTransform> videoDecoder;
     hr = mSourceReader->GetServiceForStream(MF_SOURCE_READER_FIRST_VIDEO_STREAM,
                                             GUID_NULL,
@@ -608,12 +521,12 @@ WMFReader::ReadMetadata(VideoInfo* aInfo,
       hr = ConfigureVideoDecoder();
     }
   }
-  if (mInfo.mHasVideo) {
+  if (mInfo.HasVideo()) {
     LOG("Using DXVA: %s", (mUseHwAccel ? "Yes" : "No"));
   }
 
   // Abort if both video and audio failed to initialize.
-  NS_ENSURE_TRUE(mInfo.mHasAudio || mInfo.mHasVideo, NS_ERROR_FAILURE);
+  NS_ENSURE_TRUE(mInfo.HasValidMedia(), NS_ERROR_FAILURE);
 
   // Get the duration, and report it to the decoder if we have it.
   int64_t duration = 0;
@@ -639,39 +552,6 @@ WMFReader::ReadMetadata(VideoInfo* aInfo,
   return NS_OK;
 }
 
-static int64_t
-GetSampleDuration(IMFSample* aSample)
-{
-  int64_t duration = 0;
-  aSample->GetSampleDuration(&duration);
-  return HNsToUsecs(duration);
-}
-
-HRESULT
-HNsToFrames(int64_t aHNs, uint32_t aRate, int64_t* aOutFrames)
-{
-  MOZ_ASSERT(aOutFrames);
-  const int64_t HNS_PER_S = USECS_PER_S * 10;
-  CheckedInt<int64_t> i = aHNs;
-  i *= aRate;
-  i /= HNS_PER_S;
-  NS_ENSURE_TRUE(i.isValid(), E_FAIL);
-  *aOutFrames = i.value();
-  return S_OK;
-}
-
-HRESULT
-FramesToUsecs(int64_t aSamples, uint32_t aRate, int64_t* aOutUsecs)
-{
-  MOZ_ASSERT(aOutUsecs);
-  CheckedInt<int64_t> i = aSamples;
-  i *= USECS_PER_S;
-  i /= aRate;
-  NS_ENSURE_TRUE(i.isValid(), E_FAIL);
-  *aOutUsecs = i.value();
-  return S_OK;
-}
-
 bool
 WMFReader::DecodeAudioData()
 {
@@ -763,8 +643,6 @@ WMFReader::DecodeAudioData()
       timestamp, duration, currentLength);
   #endif
 
-  NotifyBytesConsumed();
-
   return true;
 }
 
@@ -798,7 +676,7 @@ WMFReader::CreateBasicVideoFrame(IMFSample* aSample,
     hr = twoDBuffer->Lock2D(&data, &stride);
     NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
   } else {
-    hr = buffer->Lock(&data, NULL, NULL);
+    hr = buffer->Lock(&data, nullptr, nullptr);
     NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
     stride = mVideoStride;
   }
@@ -843,11 +721,11 @@ WMFReader::CreateBasicVideoFrame(IMFSample* aSample,
   b.mPlanes[2].mOffset = 0;
   b.mPlanes[2].mSkip = 0;
 
-  VideoData *v = VideoData::Create(mInfo,
+  VideoData *v = VideoData::Create(mInfo.mVideo,
                                    mDecoder->GetImageContainer(),
                                    aOffsetBytes,
                                    aTimestampUsecs,
-                                   aTimestampUsecs + aDurationUsecs,
+                                   aDurationUsecs,
                                    b,
                                    false,
                                    -1,
@@ -886,11 +764,11 @@ WMFReader::CreateD3DVideoFrame(IMFSample* aSample,
   NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
   NS_ENSURE_TRUE(image, E_FAIL);
 
-  VideoData *v = VideoData::CreateFromImage(mInfo,
+  VideoData *v = VideoData::CreateFromImage(mInfo.mVideo,
                                             mDecoder->GetImageContainer(),
                                             aOffsetBytes,
                                             aTimestampUsecs,
-                                            aTimestampUsecs + aDurationUsecs,
+                                            aDurationUsecs,
                                             image.forget(),
                                             false,
                                             -1,
@@ -993,20 +871,9 @@ WMFReader::DecodeVideoFrame(bool &aKeyframeSkip,
     return false;
   }
 
-  NotifyBytesConsumed();
-
   return true;
 }
 
-void
-WMFReader::NotifyBytesConsumed()
-{
-  uint32_t bytesConsumed = mByteStream->GetAndResetBytesConsumedCount();
-  if (bytesConsumed > 0) {
-    mDecoder->NotifyBytesConsumed(bytesConsumed);
-  }
-}
-
 nsresult
 WMFReader::Seek(int64_t aTargetUs,
                 int64_t aStartTime,
@@ -1041,17 +908,4 @@ WMFReader::Seek(int64_t aTargetUs,
   return DecodeToTarget(aTargetUs);
 }
 
-nsresult
-WMFReader::GetBuffered(mozilla::dom::TimeRanges* aBuffered, int64_t aStartTime)
-{
-  MediaResource* stream = mDecoder->GetResource();
-  int64_t durationUs = 0;
-  {
-    ReentrantMonitorAutoEnter mon(mDecoder->GetReentrantMonitor());
-    durationUs = mDecoder->GetMediaDuration();
-  }
-  GetEstimatedBufferedTimeRanges(stream, durationUs, aBuffered);
-  return NS_OK;
-}
-
 } // namespace mozilla
