@@ -46,7 +46,6 @@
 #include "nsIIdleObserver.h"
 #include "nsIDocument.h"
 #include "nsIDOMTouchEvent.h"
-
 #include "mozilla/dom/EventTarget.h"
 #include "Units.h"
 #include "nsComponentManagerUtils.h"
@@ -79,6 +78,7 @@
 class nsIArray;
 class nsIBaseWindow;
 class nsIContent;
+class nsICSSDeclaration;
 class nsIDocShellTreeOwner;
 class nsIDOMCrypto;
 class nsIDOMOfflineResourceList;
@@ -106,6 +106,7 @@ class nsWindowSizes;
 namespace mozilla {
 namespace dom {
 class BarProp;
+class Function;
 class Gamepad;
 class Navigator;
 class SpeechSynthesis;
@@ -121,6 +122,17 @@ NS_CreateJSTimeoutHandler(nsGlobalWindow *aWindow,
                           int32_t *aInterval,
                           nsIScriptTimeoutHandler **aRet);
 
+extern already_AddRefed<nsIScriptTimeoutHandler>
+NS_CreateJSTimeoutHandler(nsGlobalWindow *aWindow,
+                          mozilla::dom::Function& aFunction,
+                          const mozilla::dom::Sequence<JS::Value>& aArguments,
+                          mozilla::ErrorResult& aError);
+
+extern already_AddRefed<nsIScriptTimeoutHandler>
+NS_CreateJSTimeoutHandler(JSContext* aCx, nsGlobalWindow *aWindow,
+                          const nsAString& aExpression,
+                          mozilla::ErrorResult& aError);
+
 /*
  * Timeout struct that holds information about each script
  * timeout.  Holds a strong reference to an nsIScriptTimeoutHandler, which
@@ -306,7 +318,6 @@ class nsGlobalWindow : public mozilla::dom::EventTarget,
 public:
   typedef mozilla::TimeStamp TimeStamp;
   typedef mozilla::TimeDuration TimeDuration;
-  typedef mozilla::dom::Navigator Navigator;
   typedef nsDataHashtable<nsUint64HashKey, nsGlobalWindow*> WindowByIdTable;
 
   // public methods
@@ -330,8 +341,7 @@ public:
   virtual JSObject *GetGlobalJSObject();
 
   // nsIScriptGlobalObject
-  virtual nsIScriptContext *GetContext();
-  JSObject *FastGetGlobalJSObject()
+  JSObject *FastGetGlobalJSObject() const
   {
     return mJSObject;
   }
@@ -343,9 +353,8 @@ public:
 
   void PoisonOuterWindowProxy(JSObject *aObject);
   virtual void OnFinalize(JSObject* aObject);
-  virtual void SetScriptsEnabled(bool aEnabled, bool aFireTimeouts);
 
-  virtual bool IsBlackForCC();
+  virtual bool IsBlackForCC(bool aTracingNeeded = true);
 
   // nsIScriptObjectPrincipal
   virtual nsIPrincipal* GetPrincipal();
@@ -371,9 +380,16 @@ public:
 
   // nsIDOMEventTarget
   NS_DECL_NSIDOMEVENTTARGET
+
+  virtual nsEventListenerManager*
+  GetExistingListenerManager() const MOZ_OVERRIDE;
+
+  virtual nsEventListenerManager*
+  GetOrCreateListenerManager() MOZ_OVERRIDE;
+
   using mozilla::dom::EventTarget::RemoveEventListener;
   virtual void AddEventListener(const nsAString& aType,
-                                nsIDOMEventListener* aListener,
+                                mozilla::dom::EventListener* aListener,
                                 bool aUseCapture,
                                 const mozilla::dom::Nullable<bool>& aWantsUntrusted,
                                 mozilla::ErrorResult& aRv) MOZ_OVERRIDE;
@@ -394,9 +410,12 @@ public:
 
   // nsPIDOMWindow
   virtual NS_HIDDEN_(nsPIDOMWindow*) GetPrivateRoot();
+
+  // Outer windows only.
   virtual NS_HIDDEN_(void) ActivateOrDeactivate(bool aActivate);
   virtual NS_HIDDEN_(void) SetActive(bool aActive);
   virtual NS_HIDDEN_(void) SetIsBackground(bool aIsBackground);
+
   virtual NS_HIDDEN_(void) SetChromeEventHandler(mozilla::dom::EventTarget* aChromeEventHandler);
 
   virtual NS_HIDDEN_(void) SetInitialPrincipalToSubject();
@@ -416,6 +435,7 @@ public:
   {
     return mIsFrozen;
   }
+  virtual NS_HIDDEN_(bool) IsRunningTimeout() { return mTimeoutFiringDepth > 0; }
 
   virtual NS_HIDDEN_(bool) WouldReuseInnerWindow(nsIDocument *aNewDocument);
 
@@ -427,6 +447,7 @@ public:
   void DispatchDOMWindowCreated();
   virtual NS_HIDDEN_(void) SetOpenerWindow(nsIDOMWindow* aOpener,
                                            bool aOriginalOpener);
+
   // Outer windows only.
   virtual NS_HIDDEN_(void) EnsureSizeUpToDate();
 
@@ -449,7 +470,6 @@ public:
   NS_DECL_NSIINTERFACEREQUESTOR
 
   // WebIDL interface.
-  uint32_t GetLength();
   already_AddRefed<nsIDOMWindow> IndexedGetter(uint32_t aIndex, bool& aFound);
 
   void GetSupportedNames(nsTArray<nsString>& aNames);
@@ -462,11 +482,6 @@ public:
     // Make sure this matches the casts we do in QueryInterface().
     return (nsGlobalWindow *)(mozilla::dom::EventTarget *)supports;
   }
-  static nsISupports *ToSupports(nsGlobalWindow *win)
-  {
-    // Make sure this matches the casts we do in QueryInterface().
-    return (nsISupports *)(mozilla::dom::EventTarget *)win;
-  }
   static nsGlobalWindow *FromWrapper(nsIXPConnectWrappedNative *wrapper)
   {
     return FromSupports(wrapper->Native());
@@ -495,31 +510,31 @@ public:
   {
     nsCOMPtr<nsIDOMWindow> top;
     GetScriptableTop(getter_AddRefs(top));
-    if (top) {
-      return static_cast<nsGlobalWindow *>(top.get());
-    }
-    return nullptr;
+    return static_cast<nsGlobalWindow *>(top.get());
   }
 
   already_AddRefed<nsIDOMWindow> GetChildWindow(const nsAString& aName);
 
-  // Returns true if dialogs need to be prevented from appearings for this
-  // window. beingAbused returns whether dialogs are being abused.
-  bool DialogsAreBlocked(bool *aBeingAbused);
-
-  // Returns true if we've reached the state in this top level window where we
-  // ask the user if further dialogs should be blocked. This method must only
-  // be called on the scriptable top inner window.
+  // These return true if we've reached the state in this top level window
+  // where we ask the user if further dialogs should be blocked.
+  //
+  // DialogsAreBeingAbused must be called on the scriptable top inner window.
+  //
+  // ShouldPromptToBlockDialogs is implemented in terms of
+  // DialogsAreBeingAbused, and will get the scriptable top inner window
+  // automatically.
+  // Outer windows only.
+  bool ShouldPromptToBlockDialogs();
+  // Inner windows only.
   bool DialogsAreBeingAbused();
 
-  // Ask the user if further dialogs should be blocked, if dialogs are currently
-  // being abused. This is used in the cases where we have no modifiable UI to
-  // show, in that case we show a separate dialog to ask this question.
-  bool ConfirmDialogIfNeeded();
-
-  // Prevent further dialogs in this (top level) window
-  void PreventFurtherDialogs(bool aPermanent);
+  // These functions are used for controlling and determining whether dialogs
+  // (alert, prompt, confirm) are currently allowed in this window.
+  void EnableDialogs();
+  void DisableDialogs();
+  bool AreDialogsEnabled();
 
+  // Inner windows only.
   virtual void SetHasAudioAvailableEventListeners();
 
   nsIScriptContext *GetContextInternal()
@@ -547,7 +562,7 @@ public:
     return static_cast<nsGlobalWindow *>(GetOuterWindow());
   }
 
-  nsGlobalWindow *GetCurrentInnerWindowInternal()
+  nsGlobalWindow *GetCurrentInnerWindowInternal() const
   {
     return static_cast<nsGlobalWindow *>(mInnerWindow);
   }
@@ -574,6 +589,7 @@ public:
   nsresult Observe(nsISupports* aSubject, const char* aTopic,
                    const PRUnichar* aData);
 
+  // Outer windows only.
   void UnblockScriptedClosing();
 
   static void Init();
@@ -602,6 +618,7 @@ public:
   virtual nsresult DispatchAsyncHashchange(nsIURI *aOldURI, nsIURI *aNewURI);
   virtual nsresult DispatchSyncPopState();
 
+  // Inner windows only.
   virtual void EnableDeviceSensor(uint32_t aType);
   virtual void DisableDeviceSensor(uint32_t aType);
 
@@ -609,6 +626,7 @@ public:
   virtual void DisableTimeChangeNotifications();
 
 #ifdef MOZ_B2G
+  // Inner windows only.
   virtual void EnableNetworkEvent(uint32_t aType);
   virtual void DisableNetworkEvent(uint32_t aType);
 #endif // MOZ_B2G
@@ -654,7 +672,7 @@ public:
     return sWindowsById;
   }
 
-  void SizeOfIncludingThis(nsWindowSizes* aWindowSizes) const;
+  void AddSizeOfIncludingThis(nsWindowSizes* aWindowSizes) const;
 
   void UnmarkGrayTimers();
 
@@ -703,51 +721,41 @@ public:
 #define EVENT(name_, id_, type_, struct_)                                     \
   mozilla::dom::EventHandlerNonNull* GetOn##name_()                           \
   {                                                                           \
-    nsEventListenerManager *elm = GetListenerManager(false);                  \
+    nsEventListenerManager *elm = GetExistingListenerManager();               \
     return elm ? elm->GetEventHandler(nsGkAtoms::on##name_, EmptyString())    \
                : nullptr;                                                     \
   }                                                                           \
-  void SetOn##name_(mozilla::dom::EventHandlerNonNull* handler,               \
-                    mozilla::ErrorResult& error)                              \
+  void SetOn##name_(mozilla::dom::EventHandlerNonNull* handler)               \
   {                                                                           \
-    nsEventListenerManager *elm = GetListenerManager(true);                   \
+    nsEventListenerManager *elm = GetOrCreateListenerManager();               \
     if (elm) {                                                                \
-      error = elm->SetEventHandler(nsGkAtoms::on##name_, EmptyString(),       \
-                                   handler);                                  \
-    } else {                                                                  \
-      error.Throw(NS_ERROR_OUT_OF_MEMORY);                                    \
+      elm->SetEventHandler(nsGkAtoms::on##name_, EmptyString(), handler);     \
     }                                                                         \
   }
 #define ERROR_EVENT(name_, id_, type_, struct_)                               \
   mozilla::dom::OnErrorEventHandlerNonNull* GetOn##name_()                    \
   {                                                                           \
-    nsEventListenerManager *elm = GetListenerManager(false);                  \
+    nsEventListenerManager *elm = GetExistingListenerManager();               \
     return elm ? elm->GetOnErrorEventHandler() : nullptr;                     \
   }                                                                           \
-  void SetOn##name_(mozilla::dom::OnErrorEventHandlerNonNull* handler,        \
-                    mozilla::ErrorResult& error)                              \
+  void SetOn##name_(mozilla::dom::OnErrorEventHandlerNonNull* handler)        \
   {                                                                           \
-    nsEventListenerManager *elm = GetListenerManager(true);                   \
+    nsEventListenerManager *elm = GetOrCreateListenerManager();               \
     if (elm) {                                                                \
-      error = elm->SetEventHandler(handler);                                  \
-    } else {                                                                  \
-      error.Throw(NS_ERROR_OUT_OF_MEMORY);                                    \
+      elm->SetEventHandler(handler);                                          \
     }                                                                         \
   }
 #define BEFOREUNLOAD_EVENT(name_, id_, type_, struct_)                        \
-  mozilla::dom::BeforeUnloadEventHandlerNonNull* GetOn##name_()               \
+  mozilla::dom::OnBeforeUnloadEventHandlerNonNull* GetOn##name_()             \
   {                                                                           \
-    nsEventListenerManager *elm = GetListenerManager(false);                  \
+    nsEventListenerManager *elm = GetExistingListenerManager();               \
     return elm ? elm->GetOnBeforeUnloadEventHandler() : nullptr;              \
   }                                                                           \
-  void SetOn##name_(mozilla::dom::BeforeUnloadEventHandlerNonNull* handler,   \
-                    mozilla::ErrorResult& error)                              \
+  void SetOn##name_(mozilla::dom::OnBeforeUnloadEventHandlerNonNull* handler) \
   {                                                                           \
-    nsEventListenerManager *elm = GetListenerManager(true);                   \
+    nsEventListenerManager *elm = GetOrCreateListenerManager();               \
     if (elm) {                                                                \
-      error = elm->SetEventHandler(handler);                                  \
-    } else {                                                                  \
-      error.Throw(NS_ERROR_OUT_OF_MEMORY);                                    \
+      elm->SetEventHandler(handler);                                          \
     }                                                                         \
   }
 #define WINDOW_ONLY_EVENT EVENT
@@ -759,11 +767,180 @@ public:
 #undef ERROR_EVENT
 #undef EVENT
 
-#ifdef MOZ_WEBSPEECH
-  mozilla::dom::SpeechSynthesis* GetSpeechSynthesisInternal();
-#endif
+  nsISupports* GetParentObject()
+  {
+    return nullptr;
+  }
 
-  mozilla::dom::BarProp* Scrollbars();
+  nsIDOMWindow* GetWindow(mozilla::ErrorResult& aError);
+  nsIDOMWindow* GetSelf(mozilla::ErrorResult& aError);
+  nsIDocument* GetDocument()
+  {
+    return GetDoc();
+  }
+  void GetName(nsAString& aName, mozilla::ErrorResult& aError);
+  void SetName(const nsAString& aName, mozilla::ErrorResult& aError);
+  nsIDOMLocation* GetLocation(mozilla::ErrorResult& aError);
+  nsHistory* GetHistory(mozilla::ErrorResult& aError);
+  mozilla::dom::BarProp* GetLocationbar(mozilla::ErrorResult& aError);
+  mozilla::dom::BarProp* GetMenubar(mozilla::ErrorResult& aError);
+  mozilla::dom::BarProp* GetPersonalbar(mozilla::ErrorResult& aError);
+  mozilla::dom::BarProp* GetScrollbars(mozilla::ErrorResult& aError);
+  mozilla::dom::BarProp* GetStatusbar(mozilla::ErrorResult& aError);
+  mozilla::dom::BarProp* GetToolbar(mozilla::ErrorResult& aError);
+  void GetStatus(nsAString& aStatus, mozilla::ErrorResult& aError);
+  void SetStatus(const nsAString& aStatus, mozilla::ErrorResult& aError);
+  void Close(mozilla::ErrorResult& aError);
+  bool GetClosed(mozilla::ErrorResult& aError);
+  void Stop(mozilla::ErrorResult& aError);
+  void Focus(mozilla::ErrorResult& aError);
+  void Blur(mozilla::ErrorResult& aError);
+  already_AddRefed<nsIDOMWindow> GetFrames(mozilla::ErrorResult& aError);
+  uint32_t Length();
+  already_AddRefed<nsIDOMWindow> GetTop(mozilla::ErrorResult& aError)
+  {
+    nsCOMPtr<nsIDOMWindow> top;
+    aError = GetScriptableTop(getter_AddRefs(top));
+    return top.forget();
+  }
+  nsIDOMWindow* GetOpener(mozilla::ErrorResult& aError);
+  void SetOpener(nsIDOMWindow* aOpener, mozilla::ErrorResult& aError);
+  using nsIDOMWindow::GetParent;
+  already_AddRefed<nsIDOMWindow> GetParent(mozilla::ErrorResult& aError);
+  mozilla::dom::Element* GetFrameElement(mozilla::ErrorResult& aError);
+  already_AddRefed<nsIDOMWindow> Open(const nsAString& aUrl,
+                                      const nsAString& aName,
+                                      const nsAString& aOptions,
+                                      mozilla::ErrorResult& aError);
+  mozilla::dom::Navigator* GetNavigator(mozilla::ErrorResult& aError);
+  nsIDOMOfflineResourceList* GetApplicationCache(mozilla::ErrorResult& aError);
+  void Alert(const nsAString& aMessage, mozilla::ErrorResult& aError);
+  bool Confirm(const nsAString& aMessage, mozilla::ErrorResult& aError);
+  void Prompt(const nsAString& aMessage, const nsAString& aInitial,
+              nsAString& aReturn, mozilla::ErrorResult& aError);
+  void Print(mozilla::ErrorResult& aError);
+  JS::Value ShowModalDialog(JSContext* aCx, const nsAString& aUrl, const mozilla::dom::Optional<JS::Handle<JS::Value> >& aArgument, const nsAString& aOptions, mozilla::ErrorResult& aError);
+  void PostMessageMoz(JSContext* aCx, JS::Handle<JS::Value> aMessage,
+                      const nsAString& aTargetOrigin,
+                      const mozilla::dom::Optional<mozilla::dom::Sequence<JS::Value > >& aTransfer,
+                      mozilla::ErrorResult& aError);
+  int32_t SetTimeout(JSContext* aCx, mozilla::dom::Function& aFunction,
+                     int32_t aTimeout,
+                     const mozilla::dom::Sequence<JS::Value>& aArguments,
+                     mozilla::ErrorResult& aError);
+  int32_t SetTimeout(JSContext* aCx, const nsAString& aHandler,
+                     int32_t aTimeout,
+                     const mozilla::dom::Sequence<JS::Value>& /* unused */,
+                     mozilla::ErrorResult& aError);
+  void ClearTimeout(int32_t aHandle, mozilla::ErrorResult& aError);
+  int32_t SetInterval(JSContext* aCx, mozilla::dom::Function& aFunction,
+                      const mozilla::dom::Optional<int32_t>& aTimeout,
+                      const mozilla::dom::Sequence<JS::Value>& aArguments,
+                      mozilla::ErrorResult& aError);
+  int32_t SetInterval(JSContext* aCx, const nsAString& aHandler,
+                      const mozilla::dom::Optional<int32_t>& aTimeout,
+                      const mozilla::dom::Sequence<JS::Value>& /* unused */,
+                      mozilla::ErrorResult& aError);
+  void ClearInterval(int32_t aHandle, mozilla::ErrorResult& aError);
+  void Atob(const nsAString& aAsciiBase64String, nsAString& aBinaryData,
+            mozilla::ErrorResult& aError);
+  void Btoa(const nsAString& aBinaryData, nsAString& aAsciiBase64String,
+            mozilla::ErrorResult& aError);
+  nsIDOMStorage* GetSessionStorage(mozilla::ErrorResult& aError);
+  nsIDOMStorage* GetLocalStorage(mozilla::ErrorResult& aError);
+  nsISelection* GetSelection(mozilla::ErrorResult& aError);
+  mozilla::dom::indexedDB::IDBFactory* GetIndexedDB(mozilla::ErrorResult& aError);
+  already_AddRefed<nsICSSDeclaration>
+    GetComputedStyle(mozilla::dom::Element& aElt, const nsAString& aPseudoElt,
+                     mozilla::ErrorResult& aError);
+  already_AddRefed<nsIDOMMediaQueryList> MatchMedia(const nsAString& aQuery,
+                                                    mozilla::ErrorResult& aError);
+  nsScreen* GetScreen(mozilla::ErrorResult& aError);
+  void MoveTo(int32_t aXPos, int32_t aYPos, mozilla::ErrorResult& aError);
+  void MoveBy(int32_t aXDif, int32_t aYDif, mozilla::ErrorResult& aError);
+  void ResizeTo(int32_t aWidth, int32_t aHeight,
+                mozilla::ErrorResult& aError);
+  void ResizeBy(int32_t aWidthDif, int32_t aHeightDif,
+                mozilla::ErrorResult& aError);
+  int32_t GetInnerWidth(mozilla::ErrorResult& aError);
+  void SetInnerWidth(int32_t aInnerWidth, mozilla::ErrorResult& aError);
+  int32_t GetInnerHeight(mozilla::ErrorResult& aError);
+  void SetInnerHeight(int32_t aInnerHeight, mozilla::ErrorResult& aError);
+  int32_t GetScrollX(mozilla::ErrorResult& aError);
+  int32_t GetPageXOffset(mozilla::ErrorResult& aError)
+  {
+    return GetScrollX(aError);
+  }
+  int32_t GetScrollY(mozilla::ErrorResult& aError);
+  int32_t GetPageYOffset(mozilla::ErrorResult& aError)
+  {
+    return GetScrollY(aError);
+  }
+  int32_t GetScreenX(mozilla::ErrorResult& aError);
+  void SetScreenX(int32_t aScreenX, mozilla::ErrorResult& aError);
+  int32_t GetScreenY(mozilla::ErrorResult& aError);
+  void SetScreenY(int32_t aScreenY, mozilla::ErrorResult& aError);
+  int32_t GetOuterWidth(mozilla::ErrorResult& aError);
+  void SetOuterWidth(int32_t aOuterWidth, mozilla::ErrorResult& aError);
+  int32_t GetOuterHeight(mozilla::ErrorResult& aError);
+  void SetOuterHeight(int32_t aOuterHeight, mozilla::ErrorResult& aError);
+  int32_t RequestAnimationFrame(mozilla::dom::FrameRequestCallback& aCallback,
+                                mozilla::ErrorResult& aError);
+  void CancelAnimationFrame(int32_t aHandle, mozilla::ErrorResult& aError);
+  nsPerformance* GetPerformance(mozilla::ErrorResult& aError);
+#ifdef MOZ_WEBSPEECH
+  mozilla::dom::SpeechSynthesis*
+    GetSpeechSynthesis(mozilla::ErrorResult& aError);
+#endif
+  already_AddRefed<nsICSSDeclaration>
+    GetDefaultComputedStyle(mozilla::dom::Element& aElt,
+                            const nsAString& aPseudoElt,
+                            mozilla::ErrorResult& aError);
+  mozilla::dom::indexedDB::IDBFactory*
+    GetMozIndexedDB(mozilla::ErrorResult& aError)
+  {
+    return GetIndexedDB(aError);
+  }
+  int32_t MozRequestAnimationFrame(nsIFrameRequestCallback* aRequestCallback,
+                                   mozilla::ErrorResult& aError);
+  void MozCancelAnimationFrame(int32_t aHandle, mozilla::ErrorResult& aError)
+  {
+    return CancelAnimationFrame(aHandle, aError);
+  }
+  void MozCancelRequestAnimationFrame(int32_t aHandle,
+                                      mozilla::ErrorResult& aError)
+  {
+    return CancelAnimationFrame(aHandle, aError);
+  }
+  int64_t GetMozAnimationStartTime(mozilla::ErrorResult& aError);
+  void SizeToContent(mozilla::ErrorResult& aError);
+  nsIDOMCrypto* GetCrypto(mozilla::ErrorResult& aError);
+  nsIDOMPkcs11* GetPkcs11()
+  {
+    return nullptr;
+  }
+  nsIControllers* GetControllers(mozilla::ErrorResult& aError);
+  float GetMozInnerScreenX(mozilla::ErrorResult& aError);
+  float GetMozInnerScreenY(mozilla::ErrorResult& aError);
+  float GetDevicePixelRatio(mozilla::ErrorResult& aError);
+  int32_t GetScrollMaxX(mozilla::ErrorResult& aError);
+  int32_t GetScrollMaxY(mozilla::ErrorResult& aError);
+  bool GetFullScreen(mozilla::ErrorResult& aError);
+  void SetFullScreen(bool aFullScreen, mozilla::ErrorResult& aError);
+  void Back(mozilla::ErrorResult& aError);
+  void Forward(mozilla::ErrorResult& aError);
+  void Home(mozilla::ErrorResult& aError);
+  bool Find(const nsAString& aString, bool aCaseSensitive, bool aBackwards,
+            bool aWrapAround, bool aWholeWord, bool aSearchInFrames,
+            bool aShowDialog, mozilla::ErrorResult& aError);
+  uint64_t GetMozPaintCount(mozilla::ErrorResult& aError);
+  already_AddRefed<nsIDOMWindow> OpenDialog(JSContext* aCx,
+                                            const nsAString& aUrl,
+                                            const nsAString& aName,
+                                            const nsAString& aOptions,
+                                            const mozilla::dom::Sequence<JS::Value>& aExtraArgument,
+                                            mozilla::ErrorResult& aError);
+  JSObject* GetContent(JSContext* aCx, mozilla::ErrorResult& aError);
 
 protected:
   // Array of idle observers that are notified of idle events.
@@ -798,6 +975,7 @@ protected:
 
   // Object Management
   virtual ~nsGlobalWindow();
+  void DropOuterWindowDocs();
   void CleanUp();
   void ClearControllers();
   nsresult FinalClose();
@@ -915,7 +1093,21 @@ protected:
   nsresult SetTimeoutOrInterval(nsIScriptTimeoutHandler *aHandler,
                                 int32_t interval,
                                 bool aIsInterval, int32_t *aReturn);
-  nsresult ClearTimeoutOrInterval(int32_t aTimerID);
+  int32_t SetTimeoutOrInterval(mozilla::dom::Function& aFunction,
+                               int32_t aTimeout,
+                               const mozilla::dom::Sequence<JS::Value>& aArguments,
+                               bool aIsInterval, mozilla::ErrorResult& aError);
+  int32_t SetTimeoutOrInterval(JSContext* aCx, const nsAString& aHandler,
+                               int32_t aTimeout, bool aIsInterval,
+                               mozilla::ErrorResult& aError);
+  void ClearTimeoutOrInterval(int32_t aTimerID,
+                                  mozilla::ErrorResult& aError);
+  nsresult ClearTimeoutOrInterval(int32_t aTimerID)
+  {
+    mozilla::ErrorResult rv;
+    ClearTimeoutOrInterval(aTimerID, rv);
+    return rv.ErrorCode();
+  }
 
   // JS specific timeout functions (JS args grabbed from context).
   nsresult SetTimeoutOrInterval(bool aIsInterval, int32_t* aReturn);
@@ -941,9 +1133,8 @@ protected:
   already_AddRefed<nsIBaseWindow> GetTreeOwnerWindow();
   already_AddRefed<nsIWebBrowserChrome> GetWebBrowserChrome();
   nsresult SecurityCheckURL(const char *aURL);
-  nsresult BuildURIfromBase(const char *aURL,
-                            nsIURI **aBuiltURI,
-                            bool *aFreeSecurityPass, JSContext **aCXused);
+  nsresult BuildURIfromBase(const char *aURL, nsIURI **aBuiltURI,
+                            JSContext **aCXused);
   bool PopupWhitelisted();
   PopupControlState RevisePopupAbuseLevel(PopupControlState);
   void     FireAbuseEvents(bool aBlocked, bool aWindow,
@@ -952,6 +1143,7 @@ protected:
                            const nsAString &aPopupWindowFeatures);
   void FireOfflineStatusEvent();
 
+  // Inner windows only.
   nsresult ScheduleNextIdleObserverCallback();
   uint32_t GetFuzzTimeMS();
   nsresult ScheduleActiveTimerCallback();
@@ -961,13 +1153,17 @@ protected:
                                       int32_t* aRemoveElementIndex);
   virtual nsresult UnregisterIdleObserver(nsIIdleObserver* aIdleObserverPtr);
 
+  // Inner windows only.
   nsresult FireHashchange(const nsAString &aOldURL, const nsAString &aNewURL);
 
   void FlushPendingNotifications(mozFlushType aType);
+
+  // Outer windows only.
   void EnsureReflowFlushAndPaint();
   void CheckSecurityWidthAndHeight(int32_t* width, int32_t* height);
   void CheckSecurityLeftAndTop(int32_t* left, int32_t* top);
 
+  // Outer windows only.
   // Arguments to this function should have values in app units
   void SetCSSViewportWidthAndHeight(nscoord width, nscoord height);
   // Arguments to this function should have values in device pixels
@@ -977,21 +1173,21 @@ protected:
 
   static void MakeScriptDialogTitle(nsAString &aOutTitle);
 
+  // Outer windows only.
   bool CanMoveResizeWindows();
 
-  bool     GetBlurSuppression();
-
   // If aDoFlush is true, we'll flush our own layout; otherwise we'll try to
   // just flush our parent and only flush ourselves if we think we need to.
-  nsresult GetScrollXY(int32_t* aScrollX, int32_t* aScrollY,
-                       bool aDoFlush);
-  nsresult GetScrollMaxXY(int32_t* aScrollMaxX, int32_t* aScrollMaxY);
+  mozilla::CSSIntPoint GetScrollXY(bool aDoFlush, mozilla::ErrorResult& aError);
+  nsresult GetScrollXY(int32_t* aScrollX, int32_t* aScrollY, bool aDoFlush);
+  void GetScrollMaxXY(int32_t* aScrollMaxX, int32_t* aScrollMaxY,
+                      mozilla::ErrorResult& aError);
 
   // Outer windows only.
   nsresult GetInnerSize(mozilla::CSSIntSize& aSize);
-
-  nsresult GetOuterSize(nsIntSize* aSizeCSSPixels);
-  nsresult SetOuterSize(int32_t aLengthCSSPixels, bool aIsWidth);
+  nsIntSize GetOuterSize(mozilla::ErrorResult& aError);
+  void SetOuterSize(int32_t aLengthCSSPixels, bool aIsWidth,
+                    mozilla::ErrorResult& aError);
   nsRect GetInnerScreenRect();
 
   void ScrollTo(const mozilla::CSSIntPoint& aScroll);
@@ -1001,6 +1197,7 @@ protected:
     return GetParentInternal() != nullptr;
   }
 
+  // Outer windows only.
   // If aLookForCallerOnJSStack is true, this method will look at the JS stack
   // to determine who the caller is.  If it's false, it'll use |this| as the
   // caller.
@@ -1045,6 +1242,7 @@ protected:
   virtual void GetKeyboardIndicators(bool* aShowAccelerators,
                                      bool* aShowFocusRings);
 
+  // Inner windows only.
   void UpdateCanvasFocus(bool aFocusChanged, nsIContent* aNewContent);
 
 public:
@@ -1061,6 +1259,7 @@ protected:
 
   virtual void UpdateParentTarget();
 
+  // Outer windows only.
   bool GetIsTabModalPromptAllowed();
 
   inline int32_t DOMMinTimeoutValue() const;
@@ -1070,13 +1269,15 @@ protected:
   nsresult CloneStorageEvent(const nsAString& aType,
                              nsCOMPtr<nsIDOMStorageEvent>& aEvent);
 
-  // Implements Get{Real,Scriptable}Top.
-  nsresult GetTopImpl(nsIDOMWindow **aWindow, bool aScriptable);
-
   // Outer windows only.
   nsDOMWindowList* GetWindowList();
 
   // Helper for getComputedStyle and getDefaultComputedStyle
+  already_AddRefed<nsICSSDeclaration>
+    GetComputedStyleHelper(mozilla::dom::Element& aElt,
+                           const nsAString& aPseudoElt,
+                           bool aDefaultStylesOnly,
+                           mozilla::ErrorResult& aError);
   nsresult GetComputedStyleHelper(nsIDOMElement* aElt,
                                   const nsAString& aPseudoElt,
                                   bool aDefaultStylesOnly,
@@ -1084,8 +1285,32 @@ protected:
 
   void PreloadLocalStorage();
 
-  nsresult RequestAnimationFrame(const nsIDocument::FrameRequestCallbackHolder& aCallback,
-                                 int32_t* aHandle);
+  // Returns device pixels.  Outer windows only.
+  nsIntPoint GetScreenXY(mozilla::ErrorResult& aError);
+
+  int32_t RequestAnimationFrame(const nsIDocument::FrameRequestCallbackHolder& aCallback,
+                                mozilla::ErrorResult& aError);
+
+  nsGlobalWindow* InnerForSetTimeoutOrInterval(mozilla::ErrorResult& aError);
+
+  mozilla::dom::Element* GetRealFrameElement(mozilla::ErrorResult& aError);
+
+  void PostMessageMoz(JSContext* aCx, JS::Handle<JS::Value> aMessage,
+                      const nsAString& aTargetOrigin,
+                      JS::Handle<JS::Value> aTransfer,
+                      mozilla::ErrorResult& aError);
+
+  already_AddRefed<nsIVariant>
+    ShowModalDialog(const nsAString& aUrl, nsIVariant* aArgument,
+                    const nsAString& aOptions, mozilla::ErrorResult& aError);
+
+  already_AddRefed<nsIDOMWindow>
+    GetContentInternal(mozilla::ErrorResult& aError);
+
+  // Ask the user if further dialogs should be blocked, if dialogs are currently
+  // being abused. This is used in the cases where we have no modifiable UI to
+  // show, in that case we show a separate dialog to ask this question.
+  bool ConfirmDialogIfNeeded();
 
   // When adding new member variables, be careful not to create cycles
   // through JavaScript.  If there is any chance that a member variable
@@ -1153,6 +1378,10 @@ protected:
   // should be displayed.
   bool                   mFocusByKeyOccurred : 1;
 
+  // Ensure that a call to ResumeTimeouts() after FreeInnerObjects() does nothing.
+  // This member is only used by inner windows.
+  bool                   mInnerObjectsFreed : 1;
+
   // Indicates whether this window wants gamepad input events
   bool                   mHasGamepad : 1;
 #ifdef MOZ_GAMEPAD
@@ -1176,7 +1405,7 @@ protected:
   // For |window.dialogArguments|, via |showModalDialog|.
   nsRefPtr<DialogValueHolder> mDialogArguments;
 
-  nsRefPtr<Navigator>           mNavigator;
+  nsRefPtr<mozilla::dom::Navigator> mNavigator;
   nsRefPtr<nsScreen>            mScreen;
   nsRefPtr<nsDOMWindowList>     mFrames;
   nsRefPtr<mozilla::dom::BarProp> mMenubar;
@@ -1188,7 +1417,7 @@ protected:
   nsRefPtr<nsDOMWindowUtils>    mWindowUtils;
   nsString                      mStatus;
   nsString                      mDefaultStatus;
-  nsGlobalWindowObserver*       mObserver;
+  nsGlobalWindowObserver*       mObserver; // Inner windows only.
   nsCOMPtr<nsIDOMCrypto>        mCrypto;
 
   nsCOMPtr<nsIDOMStorage>      mLocalStorage;
@@ -1244,6 +1473,12 @@ protected:
 
   nsAutoPtr<nsJSThingHashtable<nsPtrHashKey<nsXBLPrototypeHandler>, JSObject*> > mCachedXBLPrototypeHandlers;
 
+  // mSuspendedDoc is only set on outer windows. It's useful when we get matched
+  // EnterModalState/LeaveModalState calls, in which case the outer window is
+  // responsible for unsuspending events on the document. If we don't (for
+  // example, if the outer window is closed before the LeaveModalState call),
+  // then the inner window whose mDoc is our mSuspendedDoc is responsible for
+  // unsuspending it.
   nsCOMPtr<nsIDocument> mSuspendedDoc;
 
   nsRefPtr<mozilla::dom::indexedDB::IDBFactory> mIndexedDB;
@@ -1260,15 +1495,9 @@ protected:
   // to allow disabling of further dialogs from this window.
   TimeStamp                     mLastDialogQuitTime;
 
-  // This is set to true once the user has opted-in to preventing further
-  // dialogs for this window. Subsequent dialogs may still open if
-  // mDialogAbuseCount gets reset.
-  bool                          mStopAbuseDialogs;
-
-  // This flag gets set when dialogs should be permanently disabled for this
-  // window (e.g. when we are closing the tab and therefore are guaranteed to be
-  // destroying this window).
-  bool                          mDialogsPermanentlyDisabled;
+  // This flag keeps track of whether dialogs are
+  // currently enabled on this window.
+  bool                          mAreDialogsEnabled;
 
   nsTHashtable<nsPtrHashKey<nsDOMEventTargetHelper> > mEventTargetObjects;
 
@@ -1288,6 +1517,18 @@ protected:
   static bool sWarnedAboutWindowInternal;
 };
 
+inline nsISupports*
+ToSupports(nsGlobalWindow *p)
+{
+    return static_cast<nsIDOMEventTarget*>(p);
+}
+
+inline nsISupports*
+ToCanonicalSupports(nsGlobalWindow *p)
+{
+    return static_cast<nsIDOMEventTarget*>(p);
+}
+
 /*
  * nsGlobalChromeWindow inherits from nsGlobalWindow. It is the global
  * object created for a Chrome Window only.
