@@ -6,11 +6,11 @@
 #include "nsCOMPtr.h"
 #include "nsDOMClassInfo.h"
 #include "nsHashPropertyBag.h"
-#include "jsapi.h"
 #include "nsThread.h"
 #include "DeviceStorage.h"
 #include "mozilla/dom/CameraControlBinding.h"
 #include "mozilla/dom/TabChild.h"
+#include "mozilla/MediaManager.h"
 #include "mozilla/Services.h"
 #include "mozilla/unused.h"
 #include "nsIAppsService.h"
@@ -27,6 +27,7 @@
 
 using namespace mozilla;
 using namespace mozilla::dom;
+using namespace mozilla::idl;
 
 NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE_2(nsDOMCameraControl, mDOMCapabilities, mWindow)
 
@@ -156,6 +157,83 @@ nsDOMCameraControl::SetFocusAreas(JSContext* cx, JS::Handle<JS::Value> aFocusAre
   aRv = mCameraControl->SetFocusAreas(cx, aFocusAreas);
 }
 
+static nsresult
+GetSize(JSContext* aCx, JS::Value* aValue, const CameraSize& aSize)
+{
+  JS::Rooted<JSObject*> o(aCx, JS_NewObject(aCx, nullptr, nullptr, nullptr));
+  if (!o) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  JS::Rooted<JS::Value> v(aCx);
+
+  v = INT_TO_JSVAL(aSize.width);
+  if (!JS_SetProperty(aCx, o, "width", v)) {
+    return NS_ERROR_FAILURE;
+  }
+  v = INT_TO_JSVAL(aSize.height);
+  if (!JS_SetProperty(aCx, o, "height", v)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  *aValue = JS::ObjectValue(*o);
+  return NS_OK;
+}
+
+/* attribute any pictureSize */
+JS::Value
+nsDOMCameraControl::GetPictureSize(JSContext* cx, ErrorResult& aRv)
+{
+  JS::Rooted<JS::Value> value(cx);
+  
+  CameraSize size;
+  aRv = mCameraControl->Get(CAMERA_PARAM_PICTURESIZE, size);
+  if (aRv.Failed()) {
+    return value;
+  }
+
+  aRv = GetSize(cx, value.address(), size);
+  return value;
+}
+void
+nsDOMCameraControl::SetPictureSize(JSContext* cx, JS::Handle<JS::Value> aSize, ErrorResult& aRv)
+{
+  CameraSize size;
+  aRv = size.Init(cx, aSize.address());
+  if (aRv.Failed()) {
+    return;
+  }
+
+  aRv = mCameraControl->Set(CAMERA_PARAM_PICTURESIZE, size);
+}
+
+/* attribute any thumbnailSize */
+JS::Value
+nsDOMCameraControl::GetThumbnailSize(JSContext* cx, ErrorResult& aRv)
+{
+  JS::Rooted<JS::Value> value(cx);
+  
+  CameraSize size;
+  aRv = mCameraControl->Get(CAMERA_PARAM_THUMBNAILSIZE, size);
+  if (aRv.Failed()) {
+    return value;
+  }
+
+  aRv = GetSize(cx, value.address(), size);
+  return value;
+}
+void
+nsDOMCameraControl::SetThumbnailSize(JSContext* cx, JS::Handle<JS::Value> aSize, ErrorResult& aRv)
+{
+  CameraSize size;
+  aRv = size.Init(cx, aSize.address());
+  if (aRv.Failed()) {
+    return;
+  }
+
+  aRv = mCameraControl->Set(CAMERA_PARAM_THUMBNAILSIZE, size);
+}
+
 double
 nsDOMCameraControl::GetFocalLength(ErrorResult& aRv)
 {
@@ -207,6 +285,14 @@ nsDOMCameraControl::GetExposureCompensation(ErrorResult& aRv)
   return compensation;
 }
 
+int32_t
+nsDOMCameraControl::SensorAngle()
+{
+  int32_t angle;
+  mCameraControl->Get(CAMERA_PARAM_SENSORANGLE, &angle);
+  return angle;
+}
+
 already_AddRefed<nsICameraShutterCallback>
 nsDOMCameraControl::GetOnShutter(ErrorResult& aRv)
 {
@@ -274,24 +360,7 @@ nsDOMCameraControl::StartRecording(JSContext* aCx,
     return;
   }
 
-  nsCOMPtr<nsIObserverService> obs = services::GetObserverService();
-  if (!obs) {
-    NS_WARNING("Could not get the Observer service for CameraControl::StartRecording.");
-    aRv.Throw(NS_ERROR_FAILURE);
-    return;
-  }
-
-  nsRefPtr<nsHashPropertyBag> props = CreateRecordingDeviceEventsSubject();
-  obs->NotifyObservers(static_cast<nsIPropertyBag2*>(props),
-                       "recording-device-events",
-                       NS_LITERAL_STRING("starting").get());
-  // Forward recording events to parent process.
-  // The events are gathered in chrome process and used for recording indicator
-  if (XRE_GetProcessType() != GeckoProcessType_Default) {
-    unused << TabChild::GetFrom(mWindow)->SendRecordingDeviceEvents(NS_LITERAL_STRING("starting"),
-                                                                    true /* isAudio */,
-                                                                    true /* isVideo */);
-  }
+  aRv = NotifyRecordingStatusChange(NS_LITERAL_STRING("starting"));
 
   #ifdef MOZ_B2G
   if (!mAudioChannelAgent) {
@@ -318,23 +387,7 @@ nsDOMCameraControl::StartRecording(JSContext* aCx,
 void
 nsDOMCameraControl::StopRecording(ErrorResult& aRv)
 {
-  nsCOMPtr<nsIObserverService> obs = services::GetObserverService();
-  if (!obs) {
-    NS_WARNING("Could not get the Observer service for CameraControl::StopRecording.");
-    aRv.Throw(NS_ERROR_FAILURE);
-  }
-
-  nsRefPtr<nsHashPropertyBag> props = CreateRecordingDeviceEventsSubject();
-  obs->NotifyObservers(static_cast<nsIPropertyBag2*>(props) ,
-                       "recording-device-events",
-                       NS_LITERAL_STRING("shutdown").get());
-  // Forward recording events to parent process.
-  // The events are gathered in chrome process and used for recording indicator
-  if (XRE_GetProcessType() != GeckoProcessType_Default) {
-    unused << TabChild::GetFrom(mWindow)->SendRecordingDeviceEvents(NS_LITERAL_STRING("shutdown"),
-                                                                    true /* isAudio */,
-                                                                    true /* isVideo */);
-  }
+  aRv = NotifyRecordingStatusChange(NS_LITERAL_STRING("shutdown"));
 
   #ifdef MOZ_B2G
   if (mAudioChannelAgent) {
@@ -534,38 +587,14 @@ nsDOMCameraControl::GetNativeCameraControl()
   return mCameraControl;
 }
 
-already_AddRefed<nsHashPropertyBag>
-nsDOMCameraControl::CreateRecordingDeviceEventsSubject()
+nsresult
+nsDOMCameraControl::NotifyRecordingStatusChange(const nsString& aMsg)
 {
-  MOZ_ASSERT(mWindow);
+  NS_ENSURE_TRUE(mWindow, NS_ERROR_FAILURE);
 
-  nsRefPtr<nsHashPropertyBag> props = new nsHashPropertyBag();
-  props->SetPropertyAsBool(NS_LITERAL_STRING("isAudio"), true);
-  props->SetPropertyAsBool(NS_LITERAL_STRING("isVideo"), true);
-
-  nsCOMPtr<nsIDocShell> docShell = mWindow->GetDocShell();
-  if (docShell) {
-    bool isApp;
-    DebugOnly<nsresult> rv = docShell->GetIsApp(&isApp);
-    MOZ_ASSERT(NS_SUCCEEDED(rv));
-
-    nsString requestURL;
-    if (isApp) {
-      rv = docShell->GetAppManifestURL(requestURL);
-      MOZ_ASSERT(NS_SUCCEEDED(rv));
-    } else {
-      nsCString pageURL;
-      nsCOMPtr<nsIURI> docURI = mWindow->GetDocumentURI();
-      MOZ_ASSERT(docURI);
-
-      rv = docURI->GetSpec(pageURL);
-      MOZ_ASSERT(NS_SUCCEEDED(rv));
-
-      requestURL = NS_ConvertUTF8toUTF16(pageURL);
-    }
-    props->SetPropertyAsBool(NS_LITERAL_STRING("isApp"), isApp);
-    props->SetPropertyAsAString(NS_LITERAL_STRING("requestURL"), requestURL);
-  }
-
-  return props.forget();
+  return MediaManager::NotifyRecordingStatusChange(mWindow,
+                                                   aMsg,
+                                                   true /* aIsAudio */,
+                                                   true /* aIsVideo */);
 }
+
