@@ -16,25 +16,15 @@
 #include "AudioChannelService.h"
 #include "prlog.h"
 #include "nsPrintfCString.h"
-#include "nsWeakPtr.h"
 #include "nsXULAppAPI.h"
 #include "nsIFrameLoader.h"
-#include "nsIInterfaceRequestorUtils.h"
-#include "nsITimer.h"
-#include "nsIObserver.h"
 #include "nsIObserverService.h"
-#include "nsIDocument.h"
-#include "nsIDOMEventListener.h"
-#include "nsIDOMWindow.h"
-#include "nsIDOMEvent.h"
-#include "nsIDOMDocument.h"
-#include "nsPIDOMWindow.h"
 #include "StaticPtr.h"
 #include "nsIMozBrowserFrame.h"
 #include "nsIObserver.h"
 #include "nsITimer.h"
-#include "nsPrintfCString.h"
-#include "prlog.h"
+#include "nsIPropertyBag2.h"
+#include "nsComponentManagerUtils.h"
 
 #ifdef XP_WIN
 #include <process.h>
@@ -136,7 +126,8 @@ public:
    * This function implements ProcessPriorityManager::SetProcessPriority.
    */
   void SetProcessPriority(ContentParent* aContentParent,
-                          ProcessPriority aPriority);
+                          ProcessPriority aPriority,
+                          uint32_t aBackgroundLRU = 0);
 
   /**
    * If a magic testing-only pref is set, notify the observer service on the
@@ -236,6 +227,7 @@ public:
 
   int32_t Pid() const;
   uint64_t ChildID() const;
+  bool IsPreallocated() const;
 
   /**
    * Used in logging, this method returns the ContentParent's name followed by
@@ -270,10 +262,12 @@ public:
    * This overload is equivalent to SetPriorityNow(aPriority,
    * ComputeCPUPriority()).
    */
-  void SetPriorityNow(ProcessPriority aPriority);
+  void SetPriorityNow(ProcessPriority aPriority,
+                      uint32_t aBackgroundLRU = 0);
 
   void SetPriorityNow(ProcessPriority aPriority,
-                      ProcessCPUPriority aCPUPriority);
+                      ProcessCPUPriority aCPUPriority,
+                      uint32_t aBackgroundLRU = 0);
 
   void ShutDown();
 
@@ -301,6 +295,46 @@ private:
   nsCOMPtr<nsITimer> mResetPriorityTimer;
 };
 
+class BackgroundProcessLRUPool MOZ_FINAL
+{
+public:
+  static BackgroundProcessLRUPool* Singleton();
+
+  /**
+   * Used to remove a ContentParent from background LRU pool when
+   * it is destroyed or its priority changed from BACKGROUND to others.
+   */
+  void RemoveFromBackgroundLRUPool(ContentParent* aContentParent);
+
+  /**
+   * Used to add a ContentParent into background LRU pool when
+   * its priority changed to BACKGROUND from others.
+   */
+  void AddIntoBackgroundLRUPool(ContentParent* aContentParent);
+
+private:
+  static StaticAutoPtr<BackgroundProcessLRUPool> sSingleton;
+
+  int32_t mLRUPoolLevels;
+  int32_t mLRUPoolSize;
+  int32_t mLRUPoolAvailableIndex;
+  nsTArray<ContentParent*> mLRUPool;
+
+  uint32_t CalculateLRULevel(uint32_t aBackgroundLRUPoolIndex);
+
+  nsresult UpdateAvailableIndexInLRUPool(
+      ContentParent* aContentParent,
+      int32_t aTargetIndex = -1);
+
+  void ShiftLRUPool();
+
+  void EnsureLRUPool();
+
+  BackgroundProcessLRUPool();
+  DISALLOW_EVIL_CONSTRUCTORS(BackgroundProcessLRUPool);
+
+};
+
 /* static */ bool ProcessPriorityManagerImpl::sInitialized = false;
 /* static */ bool ProcessPriorityManagerImpl::sPrefListenersRegistered = false;
 /* static */ StaticRefPtr<ProcessPriorityManagerImpl>
@@ -431,12 +465,13 @@ ProcessPriorityManagerImpl::GetParticularProcessPriorityManager(
 
 void
 ProcessPriorityManagerImpl::SetProcessPriority(ContentParent* aContentParent,
-                                               ProcessPriority aPriority)
+                                               ProcessPriority aPriority,
+                                               uint32_t aBackgroundLRU)
 {
   MOZ_ASSERT(aContentParent);
   nsRefPtr<ParticularProcessPriorityManager> pppm =
     GetParticularProcessPriorityManager(aContentParent);
-  pppm->SetPriorityNow(aPriority);
+  pppm->SetPriorityNow(aPriority, aBackgroundLRU);
 }
 
 void
@@ -667,6 +702,12 @@ ParticularProcessPriorityManager::Pid() const
   return mContentParent ? mContentParent->Pid() : -1;
 }
 
+bool
+ParticularProcessPriorityManager::IsPreallocated() const
+{
+  return mContentParent ? mContentParent->IsPreallocated() : false;
+}
+
 const nsAutoCString&
 ParticularProcessPriorityManager::NameWithComma()
 {
@@ -767,7 +808,16 @@ ParticularProcessPriorityManager::ResetPriority()
   ProcessPriority processPriority = ComputePriority();
   if (mPriority == PROCESS_PRIORITY_UNKNOWN ||
       mPriority > processPriority) {
-    ScheduleResetPriority("backgroundGracePeriodMS");
+    // Apps set at a perceivable background priority are often playing media.
+    // Most media will have short gaps while changing tracks between songs,
+    // switching videos, etc.  Give these apps a longer grace period so they
+    // can get their next track started, if there is one, before getting
+    // downgraded.
+    if (mPriority == PROCESS_PRIORITY_BACKGROUND_PERCEIVABLE) {
+      ScheduleResetPriority("backgroundPerceivableGracePeriodMS");
+    } else {
+      ScheduleResetPriority("backgroundGracePeriodMS");
+    }
     return;
   }
 
@@ -906,20 +956,35 @@ ParticularProcessPriorityManager::ResetCPUPriorityNow()
 }
 
 void
-ParticularProcessPriorityManager::SetPriorityNow(ProcessPriority aPriority)
+ParticularProcessPriorityManager::SetPriorityNow(ProcessPriority aPriority,
+                                                 uint32_t aBackgroundLRU)
 {
-  SetPriorityNow(aPriority, ComputeCPUPriority());
+  SetPriorityNow(aPriority, ComputeCPUPriority(), aBackgroundLRU);
 }
 
 void
 ParticularProcessPriorityManager::SetPriorityNow(ProcessPriority aPriority,
-                                                 ProcessCPUPriority aCPUPriority)
+                                                 ProcessCPUPriority aCPUPriority,
+                                                 uint32_t aBackgroundLRU)
 {
   if (aPriority == PROCESS_PRIORITY_UNKNOWN) {
     MOZ_ASSERT(false);
     return;
   }
 
+  if (aBackgroundLRU > 0 &&
+      aPriority == PROCESS_PRIORITY_BACKGROUND &&
+      mPriority == PROCESS_PRIORITY_BACKGROUND) {
+    hal::SetProcessPriority(Pid(), mPriority, mCPUPriority, aBackgroundLRU);
+
+    nsPrintfCString ProcessPriorityWithBackgroundLRU("%s:%d",
+      ProcessPriorityToString(mPriority, mCPUPriority),
+      aBackgroundLRU);
+
+    FireTestOnlyObserverNotification("process-priority-with-background-LRU-set",
+      ProcessPriorityWithBackgroundLRU.get());
+  }
+
   if (!mContentParent ||
       !ProcessPriorityManagerImpl::PrefsEnabled() ||
       (mPriority == aPriority && mCPUPriority == aCPUPriority)) {
@@ -934,6 +999,18 @@ ParticularProcessPriorityManager::SetPriorityNow(ProcessPriority aPriority,
     return;
   }
 
+  if (aPriority == PROCESS_PRIORITY_BACKGROUND &&
+      mPriority != PROCESS_PRIORITY_BACKGROUND &&
+      !IsPreallocated()) {
+    ProcessPriorityManager::AddIntoBackgroundLRUPool(mContentParent);
+  }
+
+  if (aPriority != PROCESS_PRIORITY_BACKGROUND &&
+      mPriority == PROCESS_PRIORITY_BACKGROUND &&
+      !IsPreallocated()) {
+    ProcessPriorityManager::RemoveFromBackgroundLRUPool(mContentParent);
+  }
+
   LOGP("Changing priority from %s to %s.",
        ProcessPriorityToString(mPriority, mCPUPriority),
        ProcessPriorityToString(aPriority, aCPUPriority));
@@ -975,6 +1052,10 @@ ParticularProcessPriorityManager::ShutDown()
     mResetPriorityTimer = nullptr;
   }
 
+  if (mPriority == PROCESS_PRIORITY_BACKGROUND && !IsPreallocated()) {
+    ProcessPriorityManager::RemoveFromBackgroundLRUPool(mContentParent);
+  }
+
   mContentParent = nullptr;
 }
 
@@ -1108,6 +1189,181 @@ ProcessPriorityManagerChild::CurrentProcessIsForeground()
          mCachedPriority >= PROCESS_PRIORITY_FOREGROUND;
 }
 
+/* static */ StaticAutoPtr<BackgroundProcessLRUPool>
+BackgroundProcessLRUPool::sSingleton;
+
+/* static */ BackgroundProcessLRUPool*
+BackgroundProcessLRUPool::Singleton()
+{
+  if (!sSingleton) {
+    sSingleton = new BackgroundProcessLRUPool();
+    ClearOnShutdown(&sSingleton);
+  }
+  return sSingleton;
+}
+
+BackgroundProcessLRUPool::BackgroundProcessLRUPool()
+{
+  EnsureLRUPool();
+}
+
+uint32_t
+BackgroundProcessLRUPool::CalculateLRULevel(uint32_t aBackgroundLRUPoolIndex)
+{
+  // Set LRU level of each background process and maintain LRU buffer as below:
+
+  // Priority background  : LRU0
+  // Priority background+1: LRU1, LRU2
+  // Priority background+2: LRU3, LRU4, LRU5, LRU6
+  // Priority background+3: LRU7, LRU8, LRU9, LRU10, LRU11, LRU12, LRU13, LRU14
+  // ...
+  // Priority background+L-1: 2^(number of background LRU pool levels - 1)
+  // (End of buffer)
+
+  return (uint32_t)(log((float)aBackgroundLRUPoolIndex) / log(2.0));
+}
+
+void
+BackgroundProcessLRUPool::EnsureLRUPool()
+{
+  // We set mBackgroundLRUPoolLevels according to our pref.
+  // This value is used to set background process LRU pool
+  if (!NS_SUCCEEDED(Preferences::GetInt(
+        "dom.ipc.processPriorityManager.backgroundLRUPoolLevels",
+        &mLRUPoolLevels))) {
+    mLRUPoolLevels = 1;
+  }
+
+  if (mLRUPoolLevels <= 0) {
+    MOZ_CRASH();
+  }
+
+  // GonkHal defines OOM_ADJUST_MAX is 15 and b2g.js defines
+  // PROCESS_PRIORITY_BACKGROUND's oom_score_adj is 667 and oom_adj is 10.
+  // This means we can only have at most (15 -10 + 1) = 6 background LRU levels.
+  // See bug 822325 comment 49
+  MOZ_ASSERT(mLRUPoolLevels <= 6);
+
+  // LRU pool size = 2 ^ (number of background LRU pool levels) - 1
+  mLRUPoolSize = (1 << mLRUPoolLevels) - 1;
+
+  mLRUPoolAvailableIndex = 0;
+
+  LOG("Making background LRU pool with size(%d)", mLRUPoolSize);
+
+  mLRUPool.InsertElementsAt(0, mLRUPoolSize, (ContentParent*)nullptr);
+}
+
+void
+BackgroundProcessLRUPool::RemoveFromBackgroundLRUPool(
+    ContentParent* aContentParent)
+{
+  for (int32_t i = 0; i < mLRUPoolSize; i++) {
+    if (mLRUPool[i]) {
+      if (mLRUPool[i]->ChildID() == aContentParent->ChildID()) {
+
+        mLRUPool[i] = nullptr;
+        LOG("Remove ChildID(%llu) from LRU pool", aContentParent->ChildID());
+
+        // After we remove this ContentParent from LRU pool, we still need to
+        // update the available index if the index of removed one is less than
+        // the available index we already have.
+        UpdateAvailableIndexInLRUPool(aContentParent, i);
+        break;
+      }
+    }
+  }
+}
+
+nsresult
+BackgroundProcessLRUPool::UpdateAvailableIndexInLRUPool(
+    ContentParent* aContentParent,
+    int32_t aTargetIndex)
+{
+  // If we specify which index we want to assign to mLRUPoolAvailableIndex,
+  // We have to make sure the index in LRUPool doesn't point to any
+  // ContentParent.
+  if (aTargetIndex >= 0 && aTargetIndex < mLRUPoolSize &&
+      aTargetIndex < mLRUPoolAvailableIndex &&
+      !mLRUPool[aTargetIndex]) {
+    mLRUPoolAvailableIndex = aTargetIndex;
+    return NS_OK;
+  }
+
+  // When we didn't specify any legal aTargetIndex, then we just check
+  // whether current mLRUPoolAvailableIndex points to any ContentParent or not.
+  if (mLRUPoolAvailableIndex >= 0 && mLRUPoolAvailableIndex < mLRUPoolSize &&
+      !(mLRUPool[mLRUPoolAvailableIndex])) {
+    return NS_OK;
+  }
+
+  // Both above way failed. So now we have to find proper value
+  // for mLRUPoolAvailableIndex.
+  // We are looking for an available index. We only shift process with
+  // LRU less than the available index should have, so we stop update
+  // mLRUPoolAvailableIndex from the for loop once we got a candidate.
+  mLRUPoolAvailableIndex = -1;
+
+  for (int32_t i = 0; i < mLRUPoolSize; i++) {
+    if (mLRUPool[i]) {
+      if (mLRUPool[i]->ChildID() == aContentParent->ChildID()) {
+        LOG("ChildID(%llu) already in LRU pool", aContentParent->ChildID());
+        MOZ_ASSERT(false);
+        return NS_ERROR_UNEXPECTED;
+      }
+      continue;
+    } else {
+      if (mLRUPoolAvailableIndex == -1) {
+        mLRUPoolAvailableIndex = i;
+      }
+    }
+  }
+
+  // If the LRUPool is already full, mLRUPoolAvailableIndex is still -1 after
+  // above loop finished. We should set mLRUPoolAvailableIndex
+  // to mLRUPoolSize - 1 in this case. Here uses the mod operator to do it:
+  // New mLRUPoolAvailableIndex either equals old mLRUPoolAvailableIndex, or
+  // mLRUPoolSize - 1 if old mLRUPoolAvailableIndex is -1.
+  mLRUPoolAvailableIndex =
+    (mLRUPoolAvailableIndex + mLRUPoolSize) % mLRUPoolSize;
+
+  return NS_OK;
+}
+
+void
+BackgroundProcessLRUPool::ShiftLRUPool()
+{
+  for (int32_t i = mLRUPoolAvailableIndex; i > 0; i--) {
+    mLRUPool[i] = mLRUPool[i - 1];
+    // Check whether i+1 is power of Two.
+    // If so, then it crossed a LRU group boundary and
+    // we need to assign its new process priority LRU.
+    if (!((i + 1) & i)) {
+      ProcessPriorityManagerImpl::GetSingleton()->SetProcessPriority(
+        mLRUPool[i], PROCESS_PRIORITY_BACKGROUND, CalculateLRULevel(i + 1));
+    }
+  }
+}
+
+void
+BackgroundProcessLRUPool::AddIntoBackgroundLRUPool(
+    ContentParent* aContentParent)
+{
+  // We have to make sure that we have correct available index in LRU pool
+  if (!NS_SUCCEEDED(
+      UpdateAvailableIndexInLRUPool(aContentParent))) {
+    return;
+  }
+
+  // Shift the list in the pool, so we have room at index 0 for the newly added
+  // ContentParent
+  ShiftLRUPool();
+
+  mLRUPool[0] = aContentParent;
+
+  LOG("Add ChildID(%llu) into LRU pool", aContentParent->ChildID());
+}
+
 } // anonymous namespace
 
 namespace mozilla {
@@ -1132,6 +1388,31 @@ ProcessPriorityManager::SetProcessPriority(ContentParent* aContentParent,
   }
 }
 
+/* static */ void
+ProcessPriorityManager::RemoveFromBackgroundLRUPool(
+    ContentParent* aContentParent)
+{
+  MOZ_ASSERT(aContentParent);
+
+  BackgroundProcessLRUPool* singleton =
+    BackgroundProcessLRUPool::Singleton();
+  if (singleton) {
+    singleton->RemoveFromBackgroundLRUPool(aContentParent);
+  }
+}
+
+/* static */ void
+ProcessPriorityManager::AddIntoBackgroundLRUPool(ContentParent* aContentParent)
+{
+  MOZ_ASSERT(aContentParent);
+
+  BackgroundProcessLRUPool* singleton =
+    BackgroundProcessLRUPool::Singleton();
+  if (singleton) {
+    singleton->AddIntoBackgroundLRUPool(aContentParent);
+  }
+}
+
 /* static */ bool
 ProcessPriorityManager::CurrentProcessIsForeground()
 {
