@@ -2,7 +2,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "MobileConnection.h"
+#include "mozilla/dom/network/MobileConnection.h"
+
 #include "GeneratedEvents.h"
 #include "mozilla/Preferences.h"
 #include "nsDOMEvent.h"
@@ -14,17 +15,17 @@
 #include "nsIDOMMozOtaStatusEvent.h"
 #include "nsIDOMUSSDReceivedEvent.h"
 #include "nsIPermissionManager.h"
+#include "nsIVariant.h"
 
 #include "nsJSUtils.h"
 #include "nsJSON.h"
-#include "jsapi.h"
 #include "mozilla/Services.h"
 
 #define NS_RILCONTENTHELPER_CONTRACTID "@mozilla.org/ril/content-helper;1"
 
 using namespace mozilla::dom::network;
 
-class MobileConnection::Listener : public nsIMobileConnectionListener
+class MobileConnection::Listener MOZ_FINAL : public nsIMobileConnectionListener
 {
   MobileConnection* mMobileConnection;
 
@@ -77,8 +78,11 @@ NS_IMPL_EVENT_HANDLER(MobileConnection, dataerror)
 NS_IMPL_EVENT_HANDLER(MobileConnection, cfstatechange)
 NS_IMPL_EVENT_HANDLER(MobileConnection, emergencycbmodechange)
 NS_IMPL_EVENT_HANDLER(MobileConnection, otastatuschange)
+NS_IMPL_EVENT_HANDLER(MobileConnection, iccchange)
+NS_IMPL_EVENT_HANDLER(MobileConnection, radiostatechange)
 
-MobileConnection::MobileConnection()
+MobileConnection::MobileConnection(uint32_t aClientId)
+: mClientId(aClientId)
 {
   mProvider = do_GetService(NS_RILCONTENTHELPER_CONTRACTID);
   mWindow = nullptr;
@@ -99,9 +103,8 @@ MobileConnection::Init(nsPIDOMWindow* aWindow)
   mWindow = do_GetWeakReference(aWindow);
   mListener = new Listener(this);
 
-  if (!CheckPermission("mobilenetwork") &&
-      CheckPermission("mobileconnection")) {
-    DebugOnly<nsresult> rv = mProvider->RegisterMobileConnectionMsg(mListener);
+  if (CheckPermission("mobileconnection")) {
+    DebugOnly<nsresult> rv = mProvider->RegisterMobileConnectionMsg(mClientId, mListener);
     NS_WARN_IF_FALSE(NS_SUCCEEDED(rv),
                      "Failed registering mobile connection messages with provider");
 
@@ -114,7 +117,7 @@ MobileConnection::Shutdown()
 {
   if (mProvider && mListener) {
     mListener->Disconnect();
-    mProvider->UnregisterMobileConnectionMsg(mListener);
+    mProvider->UnregisterMobileConnectionMsg(mClientId, mListener);
     mProvider = nullptr;
     mListener = nullptr;
   }
@@ -123,35 +126,33 @@ MobileConnection::Shutdown()
 // nsIDOMMozMobileConnection
 
 NS_IMETHODIMP
-MobileConnection::GetLastKnownNetwork(nsAString& network)
+MobileConnection::GetLastKnownNetwork(nsAString& aNetwork)
 {
-  network.SetIsVoid(true);
+  aNetwork.SetIsVoid(true);
 
   if (!CheckPermission("mobilenetwork")) {
     return NS_OK;
   }
 
-  network = mozilla::Preferences::GetString("ril.lastKnownNetwork");
-  return NS_OK;
+  return mProvider->GetLastKnownNetwork(mClientId, aNetwork);
 }
 
 NS_IMETHODIMP
-MobileConnection::GetLastKnownHomeNetwork(nsAString& network)
+MobileConnection::GetLastKnownHomeNetwork(nsAString& aNetwork)
 {
-  network.SetIsVoid(true);
+  aNetwork.SetIsVoid(true);
 
   if (!CheckPermission("mobilenetwork")) {
     return NS_OK;
   }
 
-  network = mozilla::Preferences::GetString("ril.lastKnownHomeNetwork");
-  return NS_OK;
+  return mProvider->GetLastKnownHomeNetwork(mClientId, aNetwork);
 }
 
 // All fields below require the "mobileconnection" permission.
 
 bool
-MobileConnection::CheckPermission(const char* type)
+MobileConnection::CheckPermission(const char* aType)
 {
   nsCOMPtr<nsPIDOMWindow> window = do_QueryReferent(mWindow);
   NS_ENSURE_TRUE(window, false);
@@ -161,47 +162,81 @@ MobileConnection::CheckPermission(const char* type)
   NS_ENSURE_TRUE(permMgr, false);
 
   uint32_t permission = nsIPermissionManager::DENY_ACTION;
-  permMgr->TestPermissionFromWindow(window, type, &permission);
+  permMgr->TestPermissionFromWindow(window, aType, &permission);
   return permission == nsIPermissionManager::ALLOW_ACTION;
 }
 
 NS_IMETHODIMP
-MobileConnection::GetVoice(nsIDOMMozMobileConnectionInfo** voice)
+MobileConnection::GetVoice(nsIDOMMozMobileConnectionInfo** aVoice)
 {
-  *voice = nullptr;
+  *aVoice = nullptr;
 
   if (!mProvider || !CheckPermission("mobileconnection")) {
     return NS_OK;
   }
-  return mProvider->GetVoiceConnectionInfo(voice);
+  return mProvider->GetVoiceConnectionInfo(mClientId, aVoice);
 }
 
 NS_IMETHODIMP
-MobileConnection::GetData(nsIDOMMozMobileConnectionInfo** data)
+MobileConnection::GetData(nsIDOMMozMobileConnectionInfo** aData)
 {
-  *data = nullptr;
+  *aData = nullptr;
 
   if (!mProvider || !CheckPermission("mobileconnection")) {
     return NS_OK;
   }
-  return mProvider->GetDataConnectionInfo(data);
+  return mProvider->GetDataConnectionInfo(mClientId, aData);
 }
 
 NS_IMETHODIMP
-MobileConnection::GetNetworkSelectionMode(nsAString& networkSelectionMode)
+MobileConnection::GetIccId(nsAString& aIccId)
 {
-  networkSelectionMode.SetIsVoid(true);
+  aIccId.SetIsVoid(true);
 
   if (!mProvider || !CheckPermission("mobileconnection")) {
      return NS_OK;
   }
-  return mProvider->GetNetworkSelectionMode(networkSelectionMode);
+  return mProvider->GetIccId(mClientId, aIccId);
 }
 
 NS_IMETHODIMP
-MobileConnection::GetNetworks(nsIDOMDOMRequest** request)
+MobileConnection::GetNetworkSelectionMode(nsAString& aNetworkSelectionMode)
 {
-  *request = nullptr;
+  aNetworkSelectionMode.SetIsVoid(true);
+
+  if (!mProvider || !CheckPermission("mobileconnection")) {
+     return NS_OK;
+  }
+  return mProvider->GetNetworkSelectionMode(mClientId, aNetworkSelectionMode);
+}
+
+NS_IMETHODIMP
+MobileConnection::GetRadioState(nsAString& aRadioState)
+{
+  aRadioState.SetIsVoid(true);
+
+  if (!mProvider || !CheckPermission("mobileconnection")) {
+     return NS_OK;
+  }
+  return mProvider->GetRadioState(mClientId, aRadioState);
+}
+
+NS_IMETHODIMP
+MobileConnection::GetSupportedNetworkTypes(nsIVariant** aSupportedNetworkTypes)
+{
+  *aSupportedNetworkTypes = nullptr;
+
+  if (!mProvider || !CheckPermission("mobileconnection")) {
+     return NS_OK;
+  }
+
+  return mProvider->GetSupportedNetworkTypes(mClientId, aSupportedNetworkTypes);
+}
+
+NS_IMETHODIMP
+MobileConnection::GetNetworks(nsIDOMDOMRequest** aRequest)
+{
+  *aRequest = nullptr;
 
   if (!CheckPermission("mobileconnection")) {
     return NS_OK;
@@ -211,13 +246,13 @@ MobileConnection::GetNetworks(nsIDOMDOMRequest** request)
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->GetNetworks(GetOwner(), request);
+  return mProvider->GetNetworks(mClientId, GetOwner(), aRequest);
 }
 
 NS_IMETHODIMP
-MobileConnection::SelectNetwork(nsIDOMMozMobileNetworkInfo* network, nsIDOMDOMRequest** request)
+MobileConnection::SelectNetwork(nsIDOMMozMobileNetworkInfo* aNetwork, nsIDOMDOMRequest** aRequest)
 {
-  *request = nullptr;
+  *aRequest = nullptr;
 
   if (!CheckPermission("mobileconnection")) {
     return NS_OK;
@@ -227,13 +262,13 @@ MobileConnection::SelectNetwork(nsIDOMMozMobileNetworkInfo* network, nsIDOMDOMRe
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->SelectNetwork(GetOwner(), network, request);
+  return mProvider->SelectNetwork(mClientId, GetOwner(), aNetwork, aRequest);
 }
 
 NS_IMETHODIMP
-MobileConnection::SelectNetworkAutomatically(nsIDOMDOMRequest** request)
+MobileConnection::SelectNetworkAutomatically(nsIDOMDOMRequest** aRequest)
 {
-  *request = nullptr;
+  *aRequest = nullptr;
 
   if (!CheckPermission("mobileconnection")) {
     return NS_OK;
@@ -243,7 +278,40 @@ MobileConnection::SelectNetworkAutomatically(nsIDOMDOMRequest** request)
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->SelectNetworkAutomatically(GetOwner(), request);
+  return mProvider->SelectNetworkAutomatically(mClientId, GetOwner(), aRequest);
+}
+
+NS_IMETHODIMP
+MobileConnection::SetPreferredNetworkType(const nsAString& aType,
+                                          nsIDOMDOMRequest** aDomRequest)
+{
+  *aDomRequest = nullptr;
+
+  if (!CheckPermission("mobileconnection")) {
+    return NS_OK;
+  }
+
+  if (!mProvider) {
+    return NS_ERROR_FAILURE;
+  }
+
+  return mProvider->SetPreferredNetworkType(mClientId, GetOwner(), aType, aDomRequest);
+}
+
+NS_IMETHODIMP
+MobileConnection::GetPreferredNetworkType(nsIDOMDOMRequest** aDomRequest)
+{
+  *aDomRequest = nullptr;
+
+  if (!CheckPermission("mobileconnection")) {
+    return NS_OK;
+  }
+
+  if (!mProvider) {
+    return NS_ERROR_FAILURE;
+  }
+
+  return mProvider->GetPreferredNetworkType(mClientId, GetOwner(), aDomRequest);
 }
 
 NS_IMETHODIMP
@@ -259,7 +327,7 @@ MobileConnection::SetRoamingPreference(const nsAString& aMode, nsIDOMDOMRequest*
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->SetRoamingPreference(GetOwner(), aMode, aDomRequest);
+  return mProvider->SetRoamingPreference(mClientId, GetOwner(), aMode, aDomRequest);
 }
 
 NS_IMETHODIMP
@@ -275,7 +343,7 @@ MobileConnection::GetRoamingPreference(nsIDOMDOMRequest** aDomRequest)
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->GetRoamingPreference(GetOwner(), aDomRequest);
+  return mProvider->GetRoamingPreference(mClientId, GetOwner(), aDomRequest);
 }
 
 NS_IMETHODIMP
@@ -291,7 +359,7 @@ MobileConnection::SetVoicePrivacyMode(bool aEnabled, nsIDOMDOMRequest** aDomRequ
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->SetVoicePrivacyMode(GetOwner(), aEnabled, aDomRequest);
+  return mProvider->SetVoicePrivacyMode(mClientId, GetOwner(), aEnabled, aDomRequest);
 }
 
 NS_IMETHODIMP
@@ -307,7 +375,7 @@ MobileConnection::GetVoicePrivacyMode(nsIDOMDOMRequest** aDomRequest)
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->GetVoicePrivacyMode(GetOwner(), aDomRequest);
+  return mProvider->GetVoicePrivacyMode(mClientId, GetOwner(), aDomRequest);
 }
 
 NS_IMETHODIMP
@@ -322,7 +390,7 @@ MobileConnection::SendMMI(const nsAString& aMMIString,
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->SendMMI(GetOwner(), aMMIString, aRequest);
+  return mProvider->SendMMI(mClientId, GetOwner(), aMMIString, aRequest);
 }
 
 NS_IMETHODIMP
@@ -336,7 +404,7 @@ MobileConnection::CancelMMI(nsIDOMDOMRequest** aRequest)
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->CancelMMI(GetOwner(), aRequest);
+  return mProvider->CancelMMI(mClientId, GetOwner(),aRequest);
 }
 
 NS_IMETHODIMP
@@ -353,7 +421,7 @@ MobileConnection::GetCallForwardingOption(uint16_t aReason,
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->GetCallForwardingOption(GetOwner(), aReason, aRequest);
+  return mProvider->GetCallForwardingOption(mClientId, GetOwner(), aReason, aRequest);
 }
 
 NS_IMETHODIMP
@@ -370,7 +438,7 @@ MobileConnection::SetCallForwardingOption(nsIDOMMozMobileCFInfo* aCFInfo,
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->SetCallForwardingOption(GetOwner(), aCFInfo, aRequest);
+  return mProvider->SetCallForwardingOption(mClientId, GetOwner(), aCFInfo, aRequest);
 }
 
 NS_IMETHODIMP
@@ -387,7 +455,7 @@ MobileConnection::GetCallBarringOption(const JS::Value& aOption,
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->GetCallBarringOption(GetOwner(), aOption, aRequest);
+  return mProvider->GetCallBarringOption(mClientId, GetOwner(), aOption, aRequest);
 }
 
 NS_IMETHODIMP
@@ -404,7 +472,7 @@ MobileConnection::SetCallBarringOption(const JS::Value& aOption,
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->SetCallBarringOption(GetOwner(), aOption, aRequest);
+  return mProvider->SetCallBarringOption(mClientId, GetOwner(), aOption, aRequest);
 }
 
 NS_IMETHODIMP
@@ -421,7 +489,7 @@ MobileConnection::ChangeCallBarringPassword(const JS::Value& aInfo,
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->ChangeCallBarringPassword(GetOwner(), aInfo, aRequest);
+  return mProvider->ChangeCallBarringPassword(mClientId, GetOwner(), aInfo, aRequest);
 }
 
 NS_IMETHODIMP
@@ -437,7 +505,7 @@ MobileConnection::GetCallWaitingOption(nsIDOMDOMRequest** aRequest)
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->GetCallWaitingOption(GetOwner(), aRequest);
+  return mProvider->GetCallWaitingOption(mClientId, GetOwner(), aRequest);
 }
 
 NS_IMETHODIMP
@@ -454,7 +522,7 @@ MobileConnection::SetCallWaitingOption(bool aEnabled,
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->SetCallWaitingOption(GetOwner(), aEnabled, aRequest);
+  return mProvider->SetCallWaitingOption(mClientId, GetOwner(), aEnabled, aRequest);
 }
 
 NS_IMETHODIMP
@@ -470,7 +538,7 @@ MobileConnection::GetCallingLineIdRestriction(nsIDOMDOMRequest** aRequest)
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->GetCallingLineIdRestriction(GetOwner(), aRequest);
+  return mProvider->GetCallingLineIdRestriction(mClientId, GetOwner(), aRequest);
 }
 
 NS_IMETHODIMP
@@ -487,7 +555,7 @@ MobileConnection::SetCallingLineIdRestriction(unsigned short aClirMode,
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->SetCallingLineIdRestriction(GetOwner(), aClirMode, aRequest);
+  return mProvider->SetCallingLineIdRestriction(mClientId, GetOwner(), aClirMode, aRequest);
 }
 
 NS_IMETHODIMP
@@ -503,7 +571,24 @@ MobileConnection::ExitEmergencyCbMode(nsIDOMDOMRequest** aRequest)
     return NS_ERROR_FAILURE;
   }
 
-  return mProvider->ExitEmergencyCbMode(GetOwner(), aRequest);
+  return mProvider->ExitEmergencyCbMode(mClientId, GetOwner(), aRequest);
+}
+
+NS_IMETHODIMP
+MobileConnection::SetRadioEnabled(bool aEnabled,
+                                  nsIDOMDOMRequest** aRequest)
+{
+  *aRequest = nullptr;
+
+  if (!CheckPermission("mobileconnection")) {
+    return NS_OK;
+  }
+
+  if (!mProvider) {
+    return NS_ERROR_FAILURE;
+  }
+
+  return mProvider->SetRadioEnabled(mClientId, GetOwner(), aEnabled, aRequest);
 }
 
 // nsIMobileConnectionListener
@@ -631,3 +716,23 @@ MobileConnection::NotifyOtaStatusChanged(const nsAString& aStatus)
 
   return DispatchTrustedEvent(ce);
 }
+
+NS_IMETHODIMP
+MobileConnection::NotifyIccChanged()
+{
+  if (!CheckPermission("mobileconnection")) {
+    return NS_OK;
+  }
+
+  return DispatchTrustedEvent(NS_LITERAL_STRING("iccchange"));
+}
+
+NS_IMETHODIMP
+MobileConnection::NotifyRadioStateChanged()
+{
+  if (!CheckPermission("mobileconnection")) {
+    return NS_OK;
+  }
+
+  return DispatchTrustedEvent(NS_LITERAL_STRING("radiostatechange"));
+}
