@@ -142,16 +142,15 @@ function usbTetheringSuccess(params) {
   return true;
 }
 
-function networkInterfaceStatsFail(params) {
+function networkInterfaceAlarmFail(params) {
   // Notify the main thread.
   postMessage(params);
   return true;
 }
 
-function networkInterfaceStatsSuccess(params) {
+function networkInterfaceAlarmSuccess(params) {
   // Notify the main thread.
-  params.txBytes = parseFloat(params.resultReason);
-
+  params.error = parseFloat(params.resultReason);
   postMessage(params);
   return true;
 }
@@ -213,6 +212,40 @@ self.onmessage = function onmessage(event) {
   }
 };
 
+/**
+ * Start/Stop DHCP server.
+ */
+function setDhcpServer(config) {
+  function onSuccess() {
+    postMessage({ id: config.id, success: true });
+    return true;
+  }
+
+  function onError() {
+    postMessage({ id: config.id, success: false });
+  }
+
+  let startDhcpServerChain = [setInterfaceUp,
+                              startTethering,
+                              onSuccess];
+
+  let stopDhcpServerChain = [stopTethering,
+                             onSuccess];
+
+  if (config.enabled) {
+    let params = { wifiStartIp: config.startIp,
+                   wifiEndIp: config.endIp,
+                   ip: config.serverIp,
+                   prefix: config.maskLength,
+                   ifname: config.ifname,
+                   link: "up" };
+
+    chain(params, startDhcpServerChain, onError);
+  } else {
+    chain({}, stopDhcpServerChain, onError);
+  }
+}
+
 /**
  * Set DNS servers for given network interface.
  */
@@ -455,8 +488,14 @@ function startTethering(params, callback) {
   if (params.resultReason.indexOf("started") !== -1) {
     command = DUMMY_COMMAND;
   } else {
-    command = "tether start " + params.wifiStartIp + " " + params.wifiEndIp +
-              " " + params.usbStartIp + " " + params.usbEndIp;
+    command = "tether start " + params.wifiStartIp + " " + params.wifiEndIp;
+
+    // If usbStartIp/usbEndIp is not valid, don't append them since
+    // the trailing white spaces will be parsed to extra empty args
+    // See: http://androidxref.com/4.3_r2.1/xref/system/core/libsysutils/src/FrameworkListener.cpp#78
+    if (params.usbStartIp && params.usbEndIp) {
+      command += " " + params.usbStartIp + " " + params.usbEndIp;
+    }
   }
   return doCommand(command, callback);
 }
@@ -515,16 +554,8 @@ function enableNat(params, callback) {
 }
 
 function disableNat(params, callback) {
-  let command;
-
-  // Don't disable nat because others interface still need it.
-  // Send the dummy command to continue the function chain.
-  if ("interfaceList" in params && params.interfaceList.length > 1) {
-    command = DUMMY_COMMAND;
-  } else {
-    command = "nat disable " + params.internalIfname + " " +
-              params.externalIfname + " " + "0";
-  }
+  let command = "nat disable " + params.internalIfname + " " +
+                 params.externalIfname + " " + "0";
   return doCommand(command, callback);
 }
 
@@ -563,15 +594,28 @@ function stopSoftAP(params, callback) {
   return doCommand(command, callback);
 }
 
-function getRxBytes(params, callback) {
-  let command = "interface readrxcounter " + params.ifname;
+function enableAlarm(params, callback) {
+  let command = "bandwidth enable";
   return doCommand(command, callback);
 }
 
-function getTxBytes(params, callback) {
-  params.rxBytes = parseFloat(params.resultReason);
+function disableAlarm(params, callback) {
+  let command = "bandwidth disable";
+  return doCommand(command, callback);
+}
 
-  let command = "interface readtxcounter " + params.ifname;
+function setQuota(params, callback) {
+  let command = "bandwidth setiquota " + params.ifname + " " + parseInt('0xffffffffffffffff');
+  return doCommand(command, callback);
+}
+
+function removeQuota(params, callback) {
+  let command = "bandwidth removeiquota " + params.ifname;
+  return doCommand(command, callback);
+}
+
+function setAlarm(params, callback) {
+  let command = "bandwidth setinterfacealert " + params.ifname + " " + params.threshold;
   return doCommand(command, callback);
 }
 
@@ -864,21 +908,36 @@ function setUSBTethering(params) {
   return true;
 }
 
-let gNetworkInterfaceStatsChain = [getRxBytes,
-                                   getTxBytes,
-                                   networkInterfaceStatsSuccess];
+let gNetworkInterfaceEnableAlarmChain = [enableAlarm,
+                                         setQuota,
+                                         setAlarm,
+                                         networkInterfaceAlarmSuccess];
 
-/**
- * handling main thread's get network interface stats request
- */
-function getNetworkInterfaceStats(params) {
-  debug("getNetworkInterfaceStats: " + params.ifname);
+function enableNetworkInterfaceAlarm(params) {
+  debug("enableNetworkInterfaceAlarms: " + params.ifname);
 
-  params.rxBytes = -1;
-  params.txBytes = -1;
-  params.date = new Date();
+  chain(params, gNetworkInterfaceEnableAlarmChain, networkInterfaceAlarmFail);
+  return true;
+}
 
-  chain(params, gNetworkInterfaceStatsChain, networkInterfaceStatsFail);
+let gNetworkInterfaceDisableAlarmChain = [removeQuota,
+                                          disableAlarm,
+                                          networkInterfaceAlarmSuccess];
+
+function disableNetworkInterfaceAlarm(params) {
+  debug("disableNetworkInterfaceAlarms: " + params.ifname);
+
+  chain(params, gNetworkInterfaceDisableAlarmChain, networkInterfaceAlarmFail);
+  return true;
+}
+
+let gNetworkInterfaceSetAlarmChain = [setAlarm,
+                                      networkInterfaceAlarmSuccess];
+
+function setNetworkInterfaceAlarm(params) {
+  debug("setNetworkInterfaceAlarms: " + params.ifname);
+
+  chain(params, gNetworkInterfaceSetAlarmChain, networkInterfaceAlarmFail);
   return true;
 }
 
