@@ -8,6 +8,7 @@
 
 #include "mozilla/dom/telephony/TelephonyCommon.h"
 #include "mozilla/Attributes.h"
+#include "nsIObserver.h"
 #include "nsITelephonyProvider.h"
 
 BEGIN_TELEPHONY_NAMESPACE
@@ -16,11 +17,13 @@ class PTelephonyChild;
 
 class TelephonyIPCProvider MOZ_FINAL : public nsITelephonyProvider
                                      , public nsITelephonyListener
+                                     , public nsIObserver
 {
 public:
   NS_DECL_ISUPPORTS
   NS_DECL_NSITELEPHONYPROVIDER
   NS_DECL_NSITELEPHONYLISTENER
+  NS_DECL_NSIOBSERVER
 
   TelephonyIPCProvider();
 
@@ -30,6 +33,7 @@ protected:
 private:
   nsTArray<nsCOMPtr<nsITelephonyListener> > mListeners;
   PTelephonyChild* mPTelephonyChild;
+  uint32_t mDefaultServiceId;
 };
 
 END_TELEPHONY_NAMESPACE
