@@ -9,8 +9,6 @@
 
 #include "Workers.h"
 
-#include "jspubtd.h"
-
 class nsIDOMFile;
 class nsIDOMBlob;
 
@@ -19,7 +17,7 @@ BEGIN_WORKERS_NAMESPACE
 namespace file {
 
 bool
-InitClasses(JSContext* aCx, JSObject* aGlobal);
+InitClasses(JSContext* aCx, JS::Handle<JSObject*> aGlobal);
 
 JSObject*
 CreateBlob(JSContext* aCx, nsIDOMBlob* aBlob);
