@@ -7,10 +7,11 @@
 #include <string.h>
 
 #include "mozilla/dom/DocumentFragment.h"
+#include "mozilla/ArrayUtils.h"
 #include "mozilla/Base64.h"
+#include "mozilla/BasicEvents.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/Selection.h"
-#include "mozilla/Util.h"
 #include "nsAString.h"
 #include "nsAutoPtr.h"
 #include "nsCOMArray.h"
@@ -26,7 +27,6 @@
 #include "nsEditor.h"
 #include "nsEditorUtils.h"
 #include "nsError.h"
-#include "nsGUIEvent.h"
 #include "nsGkAtoms.h"
 #include "nsHTMLEditUtils.h"
 #include "nsHTMLEditor.h"
@@ -95,8 +95,6 @@ class nsISupports;
 using namespace mozilla;
 using namespace mozilla::dom;
 
-const PRUnichar nbsp = 160;
-
 #define kInsertCookie  "_moz_Insert Here_moz_"
 
 // some little helpers
