@@ -8,6 +8,7 @@
 #include "ScaledFontBase.h"
 #include "ScaledFontCairo.h"
 #include "skia/SkDevice.h"
+#include "FilterNodeSoftware.h"
 
 #ifdef USE_SKIA_GPU
 #include "skia/SkGpuDevice.h"
@@ -25,6 +26,7 @@
 #include "Logging.h"
 #include "HelpersSkia.h"
 #include "Tools.h"
+#include "DataSurfaceHelpers.h"
 #include <algorithm>
 
 namespace mozilla {
@@ -76,6 +78,8 @@ public:
 };
 
 #ifdef USE_SKIA_GPU
+int DrawTargetSkia::sTextureCacheCount = 256;
+int DrawTargetSkia::sTextureCacheSizeInBytes = 96*1024*1024;
 
 static std::vector<DrawTargetSkia*>&
 GLDrawTargets()
@@ -84,35 +88,27 @@ GLDrawTargets()
   return targets;
 }
 
-#define SKIA_MAX_CACHE_ITEMS 256
-
-// 64MB was chosen because it seems we can pass all of our tests
-// on the current hardware (Tegra2) without running out of memory
-#define SKIA_TOTAL_CACHE_SIZE 64*1024*1024
-
-static void
-SetCacheLimits()
+void
+DrawTargetSkia::RebalanceCacheLimits()
 {
+  // Divide the global cache limits equally between all currently active GL-backed
+  // Skia DrawTargets.
   std::vector<DrawTargetSkia*>& targets = GLDrawTargets();
-  uint32_t size = targets.size();
-  if (size == 0)
+  uint32_t targetCount = targets.size();
+  if (targetCount == 0)
     return;
 
-  int individualCacheSize = SKIA_TOTAL_CACHE_SIZE / size;
-  for (uint32_t i = 0; i < size; i++) {
-    targets[i]->SetCacheLimits(SKIA_MAX_CACHE_ITEMS, individualCacheSize);
+  int individualCacheSize = sTextureCacheSizeInBytes / targetCount;
+  for (uint32_t i = 0; i < targetCount; i++) {
+    targets[i]->SetCacheLimits(sTextureCacheCount, individualCacheSize);
   }
-
 }
 
-#undef SKIA_MAX_CACHE_ITEMS
-#undef SKIA_TOTAL_CACHE_SIZE
-
 static void
 AddGLDrawTarget(DrawTargetSkia* target)
 {
   GLDrawTargets().push_back(target);
-  SetCacheLimits();
+  DrawTargetSkia::RebalanceCacheLimits();
 }
 
 static void
@@ -122,20 +118,62 @@ RemoveGLDrawTarget(DrawTargetSkia* target)
   std::vector<DrawTargetSkia*>::iterator it = std::find(targets.begin(), targets.end(), target);
   if (it != targets.end()) {
     targets.erase(it);
-    SetCacheLimits();
+    DrawTargetSkia::RebalanceCacheLimits();
+  }
+}
+
+void
+DrawTargetSkia::SetGlobalCacheLimits(int aCount, int aSizeInBytes)
+{
+  sTextureCacheCount = aCount;
+  sTextureCacheSizeInBytes = aSizeInBytes;
+
+  DrawTargetSkia::RebalanceCacheLimits();
+}
+
+void
+DrawTargetSkia::PurgeCaches()
+{
+  if (mGrContext) {
+    mGrContext->freeGpuResources();
+  }
+}
+
+/* static */ void
+DrawTargetSkia::PurgeAllCaches()
+{
+  std::vector<DrawTargetSkia*>& targets = GLDrawTargets();
+  uint32_t targetCount = targets.size();
+  for (uint32_t i = 0; i < targetCount; i++) {
+    targets[i]->PurgeCaches();
   }
 }
 
 #endif
 
+static SkBitmap
+GetBitmapForSurface(SourceSurface *aSurface)
+{
+  switch (aSurface->GetType()) {
+  case SURFACE_SKIA:
+    return static_cast<SourceSurfaceSkia*>(aSurface)->GetBitmap();
+  case SURFACE_DATA:
+    {
+      DataSourceSurface* surf = static_cast<DataSourceSurface*>(aSurface);
+      SkBitmap tmp;
+      tmp.setConfig(GfxFormatToSkiaConfig(surf->GetFormat()),
+                    surf->GetSize().width, surf->GetSize().height, surf->Stride());
+      tmp.setPixels(surf->GetData());
+      return tmp;
+    }
+  default:
+    MOZ_CRASH("Non-skia SourceSurfaces need to be DataSourceSurfaces");
+  }
+}
+
 DrawTargetSkia::DrawTargetSkia()
   : mSnapshot(nullptr)
 {
-#ifdef ANDROID
-  mSoftClipping = false;
-#else
-  mSoftClipping = true;
-#endif
 }
 
 DrawTargetSkia::~DrawTargetSkia()
@@ -227,7 +265,7 @@ void SetPaintPattern(SkPaint& aPaint, const Pattern& aPattern, Float aAlpha = 1.
     }
     case PATTERN_SURFACE: {
       const SurfacePattern& pat = static_cast<const SurfacePattern&>(aPattern);
-      const SkBitmap& bitmap = static_cast<SourceSurfaceSkia*>(pat.mSurface.get())->GetBitmap();
+      const SkBitmap& bitmap = GetBitmapForSurface(pat.mSurface);
 
       SkShader::TileMode mode = ExtendModeToTileMode(pat.mExtendMode);
       SkShader* shader = SkShader::CreateBitmapShader(bitmap, mode, mode);
@@ -276,8 +314,6 @@ struct AutoPaintSetup {
       mPaint.setAntiAlias(false);
     }
 
-    MOZ_ASSERT(aOptions.mSnapping == SNAP_NONE, "Pixel snapping not supported yet!");
-    
     // TODO: We could skip the temporary for operator_source and just
     // clear the clip rect. The other operators would be harder
     // but could be worth it to skip pushing a group.
@@ -316,7 +352,7 @@ DrawTargetSkia::DrawSurface(SourceSurface *aSurface,
                             const DrawSurfaceOptions &aSurfOptions,
                             const DrawOptions &aOptions)
 {
-  if (aSurface->GetType() != SURFACE_SKIA) {
+  if (!(aSurface->GetType() == SURFACE_SKIA || aSurface->GetType() == SURFACE_DATA)) {
     return;
   }
 
@@ -326,43 +362,27 @@ DrawTargetSkia::DrawSurface(SourceSurface *aSurface,
 
   MarkChanged();
 
-  IntRect sourceIntRect;
-  bool integerAligned = aSource.ToIntRect(&sourceIntRect);
-
   SkRect destRect = RectToSkRect(aDest);
   SkRect sourceRect = RectToSkRect(aSource);
 
-  Rect boundingSource = aSource;
-  boundingSource.RoundOut();
-
-  SkRect sourceBoundingRect = RectToSkRect(boundingSource);
-  SkIRect sourceBoundingIRect = RectToSkIRect(boundingSource);
-
-  const SkBitmap& bitmap = static_cast<SourceSurfaceSkia*>(aSurface)->GetBitmap();
+  const SkBitmap& bitmap = GetBitmapForSurface(aSurface);
  
   AutoPaintSetup paint(mCanvas.get(), aOptions);
   if (aSurfOptions.mFilter == FILTER_POINT) {
     paint.mPaint.setFilterBitmap(false);
   }
 
-  if (!integerAligned) {
-    // We need to inflate our destRect by the same amount we inflated sourceRect
-    // by when we rounded up to the nearest integer size to ensure we interpolate
-    // the edge pixels properly, but clip to the true destination rect first so
-    // we don't draw outside our designated area.
-    mCanvas->save();
-    mCanvas->clipRect(destRect);
+  mCanvas->drawBitmapRectToRect(bitmap, &sourceRect, destRect, &paint.mPaint);
+}
 
-    SkMatrix rectTransform;
-    rectTransform.setRectToRect(sourceRect, sourceBoundingRect, SkMatrix::kFill_ScaleToFit);
-    rectTransform.mapRect(&destRect);
-  }
-
-  mCanvas->drawBitmapRect(bitmap, &sourceBoundingIRect, destRect, &paint.mPaint);
-
-  if (!integerAligned) {
-    mCanvas->restore();
-  }
+void
+DrawTargetSkia::DrawFilter(FilterNode *aNode,
+                           const Rect &aSourceRect,
+                           const Point &aDestPoint,
+                           const DrawOptions &aOptions)
+{
+  FilterNodeSoftware* filter = static_cast<FilterNodeSoftware*>(aNode);
+  filter->Draw(this, aSourceRect, aDestPoint, aOptions);
 }
 
 void
@@ -379,7 +399,7 @@ DrawTargetSkia::DrawSurfaceWithShadow(SourceSurface *aSurface,
 
   uint32_t blurFlags = SkBlurMaskFilter::kHighQuality_BlurFlag |
                        SkBlurMaskFilter::kIgnoreTransform_BlurFlag;
-  const SkBitmap& bitmap = static_cast<SourceSurfaceSkia*>(aSurface)->GetBitmap();
+  const SkBitmap& bitmap = GetBitmapForSurface(aSurface);
   SkShader* shader = SkShader::CreateBitmapShader(bitmap, SkShader::kClamp_TileMode, SkShader::kClamp_TileMode);
   SkMatrix matrix;
   matrix.reset();
@@ -588,26 +608,39 @@ DrawTargetSkia::Mask(const Pattern &aSource,
   raster->addLayer(maskPaint);
   SkSafeUnref(paint.mPaint.setRasterizer(raster));
 
-  // Skia only uses the mask rasterizer when we are drawing a path/rect.
-  // Take our destination bounds and convert them into user space to use
-  // as the path to draw.
-  SkPath path;
-  path.addRect(SkRect::MakeWH(SkScalar(mSize.width), SkScalar(mSize.height)));
- 
-  Matrix temp = mTransform;
-  temp.Invert();
-  SkMatrix mat;
-  GfxMatrixToSkiaMatrix(temp, mat);
-  path.transform(mat);
+  mCanvas->drawRect(SkRectCoveringWholeSurface(), paint.mPaint);
+}
 
-  mCanvas->drawPath(path, paint.mPaint);
+void
+DrawTargetSkia::MaskSurface(const Pattern &aSource,
+                            SourceSurface *aMask,
+                            Point aOffset,
+                            const DrawOptions &aOptions)
+{
+  MarkChanged();
+  AutoPaintSetup paint(mCanvas.get(), aOptions, aSource);
+
+  SkPaint maskPaint;
+  SetPaintPattern(maskPaint, SurfacePattern(aMask, EXTEND_CLAMP));
+
+  SkMatrix transform = maskPaint.getShader()->getLocalMatrix();
+  transform.postTranslate(SkFloatToScalar(aOffset.x), SkFloatToScalar(aOffset.y));
+  maskPaint.getShader()->setLocalMatrix(transform);
+
+  SkLayerRasterizer *raster = new SkLayerRasterizer();
+  raster->addLayer(maskPaint);
+  SkSafeUnref(paint.mPaint.setRasterizer(raster));
+
+  IntSize size = aMask->GetSize();
+  Rect rect = Rect(aOffset.x, aOffset.y, size.width, size.height);
+  mCanvas->drawRect(RectToSkRect(rect), paint.mPaint);
 }
 
 TemporaryRef<SourceSurface>
 DrawTargetSkia::CreateSourceSurfaceFromData(unsigned char *aData,
-                                             const IntSize &aSize,
-                                             int32_t aStride,
-                                             SurfaceFormat aFormat) const
+                                            const IntSize &aSize,
+                                            int32_t aStride,
+                                            SurfaceFormat aFormat) const
 {
   RefPtr<SourceSurfaceSkia> newSurf = new SourceSurfaceSkia();
 
@@ -632,7 +665,20 @@ DrawTargetSkia::CreateSimilarDrawTarget(const IntSize &aSize, SurfaceFormat aFor
 TemporaryRef<SourceSurface>
 DrawTargetSkia::OptimizeSourceSurface(SourceSurface *aSurface) const
 {
-  return nullptr;
+  if (aSurface->GetType() == SURFACE_SKIA) {
+    return aSurface;
+  }
+
+  if (aSurface->GetType() != SURFACE_DATA) {
+    return nullptr;
+  }
+
+  RefPtr<DataSourceSurface> data = aSurface->GetDataSurface();
+  RefPtr<SourceSurface> surface = CreateSourceSurfaceFromData(data->GetData(),
+                                                              data->GetSize(),
+                                                              data->Stride(),
+                                                              data->GetFormat());
+  return data.forget();
 }
 
 TemporaryRef<SourceSurface>
@@ -647,14 +693,14 @@ DrawTargetSkia::CopySurface(SourceSurface *aSurface,
                             const IntPoint &aDestination)
 {
   //TODO: We could just use writePixels() here if the sourceRect is the entire source
-  
+
   if (aSurface->GetType() != SURFACE_SKIA) {
     return;
   }
 
   MarkChanged();
-  
-  const SkBitmap& bitmap = static_cast<SourceSurfaceSkia*>(aSurface)->GetBitmap();
+
+  const SkBitmap& bitmap = GetBitmapForSurface(aSurface);
 
   mCanvas->save();
   mCanvas->resetMatrix();
@@ -707,9 +753,6 @@ DrawTargetSkia::InitWithGLContextAndGrGLInterface(GenericRefCountedBase* aGLCont
   mSize = aSize;
   mFormat = aFormat;
 
-  // Always use soft clipping when we're using GL
-  mSoftClipping = true;
-
   mGrGLInterface = aGrGLInterface;
   mGrGLInterface->fCallbackData = reinterpret_cast<GrGLInterfaceCallbackData>(this);
 
@@ -740,7 +783,6 @@ DrawTargetSkia::SetCacheLimits(int aCount, int aSizeInBytes)
   MOZ_ASSERT(mGrContext, "No GrContext!");
   mGrContext->setTextureCacheLimits(aCount, aSizeInBytes);
 }
-
 #endif
 
 void
@@ -752,19 +794,14 @@ DrawTargetSkia::Init(unsigned char* aData, const IntSize &aSize, int32_t aStride
     ConvertBGRXToBGRA(aData, aSize, aStride);
     isOpaque = true;
   }
-  
-  SkAutoTUnref<SkDevice> device(new SkDevice(GfxFormatToSkiaConfig(aFormat), aSize.width, aSize.height, isOpaque));
 
-  SkBitmap bitmap = (SkBitmap)device->accessBitmap(true);
-  bitmap.lockPixels();
-  bitmap.setPixels(aData);
+  SkBitmap bitmap;
   bitmap.setConfig(GfxFormatToSkiaConfig(aFormat), aSize.width, aSize.height, aStride);
-  bitmap.unlockPixels();
-  bitmap.notifyPixelsChanged();
+  bitmap.setPixels(aData);
+  bitmap.setIsOpaque(isOpaque);
+  SkAutoTUnref<SkCanvas> canvas(new SkCanvas(new SkDevice(bitmap)));
 
-  SkAutoTUnref<SkCanvas> canvas(new SkCanvas(device.get()));
   mSize = aSize;
-
   mCanvas = canvas.get();
   mFormat = aFormat;
 }
@@ -791,7 +828,7 @@ DrawTargetSkia::ClearRect(const Rect &aRect)
   MarkChanged();
   SkPaint paint;
   mCanvas->save();
-  mCanvas->clipRect(RectToSkRect(aRect), SkRegion::kIntersect_Op, mSoftClipping);
+  mCanvas->clipRect(RectToSkRect(aRect), SkRegion::kIntersect_Op, true);
   paint.setColor(SkColorSetARGB(0, 0, 0, 0));
   paint.setXfermodeMode(SkXfermode::kSrc_Mode);
   mCanvas->drawPaint(paint);
@@ -807,7 +844,7 @@ DrawTargetSkia::PushClip(const Path *aPath)
 
   const PathSkia *skiaPath = static_cast<const PathSkia*>(aPath);
   mCanvas->save(SkCanvas::kClip_SaveFlag);
-  mCanvas->clipPath(skiaPath->GetPath(), SkRegion::kIntersect_Op, mSoftClipping);
+  mCanvas->clipPath(skiaPath->GetPath(), SkRegion::kIntersect_Op, true);
 }
 
 void
@@ -816,7 +853,7 @@ DrawTargetSkia::PushClipRect(const Rect& aRect)
   SkRect rect = RectToSkRect(aRect);
 
   mCanvas->save(SkCanvas::kClip_SaveFlag);
-  mCanvas->clipRect(rect, SkRegion::kIntersect_Op, mSoftClipping);
+  mCanvas->clipRect(rect, SkRegion::kIntersect_Op, true);
 }
 
 void
@@ -838,6 +875,12 @@ DrawTargetSkia::CreateGradientStops(GradientStop *aStops, uint32_t aNumStops, Ex
   return new GradientStopsSkia(stops, aNumStops, aExtendMode);
 }
 
+TemporaryRef<FilterNode>
+DrawTargetSkia::CreateFilter(FilterType aType)
+{
+  return FilterNodeSoftware::Create(aType);
+}
+
 void
 DrawTargetSkia::MarkChanged()
 {
@@ -847,6 +890,14 @@ DrawTargetSkia::MarkChanged()
   }
 }
 
+// Return a rect (in user space) that covers the entire surface by applying
+// the inverse of GetTransform() to (0, 0, mSize.width, mSize.height).
+SkRect
+DrawTargetSkia::SkRectCoveringWholeSurface() const
+{
+  return RectToSkRect(mTransform.TransformBounds(Rect(0, 0, mSize.width, mSize.height)));
+}
+
 void
 DrawTargetSkia::SnapshotDestroyed()
 {
