@@ -22,21 +22,6 @@ GLContextProviderNull::CreateOffscreen(const gfxIntSize&,
     return nullptr;
 }
 
-SharedTextureHandle
-GLContextProviderNull::CreateSharedHandle(SharedTextureShareType shareType,
-                                          void* buffer,
-                                          SharedTextureBufferType bufferType)
-{
-  return 0;
-}
-
-already_AddRefed<gfxASurface>
-GLContextProviderNull::GetSharedHandleAsSurface(SharedTextureShareType shareType,
-                                               SharedTextureHandle sharedHandle)
-{
-  return nullptr;
-}
-
 GLContext*
 GLContextProviderNull::GetGlobalContext(ContextFlags)
 {
