@@ -11,6 +11,7 @@
 #include "ipc/AutoOpenSurface.h"
 #include "mozilla/layers/YCbCrImageDataSerializer.h"
 #include "gfxWindowsPlatform.h"
+#include "gfx2DGlue.h"
 
 using namespace mozilla::gfx;
 
@@ -27,20 +28,39 @@ CreateDeprecatedTextureHostD3D9(SurfaceDescriptorType aDescriptorType,
     result = new DeprecatedTextureHostYCbCrD3D9();
   } else if (aDescriptorType == SurfaceDescriptor::TSurfaceDescriptorD3D9) {
     result = new DeprecatedTextureHostSystemMemD3D9();
+  } else if (aDescriptorType == SurfaceDescriptor::TSurfaceDescriptorDIB) {
+    result = new DeprecatedTextureHostDIB();
   } else {
     result = new DeprecatedTextureHostShmemD3D9();
   }
 
   result->SetFlags(aTextureFlags);
-
   return result.forget();
 }
 
+TextureSourceD3D9::~TextureSourceD3D9()
+{
+  MOZ_ASSERT(!mCreatingDeviceManager ||
+             mCreatingDeviceManager->IsInTextureHostList(this),
+             "Inconsistency in list of texture hosts.");
+  // Remove ourselves from the list of d3d9 texture hosts.
+  if (mPreviousHost) {
+    MOZ_ASSERT(mPreviousHost->mNextHost == this);
+    mPreviousHost->mNextHost = mNextHost;
+  } else if (mCreatingDeviceManager) {
+    mCreatingDeviceManager->RemoveTextureListHead(this);
+  }
+  if (mNextHost) {
+    MOZ_ASSERT(mNextHost->mPreviousHost == this);
+    mNextHost->mPreviousHost = mPreviousHost;
+  }
+}
 
 CompositingRenderTargetD3D9::CompositingRenderTargetD3D9(IDirect3DTexture9* aTexture,
                                                          SurfaceInitMode aInit,
-                                                         const gfx::IntSize& aSize)
-  : mInitMode(aInit)
+                                                         const gfx::IntRect& aRect)
+  : CompositingRenderTarget(aRect.TopLeft())
+  , mInitMode(aInit)
   , mInitialized(false)
 {
   MOZ_COUNT_CTOR(CompositingRenderTargetD3D9);
@@ -49,19 +69,20 @@ CompositingRenderTargetD3D9::CompositingRenderTargetD3D9(IDirect3DTexture9* aTex
   mTextures[0] = aTexture;
   HRESULT hr = mTextures[0]->GetSurfaceLevel(0, getter_AddRefs(mSurface));
   NS_ASSERTION(mSurface, "Couldn't create surface for texture");
-  TextureSourceD3D9::SetSize(aSize);
+  TextureSourceD3D9::SetSize(aRect.Size());
 }
 
 CompositingRenderTargetD3D9::CompositingRenderTargetD3D9(IDirect3DSurface9* aSurface,
                                                          SurfaceInitMode aInit,
-                                                         const gfx::IntSize& aSize)
-  : mSurface(aSurface)
+                                                         const gfx::IntRect& aRect)
+  : CompositingRenderTarget(aRect.TopLeft())
+  , mSurface(aSurface)
   , mInitMode(aInit)
   , mInitialized(false)
 {
   MOZ_COUNT_CTOR(CompositingRenderTargetD3D9);
   MOZ_ASSERT(mSurface);
-  TextureSourceD3D9::SetSize(aSize);
+  TextureSourceD3D9::SetSize(aRect.Size());
 }
 
 CompositingRenderTargetD3D9::~CompositingRenderTargetD3D9()
@@ -103,8 +124,8 @@ IntSize
 DeprecatedTextureHostD3D9::GetSize() const
 {
   if (mIterating) {
-    gfx::IntRect rect = GetTileRect(mCurrentTile);
-    return gfx::IntSize(rect.width, rect.height);
+    IntRect rect = GetTileRect(mCurrentTile);
+    return rect.Size();
   }
   return TextureSourceD3D9::GetSize();
 }
@@ -116,7 +137,7 @@ DeprecatedTextureHostD3D9::GetTileRect()
   return nsIntRect(rect.x, rect.y, rect.width, rect.height);
 }
 
-static uint32_t GetRequiredTiles(uint32_t aSize, uint32_t aMaxSize)
+static uint32_t GetRequiredTilesD3D9(uint32_t aSize, uint32_t aMaxSize)
 {
   uint32_t requiredTiles = aSize / aMaxSize;
   if (aSize % aMaxSize) {
@@ -129,58 +150,81 @@ void
 DeprecatedTextureHostD3D9::SetCompositor(Compositor* aCompositor)
 {
   mCompositor = static_cast<CompositorD3D9*>(aCompositor);
-  mDevice = mCompositor ? mCompositor->device() : nullptr;
 }
 
-static TemporaryRef<IDirect3DTexture9>
-DataToTexture(IDirect3DDevice9 *aDevice,
-              unsigned char *aData,
-              int aStride,
-              const gfxIntSize &aSize,
-              _D3DFORMAT aFormat,
-              uint32_t aBPP)
+/**
+ * Helper method for DataToTexture and SurfaceToTexture.
+ * The last three params are out params.
+ * Returns the created texture, or null if we fail.
+ */
+TemporaryRef<IDirect3DTexture9>
+TextureSourceD3D9::InitTextures(DeviceManagerD3D9* aDeviceManager,
+                                const IntSize &aSize,
+                                _D3DFORMAT aFormat,
+                                RefPtr<IDirect3DSurface9>& aSurface,
+                                D3DLOCKED_RECT& aLockedRect)
 {
-  RefPtr<IDirect3DTexture9> texture;
-  nsRefPtr<IDirect3DDevice9Ex> deviceEx;
-  aDevice->QueryInterface(IID_IDirect3DDevice9Ex,
-                          (void**)getter_AddRefs(deviceEx));
+  if (!aDeviceManager) {
+    return nullptr;
+  }
+  RefPtr<IDirect3DTexture9> result;
+  // D3D9Ex doesn't support managed textures and we don't want the hassle even
+  // if we don't have Ex. We could use dynamic textures
+  // here but since Images are immutable that probably isn't such a great
+  // idea.
+  result = aDeviceManager->CreateTexture(aSize, aFormat, D3DPOOL_DEFAULT, this);
+  if (!result) {
+    return nullptr;
+  }
 
+  RefPtr<IDirect3DTexture9> tmpTexture =
+    aDeviceManager->CreateTexture(aSize, aFormat, D3DPOOL_SYSTEMMEM, this);
+  if (!tmpTexture) {
+    return nullptr;
+  }
+
+  tmpTexture->GetSurfaceLevel(0, byRef(aSurface));
+  aSurface->LockRect(&aLockedRect, NULL, 0);
+  if (!aLockedRect.pBits) {
+    NS_WARNING("Could not lock surface");
+    return nullptr;
+  }
+
+  return result;
+}
+
+/**
+ * Helper method for DataToTexture and SurfaceToTexture.
+ */
+static void
+FinishTextures(DeviceManagerD3D9* aDeviceManager,
+               IDirect3DTexture9* aTexture,
+               IDirect3DSurface9* aSurface)
+{
+  if (!aDeviceManager) {
+    return;
+  }
+
+  aSurface->UnlockRect();
+  nsRefPtr<IDirect3DSurface9> dstSurface;
+  aTexture->GetSurfaceLevel(0, getter_AddRefs(dstSurface));
+  aDeviceManager->device()->UpdateSurface(aSurface, NULL, dstSurface, NULL);
+}
+
+TemporaryRef<IDirect3DTexture9>
+TextureSourceD3D9::DataToTexture(DeviceManagerD3D9* aDeviceManager,
+                                 unsigned char *aData,
+                                 int aStride,
+                                 const IntSize &aSize,
+                                 _D3DFORMAT aFormat,
+                                 uint32_t aBPP)
+{
   RefPtr<IDirect3DSurface9> surface;
   D3DLOCKED_RECT lockedRect;
-  if (deviceEx) {
-    // D3D9Ex doesn't support managed textures. We could use dynamic textures
-    // here but since Images are immutable that probably isn't such a great
-    // idea.
-    if (FAILED(aDevice->
-               CreateTexture(aSize.width, aSize.height,
-                             1, 0, aFormat, D3DPOOL_DEFAULT,
-                             byRef(texture), nullptr)))
-    {
-      return nullptr;
-    }
-
-    RefPtr<IDirect3DTexture9> tmpTexture;
-    if (FAILED(aDevice->
-               CreateTexture(aSize.width, aSize.height,
-                             1, 0, aFormat, D3DPOOL_SYSTEMMEM,
-                             byRef(tmpTexture), nullptr)))
-    {
-      return nullptr;
-    }
-
-    tmpTexture->GetSurfaceLevel(0, byRef(surface));
-    surface->LockRect(&lockedRect, NULL, 0);
-    NS_ASSERTION(lockedRect.pBits, "Could not lock surface");
-  } else {
-    if (FAILED(aDevice->
-               CreateTexture(aSize.width, aSize.height,
-                             1, 0, aFormat, D3DPOOL_MANAGED,
-                             byRef(texture), nullptr))) {
-      return nullptr;
-    }
-
-    /* lock the entire texture */
-    texture->LockRect(0, &lockedRect, nullptr, 0);
+  RefPtr<IDirect3DTexture9> texture = InitTextures(aDeviceManager, aSize, aFormat,
+                                                   surface, lockedRect);
+  if (!texture) {
+    return nullptr;
   }
 
   uint32_t width = aSize.width * aBPP;
@@ -191,18 +235,20 @@ DataToTexture(IDirect3DDevice9 *aDevice,
             width);
   }
 
-  if (deviceEx) {
-    surface->UnlockRect();
-    nsRefPtr<IDirect3DSurface9> dstSurface;
-    texture->GetSurfaceLevel(0, getter_AddRefs(dstSurface));
-    aDevice->UpdateSurface(surface, NULL, dstSurface, NULL);
-  } else {
-    texture->UnlockRect(0);
-  }
+  FinishTextures(aDeviceManager, texture, surface);
 
   return texture.forget();
 }
 
+void
+DeprecatedTextureHostD3D9::Reset()
+{
+  mSize.width = 0;
+  mSize.height = 0;
+  mTextures[0] = nullptr;
+  mIsTiled = false;
+}
+
 void
 DeprecatedTextureHostShmemD3D9::UpdateImpl(const SurfaceDescriptor& aImage,
                                  nsIntRegion *aRegion,
@@ -212,28 +258,31 @@ DeprecatedTextureHostShmemD3D9::UpdateImpl(const SurfaceDescriptor& aImage,
              aImage.type() == SurfaceDescriptor::TMemoryImage);
   MOZ_ASSERT(mCompositor, "Must have compositor to update.");
 
+  if (!mCompositor->device()) {
+    return;
+  }
+
   AutoOpenSurface openSurf(OPEN_READ_ONLY, aImage);
 
   nsRefPtr<gfxImageSurface> surf = openSurf.GetAsImage();
 
-  gfxIntSize size = surf->GetSize();
-  mSize = IntSize(size.width, size.height);
+  mSize = ToIntSize(surf->GetSize());
 
   uint32_t bpp = 0;
 
   _D3DFORMAT format = D3DFMT_A8R8G8B8;
   switch (surf->Format()) {
-  case gfxImageSurface::ImageFormatRGB24:
+  case gfxImageFormatRGB24:
     mFormat = FORMAT_B8G8R8X8;
     format = D3DFMT_X8R8G8B8;
     bpp = 4;
     break;
-  case gfxImageSurface::ImageFormatARGB32:
+  case gfxImageFormatARGB32:
     mFormat = FORMAT_B8G8R8A8;
     format = D3DFMT_A8R8G8B8;
     bpp = 4;
     break;
-  case gfxImageSurface::ImageFormatA8:
+  case gfxImageFormatA8:
     mFormat = FORMAT_A8;
     format = D3DFMT_A8;
     bpp = 1;
@@ -243,16 +292,20 @@ DeprecatedTextureHostShmemD3D9::UpdateImpl(const SurfaceDescriptor& aImage,
   }
 
   int32_t maxSize = mCompositor->GetMaxTextureSize();
-  if (size.width <= maxSize && size.height <= maxSize) {
-    mTextures[0] = DataToTexture(mDevice,
+  if (mSize.width <= maxSize && mSize.height <= maxSize) {
+    mTextures[0] = DataToTexture(gfxWindowsPlatform::GetPlatform()->GetD3D9DeviceManager(),
                                  surf->Data(), surf->Stride(),
-                                 size, format, bpp);
-    NS_ASSERTION(mTextures[0], "Could not upload texture");
+                                 mSize, format, bpp);
+    if (!mTextures[0]) {
+      NS_WARNING("Could not upload texture");
+      Reset();
+      return;
+    }
     mIsTiled = false;
   } else {
     mIsTiled = true;
-    uint32_t tileCount = GetRequiredTiles(size.width, maxSize) *
-                         GetRequiredTiles(size.height, maxSize);
+    uint32_t tileCount = GetRequiredTilesD3D9(mSize.width, maxSize) *
+                         GetRequiredTilesD3D9(mSize.height, maxSize);
     mTileTextures.resize(tileCount);
 
     for (uint32_t i = 0; i < tileCount; i++) {
@@ -260,12 +313,17 @@ DeprecatedTextureHostShmemD3D9::UpdateImpl(const SurfaceDescriptor& aImage,
       unsigned char* data = surf->Data() +
                             tileRect.y * surf->Stride() +
                             tileRect.x * bpp;
-      mTileTextures[i] = DataToTexture(mDevice,
+      mTileTextures[i] = DataToTexture(gfxWindowsPlatform::GetPlatform()->GetD3D9DeviceManager(),
                                        data,
                                        surf->Stride(),
-                                       gfxIntSize(tileRect.width, tileRect.height),
+                                       tileRect.Size(),
                                        format,
                                        bpp);
+      if (!mTileTextures[i]) {
+        NS_WARNING("Could not upload texture");
+        Reset();
+        return;
+      }
     }
   }
 }
@@ -274,8 +332,8 @@ IntRect
 DeprecatedTextureHostD3D9::GetTileRect(uint32_t aID) const
 {
   uint32_t maxSize = mCompositor->GetMaxTextureSize();
-  uint32_t horizontalTiles = GetRequiredTiles(mSize.width, maxSize);
-  uint32_t verticalTiles = GetRequiredTiles(mSize.height, maxSize);
+  uint32_t horizontalTiles = GetRequiredTilesD3D9(mSize.width, maxSize);
+  uint32_t verticalTiles = GetRequiredTilesD3D9(mSize.height, maxSize);
 
   uint32_t verticalTile = aID / horizontalTiles;
   uint32_t horizontalTile = aID % horizontalTiles;
@@ -286,11 +344,19 @@ DeprecatedTextureHostD3D9::GetTileRect(uint32_t aID) const
                  verticalTile < (verticalTiles - 1) ? maxSize : mSize.height % maxSize);
 }
 
+DeprecatedTextureHostYCbCrD3D9::DeprecatedTextureHostYCbCrD3D9()
+{
+  mFormat = gfx::FORMAT_YUV;
+}
+
+DeprecatedTextureHostYCbCrD3D9::~DeprecatedTextureHostYCbCrD3D9()
+{
+}
+
 void
 DeprecatedTextureHostYCbCrD3D9::SetCompositor(Compositor* aCompositor)
 {
-  CompositorD3D9 *d3dCompositor = static_cast<CompositorD3D9*>(aCompositor);
-  mDevice = d3dCompositor ? d3dCompositor->device() : nullptr;
+  mCompositor = static_cast<CompositorD3D9*>(aCompositor);
 }
 
 IntSize
@@ -306,47 +372,51 @@ DeprecatedTextureHostYCbCrD3D9::UpdateImpl(const SurfaceDescriptor& aImage,
 {
   MOZ_ASSERT(aImage.type() == SurfaceDescriptor::TYCbCrImage);
 
+  if (!mCompositor->device()) {
+    return;
+  }
+
   YCbCrImageDataDeserializer yuvDeserializer(aImage.get_YCbCrImage().data().get<uint8_t>());
 
-  gfxIntSize gfxCbCrSize = yuvDeserializer.GetCbCrSize();
-  gfxIntSize size = yuvDeserializer.GetYSize();
-  mSize = IntSize(size.width, size.height);
+  mSize = ToIntSize(yuvDeserializer.GetYSize());
+  IntSize cbCrSize = ToIntSize(yuvDeserializer.GetCbCrSize());
   mStereoMode = yuvDeserializer.GetStereoMode();
 
-  mTextures[0] = DataToTexture(mDevice,
+  DeviceManagerD3D9* deviceManager = gfxWindowsPlatform::GetPlatform()->GetD3D9DeviceManager();
+  mTextures[0] = DataToTexture(deviceManager,
                                yuvDeserializer.GetYData(),
                                yuvDeserializer.GetYStride(),
-                               size,
-                               D3DFMT_L8, 1);
-  mTextures[1] = DataToTexture(mDevice,
+                               mSize,
+                               D3DFMT_A8, 1);
+  mTextures[1] = DataToTexture(deviceManager,
                                yuvDeserializer.GetCbData(),
                                yuvDeserializer.GetCbCrStride(),
-                               gfxCbCrSize,
-                               D3DFMT_L8, 1);
-  mTextures[2] = DataToTexture(mDevice,
+                               cbCrSize,
+                               D3DFMT_A8, 1);
+  mTextures[2] = DataToTexture(deviceManager,
                                yuvDeserializer.GetCrData(),
                                yuvDeserializer.GetCbCrStride(),
-                               gfxCbCrSize,
-                               D3DFMT_L8, 1);
+                               cbCrSize,
+                               D3DFMT_A8, 1);
 }
 
-// aTexture should be in SYSTEMMEM, returns a texture in the default
-// pool (that is, in video memory).
-static TemporaryRef<IDirect3DTexture9>
-TextureToTexture(IDirect3DDevice9* aDevice,
-                 IDirect3DTexture9* aTexture,
-                 const IntSize& aSize,
-                 _D3DFORMAT aFormat)
+TemporaryRef<IDirect3DTexture9>
+TextureSourceD3D9::TextureToTexture(DeviceManagerD3D9* aDeviceManager,
+                                    IDirect3DTexture9* aTexture,
+                                    const IntSize& aSize,
+                                    _D3DFORMAT aFormat)
 {
-  RefPtr<IDirect3DTexture9> texture;
-  if (FAILED(aDevice->
-              CreateTexture(aSize.width, aSize.height,
-                            1, 0, aFormat, D3DPOOL_DEFAULT,
-                            byRef(texture), nullptr))) {
+  if (!aDeviceManager) {
     return nullptr;
   }
 
-  HRESULT hr = aDevice->UpdateTexture(aTexture, texture);
+  RefPtr<IDirect3DTexture9> texture =
+    aDeviceManager->CreateTexture(aSize, aFormat, D3DPOOL_DEFAULT, this);
+  if (!texture) {
+    return nullptr;
+  }
+
+  HRESULT hr = aDeviceManager->device()->UpdateTexture(aTexture, texture);
   if (FAILED(hr)) {
     return nullptr;
   }
@@ -362,16 +432,26 @@ DeprecatedTextureHostSystemMemD3D9::UpdateImpl(const SurfaceDescriptor& aImage,
   MOZ_ASSERT(aImage.type() == SurfaceDescriptor::TSurfaceDescriptorD3D9);
   MOZ_ASSERT(mCompositor, "Must have compositor to update.");
 
+  if (!mCompositor->device()) {
+    return;
+  }
+
   IDirect3DTexture9* texture =
     reinterpret_cast<IDirect3DTexture9*>(aImage.get_SurfaceDescriptorD3D9().texture());
 
   if (!texture) {
-    mTextures[0] = nullptr;
+    Reset();
     return;
   }
 
   D3DSURFACE_DESC desc;
   texture->GetLevelDesc(0, &desc);
+  HRESULT hr = texture->GetLevelDesc(0, &desc);
+  if (FAILED(hr)) {
+    Reset();
+    return;
+  }
+
   mSize.width = desc.Width;
   mSize.height = desc.Height;
 
@@ -398,13 +478,18 @@ DeprecatedTextureHostSystemMemD3D9::UpdateImpl(const SurfaceDescriptor& aImage,
   if (mSize.width <= maxSize && mSize.height <= maxSize) {
     mIsTiled = false;
 
-    mTextures[0] = TextureToTexture(mDevice, texture, mSize, format);
-    NS_ASSERTION(mTextures[0], "Could not upload texture");
+    mTextures[0] = TextureToTexture(gfxWindowsPlatform::GetPlatform()->GetD3D9DeviceManager(),
+                                    texture, mSize, format);
+    if (!mTextures[0]) {
+      NS_WARNING("Could not upload texture");
+      Reset();
+      return;
+    }
   } else {
     mIsTiled = true;
 
-    uint32_t tileCount = GetRequiredTiles(mSize.width, maxSize) *
-                         GetRequiredTiles(mSize.height, maxSize);
+    uint32_t tileCount = GetRequiredTilesD3D9(mSize.width, maxSize) *
+                         GetRequiredTilesD3D9(mSize.height, maxSize);
     mTileTextures.resize(tileCount);
 
     for (uint32_t i = 0; i < tileCount; i++) {
@@ -416,13 +501,128 @@ DeprecatedTextureHostSystemMemD3D9::UpdateImpl(const SurfaceDescriptor& aImage,
       d3dTileRect.bottom = tileRect.YMost();
       D3DLOCKED_RECT lockedRect;
       texture->LockRect(0, &lockedRect, &d3dTileRect, 0);
-      mTileTextures[i] = DataToTexture(mDevice,
+      mTileTextures[i] = DataToTexture(gfxWindowsPlatform::GetPlatform()->GetD3D9DeviceManager(),
                                        reinterpret_cast<unsigned char*>(lockedRect.pBits),
                                        lockedRect.Pitch,
-                                       gfxIntSize(tileRect.width, tileRect.height),
+                                       tileRect.Size(),
                                        format,
                                        bpp);
       texture->UnlockRect(0);
+      if (!mTileTextures[i]) {
+        NS_WARNING("Could not upload texture");
+        Reset();
+        return;
+      }
+    }
+  }
+}
+
+TemporaryRef<IDirect3DTexture9>
+TextureSourceD3D9::SurfaceToTexture(DeviceManagerD3D9* aDeviceManager,
+                                    gfxWindowsSurface* aSurface,
+                                    const IntSize& aSize,
+                                    _D3DFORMAT aFormat)
+{
+  RefPtr<IDirect3DSurface9> surface;
+  D3DLOCKED_RECT lockedRect;
+
+  RefPtr<IDirect3DTexture9> texture = InitTextures(aDeviceManager, aSize, aFormat,
+                                                   surface, lockedRect);
+  if (!texture) {
+    return nullptr;
+  }
+
+  nsRefPtr<gfxImageSurface> imgSurface =
+    new gfxImageSurface(reinterpret_cast<unsigned char*>(lockedRect.pBits),
+                        gfxIntSize(aSize.width, aSize.height),
+                        lockedRect.Pitch,
+                        gfxPlatform::GetPlatform()->OptimalFormatForContent(aSurface->GetContentType()));
+
+  nsRefPtr<gfxContext> context = new gfxContext(imgSurface);
+  context->SetSource(aSurface);
+  context->SetOperator(gfxContext::OPERATOR_SOURCE);
+  context->Paint();
+
+  FinishTextures(aDeviceManager, texture, surface);
+
+  return texture.forget();
+}
+
+void
+DeprecatedTextureHostDIB::UpdateImpl(const SurfaceDescriptor& aImage,
+                                     nsIntRegion *aRegion,
+                                     nsIntPoint *aOffset)
+{
+  MOZ_ASSERT(aImage.type() == SurfaceDescriptor::TSurfaceDescriptorDIB);
+  MOZ_ASSERT(mCompositor, "Must have compositor to update.");
+
+  if (!mCompositor->device()) {
+    return;
+  }
+
+  // We added an extra ref for transport, so we shouldn't AddRef now.
+  nsRefPtr<gfxWindowsSurface> surf =
+    dont_AddRef(reinterpret_cast<gfxWindowsSurface*>(aImage.get_SurfaceDescriptorDIB().surface()));
+
+  mSize = ToIntSize(surf->GetSize());
+
+  uint32_t bpp = 0;
+
+  _D3DFORMAT format = D3DFMT_A8R8G8B8;
+  switch (gfxPlatform::GetPlatform()->OptimalFormatForContent(surf->GetContentType())) {
+  case gfxImageFormatRGB24:
+    mFormat = FORMAT_B8G8R8X8;
+    format = D3DFMT_X8R8G8B8;
+    bpp = 4;
+    break;
+  case gfxImageFormatARGB32:
+    mFormat = FORMAT_B8G8R8A8;
+    format = D3DFMT_A8R8G8B8;
+    bpp = 4;
+    break;
+  case gfxImageFormatA8:
+    mFormat = FORMAT_A8;
+    format = D3DFMT_A8;
+    bpp = 1;
+    break;
+  default:
+    NS_ERROR("Bad image format");
+  }
+
+  int32_t maxSize = mCompositor->GetMaxTextureSize();
+  if (mSize.width <= maxSize && mSize.height <= maxSize) {
+    mTextures[0] = SurfaceToTexture(gfxWindowsPlatform::GetPlatform()->GetD3D9DeviceManager(),
+                                    surf, mSize, format);
+    if (!mTextures[0]) {
+      NS_WARNING("Could not upload texture");
+      Reset();
+      return;
+    }
+    mIsTiled = false;
+  } else {
+    mIsTiled = true;
+
+    uint32_t tileCount = GetRequiredTilesD3D9(mSize.width, maxSize) *
+                         GetRequiredTilesD3D9(mSize.height, maxSize);
+    mTileTextures.resize(tileCount);
+
+    for (uint32_t i = 0; i < tileCount; i++) {
+      IntRect tileRect = GetTileRect(i);
+      nsRefPtr<gfxImageSurface> imgSurface = surf->GetAsImageSurface();
+      unsigned char* data = imgSurface->Data() +
+                            tileRect.y * imgSurface->Stride() +
+                            tileRect.x * bpp;
+      mTileTextures[i] = DataToTexture(gfxWindowsPlatform::GetPlatform()->GetD3D9DeviceManager(),
+                                       data,
+                                       imgSurface->Stride(),
+                                       tileRect.Size(),
+                                       format,
+                                       bpp);
+      if (!mTileTextures[i]) {
+        NS_WARNING("Could not upload texture");
+        Reset();
+        return;
+      }
     }
   }
 }
@@ -430,8 +630,6 @@ DeprecatedTextureHostSystemMemD3D9::UpdateImpl(const SurfaceDescriptor& aImage,
 DeprecatedTextureClientD3D9::DeprecatedTextureClientD3D9(CompositableForwarder* aCompositableForwarder,
                                      const TextureInfo& aTextureInfo)
   : DeprecatedTextureClient(aCompositableForwarder, aTextureInfo)
-  , mDC(nullptr)
-  , mTextureLocked(false)
 {
   MOZ_COUNT_CTOR(DeprecatedTextureClientD3D9);
 }
@@ -442,12 +640,12 @@ DeprecatedTextureClientD3D9::~DeprecatedTextureClientD3D9()
   Unlock();
   mDescriptor = SurfaceDescriptor();
 
-  ClearDT();
+  mDrawTarget = nullptr;
 }
 
 bool
 DeprecatedTextureClientD3D9::EnsureAllocated(gfx::IntSize aSize,
-                                   gfxASurface::gfxContentType aType)
+                                   gfxContentType aType)
 {
   if (mTexture) {
     D3DSURFACE_DESC desc;
@@ -467,29 +665,22 @@ DeprecatedTextureClientD3D9::EnsureAllocated(gfx::IntSize aSize,
 
   _D3DFORMAT format = D3DFMT_A8R8G8B8;
   switch (aType) {
-  case gfxASurface::CONTENT_COLOR:
+  case GFX_CONTENT_COLOR:
     format = D3DFMT_X8R8G8B8;
-    mIsOpaque = true;
     break;
-  case gfxASurface::CONTENT_COLOR_ALPHA:
-    format = D3DFMT_A8R8G8B8;
-    mIsOpaque = false;
-    break;
-  case gfxASurface::CONTENT_ALPHA:
+  case GFX_CONTENT_COLOR_ALPHA:
+    // fallback to DIB texture client
+    return false;
+  case GFX_CONTENT_ALPHA:
     format = D3DFMT_A8;
-    mIsOpaque = true;
     break;
   default:
     NS_ERROR("Bad image type");
   }
 
-  IDirect3DDevice9 *device = gfxWindowsPlatform::GetPlatform()->GetD3D9Device();
-  if (!device ||
-      FAILED(device->
-               CreateTexture(aSize.width, aSize.height,
-                             1, 0, format, D3DPOOL_SYSTEMMEM,
-                             getter_AddRefs(mTexture), nullptr)))
-  {
+  DeviceManagerD3D9* deviceManager = gfxWindowsPlatform::GetPlatform()->GetD3D9DeviceManager();
+  if (!deviceManager ||
+      !(mTexture = deviceManager->CreateTexture(aSize, format, D3DPOOL_SYSTEMMEM, nullptr))) {
     NS_WARNING("Could not create texture");
     return false;
   }
@@ -497,15 +688,6 @@ DeprecatedTextureClientD3D9::EnsureAllocated(gfx::IntSize aSize,
   MOZ_ASSERT(mTexture);
   mDescriptor = SurfaceDescriptorD3D9(reinterpret_cast<uintptr_t>(mTexture.get()));
 
-  if (!mIsOpaque) {
-    nsRefPtr<gfxASurface> surface = LockSurface();
-    nsRefPtr<gfxContext> ctx = new gfxContext(surface);
-    ctx->SetOperator(gfxContext::OPERATOR_SOURCE);
-    ctx->SetColor(gfxRGBA(0.0, 0.0, 0.0, 0.0));
-    ctx->Paint();
-    Unlock();
-  }
-
   mContentType = aType;
   return true;
 }
@@ -513,47 +695,40 @@ DeprecatedTextureClientD3D9::EnsureAllocated(gfx::IntSize aSize,
 gfxASurface*
 DeprecatedTextureClientD3D9::LockSurface()
 {
+  if (!gfxWindowsPlatform::GetPlatform()->GetD3D9Device()) {
+    // If the device has failed then we should not lock the surface,
+    // even if we could.
+    Unlock();
+    mD3D9Surface = nullptr;
+    mTexture = nullptr;
+
+    return nullptr;
+  }
+
   if (mSurface) {
     return mSurface.get();
   }
 
-  MOZ_ASSERT(mTexture, "Cannot lock surface without a texture to lock");
+  if (!mTexture) {
+    NS_WARNING("No texture to lock");
+    return nullptr;
+  }
 
-  if (mIsOpaque) {
-    MOZ_ASSERT(!mTextureLocked, "Shouldn't lock texture and have a surface");
-    if (!mD3D9Surface) {
-      HRESULT hr = mTexture->GetSurfaceLevel(0, getter_AddRefs(mD3D9Surface));
-      if (FAILED(hr)) {
-        NS_WARNING("Failed to get texture surface level.");
-        return nullptr;
-      }
-    }
-
-    if (!mDC) {
-      HRESULT hr = mD3D9Surface->GetDC(&mDC);
-      if (FAILED(hr)) {
-        NS_WARNING("Failed to get device context for texture surface.");
-        return nullptr;
-      }
-    }
-    NS_ASSERTION(mDC, "We need a DC here");
-    mSurface = new gfxWindowsSurface(mDC);
-  } else {
-    // d3d9 SYSTEMMEM surfaces do not support GDI with an alpha channel
-    MOZ_ASSERT(!mD3D9Surface && !mDC, "Shouldn't lock texture and have a surface");
-    if (!mTextureLocked) {
-      D3DLOCKED_RECT lockedRect;
-      mTexture->LockRect(0, &lockedRect, nullptr, 0);
-      mTextureLocked = true;
-
-      mSurface = new gfxImageSurface(reinterpret_cast<unsigned char*>(lockedRect.pBits),
-                                     gfxIntSize(mSize.width, mSize.height),
-                                     lockedRect.Pitch,
-                                     gfxASurface::ImageFormatARGB32);
+  if (!mD3D9Surface) {
+    HRESULT hr = mTexture->GetSurfaceLevel(0, getter_AddRefs(mD3D9Surface));
+    if (FAILED(hr)) {
+      NS_WARNING("Failed to get texture surface level.");
+      return nullptr;
     }
   }
 
-  NS_ASSERTION(mSurface, "should have a surface one way or the other");
+  mSurface = new gfxWindowsSurface(mD3D9Surface);
+  if (!mSurface || mSurface->CairoStatus()) {
+    NS_WARNING("Could not create surface for d3d9 surface");
+    mSurface = nullptr;
+    return nullptr;
+  }
+
   return mSurface.get();
 }
 
@@ -561,8 +736,10 @@ DrawTarget*
 DeprecatedTextureClientD3D9::LockDrawTarget()
 {
   if (!mDrawTarget) {
-    mDrawTarget =
-      gfxPlatform::GetPlatform()->CreateDrawTargetForSurface(LockSurface(), mSize);
+    if (gfxASurface* surface = LockSurface()) {
+      mDrawTarget =
+        gfxPlatform::GetPlatform()->CreateDrawTargetForSurface(LockSurface(), mSize);
+    }
   }
 
   return mDrawTarget.get();
@@ -576,17 +753,6 @@ DeprecatedTextureClientD3D9::Unlock()
     mDrawTarget = nullptr;
   }
 
-  if (mTextureLocked) {
-    MOZ_ASSERT(!mD3D9Surface && !mDC, "Shouldn't lock texture and have a surface");
-    mTexture->UnlockRect(0);
-    mTextureLocked = false;
-  } else if (mDC) {
-    MOZ_ASSERT(mD3D9Surface, "we need a D3D9Surface to release our DC");
-    MOZ_ASSERT(!mTextureLocked, "Shouldn't lock texture and have a surface");
-    mD3D9Surface->ReleaseDC(mDC);
-    mDC = nullptr;
-  }
-
   if (mSurface) {
     mSurface = nullptr;
   }
@@ -602,7 +768,7 @@ DeprecatedTextureClientD3D9::SetDescriptor(const SurfaceDescriptor& aDescriptor)
 
   mDescriptor = aDescriptor;
   mSurface = nullptr;
-  ClearDT();
+  mDrawTarget = nullptr;
 
   if (aDescriptor.type() == SurfaceDescriptor::T__None) {
     return;
@@ -615,14 +781,114 @@ DeprecatedTextureClientD3D9::SetDescriptor(const SurfaceDescriptor& aDescriptor)
                mDescriptor.get_SurfaceDescriptorD3D9().texture());
 }
 
+DeprecatedTextureClientDIB::DeprecatedTextureClientDIB(CompositableForwarder* aCompositableForwarder,
+                                                       const TextureInfo& aTextureInfo)
+  : DeprecatedTextureClient(aCompositableForwarder, aTextureInfo)
+{
+  MOZ_COUNT_CTOR(DeprecatedTextureClientDIB);
+}
+
+DeprecatedTextureClientDIB::~DeprecatedTextureClientDIB()
+{
+  MOZ_COUNT_DTOR(DeprecatedTextureClientDIB);
+  Unlock();
+  // It is OK not to dealloc the surface descriptor because it is only a pointer
+  // to mSurface and we will release our strong reference to that automatically.
+  mDescriptor = SurfaceDescriptor();
+  mDrawTarget = nullptr;
+}
+
+bool
+DeprecatedTextureClientDIB::EnsureAllocated(gfx::IntSize aSize,
+                                            gfxContentType aType)
+{
+  if (mSurface) {
+    gfxIntSize size = mSurface->GetSize();
+    if (size.width == aSize.width &&
+        size.height == aSize.height) {
+      return true;
+    }
+
+    Unlock();
+    mSurface = nullptr;
+  }
+
+  mSurface = new gfxWindowsSurface(gfxIntSize(aSize.width, aSize.height),
+                                   gfxPlatform::GetPlatform()->OptimalFormatForContent(aType));
+  if (!mSurface || mSurface->CairoStatus())
+  {
+    NS_WARNING("Could not create surface");
+    mSurface = nullptr;
+    mDescriptor = SurfaceDescriptor();
+    return false;
+  }
+  mSize = aSize;
+  mContentType = aType;
+
+  mDescriptor = SurfaceDescriptorDIB(reinterpret_cast<uintptr_t>(mSurface.get()));
+
+  return true;
+}
+
+SurfaceDescriptor*
+DeprecatedTextureClientDIB::LockSurfaceDescriptor()
+{
+  // The host will release this ref when it receives the surface descriptor.
+  // We AddRef in case we die before the host receives the pointer.
+  NS_ASSERTION(mSurface == reinterpret_cast<gfxWindowsSurface*>(mDescriptor.get_SurfaceDescriptorDIB().surface()),
+                "SurfaceDescriptor is not up to date");
+  mSurface->AddRef();
+  return GetDescriptor();
+}
+
+gfxASurface*
+DeprecatedTextureClientDIB::LockSurface()
+{
+  if (mSurface) {
+    return mSurface.get();
+  }
+
+  return nullptr;
+}
+
+DrawTarget*
+DeprecatedTextureClientDIB::LockDrawTarget()
+{
+  if (!mDrawTarget) {
+    if (gfxASurface* surface = LockSurface()) {
+      mDrawTarget =
+        gfxPlatform::GetPlatform()->CreateDrawTargetForSurface(LockSurface(), mSize);
+    }
+  }
+
+  return mDrawTarget.get();
+}
+
 void
-DeprecatedTextureClientD3D9::ClearDT()
+DeprecatedTextureClientDIB::Unlock()
 {
-  // Perhaps this should be debug only.
   if (mDrawTarget) {
+    mDrawTarget->Flush();
     mDrawTarget = nullptr;
   }
 }
 
+void
+DeprecatedTextureClientDIB::SetDescriptor(const SurfaceDescriptor& aDescriptor)
+{
+  mDescriptor = aDescriptor;
+  mDrawTarget = nullptr;
+
+  if (aDescriptor.type() == SurfaceDescriptor::T__None) {
+    mSurface = nullptr;
+    return;
+  }
+
+  MOZ_ASSERT(aDescriptor.type() == SurfaceDescriptor::TSurfaceDescriptorDIB);
+  Unlock();
+  mSurface = reinterpret_cast<gfxWindowsSurface*>(
+               mDescriptor.get_SurfaceDescriptorDIB().surface());
+}
+
 }
 }
