@@ -3,30 +3,12 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "mozilla/Util.h"
-
-#include "nsAlgorithm.h"
-#include "nsString.h"
-#include "nsBidiUtils.h"
-
-#include "gfxTypes.h"
-
-#include "nsPromiseFlatString.h"
-
-#include "gfxContext.h"
-#include "gfxPlatform.h"
-#include "gfxPlatformMac.h"
+#include "mozilla/ArrayUtils.h"
 #include "gfxCoreTextShaper.h"
 #include "gfxMacFont.h"
-
-#include "gfxFontTest.h"
 #include "gfxFontUtils.h"
+#include "mozilla/gfx/2D.h"
 
-#include "gfxQuartzSurface.h"
-#include "gfxMacPlatformFontList.h"
-#include "gfxUserFontSet.h"
-
-#include "nsUnicodeRange.h"
 #include <algorithm>
 
 using namespace mozilla;
@@ -106,15 +88,15 @@ gfxCoreTextShaper::ShapeText(gfxContext      *aContext,
         ::CFStringAppendCharacters(mutableString,
                                    isRightToLeft ? beginRTL : beginLTR,
                                    startOffset);
-        ::CFStringAppendCharacters(mutableString, aText, length);
+        ::CFStringAppendCharacters(mutableString, reinterpret_cast<const UniChar*>(aText), length);
         ::CFStringAppendCharacters(mutableString,
                                    endBidiWrap, mozilla::ArrayLength(endBidiWrap));
         stringObj = mutableString;
     } else {
         startOffset = 0;
         stringObj = ::CFStringCreateWithCharactersNoCopy(kCFAllocatorDefault,
-                                                         aText, length,
-                                                         kCFAllocatorNull);
+                                                         reinterpret_cast<const UniChar*>(aText),
+                                                         length, kCFAllocatorNull);
     }
 
     CFDictionaryRef attrObj;
@@ -507,6 +489,8 @@ gfxCoreTextShaper::SetGlyphsFromRun(gfxShapedText *aShapedText,
     return NS_OK;
 }
 
+#undef SMALL_GLYPH_RUN
+
 // Construct the font attribute descriptor that we'll apply by default when creating a CTFontRef.
 // This will turn off line-edge swashes by default, because we don't know the actual line breaks
 // when doing glyph shaping.
