@@ -5,30 +5,29 @@
 
 #ifdef MOZ_LOGGING
 #define FORCE_PR_LOG /* Allow logging in the release build */
-#endif
 #include "prlog.h"
+#endif
 
-#include "mozilla/Util.h"
+#include "mozilla/ArrayUtils.h"
 
 #include "gfxFontUtils.h"
 
 #include "nsServiceManagerUtils.h"
 
+#include "mozilla/dom/EncodingUtils.h"
 #include "mozilla/Preferences.h"
+#include "mozilla/Services.h"
 
 #include "nsIUUIDGenerator.h"
-#include "nsMemory.h"
-#include "nsICharsetConverterManager.h"
+#include "nsIObserverService.h"
+#include "nsIUnicodeDecoder.h"
+#include "nsCRT.h"
 
 #include "harfbuzz/hb.h"
 
 #include "plbase64.h"
 #include "prlog.h"
 
-#ifdef XP_MACOSX
-#include <CoreFoundation/CoreFoundation.h>
-#endif
-
 #ifdef PR_LOGGING
 
 #define LOG(log, args) PR_LOG(gfxPlatform::GetLog(log), \
@@ -39,6 +38,7 @@
 #define UNICODE_BMP_LIMIT 0x10000
 
 using namespace mozilla;
+using mozilla::services::GetObserverService;
 
 #pragma pack(1)
 
@@ -1200,8 +1200,6 @@ const char* gfxFontUtils::gMSFontNameCharsets[] =
     /*[10] ENCODING_ID_MICROSOFT_UNICODEFULL */ ""
 };
 
-#define ARRAY_SIZE(A) (sizeof(A) / sizeof(A[0]))
-
 // Return the name of the charset we should use to decode a font name
 // given the name table attributes.
 // Special return values:
@@ -1217,7 +1215,7 @@ gfxFontUtils::GetCharsetForFontName(uint16_t aPlatform, uint16_t aScript, uint16
 
     case PLATFORM_ID_MAC:
         {
-            uint32_t lo = 0, hi = ARRAY_SIZE(gMacFontNameCharsets);
+            uint32_t lo = 0, hi = ArrayLength(gMacFontNameCharsets);
             MacFontNameCharsetMapping searchValue = { aScript, aLanguage, nullptr };
             for (uint32_t i = 0; i < 2; ++i) {
                 // binary search; if not found, set language to ANY and try again
@@ -1237,20 +1235,20 @@ gfxFontUtils::GetCharsetForFontName(uint16_t aPlatform, uint16_t aScript, uint16
                 }
 
                 // no match, so reset high bound for search and re-try
-                hi = ARRAY_SIZE(gMacFontNameCharsets);
+                hi = ArrayLength(gMacFontNameCharsets);
                 searchValue.mLanguage = ANY;
             }
         }
         break;
 
     case PLATFORM_ID_ISO:
-        if (aScript < ARRAY_SIZE(gISOFontNameCharsets)) {
+        if (aScript < ArrayLength(gISOFontNameCharsets)) {
             return gISOFontNameCharsets[aScript];
         }
         break;
 
     case PLATFORM_ID_MICROSOFT:
-        if (aScript < ARRAY_SIZE(gMSFontNameCharsets)) {
+        if (aScript < ArrayLength(gMSFontNameCharsets)) {
             return gMSFontNameCharsets[aScript];
         }
         break;
@@ -1296,23 +1294,15 @@ gfxFontUtils::DecodeFontName(const char *aNameData, int32_t aByteLen,
         return true;
     }
 
-    nsresult rv;
-    nsCOMPtr<nsICharsetConverterManager> ccm =
-        do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
-    NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get charset converter manager");
-    if (NS_FAILED(rv)) {
-        return false;
-    }
-
-    nsCOMPtr<nsIUnicodeDecoder> decoder;
-    rv = ccm->GetUnicodeDecoderRaw(csName, getter_AddRefs(decoder));
-    if (NS_FAILED(rv)) {
+    nsCOMPtr<nsIUnicodeDecoder> decoder =
+        mozilla::dom::EncodingUtils::DecoderForEncoding(csName);
+    if (!decoder) {
         NS_WARNING("failed to get the decoder for a font name string");
         return false;
     }
 
     int32_t destLength;
-    rv = decoder->GetMaxLength(aNameData, aByteLen, &destLength);
+    nsresult rv = decoder->GetMaxLength(aNameData, aByteLen, &destLength);
     if (NS_FAILED(rv)) {
         NS_WARNING("decoder->GetMaxLength failed, invalid font name?");
         return false;
@@ -1429,4 +1419,108 @@ gfxFontUtils::IsCffFont(const uint8_t* aFontData)
     return (sfntHeader->sfntVersion == TRUETYPE_TAG('O','T','T','O'));
 }
 
-#endif
\ No newline at end of file
+#endif
+
+NS_IMPL_ISUPPORTS1(gfxFontInfoLoader::ShutdownObserver, nsIObserver)
+
+NS_IMETHODIMP
+gfxFontInfoLoader::ShutdownObserver::Observe(nsISupports *aSubject,
+                                             const char *aTopic,
+                                             const PRUnichar *someData)
+{
+    if (!nsCRT::strcmp(aTopic, "quit-application")) {
+        mLoader->CancelLoader();
+    } else {
+        NS_NOTREACHED("unexpected notification topic");
+    }
+    return NS_OK;
+}
+
+void
+gfxFontInfoLoader::StartLoader(uint32_t aDelay, uint32_t aInterval)
+{
+    mInterval = aInterval;
+
+    // sanity check
+    if (mState != stateInitial && mState != stateTimerOff) {
+        CancelLoader();
+    }
+
+    // set up timer
+    if (!mTimer) {
+        mTimer = do_CreateInstance("@mozilla.org/timer;1");
+        if (!mTimer) {
+            NS_WARNING("Failure to create font info loader timer");
+            return;
+        }
+    }
+
+    // need an initial delay?
+    uint32_t timerInterval;
+
+    if (aDelay) {
+        mState = stateTimerOnDelay;
+        timerInterval = aDelay;
+    } else {
+        mState = stateTimerOnInterval;
+        timerInterval = mInterval;
+    }
+
+    InitLoader();
+
+    // start timer
+    mTimer->InitWithFuncCallback(LoaderTimerCallback, this, timerInterval,
+                                 nsITimer::TYPE_REPEATING_SLACK);
+
+    nsCOMPtr<nsIObserverService> obs = GetObserverService();
+    if (obs) {
+        mObserver = new ShutdownObserver(this);
+        obs->AddObserver(mObserver, "quit-application", false);
+    }
+}
+
+void
+gfxFontInfoLoader::CancelLoader()
+{
+    if (mState == stateInitial) {
+        return;
+    }
+    mState = stateTimerOff;
+    if (mTimer) {
+        mTimer->Cancel();
+        mTimer = nullptr;
+    }
+    RemoveShutdownObserver();
+    FinishLoader();
+}
+
+void
+gfxFontInfoLoader::LoaderTimerFire()
+{
+    if (mState == stateTimerOnDelay) {
+        mState = stateTimerOnInterval;
+        mTimer->SetDelay(mInterval);
+    }
+
+    bool done = RunLoader();
+    if (done) {
+        CancelLoader();
+    }
+}
+
+gfxFontInfoLoader::~gfxFontInfoLoader()
+{
+    RemoveShutdownObserver();
+}
+
+void
+gfxFontInfoLoader::RemoveShutdownObserver()
+{
+    if (mObserver) {
+        nsCOMPtr<nsIObserverService> obs = GetObserverService();
+        if (obs) {
+            obs->RemoveObserver(mObserver, "quit-application");
+            mObserver = nullptr;
+        }
+    }
+}
