@@ -92,8 +92,9 @@ gfxPlatformGtk::gfxPlatformGtk()
     UpdateFontList();
 #endif
     uint32_t canvasMask = (1 << BACKEND_CAIRO) | (1 << BACKEND_SKIA);
-    uint32_t contentMask = 0;
-    InitBackendPrefs(canvasMask, contentMask);
+    uint32_t contentMask = (1 << BACKEND_CAIRO) | (1 << BACKEND_SKIA);
+    InitBackendPrefs(canvasMask, BACKEND_CAIRO,
+                     contentMask, BACKEND_CAIRO);
 }
 
 gfxPlatformGtk::~gfxPlatformGtk()
@@ -135,11 +136,11 @@ gfxPlatformGtk::~gfxPlatformGtk()
 
 already_AddRefed<gfxASurface>
 gfxPlatformGtk::CreateOffscreenSurface(const gfxIntSize& size,
-                                       gfxASurface::gfxContentType contentType)
+                                       gfxContentType contentType)
 {
     nsRefPtr<gfxASurface> newSurface;
     bool needsClear = true;
-    gfxASurface::gfxImageFormat imageFormat = OptimalFormatForContent(contentType);
+    gfxImageFormat imageFormat = OptimalFormatForContent(contentType);
 #ifdef MOZ_X11
     // XXX we really need a different interface here, something that passes
     // in more context, including the display and/or target surface type that
@@ -475,10 +476,10 @@ gfxPlatformGtk::GetOffscreenFormat()
     // Make sure there is a screen
     GdkScreen *screen = gdk_screen_get_default();
     if (screen && gdk_visual_get_depth(gdk_visual_get_system()) == 16) {
-        return gfxASurface::ImageFormatRGB16_565;
+        return gfxImageFormatRGB16_565;
     }
 
-    return gfxASurface::ImageFormatRGB24;
+    return gfxImageFormatRGB24;
 }
 
 static int sDepth = 0;
@@ -722,41 +723,39 @@ gfxPlatformGtk::SetPrefFontEntries(const nsCString& aKey, nsTArray<nsRefPtr<gfxF
 
 #if (MOZ_WIDGET_GTK == 2)
 void
-gfxPlatformGtk::SetGdkDrawable(gfxASurface *target,
+gfxPlatformGtk::SetGdkDrawable(cairo_surface_t *target,
                                GdkDrawable *drawable)
 {
-    if (target->CairoStatus())
+    if (cairo_surface_status(target))
         return;
 
     g_object_ref(drawable);
 
-    cairo_surface_set_user_data (target->CairoSurface(),
+    cairo_surface_set_user_data (target,
                                  &cairo_gdk_drawable_key,
                                  drawable,
                                  g_object_unref);
 }
 
 GdkDrawable *
-gfxPlatformGtk::GetGdkDrawable(gfxASurface *target)
+gfxPlatformGtk::GetGdkDrawable(cairo_surface_t *target)
 {
-    if (target->CairoStatus())
+    if (cairo_surface_status(target))
         return nullptr;
 
     GdkDrawable *result;
 
-    result = (GdkDrawable*) cairo_surface_get_user_data (target->CairoSurface(),
+    result = (GdkDrawable*) cairo_surface_get_user_data (target,
                                                          &cairo_gdk_drawable_key);
     if (result)
         return result;
 
 #ifdef MOZ_X11
-    if (target->GetType() != gfxASurface::SurfaceTypeXlib)
+    if (cairo_surface_get_type(target) != CAIRO_SURFACE_TYPE_XLIB)
         return nullptr;
 
-    gfxXlibSurface *xs = static_cast<gfxXlibSurface*>(target);
-
     // try looking it up in gdk's table
-    result = (GdkDrawable*) gdk_xid_table_lookup(xs->XDrawable());
+    result = (GdkDrawable*) gdk_xid_table_lookup(cairo_xlib_surface_get_drawable(target));
     if (result) {
         SetGdkDrawable(target, result);
         return result;
