@@ -10,15 +10,9 @@
 namespace mozilla {
 namespace image {
 
-#ifdef DEBUG_imagelib
-static const bool allowOMTURIAccess = true;
-#else
-static const bool allowOMTURIAccess = false;
-#endif
-
 // Constructor
-ImageResource::ImageResource(imgStatusTracker* aStatusTracker, nsIURI* aURI) :
-  mURI(new nsMainThreadPtrHolder<nsIURI>(aURI, allowOMTURIAccess)),
+ImageResource::ImageResource(ImageURL* aURI) :
+  mURI(aURI),
   mInnerWindowId(0),
   mAnimationConsumers(0),
   mAnimationMode(kNormalAnimMode),
@@ -26,12 +20,6 @@ ImageResource::ImageResource(imgStatusTracker* aStatusTracker, nsIURI* aURI) :
   mAnimating(false),
   mError(false)
 {
-  if (aStatusTracker) {
-    mStatusTracker = aStatusTracker;
-    mStatusTracker->SetImage(this);
-  } else {
-    mStatusTracker = new imgStatusTracker(this);
-  }
 }
 
 uint32_t
@@ -98,12 +86,16 @@ Image::GetDecoderType(const char *aMimeType)
 void
 ImageResource::IncrementAnimationConsumers()
 {
+  MOZ_ASSERT(NS_IsMainThread(), "Main thread only to encourage serialization "
+                                "with DecrementAnimationConsumers");
   mAnimationConsumers++;
 }
 
 void
 ImageResource::DecrementAnimationConsumers()
 {
+  MOZ_ASSERT(NS_IsMainThread(), "Main thread only to encourage serialization "
+                                "with IncrementAnimationConsumers");
   NS_ABORT_IF_FALSE(mAnimationConsumers >= 1, "Invalid no. of animation consumers!");
   mAnimationConsumers--;
 }
@@ -144,7 +136,6 @@ ImageResource::EvaluateAnimation()
     mAnimating = NS_SUCCEEDED(rv);
   } else if (mAnimating && !ShouldAnimate()) {
     StopAnimation();
-    mAnimating = false;
   }
 }
 
