@@ -8,11 +8,11 @@
 #include "mozilla/DebugOnly.h"
 
 #include "WindowsMessageLoop.h"
-#include "RPCChannel.h"
+#include "MessageChannel.h"
 
 #include "nsAutoPtr.h"
 #include "nsServiceManagerUtils.h"
-#include "nsStringGlue.h"
+#include "nsString.h"
 #include "nsIXULAppInfo.h"
 
 #include "mozilla/PaintTracker.h"
@@ -61,8 +61,8 @@ using namespace mozilla::ipc::windows;
  * in a special window procedure where we can either ignore the message or
  * process it in some fashion.
  *
- * Queued and "non-queued" messages will be processed during RPC calls if
- * modal UI related api calls block an RPC in-call in the child. To prevent
+ * Queued and "non-queued" messages will be processed during Interrupt calls if
+ * modal UI related api calls block an Interrupt in-call in the child. To prevent
  * windows from freezing, and to allow concurrent processing of critical
  * events (such as painting), we spin a native event dispatch loop while
  * these in-calls are blocked.
@@ -70,7 +70,7 @@ using namespace mozilla::ipc::windows;
 
 #if defined(ACCESSIBILITY)
 // pulled from accessibility's win utils
-extern const PRUnichar* kPropNameTabContent;
+extern const wchar_t* kPropNameTabContent;
 #endif
 
 // widget related message id constants we need to defer
@@ -115,10 +115,10 @@ DeferredMessageHook(int nCode,
   // Only run deferred messages if all of these conditions are met:
   //   1. The |nCode| indicates that this hook should do something.
   //   2. We have deferred messages to run.
-  //   3. We're not being called from the PeekMessage within the WaitForNotify
-  //      function (indicated with SyncChannel::IsPumpingMessages). We really
+  //   3. We're not being called from the PeekMessage within the WaitFor*Notify
+  //      function (indicated with MessageChannel::IsPumpingMessages). We really
   //      only want to run after returning to the main event loop.
-  if (nCode >= 0 && gDeferredMessages && !SyncChannel::IsPumpingMessages()) {
+  if (nCode >= 0 && gDeferredMessages && !MessageChannel::IsPumpingMessages()) {
     NS_ASSERTION(gDeferredGetMsgHook && gDeferredCallWndProcHook,
                  "These hooks must be set if we're being called!");
     NS_ASSERTION(gDeferredMessages->Length(), "No deferred messages?!");
@@ -587,8 +587,8 @@ TimeoutHasExpired(const TimeoutData& aData)
 
 } // anonymous namespace
 
-RPCChannel::SyncStackFrame::SyncStackFrame(SyncChannel* channel, bool rpc)
-  : mRPC(rpc)
+MessageChannel::SyncStackFrame::SyncStackFrame(MessageChannel* channel, bool interrupt)
+  : mInterrupt(interrupt)
   , mSpinNestedEvents(false)
   , mListenerNotified(false)
   , mChannel(channel)
@@ -605,12 +605,12 @@ RPCChannel::SyncStackFrame::SyncStackFrame(SyncChannel* channel, bool rpc)
   }
 }
 
-RPCChannel::SyncStackFrame::~SyncStackFrame()
+MessageChannel::SyncStackFrame::~SyncStackFrame()
 {
   NS_ASSERTION(this == mChannel->mTopFrame,
-               "Mismatched RPC stack frames");
+               "Mismatched interrupt stack frames");
   NS_ASSERTION(this == sStaticTopFrame,
-               "Mismatched static RPC stack frames");
+               "Mismatched static Interrupt stack frames");
 
   mChannel->mTopFrame = mPrev;
   sStaticTopFrame = mStaticPrev;
@@ -622,45 +622,45 @@ RPCChannel::SyncStackFrame::~SyncStackFrame()
   }
 }
 
-SyncChannel::SyncStackFrame* SyncChannel::sStaticTopFrame;
+MessageChannel::SyncStackFrame* MessageChannel::sStaticTopFrame;
 
 // nsAppShell's notification that gecko events are being processed.
-// If we are here and there is an RPC Incall active, we are spinning
+// If we are here and there is an Interrupt Incall active, we are spinning
 // a nested gecko event loop. In which case the remote process needs
 // to know about it.
 void /* static */
-RPCChannel::NotifyGeckoEventDispatch()
+MessageChannel::NotifyGeckoEventDispatch()
 {
-  // sStaticTopFrame is only valid for RPC channels
+  // sStaticTopFrame is only valid for Interrupt channels
   if (!sStaticTopFrame || sStaticTopFrame->mListenerNotified)
     return;
 
   sStaticTopFrame->mListenerNotified = true;
-  RPCChannel* channel = static_cast<RPCChannel*>(sStaticTopFrame->mChannel);
-  channel->Listener()->ProcessRemoteNativeEventsInRPCCall();
+  MessageChannel* channel = static_cast<MessageChannel*>(sStaticTopFrame->mChannel);
+  channel->Listener()->ProcessRemoteNativeEventsInInterruptCall();
 }
 
 // invoked by the module that receives the spin event loop
 // message.
 void
-RPCChannel::ProcessNativeEventsInRPCCall()
+MessageChannel::ProcessNativeEventsInInterruptCall()
 {
   if (!mTopFrame) {
-    NS_ERROR("Spin logic error: no RPC frame");
+    NS_ERROR("Spin logic error: no Interrupt frame");
     return;
   }
 
   mTopFrame->mSpinNestedEvents = true;
 }
 
-// Spin loop is called in place of WaitForNotify when modal ui is being shown
+// Spin loop is called in place of WaitFor*Notify when modal ui is being shown
 // in a child. There are some intricacies in using it however. Spin loop is
-// enabled for a particular RPC frame by the client calling
-// RPCChannel::ProcessNativeEventsInRPCCall().
-// This call can be nested for multiple RPC frames in a single plugin or 
+// enabled for a particular Interrupt frame by the client calling
+// MessageChannel::ProcessNativeEventsInInterrupt().
+// This call can be nested for multiple Interrupt frames in a single plugin or 
 // multiple unrelated plugins.
 void
-RPCChannel::SpinInternalEventLoop()
+MessageChannel::SpinInternalEventLoop()
 {
   if (mozilla::PaintTracker::IsPainting()) {
     NS_RUNTIMEABORT("Don't spin an event loop while painting.");
@@ -694,7 +694,7 @@ RPCChannel::SpinInternalEventLoop()
           NS_ERROR("WM_QUIT received in SpinInternalEventLoop!");
       } else {
           TranslateMessage(&msg);
-          DispatchMessageW(&msg);
+          ::DispatchMessageW(&msg);
           return;
       }
     }
@@ -715,14 +715,14 @@ RPCChannel::SpinInternalEventLoop()
 }
 
 bool
-SyncChannel::WaitForNotify()
+MessageChannel::WaitForSyncNotify()
 {
   mMonitor->AssertCurrentThreadOwns();
 
   // Initialize global objects used in deferred messaging.
   Init();
 
-  NS_ASSERTION(mTopFrame && !mTopFrame->mRPC,
+  NS_ASSERTION(mTopFrame && !mTopFrame->mInterrupt,
                "Top frame is not a sync frame!");
 
   MonitorAutoUnlock unlock(*mMonitor);
@@ -742,10 +742,10 @@ SyncChannel::WaitForNotify()
   }
 
   // Setup deferred processing of native events while we wait for a response.
-  NS_ASSERTION(!SyncChannel::IsPumpingMessages(),
+  NS_ASSERTION(!MessageChannel::IsPumpingMessages(),
                "Shouldn't be pumping already!");
 
-  SyncChannel::SetIsPumpingMessages(true);
+  MessageChannel::SetIsPumpingMessages(true);
   HHOOK windowHook = SetWindowsHookEx(WH_CALLWNDPROC, CallWindowProcedureHook,
                                       nullptr, gUIThreadId);
   NS_ASSERTION(windowHook, "Failed to set hook!");
@@ -831,25 +831,25 @@ SyncChannel::WaitForNotify()
     KillTimer(nullptr, timerId);
   }
 
-  SyncChannel::SetIsPumpingMessages(false);
+  MessageChannel::SetIsPumpingMessages(false);
 
   return WaitResponse(timedout);
 }
 
 bool
-RPCChannel::WaitForNotify()
+MessageChannel::WaitForInterruptNotify()
 {
   mMonitor->AssertCurrentThreadOwns();
 
-  if (!StackDepth()) {
+  if (!InterruptStackDepth()) {
     // There is currently no way to recover from this condition.
-    NS_RUNTIMEABORT("StackDepth() is 0 in call to RPCChannel::WaitForNotify!");
+    NS_RUNTIMEABORT("StackDepth() is 0 in call to MessageChannel::WaitForNotify!");
   }
 
   // Initialize global objects used in deferred messaging.
   Init();
 
-  NS_ASSERTION(mTopFrame && mTopFrame->mRPC,
+  NS_ASSERTION(mTopFrame && mTopFrame->mInterrupt,
                "Top frame is not a sync frame!");
 
   MonitorAutoUnlock unlock(*mMonitor);
@@ -862,11 +862,11 @@ RPCChannel::WaitForNotify()
   // windowHook is used as a flag variable for the loop below: if it is set
   // and we start to spin a nested event loop, we need to clear the hook and
   // process deferred/pending messages.
-  // If windowHook is nullptr, SyncChannel::IsPumpingMessages should be false.
+  // If windowHook is nullptr, MessageChannel::IsPumpingMessages should be false.
   HHOOK windowHook = nullptr;
 
   while (1) {
-    NS_ASSERTION((!!windowHook) == SyncChannel::IsPumpingMessages(),
+    NS_ASSERTION((!!windowHook) == MessageChannel::IsPumpingMessages(),
                  "windowHook out of sync with reality");
 
     if (mTopFrame->mSpinNestedEvents) {
@@ -880,7 +880,7 @@ RPCChannel::WaitForNotify()
         }
 
         // Used by widget to assert on incoming native events
-        SyncChannel::SetIsPumpingMessages(false);
+        MessageChannel::SetIsPumpingMessages(false);
 
         // Unhook any neutered windows procedures so messages can be delievered
         // normally.
@@ -897,7 +897,7 @@ RPCChannel::WaitForNotify()
     }
 
     if (!windowHook) {
-      SyncChannel::SetIsPumpingMessages(true);
+      MessageChannel::SetIsPumpingMessages(true);
       windowHook = SetWindowsHookEx(WH_CALLWNDPROC, CallWindowProcedureHook,
                                     nullptr, gUIThreadId);
       NS_ASSERTION(windowHook, "Failed to set hook!");
@@ -939,7 +939,7 @@ RPCChannel::WaitForNotify()
       break;
     }
 
-    // See SyncChannel's WaitForNotify for details.
+    // See MessageChannel's WaitFor*Notify for details.
     bool haveSentMessagesPending =
       (HIWORD(GetQueueStatus(QS_SENDMESSAGE)) & QS_SENDMESSAGE) != 0;
 
@@ -971,13 +971,13 @@ RPCChannel::WaitForNotify()
     }
   }
 
-  SyncChannel::SetIsPumpingMessages(false);
+  MessageChannel::SetIsPumpingMessages(false);
 
   return WaitResponse(timedout);
 }
 
 void
-SyncChannel::NotifyWorkerThread()
+MessageChannel::NotifyWorkerThread()
 {
   mMonitor->AssertCurrentThreadOwns();
   NS_ASSERTION(mEvent, "No signal event to set, this is really bad!");
