@@ -5,7 +5,9 @@
 
 import os, sys
 
-from ipdl.ast import CxxInclude, Decl, Loc, QualifiedId, State, StructDecl, TransitionStmt, TypeSpec, UnionDecl, UsingStmt, Visitor, ASYNC, SYNC, RPC, IN, OUT, INOUT, ANSWER, CALL, RECV, SEND, URGENT
+from ipdl.ast import CxxInclude, Decl, Loc, QualifiedId, State, StructDecl, TransitionStmt
+from ipdl.ast import TypeSpec, UnionDecl, UsingStmt, Visitor, ASYNC, SYNC, INTR
+from ipdl.ast import IN, OUT, INOUT, ANSWER, CALL, RECV, SEND, URGENT, RPC
 import ipdl.builtin as builtin
 
 _DELETE_MSG = '__delete__'
@@ -204,18 +206,24 @@ class IPDLType(Type):
 
     def isAsync(self): return self.sendSemantics is ASYNC
     def isSync(self): return self.sendSemantics is SYNC
-    def isRpc(self): return self.sendSemantics is RPC or self.sendSemantics is URGENT
+    def isInterrupt(self): return self.sendSemantics is INTR
     def isUrgent(self): return self.sendSemantics is URGENT
+    def isRpc(self): return self.sendSemantics is RPC
 
     def talksAsync(self): return True
-    def talksSync(self): return self.isSync() or self.isRpc()
-    def talksRpc(self): return self.isRpc()
+    def talksSync(self): return self.isSync() or self.isRpc() or self.isInterrupt()
+    def talksRpc(self): return self.isRpc() or self.isInterrupt()
+    def talksInterrupt(self): return self.isInterrupt()
 
-    def hasReply(self):  return self.isSync() or self.isRpc()
+    def hasReply(self):  return (self.isSync()
+                                 or self.isInterrupt()
+                                 or self.isUrgent()
+                                 or self.isRpc())
 
     def needsMoreJuiceThan(self, o):
         return (o.isAsync() and not self.isAsync()
-                or o.isSync() and self.isRpc())
+                or o.isSync() and (self.isUrgent() or self.isRpc())
+                or (o.isUrgent() or o.isRpc()) and self.isInterrupt())
 
 class StateType(IPDLType):
     def __init__(self, protocol, name, start=False):
@@ -500,7 +508,7 @@ def makeBuiltinUsing(tname):
                               QualifiedId(_builtinloc, base, quals)))
 
 builtinUsing = [ makeBuiltinUsing(t) for t in builtin.Types ]
-builtinIncludes = [ CxxInclude(_builtinloc, f) for f in builtin.Includes ]
+builtinHeaderIncludes = [ CxxInclude(_builtinloc, f) for f in builtin.HeaderIncludes ]
 
 def errormsg(loc, fmt, *args):
     while not isinstance(loc, Loc):
@@ -589,8 +597,6 @@ With this information, it finally type checks the AST.'''
                 return False
             return True
 
-        tu.cxxIncludes = builtinIncludes + tu.cxxIncludes
-
         # tag each relevant node with "decl" information, giving type, name,
         # and location of declaration
         if not runpass(GatherDecls(builtinUsing, self.errors)):
@@ -871,7 +877,7 @@ class GatherDecls(TcheckVisitor):
                 "destructor declaration `%s(...)' required for managed protocol `%s'",
                 _DELETE_MSG, p.name)
 
-        p.decl.type.hasReentrantDelete = p.decl.type.hasDelete and self.symtab.lookup(_DELETE_MSG).type.isRpc()
+        p.decl.type.hasReentrantDelete = p.decl.type.hasDelete and self.symtab.lookup(_DELETE_MSG).type.isInterrupt()
 
         for managed in p.managesStmts:
             mgdname = managed.name
@@ -1459,6 +1465,18 @@ class CheckTypes(TcheckVisitor):
                 "sync parent-to-child messages are verboten (here, message `%s' in protocol `%s')",
                 mname, pname)
 
+        if mtype.isUrgent() and (mtype.isIn() or mtype.isInout()):
+            self.error(
+                loc,
+                "urgent child-to-parent messages are verboten (here, message `%s' in protocol `%s')",
+                mname, pname)
+
+        if mtype.isRpc() and (mtype.isOut() or mtype.isInout()):
+            self.error(
+                loc,
+                "rpc parent-to-child messages are verboten (here, message' `%s' in protocol `%s')",
+                mname, pname)
+
         if mtype.needsMoreJuiceThan(ptype):
             self.error(
                 loc,
@@ -1491,13 +1509,13 @@ class CheckTypes(TcheckVisitor):
         loc = t.loc
         impliedDirection, impliedSems = {
             SEND: [ OUT, _YNC ], RECV: [ IN, _YNC ],
-            CALL: [ OUT, RPC ],  ANSWER: [ IN, RPC ],
+            CALL: [ OUT, INTR ],  ANSWER: [ IN, INTR ],
          } [t.trigger]
         
         if (OUT is impliedDirection and t.msg.type.isIn()
             or IN is impliedDirection and t.msg.type.isOut()
-            or _YNC is impliedSems and t.msg.type.isRpc()
-            or RPC is impliedSems and (not t.msg.type.isRpc())):
+            or _YNC is impliedSems and t.msg.type.isInterrupt()
+            or INTR is impliedSems and (not t.msg.type.isInterrupt())):
             mtype = t.msg.type
 
             self.error(
