@@ -17,10 +17,8 @@
 #include "base/message_loop.h"
 
 #include "mozilla/Monitor.h"
-#include "mozilla/Util.h"
 #include "mozilla/FileUtils.h"
 #include "nsString.h"
-#include "nsThreadUtils.h"
 #include "nsTArray.h"
 #include "nsXULAppAPI.h"
 
@@ -44,13 +42,15 @@ class UnixSocketImpl : public MessageLoopForIO::Watcher
 {
 public:
   UnixSocketImpl(UnixSocketConsumer* aConsumer, UnixSocketConnector* aConnector,
-                 const nsACString& aAddress)
+                 const nsACString& aAddress,
+                 SocketConnectionStatus aConnectionStatus)
     : mConsumer(aConsumer)
     , mIOLoop(nullptr)
     , mConnector(aConnector)
     , mShuttingDownOnIOThread(false)
     , mAddress(aAddress)
     , mDelayedConnectTask(nullptr)
+    , mConnectionStatus(aConnectionStatus)
   {
   }
 
@@ -246,6 +246,12 @@ private:
    * Task member for delayed connect task. Should only be access on main thread.
    */
   CancelableTask* mDelayedConnectTask;
+
+  /**
+   * Socket connection status. Duplicate from UnixSocketConsumer. Should only
+   * be accessed on I/O thread.
+   */
+  SocketConnectionStatus mConnectionStatus;
 };
 
 template<class T>
@@ -521,6 +527,15 @@ UnixSocketImpl::Accept()
       return;
     }
 
+    if (!mConnector->SetUpListenSocket(mFd)) {
+      NS_WARNING("Could not set up listen socket!");
+      nsRefPtr<OnSocketEventTask> t =
+        new OnSocketEventTask(this, OnSocketEventTask::CONNECT_ERROR);
+      NS_DispatchToMainThread(t);
+      mConnectionStatus = SOCKET_DISCONNECTED;
+      return;
+    }
+
   }
 
   SetUpIO();
@@ -555,6 +570,7 @@ UnixSocketImpl::Connect()
     nsRefPtr<OnSocketEventTask> t =
       new OnSocketEventTask(this, OnSocketEventTask::CONNECT_ERROR);
     NS_DispatchToMainThread(t);
+    mConnectionStatus = SOCKET_DISCONNECTED;
     return;
   }
 
@@ -571,6 +587,7 @@ UnixSocketImpl::Connect()
         nsRefPtr<OnSocketEventTask> t =
           new OnSocketEventTask(this, OnSocketEventTask::CONNECT_ERROR);
         NS_DispatchToMainThread(t);
+        mConnectionStatus = SOCKET_DISCONNECTED;
         return;
       }
       if (-1 == fcntl(mFd.get(), F_SETFL, current_opts & ~O_NONBLOCK)) {
@@ -579,6 +596,7 @@ UnixSocketImpl::Connect()
         nsRefPtr<OnSocketEventTask> t =
           new OnSocketEventTask(this, OnSocketEventTask::CONNECT_ERROR);
         NS_DispatchToMainThread(t);
+        mConnectionStatus = SOCKET_DISCONNECTED;
         return;
       }
 
@@ -602,6 +620,7 @@ UnixSocketImpl::Connect()
     nsRefPtr<OnSocketEventTask> t =
       new OnSocketEventTask(this, OnSocketEventTask::CONNECT_ERROR);
     NS_DispatchToMainThread(t);
+    mConnectionStatus = SOCKET_DISCONNECTED;
     return;
   }
 
@@ -617,6 +636,7 @@ UnixSocketImpl::Connect()
   nsRefPtr<OnSocketEventTask> t =
     new OnSocketEventTask(this, OnSocketEventTask::CONNECT_SUCCESS);
   NS_DispatchToMainThread(t);
+  mConnectionStatus = SOCKET_CONNECTED;
 
   SetUpIO();
 }
@@ -713,8 +733,7 @@ UnixSocketImpl::OnFileCanReadWithoutBlocking(int aFd)
   MOZ_ASSERT(!NS_IsMainThread());
   MOZ_ASSERT(!mShuttingDownOnIOThread);
 
-  SocketConnectionStatus status = mConsumer->GetConnectionStatus();
-  if (status == SOCKET_CONNECTED) {
+  if (mConnectionStatus == SOCKET_CONNECTED) {
     // Read all of the incoming data.
     while (true) {
       nsAutoPtr<UnixSocketRawData> incoming(new UnixSocketRawData(MAX_READ_SIZE));
@@ -757,9 +776,7 @@ UnixSocketImpl::OnFileCanReadWithoutBlocking(int aFd)
     }
 
     MOZ_CRASH("We returned early");
-  }
-
-  if (status == SOCKET_LISTENING) {
+  } else if (mConnectionStatus == SOCKET_LISTENING) {
     int client_fd = accept(mFd.get(), (struct sockaddr*)&mAddr, &mAddrSize);
 
     if (client_fd < 0) {
@@ -784,6 +801,7 @@ UnixSocketImpl::OnFileCanReadWithoutBlocking(int aFd)
     nsRefPtr<OnSocketEventTask> t =
       new OnSocketEventTask(this, OnSocketEventTask::CONNECT_SUCCESS);
     NS_DispatchToMainThread(t);
+    mConnectionStatus = SOCKET_CONNECTED;
 
     SetUpIO();
   }
@@ -796,8 +814,7 @@ UnixSocketImpl::OnFileCanWriteWithoutBlocking(int aFd)
   MOZ_ASSERT(!mShuttingDownOnIOThread);
 
   MOZ_ASSERT(aFd >= 0);
-  SocketConnectionStatus status = mConsumer->GetConnectionStatus();
-  if (status == SOCKET_CONNECTED) {
+  if (mConnectionStatus == SOCKET_CONNECTED) {
     // Try to write the bytes of mCurrentRilRawData.  If all were written, continue.
     //
     // Otherwise, save the byte position of the next byte to write
@@ -838,7 +855,7 @@ UnixSocketImpl::OnFileCanWriteWithoutBlocking(int aFd)
       mOutgoingQ.RemoveElementAt(0);
       delete data;
     }
-  } else if (status == SOCKET_CONNECTING) {
+  } else if (mConnectionStatus == SOCKET_CONNECTING) {
     int error, ret;
     socklen_t len = sizeof(error);
     ret = getsockopt(mFd.get(), SOL_SOCKET, SO_ERROR, &error, &len);
@@ -849,6 +866,7 @@ UnixSocketImpl::OnFileCanWriteWithoutBlocking(int aFd)
       nsRefPtr<OnSocketEventTask> t =
         new OnSocketEventTask(this, OnSocketEventTask::CONNECT_ERROR);
       NS_DispatchToMainThread(t);
+      mConnectionStatus = SOCKET_DISCONNECTED;
       return;
     }
 
@@ -857,6 +875,7 @@ UnixSocketImpl::OnFileCanWriteWithoutBlocking(int aFd)
       nsRefPtr<OnSocketEventTask> t =
         new OnSocketEventTask(this, OnSocketEventTask::CONNECT_ERROR);
       NS_DispatchToMainThread(t);
+      mConnectionStatus = SOCKET_DISCONNECTED;
       return;
     }
 
@@ -866,12 +885,14 @@ UnixSocketImpl::OnFileCanWriteWithoutBlocking(int aFd)
       nsRefPtr<OnSocketEventTask> t =
         new OnSocketEventTask(this, OnSocketEventTask::CONNECT_ERROR);
       NS_DispatchToMainThread(t);
+      mConnectionStatus = SOCKET_DISCONNECTED;
       return;
     }
 
     nsRefPtr<OnSocketEventTask> t =
       new OnSocketEventTask(this, OnSocketEventTask::CONNECT_SUCCESS);
     NS_DispatchToMainThread(t);
+    mConnectionStatus = SOCKET_CONNECTED;
 
     SetUpIO();
   }
@@ -928,7 +949,7 @@ UnixSocketConsumer::ConnectSocket(UnixSocketConnector* aConnector,
   }
 
   nsCString addr(aAddress);
-  mImpl = new UnixSocketImpl(this, connector.forget(), addr);
+  mImpl = new UnixSocketImpl(this, connector.forget(), addr, SOCKET_CONNECTING);
   MessageLoop* ioLoop = XRE_GetIOMessageLoop();
   mConnectionStatus = SOCKET_CONNECTING;
   if (aDelayMs > 0) {
@@ -954,7 +975,8 @@ UnixSocketConsumer::ListenSocket(UnixSocketConnector* aConnector)
     return false;
   }
 
-  mImpl = new UnixSocketImpl(this, connector.forget(), EmptyCString());
+  mImpl = new UnixSocketImpl(this, connector.forget(), EmptyCString(),
+                             SOCKET_LISTENING);
   mConnectionStatus = SOCKET_LISTENING;
   XRE_GetIOMessageLoop()->PostTask(FROM_HERE,
                                    new SocketAcceptTask(mImpl));
