@@ -78,6 +78,8 @@ class gcstats::StatisticsSerializer
     }
 
     void appendDecimal(const char *name, const char *units, double d) {
+        if (d < 0)
+            d = 0;
         if (asJSON_)
             appendNumber(name, "%d.%d", units, (int)d, (int)(d * 10.) % 10);
         else
@@ -124,32 +126,26 @@ class gcstats::StatisticsSerializer
     jschar *finishJSString() {
         char *buf = finishCString();
         if (!buf)
-            return NULL;
+            return nullptr;
 
         size_t nchars = strlen(buf);
         jschar *out = js_pod_malloc<jschar>(nchars + 1);
         if (!out) {
             oom_ = true;
             js_free(buf);
-            return NULL;
+            return nullptr;
         }
 
-        size_t outlen = nchars;
-        bool ok = InflateStringToBuffer(NULL, buf, nchars, out, &outlen);
+        InflateStringToBuffer(buf, nchars, out);
         js_free(buf);
-        if (!ok) {
-            oom_ = true;
-            js_free(out);
-            return NULL;
-        }
-        out[nchars] = 0;
 
+        out[nchars] = 0;
         return out;
     }
 
     char *finishCString() {
         if (oom_)
-            return NULL;
+            return nullptr;
 
         buf_.append('\0');
 
@@ -285,7 +281,6 @@ static const PhaseInfo phases[] = {
     { PHASE_SWEEP, "Sweep", PHASE_NO_PARENT },
     { PHASE_SWEEP_MARK, "Mark During Sweeping", PHASE_SWEEP },
     { PHASE_SWEEP_MARK_TYPES, "Mark Types During Sweeping", PHASE_SWEEP_MARK },
-    { PHASE_SWEEP_MARK_DELAYED, "Mark Delayed During Sweeping", PHASE_SWEEP_MARK },
     { PHASE_SWEEP_MARK_INCOMING_BLACK, "Mark Incoming Black Pointers", PHASE_SWEEP_MARK },
     { PHASE_SWEEP_MARK_WEAK, "Mark Weak", PHASE_SWEEP_MARK },
     { PHASE_SWEEP_MARK_INCOMING_GRAY, "Mark Incoming Gray Pointers", PHASE_SWEEP_MARK },
@@ -315,7 +310,7 @@ static const PhaseInfo phases[] = {
     { PHASE_FINALIZE_END, "Finalize End Callback", PHASE_SWEEP },
     { PHASE_DESTROY, "Deallocate", PHASE_SWEEP },
     { PHASE_GC_END, "End Callback", PHASE_NO_PARENT },
-    { PHASE_LIMIT, NULL, PHASE_NO_PARENT }
+    { PHASE_LIMIT, nullptr, PHASE_NO_PARENT }
 };
 
 static void
@@ -360,7 +355,7 @@ Statistics::formatData(StatisticsSerializer &ss, uint64_t timestamp)
     double mmu20 = computeMMU(20 * PRMJ_USEC_PER_MSEC);
     double mmu50 = computeMMU(50 * PRMJ_USEC_PER_MSEC);
 
-    ss.beginObject(NULL);
+    ss.beginObject(nullptr);
     if (ss.isJSON())
         ss.appendNumber("Timestamp", "%llu", "", (unsigned long long)timestamp);
     if (slices.length() > 1 || ss.isJSON())
@@ -368,9 +363,9 @@ Statistics::formatData(StatisticsSerializer &ss, uint64_t timestamp)
     else
         ss.appendString("Reason", ExplainReason(slices[0].reason));
     ss.appendDecimal("Total Time", "ms", t(total));
-    ss.appendNumber("Compartments Collected", "%d", "", collectedCount);
-    ss.appendNumber("Total Compartments", "%d", "", compartmentCount);
+    ss.appendNumber("Zones Collected", "%d", "", collectedCount);
     ss.appendNumber("Total Zones", "%d", "", zoneCount);
+    ss.appendNumber("Total Compartments", "%d", "", compartmentCount);
     ss.appendNumber("MMU (20ms)", "%d", "%", int(mmu20 * 100));
     ss.appendNumber("MMU (50ms)", "%d", "%", int(mmu50 * 100));
     ss.appendDecimal("SCC Sweep Total", "ms", t(sccTotal));
@@ -394,7 +389,7 @@ Statistics::formatData(StatisticsSerializer &ss, uint64_t timestamp)
                 continue;
             }
 
-            ss.beginObject(NULL);
+            ss.beginObject(nullptr);
             ss.extra("    ");
             ss.appendNumber("Slice", "%d", "", i);
             ss.appendDecimal("Pause", "", t(width));
@@ -443,13 +438,13 @@ Statistics::formatJSON(uint64_t timestamp)
 Statistics::Statistics(JSRuntime *rt)
   : runtime(rt),
     startupTime(PRMJ_Now()),
-    fp(NULL),
+    fp(nullptr),
     fullFormat(false),
     gcDepth(0),
     collectedCount(0),
     zoneCount(0),
     compartmentCount(0),
-    nonincrementalReason(NULL),
+    nonincrementalReason(nullptr),
     preBytes(0),
     phaseNestingDepth(0)
 {
@@ -458,7 +453,7 @@ Statistics::Statistics(JSRuntime *rt)
 
     char *env = getenv("MOZ_GCTIMER");
     if (!env || strcmp(env, "none") == 0) {
-        fp = NULL;
+        fp = nullptr;
         return;
     }
 
@@ -525,7 +520,7 @@ Statistics::beginGC()
 
     slices.clearAndFree();
     sccTimes.clearAndFree();
-    nonincrementalReason = NULL;
+    nonincrementalReason = nullptr;
 
     preBytes = runtime->gcBytes;
 }
