@@ -31,7 +31,7 @@ class C1Spewer
 
   public:
     C1Spewer()
-      : graph(NULL), script(NullPtr()), spewout_(NULL)
+      : graph(nullptr), script(NullPtr()), spewout_(nullptr)
     { }
 
     bool init(const char *path);
