@@ -6,14 +6,11 @@
 
 #include "jsscript.h"
 
-#include "jit/IonFrames.h"
-#include "jit/IonLinker.h"
 #include "jit/IonSpewer.h"
 #ifdef TRACK_SNAPSHOTS
 #include "jit/LIR.h"
 #include "jit/MIR.h"
 #endif
-#include "jit/MIRGenerator.h"
 #include "jit/SnapshotReader.h"
 #include "jit/SnapshotWriter.h"
 
@@ -304,9 +301,12 @@ SnapshotWriter::startSnapshot(uint32_t frameCount, BailoutKind kind, bool resume
 void
 SnapshotWriter::startFrame(JSFunction *fun, JSScript *script, jsbytecode *pc, uint32_t exprStack)
 {
-    JS_ASSERT(CountArgSlots(script, fun) < SNAPSHOT_MAX_NARGS);
+    // Test if we honor the maximum of arguments at all times.
+    // This is a sanity check and not an algorithm limit. So check might be a bit too loose.
+    // +4 to account for scope chain, return value, this value and maybe arguments_object.
+    JS_ASSERT(CountArgSlots(script, fun) < SNAPSHOT_MAX_NARGS + 4);
 
-    uint32_t implicit = StartArgSlot(script, fun);
+    uint32_t implicit = StartArgSlot(script);
     uint32_t formalArgs = CountArgSlots(script, fun);
 
     nslots_ = formalArgs + script->nfixed + exprStack;
@@ -315,9 +315,7 @@ SnapshotWriter::startFrame(JSFunction *fun, JSScript *script, jsbytecode *pc, ui
     IonSpew(IonSpew_Snapshots, "Starting frame; implicit %u, formals %u, fixed %u, exprs %u",
             implicit, formalArgs - implicit, script->nfixed, exprStack);
 
-    JS_ASSERT(script->code <= pc && pc <= script->code + script->length);
-
-    uint32_t pcoff = uint32_t(pc - script->code);
+    uint32_t pcoff = script->pcToOffset(pc);
     IonSpew(IonSpew_Snapshots, "Writing pc offset %u, nslots %u", pcoff, nslots_);
     writer_.writeUnsigned(pcoff);
     writer_.writeUnsigned(nslots_);
@@ -362,7 +360,7 @@ SnapshotWriter::writeSlotHeader(JSValueType type, uint32_t regCode)
 void
 SnapshotWriter::addSlot(const FloatRegister &reg)
 {
-    JS_ASSERT(reg.code() < MIN_REG_FIELD_ESC);
+    JS_ASSERT(uint32_t(reg.code()) < MIN_REG_FIELD_ESC);
     IonSpew(IonSpew_Snapshots, "    slot %u: double (reg %s)", slotsWritten_, reg.name());
 
     writeSlotHeader(JSVAL_TYPE_DOUBLE, reg.code());
@@ -519,7 +517,7 @@ SnapshotWriter::addInt32Slot(int32_t value)
 void
 SnapshotWriter::addFloat32Slot(const FloatRegister &reg)
 {
-    JS_ASSERT(reg.code() < MIN_REG_FIELD_ESC);
+    JS_ASSERT(uint32_t(reg.code()) < MIN_REG_FIELD_ESC);
     IonSpew(IonSpew_Snapshots, "    slot %u: float32 (reg %s)", slotsWritten_, reg.name());
     writeSlotHeader(JSVAL_TYPE_NULL, ESC_REG_FIELD_FLOAT32_REG);
     writer_.writeUnsigned(reg.code());
