@@ -91,10 +91,10 @@ class MacroAssemblerX86Shared : public Assembler
     }
 
     void move32(const Imm32 &imm, const Register &dest) {
-        if (imm.value == 0)
-            xorl(dest, dest);
-        else
-            movl(imm, dest);
+        // Use the ImmWord version of mov to register, which has special
+        // optimizations. Casting to uint32_t here ensures that the value
+        // is zero-extended.
+        mov(ImmWord(uint32_t(imm.value)), dest);
     }
     void move32(const Imm32 &imm, const Operand &dest) {
         movl(imm, dest);
@@ -238,21 +238,43 @@ class MacroAssemblerX86Shared : public Assembler
     void jump(Register reg) {
         jmp(Operand(reg));
     }
+    void jump(const Address &addr) {
+        jmp(Operand(addr));
+    }
 
     void convertInt32ToDouble(const Register &src, const FloatRegister &dest) {
+        // cvtsi2sd and friends write only part of their output register, which
+        // causes slowdowns on out-of-order processors. Explicitly break
+        // dependencies with xorpd (and xorps elsewhere), which are handled
+        // specially in modern CPUs, for this purpose. See sections 8.14, 9.8,
+        // 10.8, 12.9, 13.16, 14.14, and 15.8 of Agner's Microarchitecture
+        // document.
+        zeroDouble(dest);
         cvtsi2sd(src, dest);
     }
     void convertInt32ToDouble(const Address &src, FloatRegister dest) {
+        convertInt32ToDouble(Operand(src), dest);
+    }
+    void convertInt32ToDouble(const Operand &src, FloatRegister dest) {
+        // Clear the output register first to break dependencies; see above;
+        zeroDouble(dest);
         cvtsi2sd(Operand(src), dest);
     }
     void convertInt32ToFloat32(const Register &src, const FloatRegister &dest) {
+        // Clear the output register first to break dependencies; see above;
+        zeroFloat32(dest);
         cvtsi2ss(src, dest);
     }
     void convertInt32ToFloat32(const Address &src, FloatRegister dest) {
-        cvtsi2ss(Operand(src), dest);
+        convertInt32ToFloat32(Operand(src), dest);
+    }
+    void convertInt32ToFloat32(const Operand &src, FloatRegister dest) {
+        // Clear the output register first to break dependencies; see above;
+        zeroFloat32(dest);
+        cvtsi2ss(src, dest);
     }
     Condition testDoubleTruthy(bool truthy, const FloatRegister &reg) {
-        xorpd(ScratchFloatReg, ScratchFloatReg);
+        zeroDouble(ScratchFloatReg);
         ucomisd(ScratchFloatReg, reg);
         return truthy ? NonZero : Zero;
     }
@@ -302,29 +324,51 @@ class MacroAssemblerX86Shared : public Assembler
         movl(src, Operand(dest));
     }
     void loadDouble(const Address &src, FloatRegister dest) {
-        movsd(Operand(src), dest);
+        movsd(src, dest);
     }
     void loadDouble(const BaseIndex &src, FloatRegister dest) {
-        movsd(Operand(src), dest);
+        movsd(src, dest);
     }
     void loadDouble(const Operand &src, FloatRegister dest) {
-        movsd(src, dest);
+        switch (src.kind()) {
+          case Operand::MEM_REG_DISP:
+            loadDouble(src.toAddress(), dest);
+            break;
+          case Operand::MEM_SCALE:
+            loadDouble(src.toBaseIndex(), dest);
+            break;
+          default:
+            MOZ_ASSUME_UNREACHABLE("unexpected operand kind");
+        }
     }
     void storeDouble(FloatRegister src, const Address &dest) {
-        movsd(src, Operand(dest));
+        movsd(src, dest);
     }
     void storeDouble(FloatRegister src, const BaseIndex &dest) {
-        movsd(src, Operand(dest));
+        movsd(src, dest);
     }
     void storeDouble(FloatRegister src, const Operand &dest) {
-        movsd(src, dest);
+        switch (dest.kind()) {
+          case Operand::MEM_REG_DISP:
+            storeDouble(src, dest.toAddress());
+            break;
+          case Operand::MEM_SCALE:
+            storeDouble(src, dest.toBaseIndex());
+            break;
+          default:
+            MOZ_ASSUME_UNREACHABLE("unexpected operand kind");
+        }
     }
     void moveDouble(FloatRegister src, FloatRegister dest) {
-        movsd(src, dest);
+        // Use movapd instead of movsd to avoid dependencies.
+        movapd(src, dest);
     }
     void zeroDouble(FloatRegister reg) {
         xorpd(reg, reg);
     }
+    void zeroFloat32(FloatRegister reg) {
+        xorps(reg, reg);
+    }
     void negateDouble(FloatRegister reg) {
         // From MacroAssemblerX86Shared::maybeInlineDouble
         pcmpeqw(ScratchFloatReg, ScratchFloatReg);
@@ -358,42 +402,61 @@ class MacroAssemblerX86Shared : public Assembler
     void convertDoubleToFloat(const FloatRegister &src, const FloatRegister &dest) {
         cvtsd2ss(src, dest);
     }
-    void loadFloatAsDouble(const Register &src, FloatRegister dest) {
+    void moveFloatAsDouble(const Register &src, FloatRegister dest) {
         movd(src, dest);
         cvtss2sd(dest, dest);
     }
     void loadFloatAsDouble(const Address &src, FloatRegister dest) {
-        movss(Operand(src), dest);
+        movss(src, dest);
         cvtss2sd(dest, dest);
     }
     void loadFloatAsDouble(const BaseIndex &src, FloatRegister dest) {
-        movss(Operand(src), dest);
+        movss(src, dest);
         cvtss2sd(dest, dest);
     }
     void loadFloatAsDouble(const Operand &src, FloatRegister dest) {
-        movss(src, dest);
+        loadFloat(src, dest);
         cvtss2sd(dest, dest);
     }
-    void loadFloat(const Register &src, FloatRegister dest) {
-        movss(Operand(src), dest);
-    }
     void loadFloat(const Address &src, FloatRegister dest) {
-        movss(Operand(src), dest);
+        movss(src, dest);
     }
     void loadFloat(const BaseIndex &src, FloatRegister dest) {
-        movss(Operand(src), dest);
+        movss(src, dest);
     }
     void loadFloat(const Operand &src, FloatRegister dest) {
-        movss(src, dest);
+        switch (src.kind()) {
+          case Operand::MEM_REG_DISP:
+            loadFloat(src.toAddress(), dest);
+            break;
+          case Operand::MEM_SCALE:
+            loadFloat(src.toBaseIndex(), dest);
+            break;
+          default:
+            MOZ_ASSUME_UNREACHABLE("unexpected operand kind");
+        }
     }
     void storeFloat(FloatRegister src, const Address &dest) {
-        movss(src, Operand(dest));
+        movss(src, dest);
     }
     void storeFloat(FloatRegister src, const BaseIndex &dest) {
-        movss(src, Operand(dest));
+        movss(src, dest);
+    }
+    void storeFloat(FloatRegister src, const Operand &dest) {
+        switch (dest.kind()) {
+          case Operand::MEM_REG_DISP:
+            storeFloat(src, dest.toAddress());
+            break;
+          case Operand::MEM_SCALE:
+            storeFloat(src, dest.toBaseIndex());
+            break;
+          default:
+            MOZ_ASSUME_UNREACHABLE("unexpected operand kind");
+        }
     }
     void moveFloat(FloatRegister src, FloatRegister dest) {
-        movss(src, dest);
+        // Use movaps instead of movss to avoid dependencies.
+        movaps(src, dest);
     }
 
     // Checks whether a double is representable as a 32-bit integer. If so, the
@@ -492,10 +555,21 @@ class MacroAssemblerX86Shared : public Assembler
         return false;
     }
 
+    bool maybeInlineFloat(float f, const FloatRegister &dest) {
+        uint32_t u = mozilla::BitwiseCast<uint32_t>(f);
+
+        // See comment above
+        if (u == 0) {
+            xorps(dest, dest);
+            return true;
+        }
+        return false;
+    }
+
     void convertBoolToInt32(Register source, Register dest) {
         // Note that C++ bool is only 1 byte, so zero extend it to clear the
         // higher-order bits.
-        movzxbl(source, dest);
+        movzbl(source, dest);
     }
 
     void emitSet(Assembler::Condition cond, const Register &dest,
@@ -504,15 +578,12 @@ class MacroAssemblerX86Shared : public Assembler
             // If the register we're defining is a single byte register,
             // take advantage of the setCC instruction
             setCC(cond, dest);
-            movzxbl(dest, dest);
+            movzbl(dest, dest);
 
             if (ifNaN != Assembler::NaN_HandledByCond) {
                 Label noNaN;
                 j(Assembler::NoParity, &noNaN);
-                if (ifNaN == Assembler::NaN_IsTrue)
-                    movl(Imm32(1), dest);
-                else
-                    xorl(dest, dest);
+                mov(ImmWord(ifNaN == Assembler::NaN_IsTrue), dest);
                 bind(&noNaN);
             }
         } else {
@@ -521,12 +592,16 @@ class MacroAssemblerX86Shared : public Assembler
 
             if (ifNaN == Assembler::NaN_IsFalse)
                 j(Assembler::Parity, &ifFalse);
+            // Note a subtlety here: FLAGS is live at this point, and the
+            // mov interface doesn't guarantee to preserve FLAGS. Use
+            // movl instead of mov, because the movl instruction
+            // preserves FLAGS.
             movl(Imm32(1), dest);
             j(cond, &end);
             if (ifNaN == Assembler::NaN_IsTrue)
                 j(Assembler::Parity, &end);
             bind(&ifFalse);
-            xorl(dest, dest);
+            mov(ImmWord(0), dest);
 
             bind(&end);
         }
@@ -563,13 +638,6 @@ class MacroAssemblerX86Shared : public Assembler
         return addCodeLabel(cl);
     }
 
-    bool buildOOLFakeExitFrame(void *fakeReturnAddr) {
-        uint32_t descriptor = MakeFrameDescriptor(framePushed(), IonFrame_OptimizedJS);
-        Push(Imm32(descriptor));
-        Push(ImmPtr(fakeReturnAddr));
-        return true;
-    }
-
     void callWithExitFrame(IonCode *target) {
         uint32_t descriptor = MakeFrameDescriptor(framePushed(), IonFrame_OptimizedJS);
         Push(Imm32(descriptor));
@@ -586,10 +654,18 @@ class MacroAssemblerX86Shared : public Assembler
     CodeOffsetLabel labelForPatch() {
         return CodeOffsetLabel(size());
     }
-    
+
     void abiret() {
         ret();
     }
+
+  protected:
+    bool buildOOLFakeExitFrame(void *fakeReturnAddr) {
+        uint32_t descriptor = MakeFrameDescriptor(framePushed(), IonFrame_OptimizedJS);
+        Push(Imm32(descriptor));
+        Push(ImmPtr(fakeReturnAddr));
+        return true;
+    }
 };
 
 } // namespace jit
