@@ -57,6 +57,16 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
     typedef HashMap<double, size_t, DefaultHasher<double>, SystemAllocPolicy> DoubleMap;
     DoubleMap doubleMap_;
 
+    struct Float {
+        float value;
+        NonAssertingLabel uses;
+        Float(float value) : value(value) {}
+    };
+    Vector<Float, 0, SystemAllocPolicy> floats_;
+
+    typedef HashMap<float, size_t, DefaultHasher<float>, SystemAllocPolicy> FloatMap;
+    FloatMap floatMap_;
+
     void setupABICall(uint32_t arg);
 
   protected:
@@ -71,7 +81,8 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
 
     enum Result {
         GENERAL,
-        DOUBLE
+        DOUBLE,
+        FLOAT
     };
 
     typedef MoveResolver::MoveOperand MoveOperand;
@@ -101,6 +112,10 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
     void call(ImmPtr target) {
         call(ImmWord(uintptr_t(target.value)));
     }
+    void call(AsmJSImmPtr target) {
+        mov(target, rax);
+        call(rax);
+    }
 
     // Refers to the upper 32 bits of a 64-bit Value operand.
     // On x86_64, the upper 32 bits do not necessarily only contain the type.
@@ -341,6 +356,48 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
         splitTag(src, ScratchReg);
         return testGCThing(cond, ScratchReg);
     }
+    Condition testPrimitive(Condition cond, const ValueOperand &src) {
+        splitTag(src, ScratchReg);
+        return testPrimitive(cond, ScratchReg);
+    }
+
+
+    Condition testUndefined(Condition cond, const Address &src) {
+        splitTag(src, ScratchReg);
+        return testUndefined(cond, ScratchReg);
+    }
+    Condition testInt32(Condition cond, const Address &src) {
+        splitTag(src, ScratchReg);
+        return testInt32(cond, ScratchReg);
+    }
+    Condition testBoolean(Condition cond, const Address &src) {
+        splitTag(src, ScratchReg);
+        return testBoolean(cond, ScratchReg);
+    }
+    Condition testDouble(Condition cond, const Address &src) {
+        splitTag(src, ScratchReg);
+        return testDouble(cond, ScratchReg);
+    }
+    Condition testNumber(Condition cond, const Address &src) {
+        splitTag(src, ScratchReg);
+        return testNumber(cond, ScratchReg);
+    }
+    Condition testNull(Condition cond, const Address &src) {
+        splitTag(src, ScratchReg);
+        return testNull(cond, ScratchReg);
+    }
+    Condition testString(Condition cond, const Address &src) {
+        splitTag(src, ScratchReg);
+        return testString(cond, ScratchReg);
+    }
+    Condition testObject(Condition cond, const Address &src) {
+        splitTag(src, ScratchReg);
+        return testObject(cond, ScratchReg);
+    }
+    Condition testPrimitive(Condition cond, const Address &src) {
+        splitTag(src, ScratchReg);
+        return testPrimitive(cond, ScratchReg);
+    }
     Condition testGCThing(Condition cond, const Address &src) {
         splitTag(src, ScratchReg);
         return testGCThing(cond, ScratchReg);
@@ -349,10 +406,6 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
         splitTag(src, ScratchReg);
         return testMagic(cond, ScratchReg);
     }
-    Condition testPrimitive(Condition cond, const ValueOperand &src) {
-        splitTag(src, ScratchReg);
-        return testPrimitive(cond, ScratchReg);
-    }
 
 
     Condition testUndefined(Condition cond, const BaseIndex &src) {
@@ -452,6 +505,7 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
     }
 
     Condition testNegativeZero(const FloatRegister &reg, const Register &scratch);
+    Condition testNegativeZeroFloat32(const FloatRegister &reg, const Register &scratch);
 
     /////////////////////////////////////////////////////////////////
     // Common interface.
@@ -535,6 +589,11 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
             branchPtr(cond, Operand(ScratchReg, 0x0), ptr, label);
         }
     }
+    void branchPtr(Condition cond, const AsmJSAbsoluteAddress &addr, const Register &ptr, Label *label) {
+        JS_ASSERT(ptr != ScratchReg);
+        mov(AsmJSImmPtr(addr.kind()), ScratchReg);
+        branchPtr(cond, Operand(ScratchReg, 0x0), ptr, label);
+    }
 
     void branchPrivatePtr(Condition cond, Address lhs, ImmPtr ptr, Label *label) {
         branchPtr(cond, lhs, ImmWord(uintptr_t(ptr.value) >> 1), label);
@@ -601,6 +660,9 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
     void movePtr(ImmPtr imm, Register dest) {
         mov(imm, dest);
     }
+    void movePtr(AsmJSImmPtr imm, const Register &dest) {
+        mov(imm, dest);
+    }
     void movePtr(ImmGCPtr imm, Register dest) {
         movq(imm, dest);
     }
@@ -627,7 +689,7 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
     }
     void storePtr(ImmWord imm, const Address &address) {
         if ((intptr_t)imm.value <= INT32_MAX && (intptr_t)imm.value >= INT32_MIN) {
-            mov(Imm32((int32_t)imm.value), Operand(address));
+            movq(Imm32((int32_t)imm.value), Operand(address));
         } else {
             mov(imm, ScratchReg);
             movq(ScratchReg, Operand(address));
@@ -749,6 +811,10 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
         cmpl(ToUpper32(operand), Imm32(Upper32Of(GetShiftedTag(JSVAL_TYPE_UNDEFINED))));
         j(cond, label);
     }
+    void branchTestUndefined(Condition cond, const Address &address, Label *label) {
+        JS_ASSERT(cond == Equal || cond == NotEqual);
+        branchTestUndefined(cond, Operand(address), label);
+    }
     void branchTestInt32(Condition cond, const Operand &operand, Label *label) {
         JS_ASSERT(cond == Equal || cond == NotEqual);
         cmpl(ToUpper32(operand), Imm32(Upper32Of(GetShiftedTag(JSVAL_TYPE_INT32))));
@@ -880,14 +946,14 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
         unboxInt32(Operand(src), dest);
     }
     void unboxDouble(const Address &src, const FloatRegister &dest) {
-        movsd(Operand(src), dest);
+        loadDouble(Operand(src), dest);
     }
 
     void unboxArgObjMagic(const ValueOperand &src, const Register &dest) {
         unboxArgObjMagic(Operand(src.valueReg()), dest);
     }
     void unboxArgObjMagic(const Operand &src, const Register &dest) {
-        xorq(dest, dest);
+        mov(ImmWord(0), dest);
     }
     void unboxArgObjMagic(const Address &src, const Register &dest) {
         unboxArgObjMagic(Operand(src), dest);
@@ -922,18 +988,21 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
     // Unbox any non-double value into dest. Prefer unboxInt32 or unboxBoolean
     // instead if the source type is known.
     void unboxNonDouble(const ValueOperand &src, const Register &dest) {
+        // In a non-trivial coupling, we're not permitted to use ScratchReg when
+        // src and dest are different registers, because of how extractObject is
+        // implemented.
         if (src.valueReg() == dest) {
-            movq(ImmWord(JSVAL_PAYLOAD_MASK), ScratchReg);
+            mov(ImmWord(JSVAL_PAYLOAD_MASK), ScratchReg);
             andq(ScratchReg, dest);
         } else {
-            movq(ImmWord(JSVAL_PAYLOAD_MASK), dest);
+            mov(ImmWord(JSVAL_PAYLOAD_MASK), dest);
             andq(src.valueReg(), dest);
         }
     }
     void unboxNonDouble(const Operand &src, const Register &dest) {
         // Explicitly permits |dest| to be used in |src|.
         JS_ASSERT(dest != ScratchReg);
-        movq(ImmWord(JSVAL_PAYLOAD_MASK), ScratchReg);
+        mov(ImmWord(JSVAL_PAYLOAD_MASK), ScratchReg);
         movq(src, dest);
         andq(ScratchReg, dest);
     }
@@ -950,6 +1019,8 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
     Register extractObject(const Address &address, Register scratch) {
         JS_ASSERT(scratch != ScratchReg);
         loadPtr(address, ScratchReg);
+        // We have a special coupling with unboxObject. As long as the registers
+        // aren't equal, it doesn't use ScratchReg.
         unboxObject(ValueOperand(ScratchReg), scratch);
         return scratch;
     }
@@ -984,7 +1055,7 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
         if (dest.isFloat()) {
             Label notInt32, end;
             branchTestInt32(Assembler::NotEqual, src, &notInt32);
-            cvtsi2sd(src.valueReg(), dest.fpu());
+            convertInt32ToDouble(src.valueReg(), dest.fpu());
             jump(&end);
             bind(&notInt32);
             unboxDouble(src, dest.fpu());
@@ -996,43 +1067,40 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
 
     // These two functions use the low 32-bits of the full value register.
     void boolValueToDouble(const ValueOperand &operand, const FloatRegister &dest) {
-        cvtsi2sd(operand.valueReg(), dest);
+        convertInt32ToDouble(operand.valueReg(), dest);
     }
     void int32ValueToDouble(const ValueOperand &operand, const FloatRegister &dest) {
-        cvtsi2sd(operand.valueReg(), dest);
+        convertInt32ToDouble(operand.valueReg(), dest);
     }
 
     void boolValueToFloat32(const ValueOperand &operand, const FloatRegister &dest) {
-        cvtsi2ss(operand.valueReg(), dest);
+        convertInt32ToFloat32(operand.valueReg(), dest);
     }
     void int32ValueToFloat32(const ValueOperand &operand, const FloatRegister &dest) {
-        cvtsi2ss(operand.valueReg(), dest);
+        convertInt32ToFloat32(operand.valueReg(), dest);
     }
 
     void loadConstantDouble(double d, const FloatRegister &dest);
-    void loadConstantFloat32(float f, const FloatRegister &dest) {
-        union FloatPun {
-            uint32_t u;
-            float f;
-        } pun;
-        pun.f = f;
-        mov(ImmWord(pun.u), ScratchReg);
-        movq(ScratchReg, dest);
-    }
-    void loadStaticDouble(const double *dp, const FloatRegister &dest) {
-        loadConstantDouble(*dp, dest);
-    }
-    void loadStaticFloat32(const float *fp, const FloatRegister &dest) {
-        loadConstantFloat32(*fp, dest);
-    }
+    void loadConstantFloat32(float f, const FloatRegister &dest);
 
     void branchTruncateDouble(const FloatRegister &src, const Register &dest, Label *fail) {
-        const uint64_t IndefiniteIntegerValue = 0x8000000000000000;
-        JS_ASSERT(dest != ScratchReg);
         cvttsd2sq(src, dest);
-        movq(ImmWord(IndefiniteIntegerValue), ScratchReg);
-        cmpq(dest, ScratchReg);
-        j(Assembler::Equal, fail);
+
+        // cvttsd2sq returns 0x8000000000000000 on failure. Test for it by
+        // subtracting 1 and testing overflow (this avoids the need to
+        // materialize that value in a register).
+        cmpq(dest, Imm32(1));
+        j(Assembler::Overflow, fail);
+
+        movl(dest, dest); // Zero upper 32-bits.
+    }
+    void branchTruncateFloat32(const FloatRegister &src, const Register &dest, Label *fail) {
+        cvttss2sq(src, dest);
+
+        // Same trick as for Doubles
+        cmpq(dest, Imm32(1));
+        j(Assembler::Overflow, fail);
+
         movl(dest, dest); // Zero upper 32-bits.
     }
 
@@ -1059,10 +1127,10 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
     void loadInt32OrDouble(const Operand &operand, const FloatRegister &dest) {
         Label notInt32, end;
         branchTestInt32(Assembler::NotEqual, operand, &notInt32);
-        cvtsi2sd(operand, dest);
+        convertInt32ToDouble(operand, dest);
         jump(&end);
         bind(&notInt32);
-        movsd(operand, dest);
+        loadDouble(operand, dest);
         bind(&end);
     }
 
@@ -1077,13 +1145,17 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
     }
 
     void loadInstructionPointerAfterCall(const Register &dest) {
-        movq(Operand(StackPointer, 0x0), dest);
+        loadPtr(Address(StackPointer, 0x0), dest);
     }
 
     void convertUInt32ToDouble(const Register &src, const FloatRegister &dest) {
         cvtsq2sd(src, dest);
     }
 
+    void convertUInt32ToFloat32(const Register &src, const FloatRegister &dest) {
+        cvtsq2ss(src, dest);
+    }
+
     void inc64(AbsoluteAddress dest) {
         if (JSC::X86Assembler::isAddressImmediate(dest.addr)) {
             addPtr(Imm32(1), Operand(dest));
@@ -1141,6 +1213,7 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
   public:
     // Emits a call to a C/C++ function, resolving all argument moves.
     void callWithABI(void *fun, Result result = GENERAL);
+    void callWithABI(AsmJSImmPtr imm, Result result = GENERAL);
     void callWithABI(Address fun, Result result = GENERAL);
 
     void handleFailureWithHandler(void *handler);
@@ -1154,8 +1227,8 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
     // Save an exit frame (which must be aligned to the stack pointer) to
     // ThreadData::ionTop of the main thread.
     void linkExitFrame() {
-        mov(ImmPtr(GetIonContext()->runtime), ScratchReg);
-        mov(StackPointer, Operand(ScratchReg, offsetof(JSRuntime, mainThread.ionTop)));
+        storePtr(StackPointer,
+                 AbsoluteAddress(GetIonContext()->runtime->addressOfIonTop()));
     }
 
     void callWithExitFrame(IonCode *target, Register dynStack) {
@@ -1168,7 +1241,7 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
     // Save an exit frame to the thread data of the current thread, given a
     // register that holds a PerThreadData *.
     void linkParallelExitFrame(const Register &pt) {
-        mov(StackPointer, Operand(pt, offsetof(PerThreadData, ionTop)));
+        storePtr(StackPointer, Address(pt, offsetof(PerThreadData, ionTop)));
     }
 
     void enterOsr(Register calleeToken, Register code) {
@@ -1180,10 +1253,10 @@ class MacroAssemblerX64 : public MacroAssemblerX86Shared
     }
 
     // See CodeGeneratorX64 calls to noteAsmJSGlobalAccess.
-    void patchAsmJSGlobalAccess(unsigned offset, uint8_t *code, uint8_t *globalData,
+    void patchAsmJSGlobalAccess(CodeOffsetLabel patchAt, uint8_t *code, uint8_t *globalData,
                                 unsigned globalDataOffset)
     {
-        uint8_t *nextInsn = code + offset;
+        uint8_t *nextInsn = code + patchAt.offset();
         JS_ASSERT(nextInsn <= globalData);
         uint8_t *target = globalData + globalDataOffset;
         ((int32_t *)nextInsn)[-1] = target - nextInsn;
