@@ -11,6 +11,7 @@
 #include "jsnum.h"
 
 #include "jit/ExecutionModeInlines.h"
+#include "jit/IonCaches.h"
 #include "jit/MIR.h"
 #include "jit/MIRGraph.h"
 #include "vm/Shape.h"
@@ -25,6 +26,10 @@ using namespace js::jit;
 using mozilla::DebugOnly;
 using mozilla::DoubleExponentBias;
 using mozilla::DoubleExponentShift;
+using mozilla::FloatExponentBias;
+using mozilla::FloatExponentShift;
+using mozilla::FloatExponentBits;
+using JS::GenericNaN;
 
 CodeGeneratorX86::CodeGeneratorX86(MIRGenerator *gen, LIRGraph *graph, MacroAssembler *masm)
   : CodeGeneratorX86Shared(gen, graph, masm)
@@ -91,17 +96,6 @@ CodeGeneratorX86::visitValue(LValue *value)
     return true;
 }
 
-bool
-CodeGeneratorX86::visitOsrValue(LOsrValue *value)
-{
-    const LAllocation *frame   = value->getOperand(0);
-    const ValueOperand out     = ToOutValue(value);
-    const ptrdiff_t frameOffset = value->mir()->frameOffset();
-
-    masm.loadValue(Operand(ToRegister(frame), frameOffset), out);
-    return true;
-}
-
 bool
 CodeGeneratorX86::visitBox(LBox *box)
 {
@@ -113,7 +107,7 @@ CodeGeneratorX86::visitBox(LBox *box)
     // On x86, the input operand and the output payload have the same
     // virtual register. All that needs to be written is the type tag for
     // the type definition.
-    masm.movl(Imm32(MIRTypeToTag(box->type())), ToRegister(type));
+    masm.mov(ImmWord(MIRTypeToTag(box->type())), ToRegister(type));
     return true;
 }
 
@@ -154,7 +148,7 @@ CodeGeneratorX86::visitLoadSlotV(LLoadSlotV *load)
     Register base = ToRegister(load->input());
     int32_t offset = load->mir()->slot() * sizeof(js::Value);
 
-    masm.loadValue(Operand(base, offset), out);
+    masm.loadValue(Address(base, offset), out);
     return true;
 }
 
@@ -167,7 +161,7 @@ CodeGeneratorX86::visitLoadSlotT(LLoadSlotT *load)
     if (load->mir()->type() == MIRType_Double)
         masm.loadInt32OrDouble(Operand(base, offset), ToFloatRegister(load->output()));
     else
-        masm.movl(Operand(base, offset + NUNBOX32_PAYLOAD_OFFSET), ToRegister(load->output()));
+        masm.load32(Address(base, offset + NUNBOX32_PAYLOAD_OFFSET), ToRegister(load->output()));
     return true;
 }
 
@@ -184,7 +178,7 @@ CodeGeneratorX86::visitStoreSlotT(LStoreSlotT *store)
         emitPreBarrier(Address(base, offset), store->mir()->slotType());
 
     if (valueType == MIRType_Double) {
-        masm.movsd(ToFloatRegister(value), Operand(base, offset));
+        masm.storeDouble(ToFloatRegister(value), Operand(base, offset));
         return true;
     }
 
@@ -236,7 +230,7 @@ CodeGeneratorX86::storeElementTyped(const LAllocation *value, MIRType valueType,
     Operand dest = createArrayElementOperand(elements, index);
 
     if (valueType == MIRType_Double) {
-        masm.movsd(ToFloatRegister(value), dest);
+        masm.storeDouble(ToFloatRegister(value), dest);
         return;
     }
 
@@ -277,8 +271,7 @@ CodeGeneratorX86::visitInterruptCheck(LInterruptCheck *lir)
     if (!ool)
         return false;
 
-    void *interrupt = (void*)&GetIonContext()->runtime->interrupt;
-    masm.cmpl(Operand(interrupt), Imm32(0));
+    masm.cmpl(Operand(AbsoluteAddress(GetIonContext()->runtime->addressOfInterrupt())), Imm32(0));
     masm.j(Assembler::NonZero, ool->entry());
     masm.bind(ool->rejoin());
     return true;
@@ -317,7 +310,7 @@ CodeGeneratorX86::visitCompareB(LCompareB *lir)
 bool
 CodeGeneratorX86::visitCompareBAndBranch(LCompareBAndBranch *lir)
 {
-    MCompare *mir = lir->mir();
+    MCompare *mir = lir->cmpMir();
     const ValueOperand lhs = ToValue(lir, LCompareBAndBranch::Lhs);
     const LAllocation *rhs = lir->rhs();
 
@@ -365,7 +358,7 @@ CodeGeneratorX86::visitCompareV(LCompareV *lir)
 bool
 CodeGeneratorX86::visitCompareVAndBranch(LCompareVAndBranch *lir)
 {
-    MCompare *mir = lir->mir();
+    MCompare *mir = lir->cmpMir();
     Assembler::Condition cond = JSOpToCondition(mir->compareType(), mir->jsop());
     const ValueOperand lhs = ToValue(lir, LCompareVAndBranch::LhsInput);
     const ValueOperand rhs = ToValue(lir, LCompareVAndBranch::RhsInput);
@@ -384,7 +377,7 @@ CodeGeneratorX86::visitCompareVAndBranch(LCompareVAndBranch *lir)
 }
 
 bool
-CodeGeneratorX86::visitUInt32ToDouble(LUInt32ToDouble *lir)
+CodeGeneratorX86::visitAsmJSUInt32ToDouble(LAsmJSUInt32ToDouble *lir)
 {
     Register input = ToRegister(lir->input());
     Register temp = ToRegister(lir->temp());
@@ -397,6 +390,21 @@ CodeGeneratorX86::visitUInt32ToDouble(LUInt32ToDouble *lir)
     return true;
 }
 
+bool
+CodeGeneratorX86::visitAsmJSUInt32ToFloat32(LAsmJSUInt32ToFloat32 *lir)
+{
+    Register input = ToRegister(lir->input());
+    Register temp = ToRegister(lir->temp());
+    FloatRegister output = ToFloatRegister(lir->output());
+
+    if (input != temp)
+        masm.mov(input, temp);
+
+    // Beware: convertUInt32ToFloat32 clobbers input.
+    masm.convertUInt32ToFloat32(temp, output);
+    return true;
+}
+
 // Load a NaN or zero into a register for an out of bounds AsmJS or static
 // typed array load.
 class jit::OutOfLineLoadTypedArrayOutOfBounds : public OutOfLineCodeBase<CodeGeneratorX86>
@@ -454,9 +462,9 @@ CodeGeneratorX86::visitLoadTypedArrayElementStatic(LLoadTypedArrayElementStatic
     Register ptr = ToRegister(ins->ptr());
     const LDefinition *out = ins->output();
 
-    OutOfLineLoadTypedArrayOutOfBounds *ool = NULL;
+    OutOfLineLoadTypedArrayOutOfBounds *ool = nullptr;
     if (!mir->fallible()) {
-        ool = new OutOfLineLoadTypedArrayOutOfBounds(ToAnyRegister(out));
+        ool = new(alloc()) OutOfLineLoadTypedArrayOutOfBounds(ToAnyRegister(out));
         if (!addOutOfLineCode(ool))
             return false;
     }
@@ -494,10 +502,11 @@ CodeGeneratorX86::visitAsmJSLoadHeap(LAsmJSLoadHeap *ins)
     const LDefinition *out = ins->output();
 
     if (ptr->isConstant()) {
-        JS_ASSERT(mir->skipBoundsCheck());
-        int32_t ptrImm = ptr->toConstant()->toInt32();
-        JS_ASSERT(ptrImm >= 0);
-        AbsoluteAddress srcAddr((void *) ptrImm);
+        // The constant displacement still needs to be added to the as-yet-unknown
+        // base address of the heap. For now, embed the displacement as an
+        // immediate in the instruction. This displacement will fixed up when the
+        // base address is known during dynamic linking (AsmJSModule::initHeap).
+        PatchedAbsoluteAddress srcAddr((void *) ptr->toConstant()->toInt32());
         return loadViewTypeElement(vt, srcAddr, out);
     }
 
@@ -507,7 +516,7 @@ CodeGeneratorX86::visitAsmJSLoadHeap(LAsmJSLoadHeap *ins)
     if (mir->skipBoundsCheck())
         return loadViewTypeElement(vt, srcAddr, out);
 
-    OutOfLineLoadTypedArrayOutOfBounds *ool = new OutOfLineLoadTypedArrayOutOfBounds(ToAnyRegister(out));
+    OutOfLineLoadTypedArrayOutOfBounds *ool = new(alloc()) OutOfLineLoadTypedArrayOutOfBounds(ToAnyRegister(out));
     if (!addOutOfLineCode(ool))
         return false;
 
@@ -534,10 +543,10 @@ bool
 CodeGeneratorX86::visitOutOfLineLoadTypedArrayOutOfBounds(OutOfLineLoadTypedArrayOutOfBounds *ool)
 {
     if (ool->dest().isFloat()) {
-        masm.movsd(&js_NaN, ool->dest().fpu());
+        masm.loadConstantDouble(GenericNaN(), ool->dest().fpu());
     } else {
         Register destReg = ool->dest().gpr();
-        masm.xorl(destReg, destReg);
+        masm.mov(ImmWord(0), destReg);
     }
     masm.jmp(ool->rejoin());
     return true;
@@ -594,9 +603,8 @@ CodeGeneratorX86::visitStoreTypedArrayElementStatic(LStoreTypedArrayElementStati
 
     Address dstAddr(ptr, (int32_t) mir->base());
     if (vt == ArrayBufferView::TYPE_FLOAT32) {
-        JS_ASSERT(mir->value()->type() == MIRType_Double);
-        masm.cvtsd2ss(ToFloatRegister(value), ScratchFloatReg);
-        masm.movssWithPatch(ScratchFloatReg, dstAddr);
+        JS_ASSERT(mir->value()->type() == MIRType_Float32);
+        masm.movssWithPatch(ToFloatRegister(value), dstAddr);
         masm.bind(&rejoin);
         return true;
     }
@@ -614,10 +622,11 @@ CodeGeneratorX86::visitAsmJSStoreHeap(LAsmJSStoreHeap *ins)
     const LAllocation *ptr = ins->ptr();
 
     if (ptr->isConstant()) {
-        JS_ASSERT(mir->skipBoundsCheck());
-        int32_t ptrImm = ptr->toConstant()->toInt32();
-        JS_ASSERT(ptrImm >= 0);
-        AbsoluteAddress dstAddr((void *) ptrImm);
+        // The constant displacement still needs to be added to the as-yet-unknown
+        // base address of the heap. For now, embed the displacement as an
+        // immediate in the instruction. This displacement will fixed up when the
+        // base address is known during dynamic linking (AsmJSModule::initHeap).
+        PatchedAbsoluteAddress dstAddr((void *) ptr->toConstant()->toInt32());
         return storeViewTypeElement(vt, value, dstAddr);
     }
 
@@ -653,9 +662,9 @@ CodeGeneratorX86::visitAsmJSLoadGlobalVar(LAsmJSLoadGlobalVar *ins)
 
     CodeOffsetLabel label;
     if (mir->type() == MIRType_Int32)
-        label = masm.movlWithPatch(NULL, ToRegister(ins->output()));
+        label = masm.movlWithPatch(PatchedAbsoluteAddress(), ToRegister(ins->output()));
     else
-        label = masm.movsdWithPatch(NULL, ToFloatRegister(ins->output()));
+        label = masm.movsdWithPatch(PatchedAbsoluteAddress(), ToFloatRegister(ins->output()));
 
     return gen->noteGlobalAccess(label.offset(), mir->globalDataOffset());
 }
@@ -670,9 +679,9 @@ CodeGeneratorX86::visitAsmJSStoreGlobalVar(LAsmJSStoreGlobalVar *ins)
 
     CodeOffsetLabel label;
     if (type == MIRType_Int32)
-        label = masm.movlWithPatch(ToRegister(ins->value()), NULL);
+        label = masm.movlWithPatch(ToRegister(ins->value()), PatchedAbsoluteAddress());
     else
-        label = masm.movsdWithPatch(ToFloatRegister(ins->value()), NULL);
+        label = masm.movsdWithPatch(ToFloatRegister(ins->value()), PatchedAbsoluteAddress());
 
     return gen->noteGlobalAccess(label.offset(), mir->globalDataOffset());
 }
@@ -684,7 +693,7 @@ CodeGeneratorX86::visitAsmJSLoadFuncPtr(LAsmJSLoadFuncPtr *ins)
 
     Register index = ToRegister(ins->index());
     Register out = ToRegister(ins->output());
-    CodeOffsetLabel label = masm.movlWithPatch(NULL, index, TimesFour, out);
+    CodeOffsetLabel label = masm.movlWithPatch(PatchedAbsoluteAddress(), index, TimesFour, out);
 
     return gen->noteGlobalAccess(label.offset(), mir->globalDataOffset());
 }
@@ -695,7 +704,7 @@ CodeGeneratorX86::visitAsmJSLoadFFIFunc(LAsmJSLoadFFIFunc *ins)
     MAsmJSLoadFFIFunc *mir = ins->mir();
 
     Register out = ToRegister(ins->output());
-    CodeOffsetLabel label = masm.movlWithPatch(NULL, out);
+    CodeOffsetLabel label = masm.movlWithPatch(PatchedAbsoluteAddress(), out);
 
     return gen->noteGlobalAccess(label.offset(), mir->globalDataOffset());
 }
@@ -709,7 +718,7 @@ CodeGeneratorX86::postAsmJSCall(LAsmJSCall *lir)
 
     masm.reserveStack(sizeof(double));
     masm.fstp(Operand(esp, 0));
-    masm.movsd(Operand(esp, 0), ReturnFloatReg);
+    masm.loadDouble(Operand(esp, 0), ReturnFloatReg);
     masm.freeStack(sizeof(double));
 }
 
@@ -745,6 +754,29 @@ GetElementParIC::initializeAddCacheState(LInstruction *ins, AddCacheState *addSt
         addState->dispatchScratch = ToRegister(ins->toGetElementCacheT()->temp());
 }
 
+void
+SetPropertyParIC::initializeAddCacheState(LInstruction *ins, AddCacheState *addState)
+{
+    // We don't have an output register to reuse, so we always need a temp.
+    JS_ASSERT(ins->isSetPropertyCacheV() || ins->isSetPropertyCacheT());
+    if (ins->isSetPropertyCacheV())
+        addState->dispatchScratch = ToRegister(ins->toSetPropertyCacheV()->tempForDispatchCache());
+    else
+        addState->dispatchScratch = ToRegister(ins->toSetPropertyCacheT()->tempForDispatchCache());
+}
+
+void
+SetElementParIC::initializeAddCacheState(LInstruction *ins, AddCacheState *addState)
+{
+    // We don't have an output register to reuse, but luckily SetElementCache
+    // already needs a temp.
+    JS_ASSERT(ins->isSetElementCacheV() || ins->isSetElementCacheT());
+    if (ins->isSetElementCacheV())
+        addState->dispatchScratch = ToRegister(ins->toSetElementCacheV()->temp());
+    else
+        addState->dispatchScratch = ToRegister(ins->toSetElementCacheT()->temp());
+}
+
 namespace js {
 namespace jit {
 
@@ -765,6 +797,23 @@ class OutOfLineTruncate : public OutOfLineCodeBase<CodeGeneratorX86>
     }
 };
 
+class OutOfLineTruncateFloat32 : public OutOfLineCodeBase<CodeGeneratorX86>
+{
+    LTruncateFToInt32 *ins_;
+
+  public:
+    OutOfLineTruncateFloat32(LTruncateFToInt32 *ins)
+      : ins_(ins)
+    { }
+
+    bool accept(CodeGeneratorX86 *codegen) {
+        return codegen->visitOutOfLineTruncateFloat32(this);
+    }
+    LTruncateFToInt32 *ins() const {
+        return ins_;
+    }
+};
+
 } // namespace jit
 } // namespace js
 
@@ -774,7 +823,7 @@ CodeGeneratorX86::visitTruncateDToInt32(LTruncateDToInt32 *ins)
     FloatRegister input = ToFloatRegister(ins->input());
     Register output = ToRegister(ins->output());
 
-    OutOfLineTruncate *ool = new OutOfLineTruncate(ins);
+    OutOfLineTruncate *ool = new(alloc()) OutOfLineTruncate(ins);
     if (!addOutOfLineCode(ool))
         return false;
 
@@ -783,6 +832,21 @@ CodeGeneratorX86::visitTruncateDToInt32(LTruncateDToInt32 *ins)
     return true;
 }
 
+bool
+CodeGeneratorX86::visitTruncateFToInt32(LTruncateFToInt32 *ins)
+{
+    FloatRegister input = ToFloatRegister(ins->input());
+    Register output = ToRegister(ins->output());
+
+    OutOfLineTruncateFloat32 *ool = new(alloc()) OutOfLineTruncateFloat32(ins);
+    if (!addOutOfLineCode(ool))
+        return false;
+
+    masm.branchTruncateFloat32(input, output, ool->entry());
+    masm.bind(ool->rejoin());
+    return true;
+}
+
 bool
 CodeGeneratorX86::visitOutOfLineTruncate(OutOfLineTruncate *ool)
 {
@@ -795,7 +859,7 @@ CodeGeneratorX86::visitOutOfLineTruncate(OutOfLineTruncate *ool)
     if (Assembler::HasSSE3()) {
         // Push double.
         masm.subl(Imm32(sizeof(double)), esp);
-        masm.movsd(input, Operand(esp, 0));
+        masm.storeDouble(input, Operand(esp, 0));
 
         static const uint32_t EXPONENT_MASK = 0x7ff00000;
         static const uint32_t EXPONENT_SHIFT = DoubleExponentShift - 32;
@@ -803,7 +867,7 @@ CodeGeneratorX86::visitOutOfLineTruncate(OutOfLineTruncate *ool)
 
         // Check exponent to avoid fp exceptions.
         Label failPopDouble;
-        masm.movl(Operand(esp, 4), output);
+        masm.load32(Address(esp, 4), output);
         masm.and32(Imm32(EXPONENT_MASK), output);
         masm.branch32(Assembler::GreaterThanOrEqual, output, Imm32(TOO_BIG_EXPONENT), &failPopDouble);
 
@@ -812,7 +876,7 @@ CodeGeneratorX86::visitOutOfLineTruncate(OutOfLineTruncate *ool)
         masm.fisttp(Operand(esp, 0));
 
         // Load low word, pop double and jump back.
-        masm.movl(Operand(esp, 0), output);
+        masm.load32(Address(esp, 0), output);
         masm.addl(Imm32(sizeof(double)), esp);
         masm.jump(ool->rejoin());
 
@@ -834,14 +898,12 @@ CodeGeneratorX86::visitOutOfLineTruncate(OutOfLineTruncate *ool)
             Label positive;
             masm.j(Assembler::Above, &positive);
 
-            static const double shiftNeg = 4294967296.0;
-            masm.loadStaticDouble(&shiftNeg, temp);
+            masm.loadConstantDouble(4294967296.0, temp);
             Label skip;
             masm.jmp(&skip);
 
             masm.bind(&positive);
-            static const double shiftPos = -4294967296.0;
-            masm.loadStaticDouble(&shiftPos, temp);
+            masm.loadConstantDouble(-4294967296.0, temp);
             masm.bind(&skip);
         }
 
@@ -860,7 +922,10 @@ CodeGeneratorX86::visitOutOfLineTruncate(OutOfLineTruncate *ool)
 
         masm.setupUnalignedABICall(1, output);
         masm.passABIArg(input);
-        masm.callWithABI(JS_FUNC_TO_DATA_PTR(void *, js::ToInt32));
+        if (gen->compilingAsmJS())
+            masm.callWithABI(AsmJSImm_ToInt32);
+        else
+            masm.callWithABI(JS_FUNC_TO_DATA_PTR(void *, js::ToInt32));
         masm.storeCallResult(output);
 
         restoreVolatile(output);
@@ -869,3 +934,92 @@ CodeGeneratorX86::visitOutOfLineTruncate(OutOfLineTruncate *ool)
     masm.jump(ool->rejoin());
     return true;
 }
+
+bool
+CodeGeneratorX86::visitOutOfLineTruncateFloat32(OutOfLineTruncateFloat32 *ool)
+{
+    LTruncateFToInt32 *ins = ool->ins();
+    FloatRegister input = ToFloatRegister(ins->input());
+    Register output = ToRegister(ins->output());
+
+    Label fail;
+
+    if (Assembler::HasSSE3()) {
+        // Push float32, but subtracts 64 bits so that the value popped by fisttp fits
+        masm.subl(Imm32(sizeof(uint64_t)), esp);
+        masm.storeFloat(input, Operand(esp, 0));
+
+        static const uint32_t EXPONENT_MASK = FloatExponentBits;
+        static const uint32_t EXPONENT_SHIFT = FloatExponentShift;
+        // Integers are still 64 bits long, so we can still test for an exponent > 63.
+        static const uint32_t TOO_BIG_EXPONENT = (FloatExponentBias + 63) << EXPONENT_SHIFT;
+
+        // Check exponent to avoid fp exceptions.
+        Label failPopFloat;
+        masm.movl(Operand(esp, 0), output);
+        masm.and32(Imm32(EXPONENT_MASK), output);
+        masm.branch32(Assembler::GreaterThanOrEqual, output, Imm32(TOO_BIG_EXPONENT), &failPopFloat);
+
+        // Load float, perform 32-bit truncation.
+        masm.fld32(Operand(esp, 0));
+        masm.fisttp(Operand(esp, 0));
+
+        // Load low word, pop 64bits and jump back.
+        masm.movl(Operand(esp, 0), output);
+        masm.addl(Imm32(sizeof(uint64_t)), esp);
+        masm.jump(ool->rejoin());
+
+        masm.bind(&failPopFloat);
+        masm.addl(Imm32(sizeof(uint64_t)), esp);
+        masm.jump(&fail);
+    } else {
+        FloatRegister temp = ToFloatRegister(ins->tempFloat());
+
+        // Try to convert float32 representing integers within 2^32 of a signed
+        // integer, by adding/subtracting 2^32 and then trying to convert to int32.
+        // This has to be an exact conversion, as otherwise the truncation works
+        // incorrectly on the modified value.
+        masm.xorps(ScratchFloatReg, ScratchFloatReg);
+        masm.ucomiss(input, ScratchFloatReg);
+        masm.j(Assembler::Parity, &fail);
+
+        {
+            Label positive;
+            masm.j(Assembler::Above, &positive);
+
+            masm.loadConstantFloat32(4294967296.f, temp);
+            Label skip;
+            masm.jmp(&skip);
+
+            masm.bind(&positive);
+            masm.loadConstantFloat32(-4294967296.f, temp);
+            masm.bind(&skip);
+        }
+
+        masm.addss(input, temp);
+        masm.cvttss2si(temp, output);
+        masm.cvtsi2ss(output, ScratchFloatReg);
+
+        masm.ucomiss(temp, ScratchFloatReg);
+        masm.j(Assembler::Parity, &fail);
+        masm.j(Assembler::Equal, ool->rejoin());
+    }
+
+    masm.bind(&fail);
+    {
+        saveVolatile(output);
+
+        masm.push(input);
+        masm.setupUnalignedABICall(1, output);
+        masm.cvtss2sd(input, input);
+        masm.passABIArg(input);
+        masm.callWithABI(JS_FUNC_TO_DATA_PTR(void *, js::ToInt32));
+        masm.storeCallResult(output);
+        masm.pop(input);
+
+        restoreVolatile(output);
+    }
+
+    masm.jump(ool->rejoin());
+    return true;
+}
