@@ -15,6 +15,7 @@
 #include "jscompartment.h"
 
 #include "jsinferinlines.h"
+#include "jsobjinlines.h"
 
 using namespace js;
 using namespace js::analyze;
@@ -36,7 +37,7 @@ analyze::PrintBytecode(JSContext *cx, HandleScript script, jsbytecode *pc)
     Sprinter sprinter(cx);
     if (!sprinter.init())
         return;
-    js_Disassemble1(cx, script, pc, pc - script->code, true, &sprinter);
+    js_Disassemble1(cx, script, pc, script->pcToOffset(pc), true, &sprinter);
     fprintf(stderr, "%s", sprinter.string());
 }
 #endif
@@ -50,7 +51,7 @@ ScriptAnalysis::addJump(JSContext *cx, unsigned offset,
                         unsigned *currentOffset, unsigned *forwardJump, unsigned *forwardLoop,
                         unsigned stackDepth)
 {
-    JS_ASSERT(offset < script_->length);
+    JS_ASSERT(offset < script_->length());
 
     Bytecode *&code = codeArray[offset];
     if (!code) {
@@ -96,7 +97,7 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
 
     numSlots = TotalSlots(script_);
 
-    unsigned length = script_->length;
+    unsigned length = script_->length();
     codeArray = alloc.newArray<Bytecode*>(length);
     escapedSlots = alloc.newArray<bool>(numSlots);
 
@@ -132,12 +133,6 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
             escapedSlots[LocalSlot(script_, bi.frameIndex())] = allVarsAliased || bi->aliased();
     }
 
-    bool heavyweight = script_->function() && script_->function()->isHeavyweight();
-
-    isIonInlineable = true;
-    if (heavyweight || cx->compartment()->debugMode())
-        isIonInlineable = false;
-
     canTrackVars = true;
 
     /*
@@ -179,7 +174,7 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
             forwardCatch = 0;
 
         Bytecode *code = maybeCode(offset);
-        jsbytecode *pc = script_->code + offset;
+        jsbytecode *pc = script_->offsetToPC(offset);
 
         JSOp op = (JSOp)*pc;
         JS_ASSERT(op < JSOP_LIMIT);
@@ -214,10 +209,8 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
 
         code->analyzed = true;
 
-        if (script_->hasBreakpointsAt(pc)) {
+        if (script_->hasBreakpointsAt(pc))
             canTrackVars = false;
-            isIonInlineable = false;
-        }
 
         unsigned stackDepth = code->stackDepth;
 
@@ -234,15 +227,10 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
         switch (op) {
 
           case JSOP_RETURN:
-          case JSOP_STOP:
+          case JSOP_RETRVAL:
             numReturnSites_++;
             break;
 
-          case JSOP_SETRVAL:
-          case JSOP_POPV:
-            isIonInlineable = false;
-            break;
-
           case JSOP_NAME:
           case JSOP_CALLNAME:
           case JSOP_BINDNAME:
@@ -261,28 +249,16 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
           case JSOP_SETCONST:
             usesScopeChain_ = true; // Requires access to VarObj via ScopeChain.
             canTrackVars = false;
-            isIonInlineable = false;
             break;
 
           case JSOP_EVAL:
-            canTrackVars = false;
-            isIonInlineable = false;
-            break;
-
+          case JSOP_SPREADEVAL:
+          case JSOP_ENTERLET2:
           case JSOP_ENTERWITH:
             canTrackVars = false;
-            isIonInlineable = false;
-            break;
-
-          case JSOP_ENTERLET0:
-          case JSOP_ENTERLET1:
-          case JSOP_ENTERBLOCK:
-          case JSOP_LEAVEBLOCK:
-            isIonInlineable = false;
             break;
 
           case JSOP_TABLESWITCH: {
-            isIonInlineable = false;
             unsigned defaultOffset = offset + GET_JUMP_OFFSET(pc);
             jsbytecode *pc2 = pc + JUMP_OFFSET_LEN;
             int32_t low = GET_JUMP_OFFSET(pc2);
@@ -311,7 +287,6 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
              * exception but is not caught by a later handler in the same function:
              * no more code will execute, and it does not matter what is defined.
              */
-            isIonInlineable = false;
             JSTryNote *tn = script_->trynotes()->vector;
             JSTryNote *tnlimit = tn + script_->trynotes()->length;
             for (; tn < tnlimit; tn++) {
@@ -368,122 +343,11 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
             numPropertyReads_++;
             break;
 
-          case JSOP_THROW:
-          case JSOP_EXCEPTION:
-          case JSOP_DEBUGGER:
-            isIonInlineable = false;
-            break;
-
           case JSOP_FINALLY:
             hasTryFinally_ = true;
             break;
 
-          /* Additional opcodes which can be both compiled both normally and inline. */
-          case JSOP_ARGUMENTS:
-          case JSOP_CALL:
-          case JSOP_NEW:
-          case JSOP_FUNCALL:
-          case JSOP_FUNAPPLY:
-          case JSOP_CALLEE:
-          case JSOP_NOP:
-          case JSOP_UNDEFINED:
-          case JSOP_GOTO:
-          case JSOP_DEFAULT:
-          case JSOP_IFEQ:
-          case JSOP_IFNE:
-          case JSOP_ITERNEXT:
-          case JSOP_DUP:
-          case JSOP_DUP2:
-          case JSOP_SWAP:
-          case JSOP_PICK:
-          case JSOP_BITOR:
-          case JSOP_BITXOR:
-          case JSOP_BITAND:
-          case JSOP_LT:
-          case JSOP_LE:
-          case JSOP_GT:
-          case JSOP_GE:
-          case JSOP_EQ:
-          case JSOP_NE:
-          case JSOP_LSH:
-          case JSOP_RSH:
-          case JSOP_URSH:
-          case JSOP_ADD:
-          case JSOP_SUB:
-          case JSOP_MUL:
-          case JSOP_DIV:
-          case JSOP_MOD:
-          case JSOP_NOT:
-          case JSOP_BITNOT:
-          case JSOP_NEG:
-          case JSOP_POS:
-          case JSOP_DELPROP:
-          case JSOP_DELELEM:
-          case JSOP_TYPEOF:
-          case JSOP_TYPEOFEXPR:
-          case JSOP_VOID:
-          case JSOP_TOID:
-          case JSOP_SETELEM:
-          case JSOP_IMPLICITTHIS:
-          case JSOP_DOUBLE:
-          case JSOP_STRING:
-          case JSOP_ZERO:
-          case JSOP_ONE:
-          case JSOP_NULL:
-          case JSOP_FALSE:
-          case JSOP_TRUE:
-          case JSOP_OR:
-          case JSOP_AND:
-          case JSOP_CASE:
-          case JSOP_STRICTEQ:
-          case JSOP_STRICTNE:
-          case JSOP_ITER:
-          case JSOP_MOREITER:
-          case JSOP_ENDITER:
-          case JSOP_POP:
-          case JSOP_GETARG:
-          case JSOP_SETARG:
-          case JSOP_CALLARG:
-          case JSOP_BINDGNAME:
-          case JSOP_UINT16:
-          case JSOP_NEWINIT:
-          case JSOP_NEWARRAY:
-          case JSOP_NEWOBJECT:
-          case JSOP_ENDINIT:
-          case JSOP_INITPROP:
-          case JSOP_INITELEM:
-          case JSOP_INITELEM_ARRAY:
-          case JSOP_SETPROP:
-          case JSOP_IN:
-          case JSOP_INSTANCEOF:
-          case JSOP_LINENO:
-          case JSOP_ENUMELEM:
-          case JSOP_CONDSWITCH:
-          case JSOP_LABEL:
-          case JSOP_RETRVAL:
-          case JSOP_GETGNAME:
-          case JSOP_CALLGNAME:
-          case JSOP_GETINTRINSIC:
-          case JSOP_SETINTRINSIC:
-          case JSOP_BINDINTRINSIC:
-          case JSOP_CALLINTRINSIC:
-          case JSOP_SETGNAME:
-          case JSOP_REGEXP:
-          case JSOP_OBJECT:
-          case JSOP_UINT24:
-          case JSOP_GETXPROP:
-          case JSOP_INT8:
-          case JSOP_INT32:
-          case JSOP_HOLE:
-          case JSOP_LOOPHEAD:
-          case JSOP_LOOPENTRY:
-          case JSOP_NOTEARG:
-          case JSOP_REST:
-          case JSOP_THIS:
-            break;
-
           default:
-            isIonInlineable = false;
             break;
         }
 
@@ -503,7 +367,7 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
 
         /* Handle any fallthrough from this opcode. */
         if (BytecodeFallsThrough(op)) {
-            JS_ASSERT(successorOffset < script_->length);
+            JS_ASSERT(successorOffset < script_->length());
 
             Bytecode *&nextcode = codeArray[successorOffset];
 
@@ -577,17 +441,17 @@ ScriptAnalysis::analyzeLifetimes(JSContext *cx)
     }
     unsigned savedCount = 0;
 
-    LoopAnalysis *loop = NULL;
+    LoopAnalysis *loop = nullptr;
 
-    uint32_t offset = script_->length - 1;
-    while (offset < script_->length) {
+    uint32_t offset = script_->length() - 1;
+    while (offset < script_->length()) {
         Bytecode *code = maybeCode(offset);
         if (!code) {
             offset--;
             continue;
         }
 
-        jsbytecode *pc = script_->code + offset;
+        jsbytecode *pc = script_->offsetToPC(offset);
 
         JSOp op = (JSOp) *pc;
 
@@ -656,13 +520,14 @@ ScriptAnalysis::analyzeLifetimes(JSContext *cx)
             /* Restore all saved variables. :FIXME: maybe do this precisely. */
             for (unsigned i = 0; i < savedCount; i++) {
                 LifetimeVariable &var = *saved[i];
-                var.lifetime = alloc.new_<Lifetime>(offset, var.savedEnd, var.saved);
+                uint32_t savedEnd = var.savedEnd;
+                var.lifetime = alloc.new_<Lifetime>(offset, savedEnd, var.saved);
                 if (!var.lifetime) {
                     js_free(saved);
                     setOOM(cx);
                     return;
                 }
-                var.saved = NULL;
+                var.saved = nullptr;
                 saved[i--] = saved[--savedCount];
             }
             savedCount = 0;
@@ -698,7 +563,7 @@ ScriptAnalysis::analyzeLifetimes(JSContext *cx)
                 /* This is a loop back edge, no lifetime to pull in yet. */
 
 #ifdef DEBUG
-                JSOp nop = JSOp(script_->code[targetOffset]);
+                JSOp nop = JSOp(script_->code()[targetOffset]);
                 JS_ASSERT(nop == JSOP_LOOPHEAD);
 #endif
 
@@ -727,7 +592,7 @@ ScriptAnalysis::analyzeLifetimes(JSContext *cx)
                         entry--;
                     } while (!maybeCode(entry));
 
-                    jsbytecode *entrypc = script_->code + entry;
+                    jsbytecode *entrypc = script_->offsetToPC(entry);
 
                     if (JSOp(*entrypc) == JSOP_GOTO)
                         entry += GET_JUMP_OFFSET(entrypc);
@@ -737,8 +602,8 @@ ScriptAnalysis::analyzeLifetimes(JSContext *cx)
                     /* Do-while loop at the start of the script. */
                     entry = targetOffset;
                 }
-                JS_ASSERT(script_->code[entry] == JSOP_LOOPHEAD ||
-                          script_->code[entry] == JSOP_LOOPENTRY);
+                JS_ASSERT(script_->code()[entry] == JSOP_LOOPHEAD ||
+                          script_->code()[entry] == JSOP_LOOPENTRY);
             } else {
                 for (unsigned i = 0; i < savedCount; i++) {
                     LifetimeVariable &var = *saved[i];
@@ -748,13 +613,14 @@ ScriptAnalysis::analyzeLifetimes(JSContext *cx)
                          * Jumping to a place where this variable is live. Make a new
                          * lifetime segment for the variable.
                          */
-                        var.lifetime = alloc.new_<Lifetime>(offset, var.savedEnd, var.saved);
+                        uint32_t savedEnd = var.savedEnd;
+                        var.lifetime = alloc.new_<Lifetime>(offset, savedEnd, var.saved);
                         if (!var.lifetime) {
                             js_free(saved);
                             setOOM(cx);
                             return;
                         }
-                        var.saved = NULL;
+                        var.saved = nullptr;
                         saved[i--] = saved[--savedCount];
                     } else if (loop && !var.savedEnd) {
                         /*
@@ -812,12 +678,13 @@ ScriptAnalysis::addVariable(JSContext *cx, LifetimeVariable &var, unsigned offse
                 }
             }
         }
-        var.lifetime = cx->typeLifoAlloc().new_<Lifetime>(offset, var.savedEnd, var.saved);
+        uint32_t savedEnd = var.savedEnd;
+        var.lifetime = cx->typeLifoAlloc().new_<Lifetime>(offset, savedEnd, var.saved);
         if (!var.lifetime) {
             setOOM(cx);
             return;
         }
-        var.saved = NULL;
+        var.saved = nullptr;
     }
 }
 
@@ -827,7 +694,8 @@ ScriptAnalysis::killVariable(JSContext *cx, LifetimeVariable &var, unsigned offs
 {
     if (!var.lifetime) {
         /* Make a point lifetime indicating the write. */
-        Lifetime *lifetime = cx->typeLifoAlloc().new_<Lifetime>(offset, var.savedEnd, var.saved);
+        uint32_t savedEnd = var.savedEnd;
+        Lifetime *lifetime = cx->typeLifoAlloc().new_<Lifetime>(offset, savedEnd, var.saved);
         if (!lifetime) {
             setOOM(cx);
             return;
@@ -867,7 +735,7 @@ ScriptAnalysis::killVariable(JSContext *cx, LifetimeVariable &var, unsigned offs
     } else {
         var.saved = var.lifetime;
         var.savedEnd = 0;
-        var.lifetime = NULL;
+        var.lifetime = nullptr;
 
         saved[savedCount++] = &var;
     }
@@ -1050,8 +918,8 @@ ScriptAnalysis::analyzeSSA(JSContext *cx)
     Vector<uint32_t> exceptionTargets(cx);
 
     uint32_t offset = 0;
-    while (offset < script_->length) {
-        jsbytecode *pc = script_->code + offset;
+    while (offset < script_->length()) {
+        jsbytecode *pc = script_->offsetToPC(offset);
         JSOp op = (JSOp)*pc;
 
         uint32_t successorOffset = offset + GetBytecodeLength(pc);
@@ -1357,7 +1225,6 @@ ScriptAnalysis::analyzeSSA(JSContext *cx)
 
           case JSOP_THROW:
           case JSOP_RETURN:
-          case JSOP_STOP:
           case JSOP_RETRVAL:
             mergeAllExceptionTargets(cx, values, exceptionTargets);
             break;
@@ -1647,7 +1514,7 @@ ScriptAnalysis::freezeNewValues(JSContext *cx, uint32_t offset)
     Bytecode &code = getCode(offset);
 
     Vector<SlotValue> *pending = code.pendingValues;
-    code.pendingValues = NULL;
+    code.pendingValues = nullptr;
 
     unsigned count = pending->length();
     if (count == 0) {
@@ -1704,19 +1571,23 @@ ScriptAnalysis::needsArgsObj(JSContext *cx, SeenVector &seen, SSAUseChain *use)
     if (!use->popped)
         return needsArgsObj(cx, seen, SSAValue::PhiValue(use->offset, use->u.phi));
 
-    jsbytecode *pc = script_->code + use->offset;
+    jsbytecode *pc = script_->offsetToPC(use->offset);
     JSOp op = JSOp(*pc);
 
     if (op == JSOP_POP || op == JSOP_POPN)
         return false;
 
     /* We can read the frame's arguments directly for f.apply(x, arguments). */
-    if (op == JSOP_FUNAPPLY && GET_ARGC(pc) == 2 && use->u.which == 0)
+    if (op == JSOP_FUNAPPLY && GET_ARGC(pc) == 2 && use->u.which == 0) {
+        argumentsContentsObserved_ = true;
         return false;
+    }
 
     /* arguments[i] can read fp->canonicalActualArg(i) directly. */
-    if (op == JSOP_GETELEM && use->u.which == 1)
+    if (op == JSOP_GETELEM && use->u.which == 1) {
+        argumentsContentsObserved_ = true;
         return false;
+    }
 
     /* arguments.length length can read fp->numActualArgs() directly. */
     if (op == JSOP_LENGTH)
@@ -1772,19 +1643,22 @@ ScriptAnalysis::needsArgsObj(JSContext *cx)
     if (localsAliasStack())
         return true;
 
-    /*
-     * If a script has explicit mentions of 'arguments' and formals which may
-     * be stored as part of a call object, don't use lazy arguments. The
-     * compiler can then assume that accesses through arguments[i] will be on
-     * unaliased variables.
-     */
-    if (script_->funHasAnyAliasedFormal)
-        return true;
-
-    unsigned pcOff = script_->argumentsBytecode() - script_->code;
+    unsigned pcOff = script_->pcToOffset(script_->argumentsBytecode());
 
     SeenVector seen(cx);
-    return needsArgsObj(cx, seen, SSAValue::PushedValue(pcOff, 0));
+    if (needsArgsObj(cx, seen, SSAValue::PushedValue(pcOff, 0)))
+        return true;
+
+    /*
+     * If a script explicitly accesses the contents of 'arguments', and has
+     * formals which may be stored as part of a call object, don't use lazy
+     * arguments. The compiler can then assume that accesses through
+     * arguments[i] will be on unaliased variables.
+     */
+    if (script_->funHasAnyAliasedFormal && argumentsContentsObserved_)
+        return true;
+
+    return false;
 }
 
 #ifdef DEBUG
@@ -1797,12 +1671,12 @@ ScriptAnalysis::printSSA(JSContext *cx)
     printf("\n");
 
     RootedScript script(cx, script_);
-    for (unsigned offset = 0; offset < script_->length; offset++) {
+    for (unsigned offset = 0; offset < script_->length(); offset++) {
         Bytecode *code = maybeCode(offset);
         if (!code)
             continue;
 
-        jsbytecode *pc = script_->code + offset;
+        jsbytecode *pc = script_->offsetToPC(offset);
 
         PrintBytecode(cx, script, pc);
 
