@@ -8,7 +8,8 @@
 
 BEGIN_TEST(testOOM)
 {
-    JS::RootedString jsstr(cx, JS_ValueToString(cx, INT_TO_JSVAL(9)));
+    JS::RootedValue v(cx, JS::Int32Value(9));
+    JS::RootedString jsstr(cx, JS::ToString(cx, v));
     mozilla::DebugOnly<const jschar *> s = JS_GetStringCharsZ(cx, jsstr);
     JS_ASSERT(s[0] == '9' && s[1] == '\0');
     return true;
@@ -18,7 +19,7 @@ virtual JSRuntime * createRuntime()
 {
     JSRuntime *rt = JS_NewRuntime(0, JS_USE_HELPER_THREADS);
     if (!rt)
-        return NULL;
+        return nullptr;
     JS_SetGCParameter(rt, JSGC_MAX_BYTES, (uint32_t)-1);
     setNativeStackQuota(rt);
     return rt;
