@@ -64,10 +64,14 @@ class JS_FRIEND_API(Wrapper);
  * default implementation for the fundamental traps. It does, however, implement
  * the derived traps in terms of the fundamental ones. This allows consumers of
  * this class to define any custom behavior they want.
+ *
+ * Important: If you add a trap here, you should probably also add a Proxy::foo
+ * entry point with an AutoEnterPolicy. If you don't, you need an explicit
+ * override for the trap in SecurityWrapper. See bug 945826 comment 0.
  */
 class JS_FRIEND_API(BaseProxyHandler)
 {
-    void *mFamily;
+    const void *mFamily;
     bool mHasPrototype;
     bool mHasPolicy;
   protected:
@@ -76,7 +80,7 @@ class JS_FRIEND_API(BaseProxyHandler)
     void setHasPolicy(bool aHasPolicy) { mHasPolicy = aHasPolicy; }
 
   public:
-    explicit BaseProxyHandler(void *family);
+    explicit BaseProxyHandler(const void *family);
     virtual ~BaseProxyHandler();
 
     bool hasPrototype() {
@@ -87,7 +91,7 @@ class JS_FRIEND_API(BaseProxyHandler)
         return mHasPolicy;
     }
 
-    inline void *family() {
+    inline const void *family() {
         return mFamily;
     }
     static size_t offsetOfFamily() {
@@ -166,8 +170,15 @@ class JS_FRIEND_API(BaseProxyHandler)
                                      uint32_t index, MutableHandleValue vp, bool *present);
     virtual bool getPrototypeOf(JSContext *cx, HandleObject proxy, MutableHandleObject protop);
 
+    // These two hooks must be overridden, or not overridden, in tandem -- no
+    // overriding just one!
+    virtual bool watch(JSContext *cx, JS::HandleObject proxy, JS::HandleId id,
+                       JS::HandleObject callable);
+    virtual bool unwatch(JSContext *cx, JS::HandleObject proxy, JS::HandleId id);
+
     /* See comment for weakmapKeyDelegateOp in js/Class.h. */
     virtual JSObject *weakmapKeyDelegate(JSObject *proxy);
+    virtual bool isScripted() { return false; }
 };
 
 /*
@@ -175,11 +186,15 @@ class JS_FRIEND_API(BaseProxyHandler)
  * reimplemented such that they forward their behavior to the target. This
  * allows consumers of this class to forward to another object as transparently
  * and efficiently as possible.
+ *
+ * Important: If you add a trap implementation here, you probably also need to
+ * add an override in CrossCompartmentWrapper. If you don't, you risk
+ * compartment mismatches. See bug 945826 comment 0.
  */
 class JS_PUBLIC_API(DirectProxyHandler) : public BaseProxyHandler
 {
   public:
-    explicit DirectProxyHandler(void *family);
+    explicit DirectProxyHandler(const void *family);
 
     /* ES5 Harmony fundamental proxy traps. */
     virtual bool preventExtensions(JSContext *cx, HandleObject proxy) MOZ_OVERRIDE;
@@ -229,7 +244,13 @@ class JS_PUBLIC_API(DirectProxyHandler) : public BaseProxyHandler
     virtual JSObject *weakmapKeyDelegate(JSObject *proxy);
 };
 
-/* Dispatch point for handlers that executes the appropriate C++ or scripted traps. */
+/*
+ * Dispatch point for handlers that executes the appropriate C++ or scripted traps.
+ *
+ * Important: All proxy traps need either (a) an AutoEnterPolicy in their
+ * Proxy::foo entry point below or (b) an override in SecurityWrapper. See bug
+ * 945826 comment 0.
+ */
 class Proxy
 {
   public:
@@ -275,38 +296,27 @@ class Proxy
     static bool defaultValue(JSContext *cx, HandleObject obj, JSType hint, MutableHandleValue vp);
     static bool getPrototypeOf(JSContext *cx, HandleObject proxy, MutableHandleObject protop);
 
+    static bool watch(JSContext *cx, JS::HandleObject proxy, JS::HandleId id,
+                      JS::HandleObject callable);
+    static bool unwatch(JSContext *cx, JS::HandleObject proxy, JS::HandleId id);
+
+    /* IC entry path for handling __noSuchMethod__ on access. */
+    static bool callProp(JSContext *cx, HandleObject proxy, HandleObject reveiver, HandleId id,
+                         MutableHandleValue vp);
+
     static JSObject * const LazyProto;
 };
 
-// These are equal to |&{Function,Object,OuterWindow}ProxyObject::class_|.  Use
-// them in places where you don't want to #include vm/ProxyObject.h.
-extern JS_FRIEND_DATA(const js::Class* const) FunctionProxyClassPtr;
-extern JS_FRIEND_DATA(const js::Class* const) ObjectProxyClassPtr;
+// Use these in places where you don't want to #include vm/ProxyObject.h.
+extern JS_FRIEND_DATA(const js::Class* const) CallableProxyClassPtr;
+extern JS_FRIEND_DATA(const js::Class* const) UncallableProxyClassPtr;
 extern JS_FRIEND_DATA(const js::Class* const) OuterWindowProxyClassPtr;
 
-inline bool IsObjectProxyClass(const Class *clasp)
-{
-    return clasp == ObjectProxyClassPtr || clasp == OuterWindowProxyClassPtr;
-}
-
-inline bool IsFunctionProxyClass(const Class *clasp)
-{
-    return clasp == FunctionProxyClassPtr;
-}
-
 inline bool IsProxyClass(const Class *clasp)
 {
-    return IsObjectProxyClass(clasp) || IsFunctionProxyClass(clasp);
-}
-
-inline bool IsObjectProxy(JSObject *obj)
-{
-    return IsObjectProxyClass(GetObjectClass(obj));
-}
-
-inline bool IsFunctionProxy(JSObject *obj)
-{
-    return IsFunctionProxyClass(GetObjectClass(obj));
+    return clasp == CallableProxyClassPtr ||
+           clasp == UncallableProxyClassPtr ||
+           clasp == OuterWindowProxyClassPtr;
 }
 
 inline bool IsProxy(JSObject *obj)
@@ -314,6 +324,17 @@ inline bool IsProxy(JSObject *obj)
     return IsProxyClass(GetObjectClass(obj));
 }
 
+BaseProxyHandler *
+GetProxyHandler(JSObject *obj);
+
+inline bool IsScriptedProxy(JSObject *obj)
+{
+    if (!IsProxy(obj))
+        return false;
+
+    return GetProxyHandler(obj)->isScripted();
+}
+
 /*
  * These are part of the API.
  *
@@ -369,15 +390,32 @@ SetProxyExtra(JSObject *obj, size_t n, const Value &extra)
     SetReservedSlot(obj, PROXY_EXTRA_SLOT + n, extra);
 }
 
-enum ProxyCallable {
-    ProxyNotCallable = false,
-    ProxyIsCallable = true
+class MOZ_STACK_CLASS ProxyOptions {
+  public:
+    ProxyOptions() : callable_(false),
+                     singleton_(false)
+    {}
+
+    bool callable() const { return callable_; }
+    ProxyOptions &setCallable(bool flag) {
+        callable_ = flag;
+        return *this;
+    }
+
+    bool singleton() const { return singleton_; }
+    ProxyOptions &setSingleton(bool flag) {
+        singleton_ = flag;
+        return *this;
+    }
+
+  private:
+    bool callable_;
+    bool singleton_;
 };
 
 JS_FRIEND_API(JSObject *)
 NewProxyObject(JSContext *cx, BaseProxyHandler *handler, HandleValue priv,
-               JSObject *proto, JSObject *parent,
-               ProxyCallable callable = ProxyNotCallable, bool singleton = false);
+               JSObject *proto, JSObject *parent, const ProxyOptions &options = ProxyOptions());
 
 JSObject *
 RenewProxyObject(JSContext *cx, JSObject *obj, BaseProxyHandler *handler, Value priv);
@@ -389,7 +427,7 @@ class JS_FRIEND_API(AutoEnterPolicy)
     AutoEnterPolicy(JSContext *cx, BaseProxyHandler *handler,
                     HandleObject wrapper, HandleId id, Action act, bool mayThrow)
 #ifdef DEBUG
-        : context(NULL)
+        : context(nullptr)
 #endif
     {
         allow = handler->hasPolicy() ? handler->enter(cx, wrapper, id, act, &rv)
@@ -412,7 +450,7 @@ class JS_FRIEND_API(AutoEnterPolicy)
     // no-op constructor for subclass
     AutoEnterPolicy()
 #ifdef DEBUG
-        : context(NULL)
+        : context(nullptr)
 #endif
         {};
     void reportErrorIfExceptionIsNotPending(JSContext *cx, jsid id);
