@@ -13,6 +13,7 @@
 #include "jsutil.h"
 #include "NamespaceImports.h"
 
+#include "gc/Rooting.h"
 #include "js/RootingAPI.h"
 #include "vm/Unicode.h"
 
@@ -47,19 +48,16 @@ SkipSpace(const jschar *s, const jschar *end)
 
 // Return less than, equal to, or greater than zero depending on whether
 // s1 is less than, equal to, or greater than s2.
-inline bool
-CompareChars(const jschar *s1, size_t l1, const jschar *s2, size_t l2, int32_t *result)
+inline int32_t
+CompareChars(const jschar *s1, size_t l1, const jschar *s2, size_t l2)
 {
     size_t n = Min(l1, l2);
     for (size_t i = 0; i < n; i++) {
-        if (int32_t cmp = s1[i] - s2[i]) {
-            *result = cmp;
-            return true;
-        }
+        if (int32_t cmp = s1[i] - s2[i])
+            return cmp;
     }
 
-    *result = (int32_t)(l1 - l2);
-    return true;
+    return (int32_t)(l1 - l2);
 }
 
 }  /* namespace js */
@@ -209,6 +207,9 @@ EqualStrings(JSLinearString *str1, JSLinearString *str2);
 extern bool
 CompareStrings(JSContext *cx, JSString *str1, JSString *str2, int32_t *result);
 
+extern int32_t
+CompareAtoms(JSAtom *atom1, JSAtom *atom2);
+
 /*
  * Return true if the string matches the given sequence of ASCII bytes.
  */
@@ -235,7 +236,7 @@ js_strncpy(jschar *dst, const jschar *src, size_t nelem)
 }
 
 extern jschar *
-js_strdup(JSContext *cx, const jschar *s);
+js_strdup(js::ThreadSafeContext *cx, const jschar *s);
 
 namespace js {
 
@@ -249,15 +250,15 @@ extern jschar *
 InflateString(ThreadSafeContext *cx, const char *bytes, size_t *length);
 
 /*
- * Inflate bytes to JS chars in an existing buffer. 'chars' must be large
- * enough for 'length' jschars. The buffer is NOT null-terminated.
- *
- * charsLength must be be initialized with the destination buffer size and, on
- * return, will contain on return the number of copied chars.
+ * Inflate bytes to JS chars in an existing buffer. 'dst' must be large
+ * enough for 'srclen' jschars. The buffer is NOT null-terminated.
  */
-extern bool
-InflateStringToBuffer(JSContext *maybecx, const char *bytes, size_t length,
-                      jschar *chars, size_t *charsLength);
+inline void
+InflateStringToBuffer(const char *src, size_t srclen, jschar *dst)
+{
+    for (size_t i = 0; i < srclen; i++)
+        dst[i] = (unsigned char) src[i];
+}
 
 /*
  * Deflate JS chars to bytes into a buffer. 'bytes' must be large enough for
@@ -318,7 +319,7 @@ PutEscapedStringImpl(char *buffer, size_t bufferSize, FILE *fp, const jschar *ch
 inline size_t
 PutEscapedString(char *buffer, size_t size, JSLinearString *str, uint32_t quote)
 {
-    size_t n = PutEscapedStringImpl(buffer, size, NULL, str, quote);
+    size_t n = PutEscapedStringImpl(buffer, size, nullptr, str, quote);
 
     /* PutEscapedStringImpl can only fail with a file. */
     JS_ASSERT(n != size_t(-1));
@@ -328,7 +329,7 @@ PutEscapedString(char *buffer, size_t size, JSLinearString *str, uint32_t quote)
 inline size_t
 PutEscapedString(char *buffer, size_t bufferSize, const jschar *chars, size_t length, uint32_t quote)
 {
-    size_t n = PutEscapedStringImpl(buffer, bufferSize, NULL, chars, length, quote);
+    size_t n = PutEscapedStringImpl(buffer, bufferSize, nullptr, chars, length, quote);
 
     /* PutEscapedStringImpl can only fail with a file. */
     JS_ASSERT(n != size_t(-1));
@@ -343,7 +344,7 @@ PutEscapedString(char *buffer, size_t bufferSize, const jschar *chars, size_t le
 inline bool
 FileEscapedString(FILE *fp, JSLinearString *str, uint32_t quote)
 {
-    return PutEscapedStringImpl(NULL, 0, fp, str, quote) != size_t(-1);
+    return PutEscapedStringImpl(nullptr, 0, fp, str, quote) != size_t(-1);
 }
 
 bool
@@ -355,6 +356,13 @@ str_search(JSContext *cx, unsigned argc, Value *vp);
 bool
 str_split(JSContext *cx, unsigned argc, Value *vp);
 
+JSObject *
+str_split_string(JSContext *cx, HandleTypeObject type, HandleString str, HandleString sep);
+
+bool
+str_resolve(JSContext *cx, HandleObject obj, HandleId id, unsigned flags,
+            MutableHandleObject objp);
+
 } /* namespace js */
 
 extern bool
