@@ -1,12 +1,15 @@
-// |reftest| skip-if(!this.hasOwnProperty("Type"))
+// |reftest| skip-if(!this.hasOwnProperty("TypedObject"))
 var BUGNUMBER = 578700;
 var summary = 'TypedObjects StructType prototype chains';
 
+var ArrayType = TypedObject.ArrayType;
+var StructType = TypedObject.StructType;
+var float32 = TypedObject.float32;
 
 function runTests() {
-  var Point = new ArrayType(float32, 3);
+  var Point = new ArrayType(float32).dimension(3);
   var Line = new StructType({from: Point, to: Point});
-  var Lines = new ArrayType(Line, 3);
+  var Lines = new ArrayType(Line).dimension(3);
 
   var lines = new Lines([
     {from: [1, 2, 3], to: [4, 5, 6]},
