@@ -1,9 +1,23 @@
-// |reftest| skip-if(!this.hasOwnProperty("Type"))
+// |reftest| skip-if(!this.hasOwnProperty("TypedObject"))
 var BUGNUMBER = 578700;
 var summary = 'Size and Alignment of TypedObjects types';
 var actual = '';
 var expect = '';
 
+
+var ArrayType = TypedObject.ArrayType;
+var StructType = TypedObject.StructType;
+var uint8 = TypedObject.uint8;
+var uint16 = TypedObject.uint16;
+var uint32 = TypedObject.uint32;
+var uint8Clamped = TypedObject.uint8Clamped;
+var int8 = TypedObject.int8;
+var int16 = TypedObject.int16;
+var int32 = TypedObject.int32;
+var float32 = TypedObject.float32;
+var float64 = TypedObject.float64;
+
+
 function runTests() {
   printBugNumber(BUGNUMBER);
   printStatus(summary);
@@ -25,9 +39,9 @@ function runTests() {
 
     {type: new StructType({a: uint8, b: uint8, c: uint16}), size: 4, alignment: 2},
 
-    {type: new ArrayType(uint8, 32), size: 32, alignment: 1},
-    {type: new ArrayType(uint16, 16), size: 32, alignment: 2},
-    {type: new ArrayType(uint32, 8), size: 32, alignment: 4},
+    {type: new ArrayType(uint8).dimension(32), size: 32, alignment: 1},
+    {type: new ArrayType(uint16).dimension(16), size: 32, alignment: 2},
+    {type: new ArrayType(uint32).dimension(8), size: 32, alignment: 4},
   ];
 
   for (var i = 0; i < typesAndAlignments.length; i++) {
