@@ -4,10 +4,10 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 387951;
-var summary = 'Do not assert: cg->stackDepth >= 0';
-var actual = 'No Crash';
-var expect = 'No Crash';
+var BUGNUMBER = 470187;
+var summary = 'Do not assert: entry->kpc == (jsbytecode*) atoms[index]';
+var actual = '';
+var expect = '';
 
 
 //-----------------------------------------------------------------------------
@@ -20,7 +20,7 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
  
-  "" + (function() { if(delete(null?0:{})){[]} });
+  for (var j=0;j<3;++j) ({valueOf: function(){return 2}}) - /x/;
 
   reportCompare(expect, actual, summary);
 
