@@ -20,10 +20,14 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-// ------- Comment #30 From Mike Shaver
+  expect = '10';
 
-  function f() { var i = 0; var i = 5; }
-  f();
+// ------- Comment #39 From Jesse Ruderman
+
+  const e = 0;
+  print(actual = ++e);
+
+  actual = String(actual) + e;
 
   reportCompare(expect, actual, summary);
 
