@@ -25,8 +25,11 @@ from results import TestOutput
 
 TESTS_LIB_DIR = os.path.dirname(os.path.abspath(__file__))
 JS_DIR = os.path.dirname(os.path.dirname(TESTS_LIB_DIR))
+TOP_SRC_DIR = os.path.dirname(os.path.dirname(JS_DIR))
 TEST_DIR = os.path.join(JS_DIR, 'jit-test', 'tests')
 LIB_DIR = os.path.join(JS_DIR, 'jit-test', 'lib') + os.path.sep
+JS_CACHE_DIR = os.path.join(JS_DIR, 'jit-test', '.js-cache')
+JS_TESTS_DIR = posixpath.join(JS_DIR, 'tests')
 
 # Backported from Python 3.1 posixpath.py
 def _relpath(path, start=None):
@@ -69,7 +72,14 @@ class Test:
     del valgrinds
 
     def __init__(self, path):
-        self.path = path       # path to test file
+        # Absolute path of the test file.
+        self.path = path
+
+        # Path relative to the top mozilla/ directory.
+        self.relpath_top = os.path.relpath(path, TOP_SRC_DIR)
+
+        # Path relative to mozilla/js/src/jit-test/tests/.
+        self.relpath_tests = os.path.relpath(path, TEST_DIR)
 
         self.jitflags = []     # jit flags to enable
         self.slow = False      # True means the test is slow-running
@@ -91,6 +101,7 @@ class Test:
         return t
 
     COOKIE = '|jit-test|'
+    CacheDir = JS_CACHE_DIR
 
     @classmethod
     def from_file(cls, path, options):
@@ -131,14 +142,8 @@ class Test:
                         test.valgrind = options.valgrind
                     elif name == 'tz-pacific':
                         test.tz_pacific = True
-                    elif name == 'mjitalways':
-                        test.jitflags.append('--always-mjit')
                     elif name == 'debug':
                         test.jitflags.append('--debugjit')
-                    elif name == 'mjit':
-                        test.jitflags.append('--jm')
-                    elif name == 'no-jm':
-                        test.jitflags.append('--no-jm')
                     elif name == 'ion-eager':
                         test.jitflags.append('--ion-eager')
                     elif name == 'no-ion':
@@ -175,7 +180,8 @@ class Test:
 
         # We may have specified '-a' or '-d' twice: once via --jitflags, once
         # via the "|jit-test|" line.  Remove dups because they are toggles.
-        cmd = prefix + list(set(self.jitflags)) + ['-e', expr, '-f', path]
+        cmd = prefix + ['--js-cache', Test.CacheDir]
+        cmd += list(set(self.jitflags)) + ['-e', expr, '-f', path]
         if self.valgrind:
             cmd = self.VALGRIND_CMD + cmd
         return cmd
@@ -296,6 +302,21 @@ def run_test(test, prefix, options):
     if test.tz_pacific:
         env['TZ'] = 'PST8PDT'
 
+    # Ensure interpreter directory is in shared library path.
+    pathvar = ''
+    if sys.platform.startswith('linux'):
+        pathvar = 'LD_LIBRARY_PATH'
+    elif sys.platform.startswith('darwin'):
+        pathvar = 'DYLD_LIBRARY_PATH'
+    elif sys.platform.startswith('win'):
+        pathvar = 'PATH'
+    if pathvar:
+        bin_dir = os.path.dirname(cmd[0])
+        if pathvar in env:
+            env[pathvar] = '%s%s%s' % (bin_dir, os.pathsep, env[pathvar])
+        else:
+            env[pathvar] = bin_dir
+
     out, err, code, timed_out = run(cmd, env, options.timeout)
     return TestOutput(test, cmd, out, err, code, None, timed_out)
 
@@ -348,20 +369,30 @@ def check_output(out, err, rc, test):
 
     return True
 
-def print_tinderbox(label, test, message=None):
+def print_tinderbox(ok, res):
     # Output test failures in a TBPL parsable format, eg:
-    # TEST-PASS | /foo/bar/baz.js | --no-jm
+    # TEST-PASS | /foo/bar/baz.js | --ion-eager
     # TEST-UNEXPECTED-FAIL | /foo/bar/baz.js | --no-ion: Assertion failure: ...
+    # INFO exit-status     : 3
+    # INFO timed-out       : False
+    # INFO stdout          > foo
+    # INFO stdout          > bar
+    # INFO stdout          > baz
+    # INFO stderr         2> TypeError: or something
     # TEST-UNEXPECTED-FAIL | jit_test.py: Test execution interrupted by user
-    if (test != None):
-        jitflags = " ".join(test.jitflags)
-        result = "%s | %s | %s" % (label, test.path, jitflags)
-    else:
-        result = "%s | jit_test.py" % label
+    label = "TEST-PASS" if ok else "TEST-UNEXPECTED-FAIL"
+    jitflags = " ".join(res.test.jitflags)
+    print("%s | %s | %s" % (label, res.test.relpath_top, jitflags))
+    if ok:
+        return
 
-    if message:
-        result += ": " + message
-    print(result)
+    # For failed tests, print as much information as we have, to aid debugging.
+    print("INFO exit-status     : {}".format(res.rc))
+    print("INFO timed-out       : {}".format(res.timed_out))
+    for line in res.out.split('\n'):
+        print("INFO stdout          > " + line.strip())
+    for line in res.err.split('\n'):
+        print("INFO stderr         2> " + line.strip())
 
 def wrap_parallel_run_test(test, prefix, resultQueue, options):
     # Ignore SIGINT in the child
@@ -537,7 +568,6 @@ def process_test_results(results, num_tests, options):
     doing = 'before starting'
     try:
         for i, res in enumerate(results):
-
             if options.show_output:
                 sys.stdout.write(res.out)
                 sys.stdout.write(res.err)
@@ -546,26 +576,17 @@ def process_test_results(results, num_tests, options):
                 sys.stdout.write(res.err)
 
             ok = check_output(res.out, res.err, res.rc, res.test)
-            doing = 'after %s' % res.test.path
+            doing = 'after %s' % res.test.relpath_tests
             if not ok:
                 failures.append(res)
                 if res.timed_out:
-                    pb.message("TIMEOUT - %s" % res.test.path)
+                    pb.message("TIMEOUT - %s" % res.test.relpath_tests)
                     timeouts += 1
                 else:
-                    pb.message("FAIL - %s" % res.test.path)
+                    pb.message("FAIL - %s" % res.test.relpath_tests)
 
             if options.tinderbox:
-                if ok:
-                    print_tinderbox("TEST-PASS", res.test);
-                else:
-                    lines = [ _ for _ in res.out.split('\n') + res.err.split('\n')
-                              if _ != '' ]
-                    if len(lines) >= 1:
-                        msg = lines[-1]
-                    else:
-                        msg = ''
-                    print_tinderbox("TEST-UNEXPECTED-FAIL", res.test, msg);
+                print_tinderbox(ok, res)
 
             n = i + 1
             pb.update(n, {
@@ -576,7 +597,8 @@ def process_test_results(results, num_tests, options):
             )
         complete = True
     except KeyboardInterrupt:
-        print_tinderbox("TEST-UNEXPECTED-FAIL", None, "Test execution interrupted by user");
+        print("TEST-UNEXPECTED-FAIL | jit_test.py" +
+              " : Test execution interrupted by user")
 
     pb.finish(True)
     return print_test_summary(num_tests, failures, complete, doing, options)
@@ -625,15 +647,23 @@ def run_tests_remote(tests, prefix, options):
             sys.exit(1)
 
     # Update the test root to point to our test directory.
-    options.remote_test_root = posixpath.join(options.remote_test_root, 'jit-tests')
+    jit_tests_dir = posixpath.join(options.remote_test_root, 'jit-tests')
+    options.remote_test_root = posixpath.join(jit_tests_dir, 'jit-tests')
 
     # Push js shell and libraries.
-    if dm.dirExists(options.remote_test_root):
-        dm.removeDir(options.remote_test_root)
-    dm.mkDir(options.remote_test_root)
+    if dm.dirExists(jit_tests_dir):
+        dm.removeDir(jit_tests_dir)
+    dm.mkDirs(options.remote_test_root)
     push_libs(options, dm)
     push_progs(options, dm, [prefix[0]])
     dm.chmodDir(options.remote_test_root)
+
+    Test.CacheDir = posixpath.join(options.remote_test_root, '.js-cache')
+    dm.mkDir(Test.CacheDir)
+
+    for path in os.listdir(JS_TESTS_DIR):
+        dm.pushDir(os.path.join(JS_TESTS_DIR, path), posixpath.join(jit_tests_dir, 'tests', path))
+
     dm.pushDir(os.path.dirname(TEST_DIR), options.remote_test_root, timeout=600)
     prefix[0] = os.path.join(options.remote_test_root, 'js')
 
