@@ -11,9 +11,12 @@
 #include "vm/Debugger.h"
 
 #include "jsobjinlines.h"
+#include "vm/Shape-inl.h"
 
 using namespace js;
 
+using JS::GenericNaN;
+
 PropDesc::PropDesc()
   : pd_(UndefinedValue()),
     value_(UndefinedValue()),
@@ -35,7 +38,7 @@ PropDesc::checkGetter(JSContext *cx)
 {
     if (hasGet_) {
         if (!js_IsCallable(get_) && !get_.isUndefined()) {
-            JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL, JSMSG_BAD_GET_SET_FIELD,
+            JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr, JSMSG_BAD_GET_SET_FIELD,
                                  js_getter_str);
             return false;
         }
@@ -48,7 +51,7 @@ PropDesc::checkSetter(JSContext *cx)
 {
     if (hasSet_) {
         if (!js_IsCallable(set_) && !set_.isUndefined()) {
-            JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL, JSMSG_BAD_GET_SET_FIELD,
+            JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr, JSMSG_BAD_GET_SET_FIELD,
                                  js_setter_str);
             return false;
         }
@@ -61,7 +64,7 @@ CheckArgCompartment(JSContext *cx, JSObject *obj, HandleValue v,
                     const char *methodname, const char *propname)
 {
     if (v.isObject() && v.toObject().compartment() != obj->compartment()) {
-        JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL, JSMSG_DEBUG_COMPARTMENT_MISMATCH,
+        JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr, JSMSG_DEBUG_COMPARTMENT_MISMATCH,
                              methodname, propname);
         return false;
     }
@@ -191,7 +194,7 @@ js::ObjectImpl::checkShapeConsistency()
     MOZ_ASSERT(isNative());
 
     Shape *shape = lastProperty();
-    Shape *prev = NULL;
+    Shape *prev = nullptr;
 
     if (inDictionaryMode()) {
         MOZ_ASSERT(shape->hasTable());
@@ -331,6 +334,25 @@ js::ObjectImpl::nativeLookupPure(jsid id)
     return Shape::searchNoHashify(lastProperty(), id);
 }
 
+uint32_t
+js::ObjectImpl::numFixedSlotsForCompilation() const
+{
+    // This is an alternative method for getting the number of fixed slots
+    // in an object. It requires more logic and memory accesses than
+    // numFixedSlots() but is safe to be called from the compilation thread,
+    // even if the main thread is actively mutating the VM.
+    if (static_cast<const JSObject *>(this)->is<ArrayObject>())
+        return 0;
+#ifdef JSGC_GENERATIONAL
+    // The compiler does not have access to nursery things, so if this object
+    // is in the nursery we can fall back to numFixedSlots().
+    if (IsInsideNursery(GetGCThingRuntime(this), this))
+        return numFixedSlots();
+#endif
+    gc::AllocKind kind = tenuredGetAllocKind();
+    return gc::GetGCKindSlots(kind, getClass());
+}
+
 void
 js::ObjectImpl::markChildren(JSTracer *trc)
 {
@@ -483,7 +505,7 @@ DenseElementsHeader::defineElement(JSContext *cx, Handle<ObjectImpl*> obj, uint3
         MOZ_ALWAYS_FALSE(js_ReportValueErrorFlags(cx, JSREPORT_ERROR, JSMSG_OBJECT_NOT_EXTENSIBLE,
                                                   JSDVG_IGNORE_STACK,
                                                   val, NullPtr(),
-                                                  NULL, NULL));
+                                                  nullptr, nullptr));
         return false;
     }
 
@@ -515,7 +537,8 @@ js::ArrayBufferDelegate(JSContext *cx, Handle<ObjectImpl*> obj)
     MOZ_ASSERT(obj->hasClass(&ArrayBufferObject::class_));
     if (obj->getPrivate())
         return static_cast<JSObject *>(obj->getPrivate());
-    JSObject *delegate = NewObjectWithGivenProto(cx, &JSObject::class_, obj->getProto(), NULL);
+    JSObject *delegate = NewObjectWithGivenProto(cx, &JSObject::class_,
+                                                 obj->getProto(), nullptr);
     obj->setPrivateGCThing(delegate);
     return delegate;
 }
@@ -532,7 +555,7 @@ TypedElementsHeader<T>::defineElement(JSContext *cx, Handle<ObjectImpl*> obj,
     RootedValue val(cx, ObjectValue(*obj));
     js_ReportValueErrorFlags(cx, JSREPORT_ERROR, JSMSG_OBJECT_NOT_EXTENSIBLE,
                              JSDVG_IGNORE_STACK,
-                             val, NullPtr(), NULL, NULL);
+                             val, NullPtr(), nullptr, nullptr);
     return false;
 }
 
@@ -571,7 +594,7 @@ js::GetOwnProperty(JSContext *cx, Handle<ObjectImpl*> obj, PropertyId pid_, unsi
             Rooted<jsid> id(cx, pid.get().asId());
             Rooted<JSObject*> robj(cx, static_cast<JSObject*>(obj.get()));
             if (clasp->flags & JSCLASS_NEW_RESOLVE) {
-                Rooted<JSObject*> obj2(cx, NULL);
+                Rooted<JSObject*> obj2(cx, nullptr);
                 JSNewResolveOp op = reinterpret_cast<JSNewResolveOp>(resolve);
                 if (!op(cx, robj, id, resolveFlags, &obj2))
                     return false;
@@ -898,13 +921,13 @@ TypedElementsHeader<T>::setElement(JSContext *cx, Handle<ObjectImpl*> obj,
             if (!StringToNumber(cx, v.toString(), &d))
                 return false;
         } else if (v.isUndefined()) {
-            d = js_NaN;
+            d = GenericNaN();
         } else {
             d = double(v.toBoolean());
         }
     } else {
         // non-primitive assignments become NaN or 0 (for float/int arrays)
-        d = js_NaN;
+        d = GenericNaN();
     }
 
     assign(index, d);
