@@ -32,92 +32,22 @@ typedef Rooted<ArgumentsObject *> RootedArgumentsObject;
 
 /*****************************************************************************/
 
-StaticScopeIter::StaticScopeIter(ExclusiveContext *cx, JSObject *objArg)
-  : obj(cx, objArg), onNamedLambda(false)
-{
-    JS_ASSERT_IF(obj, obj->is<StaticBlockObject>() || obj->is<JSFunction>());
-}
-
-bool
-StaticScopeIter::done() const
-{
-    return !obj;
-}
-
-void
-StaticScopeIter::operator++(int)
-{
-    if (obj->is<StaticBlockObject>()) {
-        obj = obj->as<StaticBlockObject>().enclosingStaticScope();
-    } else if (onNamedLambda || !obj->as<JSFunction>().isNamedLambda()) {
-        onNamedLambda = false;
-        obj = obj->as<JSFunction>().nonLazyScript()->enclosingStaticScope();
-    } else {
-        onNamedLambda = true;
-    }
-    JS_ASSERT_IF(obj, obj->is<StaticBlockObject>() || obj->is<JSFunction>());
-    JS_ASSERT_IF(onNamedLambda, obj->is<JSFunction>());
-}
-
-bool
-StaticScopeIter::hasDynamicScopeObject() const
-{
-    return obj->is<StaticBlockObject>()
-           ? obj->as<StaticBlockObject>().needsClone()
-           : obj->as<JSFunction>().isHeavyweight();
-}
-
-Shape *
-StaticScopeIter::scopeShape() const
-{
-    JS_ASSERT(hasDynamicScopeObject());
-    JS_ASSERT(type() != NAMED_LAMBDA);
-    return type() == BLOCK
-           ? block().lastProperty()
-           : funScript()->bindings.callObjShape();
-}
-
-StaticScopeIter::Type
-StaticScopeIter::type() const
-{
-    if (onNamedLambda)
-        return NAMED_LAMBDA;
-    return obj->is<StaticBlockObject>() ? BLOCK : FUNCTION;
-}
-
-StaticBlockObject &
-StaticScopeIter::block() const
-{
-    JS_ASSERT(type() == BLOCK);
-    return obj->as<StaticBlockObject>();
-}
-
-JSScript *
-StaticScopeIter::funScript() const
-{
-    JS_ASSERT(type() == FUNCTION);
-    return obj->as<JSFunction>().nonLazyScript();
-}
-
-/*****************************************************************************/
-
 static JSObject *
 InnermostStaticScope(JSScript *script, jsbytecode *pc)
 {
-    JS_ASSERT(pc >= script->code && pc < script->code + script->length);
+    JS_ASSERT(script->containsPC(pc));
     JS_ASSERT(JOF_OPTYPE(*pc) == JOF_SCOPECOORD);
 
-    uint32_t blockIndex = GET_UINT32_INDEX(pc + 2 * sizeof(uint16_t));
-
-    if (blockIndex == UINT32_MAX)
-        return script->function();
-    return &script->getObject(blockIndex)->as<StaticBlockObject>();
+    StaticBlockObject *block = GetBlockChainAtPC(script, pc);
+    if (block)
+        return block;
+    return script->function();
 }
 
 Shape *
-js::ScopeCoordinateToStaticScopeShape(JSContext *cx, JSScript *script, jsbytecode *pc)
+js::ScopeCoordinateToStaticScopeShape(JSScript *script, jsbytecode *pc)
 {
-    StaticScopeIter ssi(cx, InnermostStaticScope(script, pc));
+    StaticScopeIter<NoGC> ssi(InnermostStaticScope(script, pc));
     ScopeCoordinate sc(pc);
     while (true) {
         if (ssi.hasDynamicScopeObject()) {
@@ -130,25 +60,57 @@ js::ScopeCoordinateToStaticScopeShape(JSContext *cx, JSScript *script, jsbytecod
     return ssi.scopeShape();
 }
 
-PropertyName *
-js::ScopeCoordinateName(JSContext *cx, JSScript *script, jsbytecode *pc)
+static const uint32_t SCOPE_COORDINATE_NAME_THRESHOLD = 20;
+
+void
+ScopeCoordinateNameCache::purge()
 {
-    Shape::Range<NoGC> r(ScopeCoordinateToStaticScopeShape(cx, script, pc));
+    shape = nullptr;
+    if (map.initialized())
+        map.finish();
+}
+
+PropertyName *
+js::ScopeCoordinateName(ScopeCoordinateNameCache &cache, JSScript *script, jsbytecode *pc)
+{
+    Shape *shape = ScopeCoordinateToStaticScopeShape(script, pc);
+    if (shape != cache.shape && shape->slot() >= SCOPE_COORDINATE_NAME_THRESHOLD) {
+        cache.purge();
+        if (cache.map.init(shape->slot())) {
+            cache.shape = shape;
+            Shape::Range<NoGC> r(shape);
+            while (!r.empty()) {
+                if (!cache.map.putNew(r.front().slot(), r.front().propid())) {
+                    cache.purge();
+                    break;
+                }
+                r.popFront();
+            }
+        }
+    }
+
+    jsid id;
     ScopeCoordinate sc(pc);
-    while (r.front().slot() != sc.slot)
-        r.popFront();
-    jsid id = r.front().propid();
+    if (shape == cache.shape) {
+        ScopeCoordinateNameCache::Map::Ptr p = cache.map.lookup(sc.slot);
+        id = p->value();
+    } else {
+        Shape::Range<NoGC> r(shape);
+        while (r.front().slot() != sc.slot)
+            r.popFront();
+        id = r.front().propid();
+    }
 
     /* Beware nameless destructuring formal. */
     if (!JSID_IS_ATOM(id))
-        return cx->runtime()->atomState.empty;
+        return script->runtimeFromAnyThread()->atomState.empty;
     return JSID_TO_ATOM(id)->asPropertyName();
 }
 
 JSScript *
-js::ScopeCoordinateFunctionScript(JSContext *cx, JSScript *script, jsbytecode *pc)
+js::ScopeCoordinateFunctionScript(JSScript *script, jsbytecode *pc)
 {
-    StaticScopeIter ssi(cx, InnermostStaticScope(script, pc));
+    StaticScopeIter<NoGC> ssi(InnermostStaticScope(script, pc));
     ScopeCoordinate sc(pc);
     while (true) {
         if (ssi.hasDynamicScopeObject()) {
@@ -158,14 +120,14 @@ js::ScopeCoordinateFunctionScript(JSContext *cx, JSScript *script, jsbytecode *p
         }
         ssi++;
     }
-    if (ssi.type() != StaticScopeIter::FUNCTION)
-        return NULL;
+    if (ssi.type() != StaticScopeIter<NoGC>::FUNCTION)
+        return nullptr;
     return ssi.funScript();
 }
 
 /*****************************************************************************/
 
-inline void
+void
 ScopeObject::setEnclosingScope(HandleObject obj)
 {
     JS_ASSERT_IF(obj->is<CallObject>() || obj->is<DeclEnvObject>() || obj->is<BlockObject>(),
@@ -187,12 +149,12 @@ CallObject::create(JSContext *cx, HandleScript script, HandleShape shape, Handle
     gc::InitialHeap heap = script->treatAsRunOnce ? gc::TenuredHeap : gc::DefaultHeap;
     JSObject *obj = JSObject::create(cx, kind, heap, shape, type, slots);
     if (!obj)
-        return NULL;
+        return nullptr;
 
     if (script->treatAsRunOnce) {
         RootedObject nobj(cx, obj);
         if (!JSObject::setSingletonType(cx, nobj))
-            return NULL;
+            return nullptr;
         return &nobj->as<CallObject>();
     }
 
@@ -210,9 +172,9 @@ CallObject::createTemplateObject(JSContext *cx, HandleScript script, gc::Initial
     RootedShape shape(cx, script->bindings.callObjShape());
     JS_ASSERT(shape->getObjectClass() == &class_);
 
-    RootedTypeObject type(cx, cx->getNewType(&class_, NULL));
+    RootedTypeObject type(cx, cx->getNewType(&class_, nullptr));
     if (!type)
-        return NULL;
+        return nullptr;
 
     gc::AllocKind kind = gc::GetGCObjectKind(shape->numFixedSlots());
     JS_ASSERT(CanBeFinalizedInBackground(kind, &class_));
@@ -220,7 +182,7 @@ CallObject::createTemplateObject(JSContext *cx, HandleScript script, gc::Initial
 
     JSObject *obj = JSObject::create(cx, kind, heap, shape, type);
     if (!obj)
-        return NULL;
+        return nullptr;
 
     return &obj->as<CallObject>();
 }
@@ -237,7 +199,7 @@ CallObject::create(JSContext *cx, HandleScript script, HandleObject enclosing, H
     gc::InitialHeap heap = script->treatAsRunOnce ? gc::TenuredHeap : gc::DefaultHeap;
     CallObject *callobj = CallObject::createTemplateObject(cx, script, heap);
     if (!callobj)
-        return NULL;
+        return nullptr;
 
     callobj->as<ScopeObject>().setEnclosingScope(enclosing);
     callobj->initFixedSlot(CALLEE_SLOT, ObjectOrNullValue(callee));
@@ -245,7 +207,7 @@ CallObject::create(JSContext *cx, HandleScript script, HandleObject enclosing, H
     if (script->treatAsRunOnce) {
         Rooted<CallObject*> ncallobj(cx, callobj);
         if (!JSObject::setSingletonType(cx, ncallobj))
-            return NULL;
+            return nullptr;
         return ncallobj;
     }
 
@@ -265,7 +227,7 @@ CallObject::createForFunction(JSContext *cx, HandleObject enclosing, HandleFunct
     if (callee->isNamedLambda()) {
         scopeChain = DeclEnvObject::create(cx, scopeChain, callee);
         if (!scopeChain)
-            return NULL;
+            return nullptr;
     }
 
     RootedScript script(cx, callee->nonLazyScript());
@@ -283,7 +245,7 @@ CallObject::createForFunction(JSContext *cx, AbstractFramePtr frame)
 
     CallObject *callobj = createForFunction(cx, scopeChain, callee);
     if (!callobj)
-        return NULL;
+        return nullptr;
 
     /* Copy in the closed-over formal arguments. */
     for (AliasedFormalIter i(frame.script()); i; i++) {
@@ -299,7 +261,7 @@ CallObject::createForStrictEval(JSContext *cx, AbstractFramePtr frame)
 {
     JS_ASSERT(frame.isStrictEvalFrame());
     JS_ASSERT_IF(frame.isStackFrame(), cx->interpreterFrame() == frame.asStackFrame());
-    JS_ASSERT_IF(frame.isStackFrame(), cx->interpreterRegs().pc == frame.script()->code);
+    JS_ASSERT_IF(frame.isStackFrame(), cx->interpreterRegs().pc == frame.script()->code());
 
     RootedFunction callee(cx);
     RootedScript script(cx, frame.script());
@@ -316,7 +278,7 @@ const Class CallObject::class_ = {
     JS_StrictPropertyStub,   /* setProperty */
     JS_EnumerateStub,
     JS_ResolveStub,
-    NULL                     /* convert: Leave it NULL so we notice if calls ever escape */
+    nullptr                  /* convert: Leave it nullptr so we notice if calls ever escape */
 };
 
 const Class DeclEnvObject::class_ = {
@@ -342,29 +304,29 @@ DeclEnvObject::createTemplateObject(JSContext *cx, HandleFunction fun, gc::Initi
 {
     JS_ASSERT(IsNurseryAllocable(FINALIZE_KIND));
 
-    RootedTypeObject type(cx, cx->getNewType(&class_, NULL));
+    RootedTypeObject type(cx, cx->getNewType(&class_, nullptr));
     if (!type)
-        return NULL;
+        return nullptr;
 
     RootedShape emptyDeclEnvShape(cx);
-    emptyDeclEnvShape = EmptyShape::getInitialShape(cx, &class_, NULL,
-                                                    cx->global(), NULL, FINALIZE_KIND,
+    emptyDeclEnvShape = EmptyShape::getInitialShape(cx, &class_, nullptr,
+                                                    cx->global(), nullptr, FINALIZE_KIND,
                                                     BaseShape::DELEGATE);
     if (!emptyDeclEnvShape)
-        return NULL;
+        return nullptr;
 
     RootedObject obj(cx, JSObject::create(cx, FINALIZE_KIND, heap, emptyDeclEnvShape, type));
     if (!obj)
-        return NULL;
+        return nullptr;
 
     // Assign a fixed slot to a property with the same name as the lambda.
     Rooted<jsid> id(cx, AtomToId(fun->atom()));
     const Class *clasp = obj->getClass();
     unsigned attrs = JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY;
-    if (!JSObject::putProperty(cx, obj, id, clasp->getProperty, clasp->setProperty,
-                               lambdaSlot(), attrs, 0, 0))
+    if (!JSObject::putProperty<SequentialExecution>(cx, obj, id, clasp->getProperty,
+                                                    clasp->setProperty, lambdaSlot(), attrs, 0, 0))
     {
-        return NULL;
+        return nullptr;
     }
 
     JS_ASSERT(!obj->hasDynamicSlots());
@@ -376,7 +338,7 @@ DeclEnvObject::create(JSContext *cx, HandleObject enclosing, HandleFunction call
 {
     RootedObject obj(cx, createTemplateObject(cx, callee, gc::DefaultHeap));
     if (!obj)
-        return NULL;
+        return nullptr;
 
     obj->as<ScopeObject>().setEnclosingScope(enclosing);
     obj->setFixedSlot(lambdaSlot(), ObjectValue(*callee));
@@ -388,23 +350,24 @@ WithObject::create(JSContext *cx, HandleObject proto, HandleObject enclosing, ui
 {
     RootedTypeObject type(cx, cx->getNewType(&class_, proto.get()));
     if (!type)
-        return NULL;
+        return nullptr;
 
     RootedShape shape(cx, EmptyShape::getInitialShape(cx, &class_, TaggedProto(proto),
-                                                      &enclosing->global(), NULL, FINALIZE_KIND));
+                                                      &enclosing->global(), nullptr,
+                                                      FINALIZE_KIND));
     if (!shape)
-        return NULL;
+        return nullptr;
 
     RootedObject obj(cx, JSObject::create(cx, FINALIZE_KIND, gc::DefaultHeap, shape, type));
     if (!obj)
-        return NULL;
+        return nullptr;
 
     obj->as<ScopeObject>().setEnclosingScope(enclosing);
     obj->setReservedSlot(DEPTH_SLOT, PrivateUint32Value(depth));
 
     JSObject *thisp = JSObject::thisObject(cx, proto);
     if (!thisp)
-        return NULL;
+        return nullptr;
 
     obj->setFixedSlot(THIS_SLOT, ObjectValue(*thisp));
 
@@ -432,7 +395,7 @@ with_LookupElement(JSContext *cx, HandleObject obj, uint32_t index,
                    MutableHandleObject objp, MutableHandleShape propp)
 {
     RootedId id(cx);
-    if (!IndexToId(cx, index, &id))
+    if (!IndexToId(cx, index, id.address()))
         return false;
     return with_LookupGeneric(cx, obj, id, objp, propp);
 }
@@ -466,7 +429,7 @@ with_GetElement(JSContext *cx, HandleObject obj, HandleObject receiver, uint32_t
                 MutableHandleValue vp)
 {
     RootedId id(cx);
-    if (!IndexToId(cx, index, &id))
+    if (!IndexToId(cx, index, id.address()))
         return false;
     return with_GetGeneric(cx, obj, receiver, id, vp);
 }
@@ -574,26 +537,26 @@ const Class WithObject::class_ = {
     JS_EnumerateStub,
     JS_ResolveStub,
     JS_ConvertStub,
-    NULL,                    /* finalize */
-    NULL,                    /* checkAccess */
-    NULL,                    /* call        */
-    NULL,                    /* hasInstance */
-    NULL,                    /* construct   */
-    NULL,                    /* trace       */
+    nullptr,                 /* finalize */
+    nullptr,                 /* checkAccess */
+    nullptr,                 /* call        */
+    nullptr,                 /* hasInstance */
+    nullptr,                 /* construct   */
+    nullptr,                 /* trace       */
     JS_NULL_CLASS_EXT,
     {
         with_LookupGeneric,
         with_LookupProperty,
         with_LookupElement,
         with_LookupSpecial,
-        NULL,             /* defineGeneric */
-        NULL,             /* defineProperty */
-        NULL,             /* defineElement */
-        NULL,             /* defineSpecial */
+        nullptr,             /* defineGeneric */
+        nullptr,             /* defineProperty */
+        nullptr,             /* defineElement */
+        nullptr,             /* defineSpecial */
         with_GetGeneric,
         with_GetProperty,
         with_GetElement,
-        NULL,             /* getElementIfPresent */
+        nullptr,             /* getElementIfPresent */
         with_GetSpecial,
         with_SetGeneric,
         with_SetProperty,
@@ -604,6 +567,7 @@ const Class WithObject::class_ = {
         with_DeleteProperty,
         with_DeleteElement,
         with_DeleteSpecial,
+        nullptr, nullptr, /* watch/unwatch */
         with_Enumerate,
         with_ThisObject,
     }
@@ -619,20 +583,20 @@ ClonedBlockObject::create(JSContext *cx, Handle<StaticBlockObject *> block, Abst
 
     RootedTypeObject type(cx, cx->getNewType(&BlockObject::class_, block.get()));
     if (!type)
-        return NULL;
+        return nullptr;
 
     RootedShape shape(cx, block->lastProperty());
 
     RootedObject obj(cx, JSObject::create(cx, FINALIZE_KIND, gc::TenuredHeap, shape, type));
     if (!obj)
-        return NULL;
+        return nullptr;
 
     /* Set the parent if necessary, as for call objects. */
     if (&frame.scopeChain()->global() != obj->getParent()) {
-        JS_ASSERT(obj->getParent() == NULL);
+        JS_ASSERT(obj->getParent() == nullptr);
         Rooted<GlobalObject*> global(cx, &frame.scopeChain()->global());
         if (!JSObject::setParent(cx, obj, global))
-            return NULL;
+            return nullptr;
     }
 
     JS_ASSERT(!obj->inDictionaryMode());
@@ -671,19 +635,19 @@ ClonedBlockObject::copyUnaliasedValues(AbstractFramePtr frame)
 StaticBlockObject *
 StaticBlockObject::create(ExclusiveContext *cx)
 {
-    RootedTypeObject type(cx, cx->getNewType(&BlockObject::class_, NULL));
+    RootedTypeObject type(cx, cx->getNewType(&BlockObject::class_, nullptr));
     if (!type)
-        return NULL;
+        return nullptr;
 
     RootedShape emptyBlockShape(cx);
-    emptyBlockShape = EmptyShape::getInitialShape(cx, &BlockObject::class_, NULL, NULL, NULL,
-                                                  FINALIZE_KIND, BaseShape::DELEGATE);
+    emptyBlockShape = EmptyShape::getInitialShape(cx, &BlockObject::class_, nullptr, nullptr,
+                                                  nullptr, FINALIZE_KIND, BaseShape::DELEGATE);
     if (!emptyBlockShape)
-        return NULL;
+        return nullptr;
 
     JSObject *obj = JSObject::create(cx, FINALIZE_KIND, gc::TenuredHeap, emptyBlockShape, type);
     if (!obj)
-        return NULL;
+        return nullptr;
 
     return &obj->as<StaticBlockObject>();
 }
@@ -700,7 +664,7 @@ StaticBlockObject::addVar(ExclusiveContext *cx, Handle<StaticBlockObject*> block
     Shape **spp;
     if (Shape::search(cx, block->lastProperty(), id, &spp, true)) {
         *redeclared = true;
-        return NULL;
+        return nullptr;
     }
 
     /*
@@ -708,10 +672,12 @@ StaticBlockObject::addVar(ExclusiveContext *cx, Handle<StaticBlockObject*> block
      * block's shape later.
      */
     uint32_t slot = JSSLOT_FREE(&BlockObject::class_) + index;
-    return JSObject::addPropertyInternal(cx, block, id, /* getter = */ NULL, /* setter = */ NULL,
-                                         slot, JSPROP_ENUMERATE | JSPROP_PERMANENT,
-                                         Shape::HAS_SHORTID, index, spp,
-                                         /* allowDictionary = */ false);
+    return JSObject::addPropertyInternal<SequentialExecution>(
+        cx, block, id,
+        /* getter = */ nullptr, /* setter = */ nullptr,
+        slot, JSPROP_ENUMERATE | JSPROP_PERMANENT,
+        Shape::HAS_SHORTID, index, spp,
+        /* allowDictionary = */ false);
 }
 
 const Class BlockObject::class_ = {
@@ -730,7 +696,7 @@ const Class BlockObject::class_ = {
 
 template<XDRMode mode>
 bool
-js::XDRStaticBlockObject(XDRState<mode> *xdr, HandleObject enclosingScope, HandleScript script,
+js::XDRStaticBlockObject(XDRState<mode> *xdr, HandleObject enclosingScope,
                          StaticBlockObject **objp)
 {
     /* NB: Keep this in sync with CloneStaticBlockObject. */
@@ -834,10 +800,10 @@ js::XDRStaticBlockObject(XDRState<mode> *xdr, HandleObject enclosingScope, Handl
 }
 
 template bool
-js::XDRStaticBlockObject(XDRState<XDR_ENCODE> *, HandleObject, HandleScript, StaticBlockObject **);
+js::XDRStaticBlockObject(XDRState<XDR_ENCODE> *, HandleObject, StaticBlockObject **);
 
 template bool
-js::XDRStaticBlockObject(XDRState<XDR_DECODE> *, HandleObject, HandleScript, StaticBlockObject **);
+js::XDRStaticBlockObject(XDRState<XDR_DECODE> *, HandleObject, StaticBlockObject **);
 
 JSObject *
 js::CloneStaticBlockObject(JSContext *cx, HandleObject enclosingScope, Handle<StaticBlockObject*> srcBlock)
@@ -846,7 +812,7 @@ js::CloneStaticBlockObject(JSContext *cx, HandleObject enclosingScope, Handle<St
 
     Rooted<StaticBlockObject*> clone(cx, StaticBlockObject::create(cx));
     if (!clone)
-        return NULL;
+        return nullptr;
 
     clone->initEnclosingStaticScope(enclosingScope);
     clone->setStackDepth(srcBlock->stackDepth());
@@ -854,7 +820,7 @@ js::CloneStaticBlockObject(JSContext *cx, HandleObject enclosingScope, Handle<St
     /* Shape::Range is reverse order, so build a list in forward order. */
     AutoShapeVector shapes(cx);
     if (!shapes.growBy(srcBlock->slotCount()))
-        return NULL;
+        return nullptr;
     for (Shape::Range<NoGC> r(srcBlock->lastProperty()); !r.empty(); r.popFront())
         shapes[r.front().shortid()] = &r.front();
 
@@ -865,7 +831,7 @@ js::CloneStaticBlockObject(JSContext *cx, HandleObject enclosingScope, Handle<St
         bool redeclared;
         if (!StaticBlockObject::addVar(cx, clone, id, i, &redeclared)) {
             JS_ASSERT(!redeclared);
-            return NULL;
+            return nullptr;
         }
 
         clone->setAliased(i, srcBlock->isAliased(i));
@@ -881,7 +847,7 @@ js::CloneStaticBlockObject(JSContext *cx, HandleObject enclosingScope, Handle<St
 static inline JSAtom *
 CallObjectLambdaName(JSFunction &fun)
 {
-    return fun.isNamedLambda() ? fun.atom() : NULL;
+    return fun.isNamedLambda() ? fun.atom() : nullptr;
 }
 
 ScopeIter::ScopeIter(const ScopeIter &si, JSContext *cx
@@ -901,7 +867,7 @@ ScopeIter::ScopeIter(JSObject &enclosingScope, JSContext *cx
   : cx(cx),
     frame_(NullFramePtr()),
     cur_(cx, &enclosingScope),
-    block_(cx, NULL),
+    block_(cx, nullptr),
     type_(Type(-1))
 {
     MOZ_GUARD_OBJECT_NOTIFIER_INIT;
@@ -963,7 +929,7 @@ ScopeIter::ScopeIter(AbstractFramePtr frame, ScopeObject &scope, JSContext *cx
         JS_ASSERT_IF(cur_->is<ClonedBlockObject>(),
                      cur_->as<ClonedBlockObject>().staticBlock() == *block_);
     } else {
-        block_ = NULL;
+        block_ = nullptr;
     }
     settle();
     MOZ_GUARD_OBJECT_NOTIFIER_INIT;
@@ -1288,7 +1254,7 @@ class DebugScopeProxy : public BaseProxyHandler
     static bool checkForMissingArguments(JSContext *cx, jsid id, ScopeObject &scope,
                                          ArgumentsObject **maybeArgsObj)
     {
-        *maybeArgsObj = NULL;
+        *maybeArgsObj = nullptr;
 
         if (!isArguments(cx, id) || !isFunctionScope(scope))
             return true;
@@ -1298,7 +1264,7 @@ class DebugScopeProxy : public BaseProxyHandler
 
         AbstractFramePtr maybeframe = DebugScopes::hasLiveFrame(scope);
         if (!maybeframe) {
-            JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL, JSMSG_DEBUG_NOT_LIVE,
+            JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr, JSMSG_DEBUG_NOT_LIVE,
                                  "Debugger scope");
             return false;
         }
@@ -1324,7 +1290,7 @@ class DebugScopeProxy : public BaseProxyHandler
     bool preventExtensions(JSContext *cx, HandleObject proxy) MOZ_OVERRIDE
     {
         // See above.
-        JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL, JSMSG_CANT_CHANGE_EXTENSIBILITY);
+        JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr, JSMSG_CANT_CHANGE_EXTENSIBILITY);
         return false;
     }
 
@@ -1351,8 +1317,8 @@ class DebugScopeProxy : public BaseProxyHandler
             desc.setAttributes(JSPROP_READONLY | JSPROP_ENUMERATE | JSPROP_PERMANENT);
             desc.value().setObject(*maybeArgsObj);
             desc.setShortId(0);
-            desc.setGetter(NULL);
-            desc.setSetter(NULL);
+            desc.setGetter(nullptr);
+            desc.setSetter(nullptr);
             return true;
         }
 
@@ -1362,8 +1328,8 @@ class DebugScopeProxy : public BaseProxyHandler
             desc.setAttributes(JSPROP_READONLY | JSPROP_ENUMERATE | JSPROP_PERMANENT);
             desc.value().set(v);
             desc.setShortId(0);
-            desc.setGetter(NULL);
-            desc.setSetter(NULL);
+            desc.setGetter(nullptr);
+            desc.setSetter(nullptr);
             return true;
         }
 
@@ -1491,7 +1457,7 @@ class DebugScopeProxy : public BaseProxyHandler
     {
         RootedValue idval(cx, IdToValue(id));
         return js_ReportValueErrorFlags(cx, JSREPORT_ERROR, JSMSG_CANT_DELETE,
-                                        JSDVG_IGNORE_STACK, idval, NullPtr(), NULL, NULL);
+                                        JSDVG_IGNORE_STACK, idval, NullPtr(), nullptr, nullptr);
     }
 };
 
@@ -1506,9 +1472,9 @@ DebugScopeObject::create(JSContext *cx, ScopeObject &scope, HandleObject enclosi
     JS_ASSERT(scope.compartment() == cx->compartment());
     RootedValue priv(cx, ObjectValue(scope));
     JSObject *obj = NewProxyObject(cx, &DebugScopeProxy::singleton, priv,
-                                   NULL /* proto */, &scope.global(), ProxyNotCallable);
+                                   nullptr /* proto */, &scope.global());
     if (!obj)
-        return NULL;
+        return nullptr;
 
     JS_ASSERT(!enclosing->is<ScopeObject>());
 
@@ -1541,7 +1507,7 @@ DebugScopeObject::maybeSnapshot() const
 void
 DebugScopeObject::initSnapshot(JSObject &o)
 {
-    JS_ASSERT(maybeSnapshot() == NULL);
+    JS_ASSERT(maybeSnapshot() == nullptr);
     setExtra(SNAPSHOT_EXTRA, ObjectValue(o));
 }
 
@@ -1553,14 +1519,39 @@ DebugScopeObject::isForDeclarative() const
 }
 
 bool
-js_IsDebugScopeSlow(ObjectProxyObject *proxy)
+js_IsDebugScopeSlow(ProxyObject *proxy)
 {
-    JS_ASSERT(proxy->hasClass(&ObjectProxyObject::class_));
+    JS_ASSERT(proxy->hasClass(&ProxyObject::uncallableClass_));
     return proxy->handler() == &DebugScopeProxy::singleton;
 }
 
 /*****************************************************************************/
 
+/* static */ JS_ALWAYS_INLINE void
+DebugScopes::proxiedScopesPostWriteBarrier(JSRuntime *rt, ObjectWeakMap *map,
+                                           const EncapsulatedPtr<JSObject> &key)
+{
+#ifdef JSGC_GENERATIONAL
+    /*
+     * Strip the barriers from the type before inserting into the store buffer.
+     * This will automatically ensure that barriers do not fire during GC.
+     */
+    typedef WeakMap<JSObject *, JSObject *> UnbarrieredMap;
+    typedef gc::HashKeyRef<UnbarrieredMap, JSObject *> Ref;
+    if (key && IsInsideNursery(rt, key))
+        rt->gcStoreBuffer.putGeneric(Ref(reinterpret_cast<UnbarrieredMap *>(map), key.get()));
+#endif
+}
+
+/* static */ JS_ALWAYS_INLINE void
+DebugScopes::liveScopesPostWriteBarrier(JSRuntime *rt, LiveScopeMap *map, ScopeObject *key)
+{
+#ifdef JSGC_GENERATIONAL
+    if (key && IsInsideNursery(rt, key))
+        rt->gcStoreBuffer.putGeneric(gc::HashKeyRef<LiveScopeMap, ScopeObject *>(map, key));
+#endif
+}
+
 DebugScopes::DebugScopes(JSContext *cx)
  : proxiedScopes(cx),
    missingScopes(cx->runtime()),
@@ -1600,13 +1591,13 @@ DebugScopes::sweep(JSRuntime *rt)
      * creating an uncollectable cycle with suspended generator frames.
      */
     for (MissingScopeMap::Enum e(missingScopes); !e.empty(); e.popFront()) {
-        if (IsObjectAboutToBeFinalized(e.front().value.unsafeGet()))
+        if (IsObjectAboutToBeFinalized(e.front().value().unsafeGet()))
             e.removeFront();
     }
 
     for (LiveScopeMap::Enum e(liveScopes); !e.empty(); e.popFront()) {
-        ScopeObject *scope = e.front().key;
-        AbstractFramePtr frame = e.front().value;
+        ScopeObject *scope = e.front().key();
+        AbstractFramePtr frame = e.front().value();
 
         /*
          * Scopes can be finalized when a debugger-synthesized ScopeObject is
@@ -1657,7 +1648,7 @@ DebugScopes::ensureCompartmentData(JSContext *cx)
         return c->debugScopes;
 
     js_ReportOutOfMemory(cx);
-    return NULL;
+    return nullptr;
 }
 
 DebugScopeObject *
@@ -1665,14 +1656,14 @@ DebugScopes::hasDebugScope(JSContext *cx, ScopeObject &scope)
 {
     DebugScopes *scopes = scope.compartment()->debugScopes;
     if (!scopes)
-        return NULL;
+        return nullptr;
 
     if (ObjectWeakMap::Ptr p = scopes->proxiedScopes.lookup(&scope)) {
         JS_ASSERT(CanUseDebugScopeMaps(cx));
-        return &p->value->as<DebugScopeObject>();
+        return &p->value()->as<DebugScopeObject>();
     }
 
-    return NULL;
+    return nullptr;
 }
 
 bool
@@ -1694,7 +1685,7 @@ DebugScopes::addDebugScope(JSContext *cx, ScopeObject &scope, DebugScopeObject &
         return false;
     }
 
-    HashTableWriteBarrierPost(cx->runtime(), &scopes->proxiedScopes, &scope);
+    proxiedScopesPostWriteBarrier(cx->runtime(), &scopes->proxiedScopes, &scope);
     return true;
 }
 
@@ -1705,13 +1696,13 @@ DebugScopes::hasDebugScope(JSContext *cx, const ScopeIter &si)
 
     DebugScopes *scopes = cx->compartment()->debugScopes;
     if (!scopes)
-        return NULL;
+        return nullptr;
 
     if (MissingScopeMap::Ptr p = scopes->missingScopes.lookup(si)) {
         JS_ASSERT(CanUseDebugScopeMaps(cx));
-        return p->value;
+        return p->value();
     }
-    return NULL;
+    return nullptr;
 }
 
 bool
@@ -1738,7 +1729,7 @@ DebugScopes::addDebugScope(JSContext *cx, const ScopeIter &si, DebugScopeObject
         js_ReportOutOfMemory(cx);
         return false;
     }
-    HashTableWriteBarrierPost(cx->runtime(), &scopes->liveScopes, &debugScope.scope());
+    liveScopesPostWriteBarrier(cx->runtime(), &scopes->liveScopes, &debugScope.scope());
 
     return true;
 }
@@ -1753,7 +1744,7 @@ DebugScopes::onPopCall(AbstractFramePtr frame, JSContext *cx)
     if (!scopes)
         return;
 
-    Rooted<DebugScopeObject*> debugScope(cx, NULL);
+    Rooted<DebugScopeObject*> debugScope(cx, nullptr);
 
     if (frame.fun()->isHeavyweight()) {
         /*
@@ -1766,11 +1757,11 @@ DebugScopes::onPopCall(AbstractFramePtr frame, JSContext *cx)
         CallObject &callobj = frame.scopeChain()->as<CallObject>();
         scopes->liveScopes.remove(&callobj);
         if (ObjectWeakMap::Ptr p = scopes->proxiedScopes.lookup(&callobj))
-            debugScope = &p->value->as<DebugScopeObject>();
+            debugScope = &p->value()->as<DebugScopeObject>();
     } else {
         ScopeIter si(frame, cx);
         if (MissingScopeMap::Ptr p = scopes->missingScopes.lookup(si)) {
-            debugScope = p->value;
+            debugScope = p->value();
             scopes->liveScopes.remove(&debugScope->scope().as<CallObject>());
             scopes->missingScopes.remove(p);
         }
@@ -1784,7 +1775,7 @@ DebugScopes::onPopCall(AbstractFramePtr frame, JSContext *cx)
      *
      * Note: since it is simplest for this function to be infallible, failure
      * in this code will be silently ignored. This does not break any
-     * invariants since DebugScopeObject::maybeSnapshot can already be NULL.
+     * invariants since DebugScopeObject::maybeSnapshot can already be nullptr.
      */
     if (debugScope) {
         /*
@@ -1801,7 +1792,7 @@ DebugScopes::onPopCall(AbstractFramePtr frame, JSContext *cx)
          * but are aliased via the arguments object.
          */
         RootedScript script(cx, frame.script());
-        if (script->needsArgsObj() && frame.hasArgsObj()) {
+        if (script->analyzedArgsUsage() && script->needsArgsObj() && frame.hasArgsObj()) {
             for (unsigned i = 0; i < frame.numFormalArgs(); ++i) {
                 if (script->formalLivesInArgumentsObject(i))
                     vec[i] = frame.argsObj().arg(i);
@@ -1839,7 +1830,7 @@ DebugScopes::onPopBlock(JSContext *cx, AbstractFramePtr frame)
     } else {
         ScopeIter si(frame, cx);
         if (MissingScopeMap::Ptr p = scopes->missingScopes.lookup(si)) {
-            ClonedBlockObject &clone = p->value->scope().as<ClonedBlockObject>();
+            ClonedBlockObject &clone = p->value()->scope().as<ClonedBlockObject>();
             clone.copyUnaliasedValues(frame);
             scopes->liveScopes.remove(&clone);
             scopes->missingScopes.remove(p);
@@ -1889,17 +1880,17 @@ DebugScopes::onGeneratorFrameChange(AbstractFramePtr from, AbstractFramePtr to,
             JS_ASSERT(toIter.scope().compartment() == cx->compartment());
             LiveScopeMap::AddPtr livePtr = scopes->liveScopes.lookupForAdd(&toIter.scope());
             if (livePtr) {
-                livePtr->value = to;
+                livePtr->value() = to;
             } else {
                 scopes->liveScopes.add(livePtr, &toIter.scope(), to);  // OOM here?
-                HashTableWriteBarrierPost(cx->runtime(), &scopes->liveScopes, &toIter.scope());
+                liveScopesPostWriteBarrier(cx->runtime(), &scopes->liveScopes, &toIter.scope());
             }
         } else {
             ScopeIter si(toIter, from, cx);
             JS_ASSERT(si.frame().scopeChain()->compartment() == cx->compartment());
             if (MissingScopeMap::Ptr p = scopes->missingScopes.lookup(si)) {
-                DebugScopeObject &debugScope = *p->value;
-                scopes->liveScopes.lookup(&debugScope.scope())->value = to;
+                DebugScopeObject &debugScope = *p->value();
+                scopes->liveScopes.lookup(&debugScope.scope())->value() = to;
                 scopes->missingScopes.remove(p);
                 scopes->missingScopes.put(toIter, &debugScope);  // OOM here?
             }
@@ -1954,7 +1945,7 @@ DebugScopes::updateLiveScopes(JSContext *cx)
                     return false;
                 if (!scopes->liveScopes.put(&si.scope(), frame))
                     return false;
-                HashTableWriteBarrierPost(cx->runtime(), &scopes->liveScopes, &si.scope());
+                liveScopesPostWriteBarrier(cx->runtime(), &scopes->liveScopes, &si.scope());
             }
         }
 
@@ -1975,7 +1966,7 @@ DebugScopes::hasLiveFrame(ScopeObject &scope)
         return NullFramePtr();
 
     if (LiveScopeMap::Ptr p = scopes->liveScopes.lookup(&scope)) {
-        AbstractFramePtr frame = p->value;
+        AbstractFramePtr frame = p->value();
 
         /*
          * Since liveScopes is effectively a weak pointer, we need a read
@@ -2009,22 +2000,22 @@ GetDebugScopeForScope(JSContext *cx, Handle<ScopeObject*> scope, const ScopeIter
 
     RootedObject enclosingDebug(cx, GetDebugScope(cx, enclosing));
     if (!enclosingDebug)
-        return NULL;
+        return nullptr;
 
     JSObject &maybeDecl = scope->enclosingScope();
     if (maybeDecl.is<DeclEnvObject>()) {
         JS_ASSERT(CallObjectLambdaName(scope->as<CallObject>().callee()));
         enclosingDebug = DebugScopeObject::create(cx, maybeDecl.as<DeclEnvObject>(), enclosingDebug);
         if (!enclosingDebug)
-            return NULL;
+            return nullptr;
     }
 
     DebugScopeObject *debugScope = DebugScopeObject::create(cx, *scope, enclosingDebug);
     if (!debugScope)
-        return NULL;
+        return nullptr;
 
     if (!DebugScopes::addDebugScope(cx, *scope, *debugScope))
-        return NULL;
+        return nullptr;
 
     return debugScope;
 }
@@ -2038,7 +2029,7 @@ GetDebugScopeForMissing(JSContext *cx, const ScopeIter &si)
     ScopeIter copy(si, cx);
     RootedObject enclosingDebug(cx, GetDebugScope(cx, ++copy));
     if (!enclosingDebug)
-        return NULL;
+        return nullptr;
 
     /*
      * Create the missing scope object. For block objects, this takes care of
@@ -2051,19 +2042,19 @@ GetDebugScopeForMissing(JSContext *cx, const ScopeIter &si)
      * scopes must not be put on the frame's scope chain; instead, they are
      * maintained via DebugScopes hooks.
      */
-    DebugScopeObject *debugScope = NULL;
+    DebugScopeObject *debugScope = nullptr;
     switch (si.type()) {
       case ScopeIter::Call: {
         Rooted<CallObject*> callobj(cx, CallObject::createForFunction(cx, si.frame()));
         if (!callobj)
-            return NULL;
+            return nullptr;
 
         if (callobj->enclosingScope().is<DeclEnvObject>()) {
             JS_ASSERT(CallObjectLambdaName(callobj->callee()));
             DeclEnvObject &declenv = callobj->enclosingScope().as<DeclEnvObject>();
             enclosingDebug = DebugScopeObject::create(cx, declenv, enclosingDebug);
             if (!enclosingDebug)
-                return NULL;
+                return nullptr;
         }
 
         debugScope = DebugScopeObject::create(cx, *callobj, enclosingDebug);
@@ -2073,7 +2064,7 @@ GetDebugScopeForMissing(JSContext *cx, const ScopeIter &si)
         Rooted<StaticBlockObject *> staticBlock(cx, &si.staticBlock());
         ClonedBlockObject *block = ClonedBlockObject::create(cx, staticBlock, si.frame());
         if (!block)
-            return NULL;
+            return nullptr;
 
         debugScope = DebugScopeObject::create(cx, *block, enclosingDebug);
         break;
@@ -2083,10 +2074,10 @@ GetDebugScopeForMissing(JSContext *cx, const ScopeIter &si)
         MOZ_ASSUME_UNREACHABLE("should already have a scope");
     }
     if (!debugScope)
-        return NULL;
+        return nullptr;
 
     if (!DebugScopes::addDebugScope(cx, si, *debugScope))
-        return NULL;
+        return nullptr;
 
     return debugScope;
 }
@@ -2121,7 +2112,7 @@ GetDebugScope(JSContext *cx, JSObject &obj)
 static JSObject *
 GetDebugScope(JSContext *cx, const ScopeIter &si)
 {
-    JS_CHECK_RECURSION(cx, return NULL);
+    JS_CHECK_RECURSION(cx, return nullptr);
 
     if (si.done())
         return GetDebugScope(cx, si.enclosingScope());
@@ -2141,7 +2132,7 @@ js::GetDebugScopeForFunction(JSContext *cx, HandleFunction fun)
     assertSameCompartment(cx, fun);
     JS_ASSERT(cx->compartment()->debugMode());
     if (!DebugScopes::updateLiveScopes(cx))
-        return NULL;
+        return nullptr;
     return GetDebugScope(cx, *fun->environment());
 }
 
@@ -2150,7 +2141,7 @@ js::GetDebugScopeForFrame(JSContext *cx, AbstractFramePtr frame)
 {
     assertSameCompartment(cx, frame);
     if (CanUseDebugScopeMaps(cx) && !DebugScopes::updateLiveScopes(cx))
-        return NULL;
+        return nullptr;
     ScopeIter si(frame, cx);
     return GetDebugScope(cx, si);
 }
@@ -2176,10 +2167,7 @@ RemoveReferencedNames(JSContext *cx, HandleScript script, PropertyNameSet &remai
     //   these names and putting eval in an inner script is bad news if you
     //   care about entraining variables unnecessarily.
 
-    for (jsbytecode *pc = script->code;
-         pc != script->code + script->length;
-         pc += GetBytecodeLength(pc))
-    {
+    for (jsbytecode *pc = script->code(); pc != script->codeEnd(); pc += GetBytecodeLength(pc)) {
         PropertyName *name;
 
         switch (JSOp(*pc)) {
@@ -2192,11 +2180,11 @@ RemoveReferencedNames(JSContext *cx, HandleScript script, PropertyNameSet &remai
           case JSOP_GETALIASEDVAR:
           case JSOP_CALLALIASEDVAR:
           case JSOP_SETALIASEDVAR:
-            name = ScopeCoordinateName(cx, script, pc);
+            name = ScopeCoordinateName(cx->runtime()->scopeCoordinateNameCache, script, pc);
             break;
 
           default:
-            name = NULL;
+            name = nullptr;
             break;
         }
 
