@@ -4,14 +4,19 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "vm/SelfHosting.h"
+
 #include "jscntxt.h"
 #include "jscompartment.h"
 #include "jsfriendapi.h"
+#include "jshashutil.h"
 #include "jsobj.h"
+#include "jswrapper.h"
 #include "selfhosted.out.h"
 
 #include "builtin/Intl.h"
 #include "builtin/ParallelArray.h"
+#include "builtin/TypedObject.h"
 #include "gc/Marking.h"
 #include "vm/ForkJoin.h"
 #include "vm/Interpreter.h"
@@ -26,78 +31,6 @@
 using namespace js;
 using namespace js::selfhosted;
 
-namespace js {
-
-/*
- * A linked-list container for self-hosted prototypes that have need of a
- * Class for reserved slots. These are freed when self-hosting is destroyed at
- * the destruction of the last context.
- */
-struct SelfHostedClass
-{
-    /* Next class in the list. */
-    SelfHostedClass *next;
-
-    /* Class of instances. */
-    Class class_;
-
-    /*
-     * Create a new self-hosted proto with its class set to a new dynamically
-     * allocated class with numSlots reserved slots.
-     */
-    static JSObject *newPrototype(JSContext *cx, uint32_t numSlots);
-
-    static bool is(JSContext *cx, const Class *clasp);
-
-    SelfHostedClass(const char *name, uint32_t numSlots);
-};
-
-} /* namespace js */
-
-JSObject *
-SelfHostedClass::newPrototype(JSContext *cx, uint32_t numSlots)
-{
-    /* Allocate a new self hosted class and prepend it to the list. */
-    SelfHostedClass *shClass = cx->new_<SelfHostedClass>("Self-hosted Class", numSlots);
-    if (!shClass)
-        return NULL;
-    cx->runtime()->addSelfHostedClass(shClass);
-
-    Rooted<GlobalObject *> global(cx, cx->global());
-    RootedObject proto(cx, global->createBlankPrototype(cx, &shClass->class_));
-    if (!proto)
-        return NULL;
-
-    return proto;
-}
-
-bool
-SelfHostedClass::is(JSContext *cx, const Class *clasp)
-{
-    SelfHostedClass *shClass = cx->runtime()->selfHostedClasses();
-    while (shClass) {
-        if (clasp == &shClass->class_)
-            return true;
-        shClass = shClass->next;
-    }
-    return false;
-}
-
-SelfHostedClass::SelfHostedClass(const char *name, uint32_t numSlots)
-{
-    mozilla::PodZero(this);
-
-    class_.name = name;
-    class_.flags = JSCLASS_HAS_RESERVED_SLOTS(numSlots);
-    class_.addProperty = JS_PropertyStub;
-    class_.delProperty = JS_DeletePropertyStub;
-    class_.getProperty = JS_PropertyStub;
-    class_.setProperty = JS_StrictPropertyStub;
-    class_.enumerate = JS_EnumerateStub;
-    class_.resolve = JS_ResolveStub;
-    class_.convert = JS_ConvertStub;
-}
-
 static void
 selfHosting_ErrorReporter(JSContext *cx, const char *message, JSErrorReport *report)
 {
@@ -109,7 +42,7 @@ static const JSClass self_hosting_global_class = {
     JS_PropertyStub,  JS_DeletePropertyStub,
     JS_PropertyStub,  JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub,
-    JS_ConvertStub,   NULL
+    JS_ConvertStub,   nullptr
 };
 
 bool
@@ -152,7 +85,13 @@ js::intrinsic_ThrowError(JSContext *cx, unsigned argc, Value *vp)
     JS_ASSERT(args.length() >= 1);
     uint32_t errorNumber = args[0].toInt32();
 
-    char *errorArgs[3] = {NULL, NULL, NULL};
+#ifdef DEBUG
+    const JSErrorFormatString *efs =
+        js_GetLocalizedErrorMessage(cx, nullptr, nullptr, errorNumber);
+    JS_ASSERT(efs->argCount == args.length() - 1);
+#endif
+
+    char *errorArgs[3] = {nullptr, nullptr, nullptr};
     for (unsigned i = 1; i < 4 && i < args.length(); i++) {
         RootedValue val(cx, args[i]);
         if (val.isInt32()) {
@@ -169,7 +108,7 @@ js::intrinsic_ThrowError(JSContext *cx, unsigned argc, Value *vp)
             return false;
     }
 
-    JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL, errorNumber,
+    JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr, errorNumber,
                          errorArgs[0], errorArgs[1], errorArgs[2]);
     for (unsigned i = 0; i < 3; i++)
         js_free(errorArgs[i]);
@@ -214,7 +153,7 @@ intrinsic_MakeConstructible(JSContext *cx, unsigned argc, Value *vp)
     // Normal .prototype properties aren't enumerable.  But for this to clone
     // correctly, it must be enumerable.
     RootedObject ctor(cx, &args[0].toObject());
-    if (!JSObject::defineProperty(cx, ctor, cx->names().classPrototype, args[1],
+    if (!JSObject::defineProperty(cx, ctor, cx->names().prototype, args[1],
                                   JS_PropertyStub, JS_StrictPropertyStub,
                                   JSPROP_READONLY | JSPROP_ENUMERATE | JSPROP_PERMANENT))
     {
@@ -476,7 +415,7 @@ js::intrinsic_UnsafePutElements(JSContext *cx, unsigned argc, Value *vp)
 
         if (arrobj->isNative()) {
             JS_ASSERT(idx < arrobj->getDenseInitializedLength());
-            JSObject::setDenseElementWithType(cx, arrobj, idx, args[elemi]);
+            arrobj->setDenseElementWithType(cx, idx, args[elemi]);
         } else {
             JS_ASSERT(idx < arrobj->as<TypedArrayObject>().length());
             RootedValue tmp(cx, args[elemi]);
@@ -515,37 +454,6 @@ js::intrinsic_UnsafeGetReservedSlot(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
-static bool
-intrinsic_NewClassPrototype(JSContext *cx, unsigned argc, Value *vp)
-{
-    CallArgs args = CallArgsFromVp(argc, vp);
-    JS_ASSERT(args.length() == 1);
-    JS_ASSERT(args[0].isInt32());
-
-    JSObject *proto = SelfHostedClass::newPrototype(cx, args[0].toPrivateUint32());
-    if (!proto)
-        return false;
-
-    args.rval().setObject(*proto);
-    return true;
-}
-
-bool
-js::intrinsic_NewObjectWithClassPrototype(JSContext *cx, unsigned argc, Value *vp)
-{
-    CallArgs args = CallArgsFromVp(argc, vp);
-    JS_ASSERT(args.length() == 1);
-    JS_ASSERT(args[0].isObject());
-
-    RootedObject proto(cx, &args[0].toObject());
-    JSObject *result = NewObjectWithGivenProto(cx, proto->getClass(), proto, cx->global());
-    if (!result)
-        return false;
-
-    args.rval().setObject(*result);
-    return true;
-}
-
 bool
 js::intrinsic_HaveSameClass(JSContext *cx, unsigned argc, Value *vp)
 {
@@ -558,6 +466,94 @@ js::intrinsic_HaveSameClass(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
+bool
+js::intrinsic_IsPackedArray(JSContext *cx, unsigned argc, Value *vp)
+{
+    CallArgs args = CallArgsFromVp(argc, vp);
+    JS_ASSERT(args.length() == 1);
+    JS_ASSERT(args[0].isObject());
+
+    JSObject *obj = &args[0].toObject();
+    bool isPacked = obj->is<ArrayObject>() && !obj->hasLazyType() &&
+                    !obj->type()->hasAllFlags(types::OBJECT_FLAG_NON_PACKED) &&
+                    obj->getDenseInitializedLength() == obj->as<ArrayObject>().length();
+
+    args.rval().setBoolean(isPacked);
+    return true;
+}
+
+static bool
+intrinsic_GetIteratorPrototype(JSContext *cx, unsigned argc, Value *vp)
+{
+    CallArgs args = CallArgsFromVp(argc, vp);
+    JS_ASSERT(args.length() == 0);
+
+    JSObject *obj = cx->global()->getOrCreateIteratorPrototype(cx);
+    if (!obj)
+        return false;
+
+    args.rval().setObject(*obj);
+    return true;
+}
+
+static bool
+intrinsic_NewArrayIterator(JSContext *cx, unsigned argc, Value *vp)
+{
+    CallArgs args = CallArgsFromVp(argc, vp);
+    JS_ASSERT(args.length() == 0);
+
+    RootedObject proto(cx, cx->global()->getOrCreateArrayIteratorPrototype(cx));
+    if (!proto)
+        return false;
+
+    JSObject *obj = NewObjectWithGivenProto(cx, proto->getClass(), proto, cx->global());
+    if (!obj)
+        return false;
+
+    args.rval().setObject(*obj);
+    return true;
+}
+
+static bool
+intrinsic_IsArrayIterator(JSContext *cx, unsigned argc, Value *vp)
+{
+    CallArgs args = CallArgsFromVp(argc, vp);
+    JS_ASSERT(args.length() == 1);
+    JS_ASSERT(args[0].isObject());
+
+    args.rval().setBoolean(args[0].toObject().is<ArrayIteratorObject>());
+    return true;
+}
+
+static bool
+intrinsic_NewStringIterator(JSContext *cx, unsigned argc, Value *vp)
+{
+    CallArgs args = CallArgsFromVp(argc, vp);
+    JS_ASSERT(args.length() == 0);
+
+    RootedObject proto(cx, cx->global()->getOrCreateStringIteratorPrototype(cx));
+    if (!proto)
+        return false;
+
+    JSObject *obj = NewObjectWithGivenProto(cx, &StringIteratorObject::class_, proto, cx->global());
+    if (!obj)
+        return false;
+
+    args.rval().setObject(*obj);
+    return true;
+}
+
+static bool
+intrinsic_IsStringIterator(JSContext *cx, unsigned argc, Value *vp)
+{
+    CallArgs args = CallArgsFromVp(argc, vp);
+    JS_ASSERT(args.length() == 1);
+    JS_ASSERT(args[0].isObject());
+
+    args.rval().setBoolean(args[0].toObject().is<StringIteratorObject>());
+    return true;
+}
+
 /*
  * ParallelTestsShouldPass(): Returns false if we are running in a
  * mode (such as --ion-eager) that is known to cause additional
@@ -607,7 +603,7 @@ intrinsic_RuntimeDefaultLocale(JSContext *cx, unsigned argc, Value *vp)
 
     const char *locale = cx->runtime()->getDefaultLocale();
     if (!locale) {
-        JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL, JSMSG_DEFAULT_LOCALE_ERROR);
+        JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr, JSMSG_DEFAULT_LOCALE_ERROR);
         return false;
     }
 
@@ -619,7 +615,7 @@ intrinsic_RuntimeDefaultLocale(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
-const JSFunctionSpec intrinsic_functions[] = {
+static const JSFunctionSpec intrinsic_functions[] = {
     JS_FN("ToObject",                intrinsic_ToObject,                1,0),
     JS_FN("ToInteger",               intrinsic_ToInteger,               1,0),
     JS_FN("IsCallable",              intrinsic_IsCallable,              1,0),
@@ -631,13 +627,19 @@ const JSFunctionSpec intrinsic_functions[] = {
     JS_FN("DecompileArg",            intrinsic_DecompileArg,            2,0),
     JS_FN("RuntimeDefaultLocale",    intrinsic_RuntimeDefaultLocale,    0,0),
 
-    JS_FN("UnsafePutElements",               intrinsic_UnsafePutElements,               3,0),
+    JS_FN("UnsafePutElements",       intrinsic_UnsafePutElements,       3,0),
     JS_FN("UnsafeSetReservedSlot",   intrinsic_UnsafeSetReservedSlot,   3,0),
     JS_FN("UnsafeGetReservedSlot",   intrinsic_UnsafeGetReservedSlot,   2,0),
-
-    JS_FN("NewClassPrototype",       intrinsic_NewClassPrototype,       1,0),
-    JS_FN("NewObjectWithClassPrototype", intrinsic_NewObjectWithClassPrototype, 1,0),
     JS_FN("HaveSameClass",           intrinsic_HaveSameClass,           2,0),
+    JS_FN("IsPackedArray",           intrinsic_IsPackedArray,           1,0),
+
+    JS_FN("GetIteratorPrototype",    intrinsic_GetIteratorPrototype,    0,0),
+
+    JS_FN("NewArrayIterator",        intrinsic_NewArrayIterator,        0,0),
+    JS_FN("IsArrayIterator",         intrinsic_IsArrayIterator,         1,0),
+
+    JS_FN("NewStringIterator",       intrinsic_NewStringIterator,       0,0),
+    JS_FN("IsStringIterator",        intrinsic_IsStringIterator,        1,0),
 
     JS_FN("ForkJoin",                intrinsic_ForkJoin,                2,0),
     JS_FN("ForkJoinSlices",          intrinsic_ForkJoinSlices,          0,0),
@@ -646,6 +648,57 @@ const JSFunctionSpec intrinsic_functions[] = {
     JS_FN("ShouldForceSequential",   intrinsic_ShouldForceSequential,   0,0),
     JS_FN("ParallelTestsShouldPass", intrinsic_ParallelTestsShouldPass, 0,0),
 
+    // See builtin/TypedObject.h for descriptors of the typedobj functions.
+    JS_FN("NewTypedHandle",
+          js::NewTypedHandle,
+          1, 0),
+    JS_FN("NewDerivedTypedDatum",
+          js::NewDerivedTypedDatum,
+          3, 0),
+    JS_FNINFO("AttachHandle",
+              JSNativeThreadSafeWrapper<js::AttachHandle>,
+              &js::AttachHandleJitInfo, 5, 0),
+    JS_FNINFO("ObjectIsTypeObject",
+              JSNativeThreadSafeWrapper<js::ObjectIsTypeObject>,
+              &js::ObjectIsTypeObjectJitInfo, 5, 0),
+    JS_FNINFO("ObjectIsTypeRepresentation",
+              JSNativeThreadSafeWrapper<js::ObjectIsTypeRepresentation>,
+              &js::ObjectIsTypeRepresentationJitInfo, 5, 0),
+    JS_FNINFO("ObjectIsTypedObject",
+              JSNativeThreadSafeWrapper<js::ObjectIsTypedObject>,
+              &js::ObjectIsTypedObjectJitInfo, 5, 0),
+    JS_FNINFO("ObjectIsTypedHandle",
+              JSNativeThreadSafeWrapper<js::ObjectIsTypedHandle>,
+              &js::ObjectIsTypedHandleJitInfo, 5, 0),
+    JS_FN("NewHandle",
+          js::NewTypedHandle,
+          1, 0),
+    JS_FNINFO("ClampToUint8",
+              JSNativeThreadSafeWrapper<js::ClampToUint8>,
+              &js::ClampToUint8JitInfo, 1, 0),
+    JS_FNINFO("Memcpy",
+              JSNativeThreadSafeWrapper<js::Memcpy>,
+              &js::MemcpyJitInfo, 5, 0),
+    JS_FN("GetTypedObjectModule", js::GetTypedObjectModule, 0, 0),
+
+#define LOAD_AND_STORE_SCALAR_FN_DECLS(_constant, _type, _name)               \
+    JS_FNINFO("Store_" #_name,                                                \
+              JSNativeThreadSafeWrapper<js::StoreScalar##_type::Func>,        \
+              &js::StoreScalar##_type::JitInfo, 3, 0),                        \
+    JS_FNINFO("Load_" #_name,                                                 \
+              JSNativeThreadSafeWrapper<js::LoadScalar##_type::Func>,         \
+              &js::LoadScalar##_type::JitInfo, 3, 0),
+    JS_FOR_EACH_UNIQUE_SCALAR_TYPE_REPR_CTYPE(LOAD_AND_STORE_SCALAR_FN_DECLS)
+
+#define LOAD_AND_STORE_REFERENCE_FN_DECLS(_constant, _type, _name)              \
+    JS_FNINFO("Store_" #_name,                                                  \
+              JSNativeThreadSafeWrapper<js::StoreReference##_type::Func>,       \
+              &js::StoreReference##_type::JitInfo, 3, 0),                       \
+    JS_FNINFO("Load_" #_name,                                                   \
+              JSNativeThreadSafeWrapper<js::LoadReference##_type::Func>,        \
+              &js::LoadReference##_type::JitInfo, 3, 0),
+    JS_FOR_EACH_REFERENCE_TYPE_REPR(LOAD_AND_STORE_REFERENCE_FN_DECLS)
+
     // See builtin/Intl.h for descriptions of the intl_* functions.
     JS_FN("intl_availableCalendars", intl_availableCalendars, 1,0),
     JS_FN("intl_availableCollations", intl_availableCollations, 1,0),
@@ -683,12 +736,12 @@ JSRuntime::initSelfHosting(JSContext *cx)
 {
     JS_ASSERT(!selfHostingGlobal_);
 
-    bool receivesDefaultObject = !cx->hasOption(JSOPTION_NO_DEFAULT_COMPARTMENT_OBJECT);
+    bool receivesDefaultObject = !cx->options().noDefaultCompartmentObject();
     RootedObject savedGlobal(cx, receivesDefaultObject
                                  ? js::DefaultObjectForContextOrNull(cx)
-                                 : NULL);
+                                 : nullptr);
     if (!(selfHostingGlobal_ = JS_NewGlobalObject(cx, &self_hosting_global_class,
-                                                  NULL, JS::DontFireOnNewGlobalHook)))
+                                                  nullptr, JS::DontFireOnNewGlobalHook)))
         return false;
     JSAutoCompartment ac(cx, selfHostingGlobal_);
     if (receivesDefaultObject)
@@ -727,6 +780,12 @@ JSRuntime::initSelfHosting(JSContext *cx)
     options.setCanLazilyParse(false);
     options.setSourcePolicy(CompileOptions::NO_SOURCE);
     options.setVersion(JSVERSION_LATEST);
+    options.werrorOption = true;
+
+#ifdef DEBUG
+    options.strictOption = true;
+    options.extraWarningsOption = true;
+#endif
 
     /*
      * Set a temporary error reporter printing to stderr because it is too
@@ -769,15 +828,7 @@ JSRuntime::initSelfHosting(JSContext *cx)
 void
 JSRuntime::finishSelfHosting()
 {
-    selfHostingGlobal_ = NULL;
-
-    SelfHostedClass *shClass = selfHostedClasses_;
-    while (shClass) {
-        SelfHostedClass *tmp = shClass;
-        shClass = shClass->next;
-        js_delete(tmp);
-    }
-    selfHostedClasses_ = NULL;
+    selfHostingGlobal_ = nullptr;
 }
 
 void
@@ -787,13 +838,6 @@ JSRuntime::markSelfHostingGlobal(JSTracer *trc)
         MarkObjectRoot(trc, &selfHostingGlobal_, "self-hosting global");
 }
 
-void
-JSRuntime::addSelfHostedClass(SelfHostedClass *shClass)
-{
-    shClass->next = selfHostedClasses_;
-    selfHostedClasses_ = shClass;
-}
-
 typedef AutoObjectObjectHashMap CloneMemory;
 static bool CloneValue(JSContext *cx, MutableHandleValue vp, CloneMemory &clonedObjects);
 
@@ -819,25 +863,12 @@ CloneProperties(JSContext *cx, HandleObject obj, HandleObject clone, CloneMemory
         id = ids[i];
         if (!GetUnclonedValue(cx, obj, id, &val) ||
             !CloneValue(cx, &val, clonedObjects) ||
-            !JS_DefinePropertyById(cx, clone, id, val.get(), NULL, NULL, 0))
+            !JS_DefinePropertyById(cx, clone, id, val.get(), nullptr, nullptr, 0))
         {
             return false;
         }
     }
 
-    if (SelfHostedClass::is(cx, obj->getClass())) {
-        for (uint32_t i = 0; i < JSCLASS_RESERVED_SLOTS(obj->getClass()); i++) {
-            val = obj->getReservedSlot(i);
-            if (!CloneValue(cx, &val, clonedObjects))
-                return false;
-            clone->setReservedSlot(i, val);
-        }
-
-        /* Privates are not cloned, so be careful! */
-        if (obj->hasPrivate())
-            clone->setPrivate(obj->getPrivate());
-    }
-
     return true;
 }
 
@@ -859,15 +890,15 @@ GetObjectAllocKindForClone(JSRuntime *rt, JSObject *obj)
 static JSObject *
 CloneObject(JSContext *cx, HandleObject srcObj, CloneMemory &clonedObjects)
 {
-    CloneMemory::AddPtr p = clonedObjects.lookupForAdd(srcObj.get());
+    DependentAddPtr<CloneMemory> p(cx, clonedObjects, srcObj.get());
     if (p)
-        return p->value;
+        return p->value();
     RootedObject clone(cx);
     if (srcObj->is<JSFunction>()) {
         if (srcObj->as<JSFunction>().isWrappable()) {
             clone = srcObj;
             if (!cx->compartment()->wrap(cx, &clone))
-                return NULL;
+                return nullptr;
         } else {
             RootedFunction fun(cx, &srcObj->as<JSFunction>());
             clone = CloneFunctionObject(cx, fun, cx->global(), fun->getAllocKind(), TenuredObject);
@@ -875,7 +906,7 @@ CloneObject(JSContext *cx, HandleObject srcObj, CloneMemory &clonedObjects)
     } else if (srcObj->is<RegExpObject>()) {
         RegExpObject &reobj = srcObj->as<RegExpObject>();
         RootedAtom source(cx, reobj.getSource());
-        clone = RegExpObject::createNoStatics(cx, source, reobj.getFlags(), NULL);
+        clone = RegExpObject::createNoStatics(cx, source, reobj.getFlags(), nullptr);
     } else if (srcObj->is<DateObject>()) {
         clone = JS_NewDateObjectMsec(cx, srcObj->as<DateObject>().UTCTime().toNumber());
     } else if (srcObj->is<BooleanObject>()) {
@@ -885,23 +916,26 @@ CloneObject(JSContext *cx, HandleObject srcObj, CloneMemory &clonedObjects)
     } else if (srcObj->is<StringObject>()) {
         Rooted<JSStableString*> str(cx, srcObj->as<StringObject>().unbox()->ensureStable(cx));
         if (!str)
-            return NULL;
+            return nullptr;
         str = js_NewStringCopyN<CanGC>(cx, str->chars().get(), str->length())->ensureStable(cx);
         if (!str)
-            return NULL;
+            return nullptr;
         clone = StringObject::create(cx, str);
     } else if (srcObj->is<ArrayObject>()) {
-        clone = NewDenseEmptyArray(cx, NULL, TenuredObject);
+        clone = NewDenseEmptyArray(cx, nullptr, TenuredObject);
     } else {
         JS_ASSERT(srcObj->isNative());
-        clone = NewObjectWithGivenProto(cx, srcObj->getClass(), NULL, cx->global(),
+        clone = NewObjectWithGivenProto(cx, srcObj->getClass(), nullptr, cx->global(),
                                         GetObjectAllocKindForClone(cx->runtime(), srcObj),
                                         SingletonObject);
     }
-    if (!clone || !clonedObjects.relookupOrAdd(p, srcObj.get(), clone.get()) ||
-        !CloneProperties(cx, srcObj, clone, clonedObjects))
-    {
-        return NULL;
+    if (!clone)
+        return nullptr;
+    if (!p.add(clonedObjects, srcObj, clone))
+        return nullptr;
+    if (!CloneProperties(cx, srcObj, clone, clonedObjects)) {
+        clonedObjects.remove(srcObj);
+        return nullptr;
     }
     return clone;
 }
@@ -980,12 +1014,67 @@ JSRuntime::cloneSelfHostedValue(JSContext *cx, Handle<PropertyName*> name, Mutab
     return true;
 }
 
+class OpaqueWrapper : public CrossCompartmentSecurityWrapper
+{
+  public:
+    OpaqueWrapper() : CrossCompartmentSecurityWrapper(0) {}
+    virtual bool enter(JSContext *cx, HandleObject wrapper, HandleId id,
+                       Wrapper::Action act, bool *bp) MOZ_OVERRIDE
+    {
+        *bp = false;
+        return false;
+    }
+    static OpaqueWrapper singleton;
+};
+
+OpaqueWrapper OpaqueWrapper::singleton;
+
+class OpaqueWrapperWithCall : public OpaqueWrapper
+{
+  public:
+    OpaqueWrapperWithCall() : OpaqueWrapper() {}
+    virtual bool enter(JSContext *cx, HandleObject wrapper, HandleId id,
+                       Wrapper::Action act, bool *bp) MOZ_OVERRIDE
+    {
+        if (act != Wrapper::CALL) {
+            *bp = false;
+            return false;
+        }
+        return true;
+    }
+    static OpaqueWrapperWithCall singleton;
+};
+
+OpaqueWrapperWithCall OpaqueWrapperWithCall::singleton;
+
+static JSObject *
+SelfHostingWrapObjectCallback(JSContext *cx, HandleObject existing, HandleObject obj,
+                              HandleObject proto, HandleObject parent, unsigned flags)
+{
+    RootedObject objGlobal(cx, &obj->global());
+    bool wrappingSelfHostedFunction = cx->runtime()->isSelfHostingGlobal(objGlobal);
+    JS_ASSERT_IF(!wrappingSelfHostedFunction, cx->runtime()->isSelfHostingGlobal(cx->global()));
+
+    OpaqueWrapper *handler = wrappingSelfHostedFunction
+                             ? &OpaqueWrapperWithCall::singleton
+                             : &OpaqueWrapper::singleton;
+    if (existing)
+        return Wrapper::Renew(cx, existing, obj, handler);
+    else
+        return Wrapper::New(cx, obj, proto, parent, handler);
+}
+
+const JSWrapObjectCallbacks
+js::SelfHostingWrapObjectCallbacks = {
+    SelfHostingWrapObjectCallback,
+    nullptr,
+    nullptr
+};
+
 bool
-JSRuntime::maybeWrappedSelfHostedFunction(JSContext *cx, Handle<PropertyName*> name,
-                                          MutableHandleValue funVal)
+JSRuntime::maybeWrappedSelfHostedFunction(JSContext *cx, HandleId id, MutableHandleValue funVal)
 {
     RootedObject shg(cx, selfHostingGlobal_);
-    RootedId id(cx, NameToId(name));
     if (!GetUnclonedValue(cx, shg, id, funVal))
         return false;
 
@@ -999,3 +1088,15 @@ JSRuntime::maybeWrappedSelfHostedFunction(JSContext *cx, Handle<PropertyName*> n
 
     return cx->compartment()->wrap(cx, funVal);
 }
+
+JSFunction *
+js::SelfHostedFunction(JSContext *cx, HandlePropertyName propName)
+{
+    RootedValue func(cx);
+    if (!cx->global()->getIntrinsicValue(cx, propName, &func))
+        return nullptr;
+
+    JS_ASSERT(func.isObject());
+    JS_ASSERT(func.toObject().is<JSFunction>());
+    return &func.toObject().as<JSFunction>();
+}
