@@ -12,9 +12,9 @@
 #include "mozilla/MathAlgorithms.h"
 #include "mozilla/PodOperations.h"
 
-#include "jsapi.h"
 #include "jsatom.h"
 #include "jscntxt.h"
+#include "jshashutil.h"
 #include "jsobj.h"
 
 #include "js/HashTable.h"
@@ -33,7 +33,7 @@ using mozilla::PodZero;
 using mozilla::CeilingLog2Size;
 
 bool
-ShapeTable::init(ExclusiveContext *cx, Shape *lastProp)
+ShapeTable::init(ThreadSafeContext *cx, Shape *lastProp)
 {
     /*
      * Either we're creating a table for a large scope that was populated
@@ -57,6 +57,7 @@ ShapeTable::init(ExclusiveContext *cx, Shape *lastProp)
     hashShift = HASH_BITS - sizeLog2;
     for (Shape::Range<NoGC> r(lastProp); !r.empty(); r.popFront()) {
         Shape &shape = r.front();
+        JS_ASSERT(cx->isThreadLocal(&shape));
         Shape **spp = search(shape.propid(), true);
 
         /*
@@ -82,7 +83,7 @@ Shape::removeFromDictionary(ObjectImpl *obj)
     if (parent)
         parent->listp = listp;
     *listp = parent;
-    listp = NULL;
+    listp = nullptr;
 }
 
 void
@@ -105,9 +106,10 @@ Shape::insertIntoDictionary(HeapPtrShape *dictp)
 }
 
 bool
-Shape::makeOwnBaseShape(ExclusiveContext *cx)
+Shape::makeOwnBaseShape(ThreadSafeContext *cx)
 {
     JS_ASSERT(!base()->isOwned());
+    JS_ASSERT(cx->isThreadLocal(this));
     assertSameCompartmentDebugOnly(cx, compartment());
 
     BaseShape *nbase = js_NewGCBaseShape<NoGC>(cx);
@@ -143,7 +145,7 @@ Shape::handoffTableTo(Shape *shape)
 }
 
 /* static */ bool
-Shape::hashify(ExclusiveContext *cx, Shape *shape)
+Shape::hashify(ThreadSafeContext *cx, Shape *shape)
 {
     JS_ASSERT(!shape->hasTable());
 
@@ -209,7 +211,7 @@ ShapeTable::search(jsid id, bool adding)
     if (SHAPE_IS_REMOVED(stored)) {
         firstRemoved = spp;
     } else {
-        firstRemoved = NULL;
+        firstRemoved = nullptr;
         if (adding && !SHAPE_HAD_COLLISION(stored))
             SHAPE_FLAG_COLLISION(spp, shape);
 #ifdef DEBUG
@@ -245,11 +247,11 @@ ShapeTable::search(jsid id, bool adding)
     }
 
     /* NOTREACHED */
-    return NULL;
+    return nullptr;
 }
 
 bool
-ShapeTable::change(int log2Delta, ExclusiveContext *cx)
+ShapeTable::change(int log2Delta, ThreadSafeContext *cx)
 {
     JS_ASSERT(entries);
 
@@ -273,6 +275,7 @@ ShapeTable::change(int log2Delta, ExclusiveContext *cx)
     /* Copy only live entries, leaving removed and free ones behind. */
     for (Shape **oldspp = oldTable; oldsize != 0; oldspp++) {
         Shape *shape = SHAPE_FETCH(oldspp);
+        JS_ASSERT(cx->isThreadLocal(shape));
         if (shape) {
             Shape **spp = search(shape->propid(), true);
             JS_ASSERT(SHAPE_IS_FREE(*spp));
@@ -287,7 +290,7 @@ ShapeTable::change(int log2Delta, ExclusiveContext *cx)
 }
 
 bool
-ShapeTable::grow(ExclusiveContext *cx)
+ShapeTable::grow(ThreadSafeContext *cx)
 {
     JS_ASSERT(needsToGrow());
 
@@ -328,15 +331,12 @@ Shape::replaceLastProperty(ExclusiveContext *cx, const StackBaseShape &base,
                                            base.flags & BaseShape::OBJECT_FLAG_MASK);
     }
 
-    StackShape child(shape);
-    StackShape::AutoRooter childRoot(cx, &child);
-    {
-        UnownedBaseShape *nbase = BaseShape::getUnowned(cx, base);
-        if (!nbase)
-            return NULL;
+    UnownedBaseShape *nbase = BaseShape::getUnowned(cx, base);
+    if (!nbase)
+        return nullptr;
 
-        child.base = nbase;
-    }
+    StackShape child(shape);
+    child.base = nbase;
 
     return cx->compartment()->propertyTree.getChild(cx, shape->parent,
                                                     shape->numFixedSlots(), child);
@@ -348,8 +348,8 @@ Shape::replaceLastProperty(ExclusiveContext *cx, const StackBaseShape &base,
  * one of lastProperty() or lastProperty()->parent.
  */
 /* static */ Shape *
-JSObject::getChildProperty(ExclusiveContext *cx,
-                           HandleObject obj, HandleShape parent, StackShape &child)
+JSObject::getChildPropertyOnDictionary(ThreadSafeContext *cx, JS::HandleObject obj,
+                                       HandleShape parent, js::StackShape &child)
 {
     /*
      * Shared properties have no slot, but slot_ will reflect that of parent.
@@ -362,13 +362,23 @@ JSObject::getChildProperty(ExclusiveContext *cx,
         if (child.hasMissingSlot()) {
             uint32_t slot;
             if (!allocSlot(cx, obj, &slot))
-                return NULL;
+                return nullptr;
             child.setSlot(slot);
         } else {
-            /* Slots can only be allocated out of order on objects in dictionary mode. */
+            /*
+             * Slots can only be allocated out of order on objects in
+             * dictionary mode.  Otherwise the child's slot must be after the
+             * parent's slot (if it has one), because slot number determines
+             * slot span for objects with that shape.  Usually child slot
+             * *immediately* follows parent slot, but there may be a slot gap
+             * when the object uses some -- but not all -- of its reserved
+             * slots to store properties.
+             */
             JS_ASSERT(obj->inDictionaryMode() ||
                       parent->hasMissingSlot() ||
-                      child.slot() == parent->maybeSlot() + 1);
+                      child.slot() == parent->maybeSlot() + 1 ||
+                      (parent->maybeSlot() + 1 < JSSLOT_FREE(obj->getClass()) &&
+                       child.slot() == JSSLOT_FREE(obj->getClass())));
         }
     }
 
@@ -379,33 +389,72 @@ JSObject::getChildProperty(ExclusiveContext *cx,
         StackShape::AutoRooter childRoot(cx, &child);
         shape = js_NewGCShape(cx);
         if (!shape)
-            return NULL;
+            return nullptr;
         if (child.hasSlot() && child.slot() >= obj->lastProperty()->base()->slotSpan()) {
             if (!JSObject::setSlotSpan(cx, obj, child.slot() + 1))
-                return NULL;
+                return nullptr;
         }
         shape->initDictionaryShape(child, obj->numFixedSlots(), &obj->shape_);
-    } else {
+    }
+
+    return shape;
+}
+
+/* static */ Shape *
+JSObject::getChildProperty(ExclusiveContext *cx,
+                           HandleObject obj, HandleShape parent, StackShape &child)
+{
+    StackShape::AutoRooter childRoot(cx, &child);
+    RootedShape shape(cx, getChildPropertyOnDictionary(cx, obj, parent, child));
+
+    if (!obj->inDictionaryMode()) {
         shape = cx->compartment()->propertyTree.getChild(cx, parent, obj->numFixedSlots(), child);
         if (!shape)
-            return NULL;
+            return nullptr;
         //JS_ASSERT(shape->parent == parent);
         //JS_ASSERT_IF(parent != lastProperty(), parent == lastProperty()->parent);
         if (!JSObject::setLastProperty(cx, obj, shape))
-            return NULL;
+            return nullptr;
+    }
+
+    return shape;
+}
+
+/* static */ Shape *
+JSObject::lookupChildProperty(ThreadSafeContext *cx,
+                              HandleObject obj, HandleShape parent, StackShape &child)
+{
+    StackShape::AutoRooter childRoot(cx, &child);
+    JS_ASSERT(cx->isThreadLocal(obj));
+
+    RootedShape shape(cx, getChildPropertyOnDictionary(cx, obj, parent, child));
+
+    if (!obj->inDictionaryMode()) {
+        shape = cx->compartment_->propertyTree.lookupChild(cx, parent, child);
+        if (!shape)
+            return nullptr;
+        if (!JSObject::setLastProperty(cx, obj, shape))
+            return nullptr;
     }
 
     return shape;
 }
 
 bool
-js::ObjectImpl::toDictionaryMode(ExclusiveContext *cx)
+js::ObjectImpl::toDictionaryMode(ThreadSafeContext *cx)
 {
     JS_ASSERT(!inDictionaryMode());
 
     /* We allocate the shapes from cx->compartment(), so make sure it's right. */
     JS_ASSERT(cx->isInsideCurrentCompartment(this));
 
+    /*
+     * This function is thread safe as long as the object is thread local. It
+     * does not modify the shared shapes, and only allocates newly allocated
+     * (and thus also thread local) shapes.
+     */
+    JS_ASSERT(cx->isThreadLocal(this));
+
     uint32_t span = slotSpan();
 
     Rooted<ObjectImpl*> self(cx, this);
@@ -468,11 +517,11 @@ NormalizeGetterAndSetter(JSObject *obj,
 {
     if (setter == JS_StrictPropertyStub) {
         JS_ASSERT(!(attrs & JSPROP_SETTER));
-        setter = NULL;
+        setter = nullptr;
     }
     if (getter == JS_PropertyStub) {
         JS_ASSERT(!(attrs & JSPROP_GETTER));
-        getter = NULL;
+        getter = nullptr;
     }
 
     return true;
@@ -488,21 +537,21 @@ JSObject::addProperty(ExclusiveContext *cx, HandleObject obj, HandleId id,
 
     bool extensible;
     if (!JSObject::isExtensible(cx, obj, &extensible))
-        return NULL;
+        return nullptr;
     if (!extensible) {
         if (cx->isJSContext())
             obj->reportNotExtensible(cx->asJSContext());
-        return NULL;
+        return nullptr;
     }
 
     NormalizeGetterAndSetter(obj, id, attrs, flags, getter, setter);
 
-    Shape **spp = NULL;
+    Shape **spp = nullptr;
     if (obj->inDictionaryMode())
         spp = obj->lastProperty()->table().search(id, true);
 
-    return addPropertyInternal(cx, obj, id, getter, setter, slot, attrs, flags, shortid,
-                               spp, allowDictionary);
+    return addPropertyInternal<SequentialExecution>(cx, obj, id, getter, setter, slot, attrs,
+                                                    flags, shortid, spp, allowDictionary);
 }
 
 static bool
@@ -517,18 +566,36 @@ ShouldConvertToDictionary(JSObject *obj)
     return obj->lastProperty()->entryCount() >= PropertyTree::MAX_HEIGHT;
 }
 
+template <ExecutionMode mode>
+static inline UnownedBaseShape *
+GetOrLookupUnownedBaseShape(typename ExecutionModeTraits<mode>::ExclusiveContextType cx,
+                            const StackBaseShape &base)
+{
+    if (mode == ParallelExecution)
+        return BaseShape::lookupUnowned(cx, base);
+    return BaseShape::getUnowned(cx->asExclusiveContext(), base);
+}
+
+template <ExecutionMode mode>
 /* static */ Shape *
-JSObject::addPropertyInternal(ExclusiveContext *cx, HandleObject obj, HandleId id,
+JSObject::addPropertyInternal(typename ExecutionModeTraits<mode>::ExclusiveContextType cx,
+                              HandleObject obj, HandleId id,
                               PropertyOp getter, StrictPropertyOp setter,
                               uint32_t slot, unsigned attrs,
                               unsigned flags, int shortid, Shape **spp,
                               bool allowDictionary)
 {
+    JS_ASSERT(cx->isThreadLocal(obj));
     JS_ASSERT_IF(!allowDictionary, !obj->inDictionaryMode());
 
     AutoRooterGetterSetter gsRoot(cx, attrs, &getter, &setter);
 
-    ShapeTable *table = NULL;
+    /*
+     * The code below deals with either converting obj to dictionary mode or
+     * growing an object that's already in dictionary mode. Either way,
+     * dictionray operations are safe if thread local.
+     */
+    ShapeTable *table = nullptr;
     if (!obj->inDictionaryMode()) {
         bool stableSlot =
             (slot == SHAPE_INVALID_SLOT) ||
@@ -539,7 +606,7 @@ JSObject::addPropertyInternal(ExclusiveContext *cx, HandleObject obj, HandleId i
             (!stableSlot || ShouldConvertToDictionary(obj)))
         {
             if (!obj->toDictionaryMode(cx))
-                return NULL;
+                return nullptr;
             table = &obj->lastProperty()->table();
             spp = table->search(id, true);
         }
@@ -547,7 +614,7 @@ JSObject::addPropertyInternal(ExclusiveContext *cx, HandleObject obj, HandleId i
         table = &obj->lastProperty()->table();
         if (table->needsToGrow()) {
             if (!table->grow(cx))
-                return NULL;
+                return nullptr;
             spp = table->search(id, true);
             JS_ASSERT(!SHAPE_FETCH(spp));
         }
@@ -571,13 +638,13 @@ JSObject::addPropertyInternal(ExclusiveContext *cx, HandleObject obj, HandleId i
             base.updateGetterSetter(attrs, getter, setter);
             if (indexed)
                 base.flags |= BaseShape::INDEXED;
-            nbase = BaseShape::getUnowned(cx, base);
+            nbase = GetOrLookupUnownedBaseShape<mode>(cx, base);
             if (!nbase)
-                return NULL;
+                return nullptr;
         }
 
         StackShape child(nbase, id, slot, obj->numFixedSlots(), attrs, flags, shortid);
-        shape = getChildProperty(cx, obj, last, child);
+        shape = getOrLookupChildProperty<mode>(cx, obj, last, child);
     }
 
     if (shape) {
@@ -598,7 +665,77 @@ JSObject::addPropertyInternal(ExclusiveContext *cx, HandleObject obj, HandleId i
     }
 
     obj->checkShapeConsistency();
-    return NULL;
+    return nullptr;
+}
+
+template /* static */ Shape *
+JSObject::addPropertyInternal<SequentialExecution>(ExclusiveContext *cx,
+                                                   HandleObject obj, HandleId id,
+                                                   PropertyOp getter, StrictPropertyOp setter,
+                                                   uint32_t slot, unsigned attrs,
+                                                   unsigned flags, int shortid, Shape **spp,
+                                                   bool allowDictionary);
+template /* static */ Shape *
+JSObject::addPropertyInternal<ParallelExecution>(ForkJoinSlice *cx,
+                                                 HandleObject obj, HandleId id,
+                                                 PropertyOp getter, StrictPropertyOp setter,
+                                                 uint32_t slot, unsigned attrs,
+                                                 unsigned flags, int shortid, Shape **spp,
+                                                 bool allowDictionary);
+
+JSObject *
+js::NewReshapedObject(JSContext *cx, HandleTypeObject type, JSObject *parent,
+                      gc::AllocKind allocKind, HandleShape shape, NewObjectKind newKind)
+{
+    RootedObject res(cx, NewObjectWithType(cx, type, parent, allocKind, newKind));
+    if (!res)
+        return nullptr;
+
+    if (shape->isEmptyShape())
+        return res;
+
+    /* Get all the ids in the object, in order. */
+    js::AutoIdVector ids(cx);
+    {
+        for (unsigned i = 0; i <= shape->slot(); i++) {
+            if (!ids.append(JSID_VOID))
+                return nullptr;
+        }
+        Shape *nshape = shape;
+        while (!nshape->isEmptyShape()) {
+            ids[nshape->slot()] = nshape->propid();
+            nshape = nshape->previous();
+        }
+    }
+
+    /* Construct the new shape, without updating type information. */
+    RootedId id(cx);
+    RootedShape newShape(cx, res->lastProperty());
+    for (unsigned i = 0; i < ids.length(); i++) {
+        id = ids[i];
+        JS_ASSERT(!res->nativeContains(cx, id));
+
+        uint32_t index;
+        bool indexed = js_IdIsIndex(id, &index);
+
+        Rooted<UnownedBaseShape*> nbase(cx, newShape->base()->unowned());
+        if (indexed) {
+            StackBaseShape base(nbase);
+            base.flags |= BaseShape::INDEXED;
+            nbase = GetOrLookupUnownedBaseShape<SequentialExecution>(cx, base);
+            if (!nbase)
+                return nullptr;
+        }
+
+        StackShape child(nbase, id, i, res->numFixedSlots(), JSPROP_ENUMERATE, 0, 0);
+        newShape = cx->compartment()->propertyTree.getChild(cx, newShape, res->numFixedSlots(), child);
+        if (!newShape)
+            return nullptr;
+        if (!JSObject::setLastProperty(cx, res, newShape))
+            return nullptr;
+    }
+
+    return res;
 }
 
 /*
@@ -606,8 +743,8 @@ JSObject::addPropertyInternal(ExclusiveContext *cx, HandleObject obj, HandleId i
  * slot access optimizations are sound. It is responsibility of the callers to
  * enforce all restrictions from ECMA-262 v5 8.12.9 [[DefineOwnProperty]].
  */
-inline bool
-CheckCanChangeAttrs(ExclusiveContext *cx, JSObject *obj, Shape *shape, unsigned *attrsp)
+static inline bool
+CheckCanChangeAttrs(ThreadSafeContext *cx, JSObject *obj, Shape *shape, unsigned *attrsp)
 {
     if (shape->configurable())
         return true;
@@ -627,12 +764,15 @@ CheckCanChangeAttrs(ExclusiveContext *cx, JSObject *obj, Shape *shape, unsigned
     return true;
 }
 
+template <ExecutionMode mode>
 /* static */ Shape *
-JSObject::putProperty(ExclusiveContext *cx, HandleObject obj, HandleId id,
+JSObject::putProperty(typename ExecutionModeTraits<mode>::ExclusiveContextType cx,
+                      HandleObject obj, HandleId id,
                       PropertyOp getter, StrictPropertyOp setter,
                       uint32_t slot, unsigned attrs,
                       unsigned flags, int shortid)
 {
+    JS_ASSERT(cx->isThreadLocal(obj));
     JS_ASSERT(!JSID_IS_VOID(id));
 
 #ifdef DEBUG
@@ -648,31 +788,53 @@ JSObject::putProperty(ExclusiveContext *cx, HandleObject obj, HandleId id,
 
     AutoRooterGetterSetter gsRoot(cx, attrs, &getter, &setter);
 
-    /* Search for id in order to claim its entry if table has been allocated. */
+    /*
+     * Search for id in order to claim its entry if table has been allocated.
+     *
+     * Note that we can only try to claim an entry in a table that is thread
+     * local. An object may be thread local *without* its shape being thread
+     * local. The only thread local objects that *also* have thread local
+     * shapes are dictionaries that were allocated/converted thread
+     * locally. Only for those objects we can try to claim an entry in its
+     * shape table.
+     */
     Shape **spp;
-    RootedShape shape(cx, Shape::search(cx, obj->lastProperty(), id, &spp, true));
+    RootedShape shape(cx, (mode == ParallelExecution
+                           ? Shape::searchThreadLocal(cx, obj->lastProperty(), id, &spp,
+                                                      cx->isThreadLocal(obj->lastProperty()))
+                           : Shape::search(cx->asExclusiveContext(), obj->lastProperty(), id,
+                                           &spp, true)));
     if (!shape) {
         /*
          * You can't add properties to a non-extensible object, but you can change
          * attributes of properties in such objects.
          */
         bool extensible;
-        if (!JSObject::isExtensible(cx, obj, &extensible))
-            return NULL;
+
+        if (mode == ParallelExecution) {
+            if (obj->is<ProxyObject>())
+                return nullptr;
+            extensible = obj->nonProxyIsExtensible();
+        } else {
+            if (!JSObject::isExtensible(cx->asExclusiveContext(), obj, &extensible))
+                return nullptr;
+        }
+
         if (!extensible) {
             if (cx->isJSContext())
                 obj->reportNotExtensible(cx->asJSContext());
-            return NULL;
+            return nullptr;
         }
 
-        return addPropertyInternal(cx, obj, id, getter, setter, slot, attrs, flags, shortid, spp, true);
+        return addPropertyInternal<mode>(cx, obj, id, getter, setter, slot, attrs, flags,
+                                         shortid, spp, true);
     }
 
     /* Property exists: search must have returned a valid *spp. */
     JS_ASSERT_IF(spp, !SHAPE_IS_REMOVED(*spp));
 
     if (!CheckCanChangeAttrs(cx, obj, shape, &attrs))
-        return NULL;
+        return nullptr;
 
     /*
      * If the caller wants to allocate a slot, but doesn't care which slot,
@@ -692,9 +854,9 @@ JSObject::putProperty(ExclusiveContext *cx, HandleObject obj, HandleId id,
         base.updateGetterSetter(attrs, getter, setter);
         if (indexed)
             base.flags |= BaseShape::INDEXED;
-        nbase = BaseShape::getUnowned(cx, base);
+        nbase = GetOrLookupUnownedBaseShape<mode>(cx, base);
         if (!nbase)
-            return NULL;
+            return nullptr;
     }
 
     /*
@@ -711,7 +873,7 @@ JSObject::putProperty(ExclusiveContext *cx, HandleObject obj, HandleId id,
      */
     if (shape != obj->lastProperty() && !obj->inDictionaryMode()) {
         if (!obj->toDictionaryMode(cx))
-            return NULL;
+            return nullptr;
         spp = obj->lastProperty()->table().search(shape->propid(), false);
         shape = SHAPE_FETCH(spp);
     }
@@ -728,14 +890,14 @@ JSObject::putProperty(ExclusiveContext *cx, HandleObject obj, HandleId id,
         bool updateLast = (shape == obj->lastProperty());
         shape = obj->replaceWithNewEquivalentShape(cx, shape);
         if (!shape)
-            return NULL;
+            return nullptr;
         if (!updateLast && !obj->generateOwnShape(cx))
-            return NULL;
+            return nullptr;
 
         /* FIXME bug 593129 -- slot allocation and JSObject *this must move out of here! */
         if (slot == SHAPE_INVALID_SLOT && !(attrs & JSPROP_SHARED)) {
             if (!allocSlot(cx, obj, &slot))
-                return NULL;
+                return nullptr;
         }
 
         if (updateLast)
@@ -755,20 +917,20 @@ JSObject::putProperty(ExclusiveContext *cx, HandleObject obj, HandleId id,
         StackBaseShape base(obj->lastProperty()->base());
         base.updateGetterSetter(attrs, getter, setter);
 
-        UnownedBaseShape *nbase = BaseShape::getUnowned(cx, base);
+        UnownedBaseShape *nbase = GetOrLookupUnownedBaseShape<mode>(cx, base);
         if (!nbase)
-            return NULL;
+            return nullptr;
 
         JS_ASSERT(shape == obj->lastProperty());
 
         /* Find or create a property tree node labeled by our arguments. */
         StackShape child(nbase, id, slot, obj->numFixedSlots(), attrs, flags, shortid);
         RootedShape parent(cx, shape->parent);
-        Shape *newShape = JSObject::getChildProperty(cx, obj, parent, child);
+        Shape *newShape = getOrLookupChildProperty<mode>(cx, obj, parent, child);
 
         if (!newShape) {
             obj->checkShapeConsistency();
-            return NULL;
+            return nullptr;
         }
 
         shape = newShape;
@@ -793,11 +955,27 @@ JSObject::putProperty(ExclusiveContext *cx, HandleObject obj, HandleId id,
     return shape;
 }
 
+template /* static */ Shape *
+JSObject::putProperty<SequentialExecution>(ExclusiveContext *cx,
+                                           HandleObject obj, HandleId id,
+                                           PropertyOp getter, StrictPropertyOp setter,
+                                           uint32_t slot, unsigned attrs,
+                                           unsigned flags, int shortid);
+template /* static */ Shape *
+JSObject::putProperty<ParallelExecution>(ForkJoinSlice *cx,
+                                         HandleObject obj, HandleId id,
+                                         PropertyOp getter, StrictPropertyOp setter,
+                                         uint32_t slot, unsigned attrs,
+                                         unsigned flags, int shortid);
+
+template <ExecutionMode mode>
 /* static */ Shape *
-JSObject::changeProperty(ExclusiveContext *cx, HandleObject obj, HandleShape shape, unsigned attrs,
+JSObject::changeProperty(typename ExecutionModeTraits<mode>::ExclusiveContextType cx,
+                         HandleObject obj, HandleShape shape, unsigned attrs,
                          unsigned mask, PropertyOp getter, StrictPropertyOp setter)
 {
-    JS_ASSERT(obj->nativeContains(cx, shape));
+    JS_ASSERT(cx->isThreadLocal(obj));
+    JS_ASSERT(obj->nativeContainsPure(shape));
 
     attrs |= shape->attrs & mask;
 
@@ -805,17 +983,20 @@ JSObject::changeProperty(ExclusiveContext *cx, HandleObject obj, HandleShape sha
     JS_ASSERT(!((attrs ^ shape->attrs) & JSPROP_SHARED) ||
               !(attrs & JSPROP_SHARED));
 
-    types::MarkTypePropertyConfigured(cx, obj, shape->propid());
-    if (attrs & (JSPROP_GETTER | JSPROP_SETTER))
-        types::AddTypePropertyId(cx, obj, shape->propid(), types::Type::UnknownType());
+    if (mode == ParallelExecution) {
+        if (!types::IsTypePropertyIdMarkedConfigured(obj, shape->propid()))
+            return nullptr;
+    } else {
+        types::MarkTypePropertyConfigured(cx->asExclusiveContext(), obj, shape->propid());
+    }
 
     if (getter == JS_PropertyStub)
-        getter = NULL;
+        getter = nullptr;
     if (setter == JS_StrictPropertyStub)
-        setter = NULL;
+        setter = nullptr;
 
     if (!CheckCanChangeAttrs(cx, obj, shape, &attrs))
-        return NULL;
+        return nullptr;
 
     if (shape->attrs == attrs && shape->getter() == getter && shape->setter() == setter)
         return shape;
@@ -827,13 +1008,25 @@ JSObject::changeProperty(ExclusiveContext *cx, HandleObject obj, HandleShape sha
      * putProperty won't re-allocate it.
      */
     RootedId propid(cx, shape->propid());
-    Shape *newShape = putProperty(cx, obj, propid, getter, setter, shape->maybeSlot(),
-                                    attrs, shape->flags, shape->maybeShortid());
+    Shape *newShape = putProperty<mode>(cx, obj, propid, getter, setter,
+                                        shape->maybeSlot(), attrs, shape->flags,
+                                        shape->maybeShortid());
 
     obj->checkShapeConsistency();
     return newShape;
 }
 
+template /* static */ Shape *
+JSObject::changeProperty<SequentialExecution>(ExclusiveContext *cx,
+                                              HandleObject obj, HandleShape shape,
+                                              unsigned attrs, unsigned mask,
+                                              PropertyOp getter, StrictPropertyOp setter);
+template /* static */ Shape *
+JSObject::changeProperty<ParallelExecution>(ForkJoinSlice *slice,
+                                            HandleObject obj, HandleShape shape,
+                                            unsigned attrs, unsigned mask,
+                                            PropertyOp getter, StrictPropertyOp setter);
+
 bool
 JSObject::removeProperty(ExclusiveContext *cx, jsid id_)
 {
@@ -906,7 +1099,7 @@ JSObject::removeProperty(ExclusiveContext *cx, jsid id_)
             ++table.removedCount;
             --table.entryCount;
         } else {
-            *spp = NULL;
+            *spp = nullptr;
             --table.entryCount;
 
 #ifdef DEBUG
@@ -989,12 +1182,16 @@ JSObject::rollbackProperties(ExclusiveContext *cx, uint32_t slotSpan)
 }
 
 Shape *
-ObjectImpl::replaceWithNewEquivalentShape(ExclusiveContext *cx, Shape *oldShape, Shape *newShape)
+ObjectImpl::replaceWithNewEquivalentShape(ThreadSafeContext *cx, Shape *oldShape, Shape *newShape)
 {
+    JS_ASSERT(cx->isThreadLocal(this));
+    JS_ASSERT(cx->isThreadLocal(oldShape));
     JS_ASSERT(cx->isInsideCurrentCompartment(oldShape));
     JS_ASSERT_IF(oldShape != lastProperty(),
                  inDictionaryMode() &&
-                 nativeLookup(cx, oldShape->propidRef()) == oldShape);
+                 ((cx->isExclusiveContext()
+                   ? nativeLookup(cx->asExclusiveContext(), oldShape->propidRef())
+                   : nativeLookupPure(oldShape->propidRef())) == oldShape));
 
     ObjectImpl *self = this;
 
@@ -1002,7 +1199,7 @@ ObjectImpl::replaceWithNewEquivalentShape(ExclusiveContext *cx, Shape *oldShape,
         Rooted<ObjectImpl*> selfRoot(cx, self);
         RootedShape newRoot(cx, newShape);
         if (!toDictionaryMode(cx))
-            return NULL;
+            return nullptr;
         oldShape = selfRoot->lastProperty();
         self = selfRoot;
         newShape = newRoot;
@@ -1013,7 +1210,7 @@ ObjectImpl::replaceWithNewEquivalentShape(ExclusiveContext *cx, Shape *oldShape,
         RootedShape oldRoot(cx, oldShape);
         newShape = js_NewGCShape(cx);
         if (!newShape)
-            return NULL;
+            return nullptr;
         new (newShape) Shape(oldRoot->base()->unowned(), 0);
         self = selfRoot;
         oldShape = oldRoot;
@@ -1021,7 +1218,7 @@ ObjectImpl::replaceWithNewEquivalentShape(ExclusiveContext *cx, Shape *oldShape,
 
     ShapeTable &table = self->lastProperty()->table();
     Shape **spp = oldShape->isEmptyShape()
-                  ? NULL
+                  ? nullptr
                   : table.search(oldShape->propidRef(), false);
 
     /*
@@ -1280,10 +1477,9 @@ BaseShape::getUnowned(ExclusiveContext *cx, const StackBaseShape &base)
     BaseShapeSet &table = cx->compartment()->baseShapes;
 
     if (!table.initialized() && !table.init())
-        return NULL;
-
-    BaseShapeSet::AddPtr p = table.lookupForAdd(&base);
+        return nullptr;
 
+    DependentAddPtr<BaseShapeSet> p(cx, table, &base);
     if (p)
         return *p;
 
@@ -1291,18 +1487,30 @@ BaseShape::getUnowned(ExclusiveContext *cx, const StackBaseShape &base)
 
     BaseShape *nbase_ = js_NewGCBaseShape<CanGC>(cx);
     if (!nbase_)
-        return NULL;
+        return nullptr;
 
     new (nbase_) BaseShape(base);
 
     UnownedBaseShape *nbase = static_cast<UnownedBaseShape *>(nbase_);
 
-    if (!table.relookupOrAdd(p, &base, nbase))
-        return NULL;
+    if (!p.add(table, &base, nbase))
+        return nullptr;
 
     return nbase;
 }
 
+/* static */ UnownedBaseShape *
+BaseShape::lookupUnowned(ThreadSafeContext *cx, const StackBaseShape &base)
+{
+    BaseShapeSet &table = cx->compartment_->baseShapes;
+
+    if (!table.initialized())
+        return nullptr;
+
+    BaseShapeSet::Ptr p = table.readonlyThreadsafeLookup(&base);
+    return *p;
+}
+
 void
 BaseShape::assertConsistency()
 {
@@ -1340,12 +1548,12 @@ BaseShape::finalize(FreeOp *fop)
 {
     if (table_) {
         fop->delete_(table_);
-        table_ = NULL;
+        table_ = nullptr;
     }
 }
 
 inline
-InitialShapeEntry::InitialShapeEntry() : shape(NULL), proto(NULL)
+InitialShapeEntry::InitialShapeEntry() : shape(nullptr), proto(nullptr)
 {
 }
 
@@ -1394,12 +1602,11 @@ EmptyShape::getInitialShape(ExclusiveContext *cx, const Class *clasp, TaggedProt
     InitialShapeSet &table = cx->compartment()->initialShapes;
 
     if (!table.initialized() && !table.init())
-        return NULL;
+        return nullptr;
 
     typedef InitialShapeEntry::Lookup Lookup;
-    InitialShapeSet::AddPtr p =
-        table.lookupForAdd(Lookup(clasp, proto, parent, metadata, nfixed, objectFlags));
-
+    DependentAddPtr<InitialShapeSet>
+        p(cx, table, Lookup(clasp, proto, parent, metadata, nfixed, objectFlags));
     if (p)
         return p->shape;
 
@@ -1411,18 +1618,16 @@ EmptyShape::getInitialShape(ExclusiveContext *cx, const Class *clasp, TaggedProt
     StackBaseShape base(cx, clasp, parent, metadata, objectFlags);
     Rooted<UnownedBaseShape*> nbase(cx, BaseShape::getUnowned(cx, base));
     if (!nbase)
-        return NULL;
+        return nullptr;
 
     Shape *shape = cx->compartment()->propertyTree.newShape(cx);
     if (!shape)
-        return NULL;
+        return nullptr;
     new (shape) EmptyShape(nbase, nfixed);
 
-    if (!table.relookupOrAdd(p, Lookup(clasp, protoRoot, parentRoot, metadataRoot, nfixed, objectFlags),
-                             InitialShapeEntry(shape, protoRoot)))
-    {
-        return NULL;
-    }
+    Lookup lookup(clasp, protoRoot, parentRoot, metadataRoot, nfixed, objectFlags);
+    if (!p.add(table, lookup, InitialShapeEntry(shape, protoRoot)))
+        return nullptr;
 
     return shape;
 }
@@ -1463,6 +1668,10 @@ EmptyShape::insertInitialShape(ExclusiveContext *cx, HandleShape shape, HandleOb
                                      shape->getObjectParent(), shape->getObjectMetadata(),
                                      shape->numFixedSlots(), shape->getObjectFlags());
 
+    /* Bug 929547 - we do not rekey based on metadata object moves */
+    DebugOnly<JSObject *> metadata = shape->getObjectMetadata();
+    JS_ASSERT_IF(metadata, !gc::IsInsideNursery(cx->compartment()->runtimeFromAnyThread(), metadata));
+
     InitialShapeSet::Ptr p = cx->compartment()->initialShapes.lookup(lookup);
     JS_ASSERT(p);
 
