@@ -345,23 +345,13 @@ nsColumnSetFrame::ReflowColumns(nsHTMLReflowMetrics& aDesiredSize,
   return feasible;
 }
 
-// XXX copied from nsBlockFrame, should this be moved to nsContainerFrame?
-static void
-PlaceFrameView(nsIFrame* aFrame)
-{
-  if (aFrame->HasView())
-    nsContainerFrame::PositionFrameView(aFrame);
-  else
-    nsContainerFrame::PositionChildViews(aFrame);
-}
-
 static void MoveChildTo(nsIFrame* aParent, nsIFrame* aChild, nsPoint aOrigin) {
   if (aChild->GetPosition() == aOrigin) {
     return;
   }
-  
+
   aChild->SetPosition(aOrigin);
-  PlaceFrameView(aChild);
+  nsContainerFrame::PlaceFrameView(aChild);
 }
 
 nscoord
@@ -1039,10 +1029,12 @@ nsColumnSetFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
                                    const nsDisplayListSet& aLists) {
   DisplayBorderBackgroundOutline(aBuilder, aLists);
 
-  aLists.BorderBackground()->AppendNewToTop(new (aBuilder)
+  if (IsVisibleForPainting(aBuilder)) {
+    aLists.BorderBackground()->AppendNewToTop(new (aBuilder)
       nsDisplayGenericOverflow(aBuilder, this, ::PaintColumnRule, "ColumnRule",
                                nsDisplayItem::TYPE_COLUMN_RULE));
-  
+  }
+
   // Our children won't have backgrounds so it doesn't matter where we put them.
   for (nsFrameList::Enumerator e(mFrames); !e.AtEnd(); e.Next()) {
     BuildDisplayListForChild(aBuilder, e.get(), aDirtyRect, aLists);
