@@ -25,9 +25,7 @@
 #include "nsAccessibilityService.h"
 #endif
 
-namespace dom = mozilla::dom;
-
-static NS_DEFINE_CID(kCStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
+using namespace mozilla;
 
 class Area {
 public:
