@@ -178,6 +178,10 @@ CSS_KEY(block, block)
 CSS_KEY(block-axis, block_axis)
 CSS_KEY(blur, blur)
 CSS_KEY(bold, bold)
+CSS_KEY(bold-fraktur, bold_fraktur)
+CSS_KEY(bold-italic, bold_italic)
+CSS_KEY(bold-sans-serif, bold_sans_serif)
+CSS_KEY(bold-script, bold_script)
 CSS_KEY(bolder, bolder)
 CSS_KEY(border-box, border_box)
 CSS_KEY(both, both)
@@ -200,6 +204,7 @@ CSS_KEY(center, center)
 CSS_KEY(ch, ch)
 CSS_KEY(character-variant, character_variant)
 CSS_KEY(circle, circle)
+CSS_KEY(cjk-decimal, cjk_decimal)
 CSS_KEY(cjk-ideographic, cjk_ideographic)
 CSS_KEY(clip, clip)
 CSS_KEY(close-quote, close_quote)
@@ -247,6 +252,7 @@ CSS_KEY(disc, disc)
 CSS_KEY(discretionary-ligatures, discretionary_ligatures)
 CSS_KEY(dotted, dotted)
 CSS_KEY(double, double)
+CSS_KEY(double-struck, double_struck)
 CSS_KEY(drop-shadow, drop_shadow)
 CSS_KEY(e-resize, e_resize)
 CSS_KEY(each-box, each_box)
@@ -278,9 +284,12 @@ CSS_KEY(flex-end, flex_end)
 CSS_KEY(flex-start, flex_start)
 CSS_KEY(flip, flip)
 CSS_KEY(forwards, forwards)
+CSS_KEY(fraktur, fraktur)
 CSS_KEY(from-image, from_image)
 CSS_KEY(full-width, full_width)
 CSS_KEY(georgian, georgian)
+CSS_KEY(grab, grab)
+CSS_KEY(grabbing, grabbing)
 CSS_KEY(grad, grad)
 CSS_KEY(grayscale, grayscale)
 CSS_KEY(graytext, graytext)
@@ -324,6 +333,8 @@ CSS_KEY(inside, inside)
 CSS_KEY(interpolatematrix, interpolatematrix)
 CSS_KEY(invert, invert)
 CSS_KEY(italic, italic)
+CSS_KEY(japanese-formal, japanese_formal)
+CSS_KEY(japanese-informal, japanese_informal)
 CSS_KEY(jis78, jis78)
 CSS_KEY(jis83, jis83)
 CSS_KEY(jis90, jis90)
@@ -333,6 +344,9 @@ CSS_KEY(katakana, katakana)
 CSS_KEY(katakana-iroha, katakana_iroha)
 CSS_KEY(keep-all, keep_all)
 CSS_KEY(khz, khz)
+CSS_KEY(korean-hangul-formal, korean_hangul_formal)
+CSS_KEY(korean-hanja-formal, korean_hanja_formal)
+CSS_KEY(korean-hanja-informal, korean_hanja_informal)
 CSS_KEY(landscape, landscape)
 CSS_KEY(large, large)
 CSS_KEY(larger, larger)
@@ -345,6 +359,7 @@ CSS_KEY(lining-nums, lining_nums)
 CSS_KEY(list-item, list_item)
 CSS_KEY(local, local)
 CSS_KEY(logical, logical)
+CSS_KEY(looped, looped)
 CSS_KEY(lower-alpha, lower_alpha)
 CSS_KEY(lower-greek, lower_greek)
 CSS_KEY(lower-latin, lower_latin)
@@ -365,6 +380,7 @@ CSS_KEY(message-box, message_box)
 CSS_KEY(middle, middle)
 CSS_KEY(mix, mix)
 CSS_KEY(mm, mm)
+CSS_KEY(monospace, monospace)
 CSS_KEY(move, move)
 CSS_KEY(ms, ms)
 CSS_KEY(multiply, multiply)
@@ -448,6 +464,7 @@ CSS_KEY(scalex, scalex)
 CSS_KEY(scaley, scaley)
 CSS_KEY(scalez, scalez)
 CSS_KEY(screen, screen)
+CSS_KEY(script, script)
 CSS_KEY(scroll, scroll)
 CSS_KEY(scrollbar, scrollbar)
 CSS_KEY(scrollbar-small, scrollbar_small)
@@ -463,6 +480,8 @@ CSS_KEY(separate, separate)
 CSS_KEY(sepia, sepia)
 CSS_KEY(show, show)
 CSS_KEY(sideways, sideways)
+CSS_KEY(simp-chinese-formal, simp_chinese_formal)
+CSS_KEY(simp-chinese-informal, simp_chinese_informal)
 CSS_KEY(simplified, simplified)
 CSS_KEY(skew, skew)
 CSS_KEY(skewx, skewx)
@@ -487,6 +506,7 @@ CSS_KEY(step-start, step_start)
 CSS_KEY(sticky, sticky)
 CSS_KEY(stretch, stretch)
 CSS_KEY(stretch-to-fit, stretch_to_fit)
+CSS_KEY(stretched, stretched)
 CSS_KEY(stroke, stroke)
 CSS_KEY(style, style)
 CSS_KEY(styleset, styleset)
@@ -505,6 +525,7 @@ CSS_KEY(table-header-group, table_header_group)
 CSS_KEY(table-row, table_row)
 CSS_KEY(table-row-group, table_row_group)
 CSS_KEY(tabular-nums, tabular_nums)
+CSS_KEY(tailed, tailed)
 CSS_KEY(text, text)
 CSS_KEY(text-bottom, text_bottom)
 CSS_KEY(text-top, text_top)
@@ -519,6 +540,8 @@ CSS_KEY(titling-caps, titling_caps)
 CSS_KEY(toggle, toggle)
 CSS_KEY(top, top)
 CSS_KEY(top-outside, top_outside)
+CSS_KEY(trad-chinese-formal, trad_chinese_formal)
+CSS_KEY(trad-chinese-informal, trad_chinese_informal)
 CSS_KEY(traditional, traditional)
 CSS_KEY(translate, translate)
 CSS_KEY(translate3d, translate3d)
@@ -527,10 +550,12 @@ CSS_KEY(translatey, translatey)
 CSS_KEY(translatez, translatez)
 CSS_KEY(transparent, transparent) // for nsComputedDOMStyle only
 CSS_KEY(tri-state, tri_state)
+CSS_KEY(true, true)
 CSS_KEY(ultra-condensed, ultra_condensed)
 CSS_KEY(ultra-expanded, ultra_expanded)
 CSS_KEY(underline, underline)
 CSS_KEY(unicase, unicase)
+CSS_KEY(unset, unset)
 CSS_KEY(upper-alpha, upper_alpha)
 CSS_KEY(upper-latin, upper_latin)
 CSS_KEY(upper-roman, upper_roman)
@@ -552,6 +577,8 @@ CSS_KEY(wider, wider)
 CSS_KEY(window, window)
 CSS_KEY(windowframe, windowframe)
 CSS_KEY(windowtext, windowtext)
+CSS_KEY(wrap, wrap)
+CSS_KEY(wrap-reverse, wrap_reverse)
 CSS_KEY(write-only, write_only)
 CSS_KEY(x-large, x_large)
 CSS_KEY(x-small, x_small)
@@ -582,6 +609,7 @@ CSS_KEY(resizerpanel, resizerpanel)
 CSS_KEY(resizer, resizer)
 CSS_KEY(listbox, listbox)
 CSS_KEY(listitem, listitem)
+CSS_KEY(number-input, number_input)
 CSS_KEY(treeview, treeview)
 CSS_KEY(treeitem, treeitem)
 CSS_KEY(treetwisty, treetwisty)
@@ -636,6 +664,9 @@ CSS_KEY(meterbar, meterbar)
 CSS_KEY(meterchunk, meterchunk)
 CSS_KEY(range, range)
 CSS_KEY(range-thumb, range_thumb)
+CSS_KEY(sans-serif, sans_serif)
+CSS_KEY(sans-serif-bold-italic, sans_serif_bold_italic)
+CSS_KEY(sans-serif-italic, sans_serif_italic)
 CSS_KEY(scale-horizontal, scale_horizontal)
 CSS_KEY(scale-vertical, scale_vertical)
 CSS_KEY(scalethumb-horizontal, scalethumb_horizontal)
