@@ -161,12 +161,12 @@ GroupRuleRuleList::~GroupRuleRuleList()
 {
 }
 
-// QueryInterface implementation for CSSGroupRuleRuleList
+// QueryInterface implementation for GroupRuleRuleList
 NS_INTERFACE_MAP_BEGIN(GroupRuleRuleList)
   NS_INTERFACE_MAP_ENTRY(nsICSSRuleList)
   NS_INTERFACE_MAP_ENTRY(nsIDOMCSSRuleList)
   NS_INTERFACE_MAP_ENTRY(nsISupports)
-  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(CSSGroupRuleRuleList)
+  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(CSSRuleList)
 NS_INTERFACE_MAP_END
 
 
@@ -202,9 +202,6 @@ GroupRuleRuleList::IndexedGetter(uint32_t aIndex, bool& aFound)
 } // namespace css
 } // namespace mozilla
 
-// Must be outside the namespace
-DOMCI_DATA(CSSGroupRuleRuleList, css::GroupRuleRuleList)
-
 // -------------------------------------------
 // CharsetRule
 //
@@ -416,10 +413,7 @@ ImportRule::SetSheet(nsCSSStyleSheet* aSheet)
   aSheet->SetOwnerRule(this);
 
   // set our medialist to be the same as the sheet's medialist
-  nsCOMPtr<nsIDOMMediaList> mediaList;
-  mChildSheet->GetMedia(getter_AddRefs(mediaList));
-  NS_ABORT_IF_FALSE(mediaList, "GetMedia returned null");
-  mMedia = static_cast<nsMediaList*>(mediaList.get());
+  mMedia = mChildSheet->Media();
 }
 
 NS_IMETHODIMP
@@ -511,14 +505,6 @@ ImportRule::SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const
 // must be outside the namespace
 DOMCI_DATA(CSSImportRule, css::ImportRule)
 
-static bool
-CloneRuleInto(css::Rule* aRule, void* aArray)
-{
-  nsRefPtr<css::Rule> clone = aRule->Clone();
-  static_cast<nsCOMArray<css::Rule>*>(aArray)->AppendObject(clone);
-  return true;
-}
-
 namespace mozilla {
 namespace css {
 
@@ -538,7 +524,7 @@ SetParentRuleReference(Rule* aRule, void* aParentRule)
 GroupRule::GroupRule(const GroupRule& aCopy)
   : Rule(aCopy)
 {
-  const_cast<GroupRule&>(aCopy).mRules.EnumerateForwards(CloneRuleInto, &mRules);
+  const_cast<GroupRule&>(aCopy).mRules.EnumerateForwards(GroupRule::CloneRuleInto, &mRules);
   mRules.EnumerateForwards(SetParentRuleReference, this);
 }
 
@@ -776,11 +762,9 @@ MediaRule::MediaRule(const MediaRule& aCopy)
   : GroupRule(aCopy)
 {
   if (aCopy.mMedia) {
-    aCopy.mMedia->Clone(getter_AddRefs(mMedia));
-    if (mMedia) {
-      // XXXldb This doesn't really make sense.
-      mMedia->SetStyleSheet(aCopy.GetStyleSheet());
-    }
+    mMedia = aCopy.mMedia->Clone();
+    // XXXldb This doesn't really make sense.
+    mMedia->SetStyleSheet(aCopy.GetStyleSheet());
   }
 }
 
