@@ -221,16 +221,16 @@ nsAutoFilterInstance::nsAutoFilterInstance(nsIFrame *aTarget,
       // nothing to draw
       return;
     }
-    float scale = nsSVGUtils::MaxExpansion(canvasTM);
 
-    filterRegion.Scale(scale);
+    gfxSize scale = canvasTM.ScaleFactors(true);
+    filterRegion.Scale(scale.width, scale.height);
     filterRegion.RoundOut();
     // We don't care if this overflows, because we can handle upscaling/
     // downscaling to filterRes
     bool overflow;
     filterRes = nsSVGUtils::ConvertToSurfaceSize(filterRegion.Size(),
                                                  &overflow);
-    filterRegion.Scale(1.0 / scale);
+    filterRegion.Scale(1.0 / scale.width, 1.0 / scale.height);
   }
 
   // Get various transforms:
@@ -451,13 +451,7 @@ nsSVGFilterFrame::PaintFilteredFrame(nsRenderingContext *aContext,
   if (!instance.get()) {
     return NS_OK;
   }
-  nsRefPtr<gfxASurface> result;
-  nsresult rv = instance.get()->Render(getter_AddRefs(result));
-  if (NS_SUCCEEDED(rv) && result) {
-    nsSVGUtils::CompositeSurfaceMatrix(aContext->ThebesContext(),
-      result, instance.get()->GetFilterSpaceToDeviceSpaceTransform(), 1.0);
-  }
-  return rv;
+  return instance.get()->Render(aContext->ThebesContext());
 }
 
 static nsRect
@@ -531,11 +525,8 @@ nsSVGFilterFrame::GetPostFilterBounds(nsIFrame *aFilteredFrame,
   if (!instance.get()) {
     return nsRect();
   }
-  // We've passed in the source's bounding box so the instance knows about
-  // it. Now we can ask the instance to compute the bounding box of
-  // the filter output.
   nsIntRect bbox;
-  nsresult rv = instance.get()->ComputeOutputBBox(&bbox);
+  nsresult rv = instance.get()->ComputePostFilterExtents(&bbox);
   if (NS_SUCCEEDED(rv)) {
     return TransformFilterSpaceToFrameSpace(instance.get(), &bbox);
   }
