@@ -4,6 +4,7 @@
  * This program is made available under an ISC-style license.  See the
  * accompanying file LICENSE for details.
  */
+#undef NDEBUG
 #if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
@@ -40,6 +41,12 @@ ms_to_hns(uint32_t ms)
   return ms * 10000;
 }
 
+uint32_t
+hns_to_ms(uint32_t hns)
+{
+  return hns / 10000;
+}
+
 double
 hns_to_s(uint32_t hns)
 {
@@ -81,7 +88,6 @@ extern cubeb_ops const wasapi_ops;
 struct cubeb
 {
   cubeb_ops const * ops;
-  IMMDevice * device;
   /* Library dynamically opened to increase the render
    * thread priority, and the two function pointers we need. */
   HMODULE mmcss_module;
@@ -129,10 +135,8 @@ struct cubeb_stream
   uint32_t leftover_frame_size;
   float * leftover_frames_buffer;
   /* Buffer used to downmix or upmix to the number of channels the mixer has.
-   * its size is |buffer_frame_count * bytes_per_frame * mixer_channels|. */
+   * its size is |frames_to_bytes_before_mix(buffer_frame_count)|. */
   float * mix_buffer;
-  /* Number of bytes per frame. Prefer to use frames_to_bytes_before_mix. */
-  uint8_t bytes_per_frame;
   /* True if the stream is draining. */
   bool draining;
 };
@@ -155,7 +159,7 @@ void
 mono_to_stereo(T * in, long insamples, T * out)
 {
   int j = 0;
-  for (int i = 0; i < insamples; i++, j+=2) {
+  for (int i = 0; i < insamples; ++i, j += 2) {
     out[j] = out[j + 1] = in[i];
   }
 }
@@ -164,6 +168,7 @@ template<typename T>
 void
 upmix(T * in, long inframes, T * out, int32_t in_channels, int32_t out_channels)
 {
+  assert(out_channels >= in_channels);
   /* If we are playing a mono stream over stereo speakers, copy the data over. */
   if (in_channels == 1 && out_channels == 2) {
     mono_to_stereo(in, inframes, out);
@@ -171,14 +176,11 @@ upmix(T * in, long inframes, T * out, int32_t in_channels, int32_t out_channels)
   }
   /* Otherwise, put silence in other channels. */
   long out_index = 0;
-  for (long i = 0; i < inframes * in_channels; i+=in_channels) {
-    for (int j = 0; j < in_channels; j++) {
+  for (long i = 0; i < inframes * in_channels; i += in_channels) {
+    for (int j = 0; j < in_channels; ++j) {
       out[out_index + j] = in[i + j];
-      if (in_channels == 1) {
-        out[out_index + j + 1] = in[i + j];
-      }
     }
-    for (int j = in_channels; j < out_channels; j++) {
+    for (int j = in_channels; j < out_channels; ++j) {
       out[out_index + j] = 0.0;
     }
     out_index += out_channels;
@@ -187,21 +189,25 @@ upmix(T * in, long inframes, T * out, int32_t in_channels, int32_t out_channels)
 
 template<typename T>
 void
-downmix_to_stereo(T * in, long inframes, T * out, int32_t in_channels)
+downmix(T * in, long inframes, T * out, int32_t in_channels, int32_t out_channels)
 {
+  assert(in_channels >= out_channels);
   /* We could use a downmix matrix here, applying mixing weight based on the
    * channel, but directsound and winmm simply drop the channels that cannot be
    * rendered by the hardware, so we do the same for consistency. */
-  for (int32_t i = 0; i < inframes; i++) {
-    out[i * 2] = in[i * in_channels];
-    out[i * 2 + 1] = in[i * in_channels + 1];
+  long out_index = 0;
+  for (long i = 0; i < inframes * in_channels; i += in_channels) {
+    for (int j = 0; j < out_channels; ++j) {
+      out[out_index + j] = in[i + j];
+    }
+    out_index += out_channels;
   }
 }
 
 /* This returns the size of a frame in the stream,
  * before the eventual upmix occurs. */
 static size_t
-frame_to_bytes_before_mix(cubeb_stream * stm, size_t frames)
+frames_to_bytes_before_mix(cubeb_stream * stm, size_t frames)
 {
   size_t stream_frame_size = stm->stream_params.channels * sizeof(float);
   return stream_frame_size * frames;
@@ -221,7 +227,7 @@ refill_with_resampling(cubeb_stream * stm, float * data, long frames_needed)
   long frame_requested = before_resampling - stm->leftover_frame_count;
 
   size_t leftover_bytes =
-    frame_to_bytes_before_mix(stm, stm->leftover_frame_count);
+    frames_to_bytes_before_mix(stm, stm->leftover_frame_count);
 
   /* Copy the previous leftover frames to the front of the buffer. */
   memcpy(stm->resampling_src_buffer, stm->leftover_frames_buffer, leftover_bytes);
@@ -255,11 +261,11 @@ refill_with_resampling(cubeb_stream * stm, float * data, long frames_needed)
   /* Copy the leftover frames to buffer for the next time. */
   stm->leftover_frame_count = before_resampling - in_frames;
   size_t unresampled_bytes =
-    frame_to_bytes_before_mix(stm, stm->leftover_frame_count);
+    frames_to_bytes_before_mix(stm, stm->leftover_frame_count);
 
   uint8_t * leftover_frames_start =
     reinterpret_cast<uint8_t *>(stm->resampling_src_buffer);
-  leftover_frames_start += frame_to_bytes_before_mix(stm, in_frames);
+  leftover_frames_start += frames_to_bytes_before_mix(stm, in_frames);
 
   assert(stm->leftover_frame_count <= stm->leftover_frame_size);
   memcpy(stm->leftover_frames_buffer, leftover_frames_start, unresampled_bytes);
@@ -272,8 +278,8 @@ refill_with_resampling(cubeb_stream * stm, float * data, long frames_needed)
     upmix(resample_dest, out_frames, data,
           stm->stream_params.channels, stm->mix_params.channels);
   } else if (should_downmix(stm)) {
-    downmix_to_stereo(resample_dest, out_frames, data,
-                      stm->stream_params.channels);
+    downmix(resample_dest, out_frames, data,
+            stm->stream_params.channels, stm->mix_params.channels);
   }
 }
 
@@ -290,7 +296,7 @@ refill(cubeb_stream * stm, float * data, long frames_needed)
   }
 
   long got = stm->data_callback(stm, stm->user_ptr, dest, frames_needed);
-
+  assert(got <= frames_needed);
   if (got != frames_needed) {
     LOG("draining.");
     stm->draining = true;
@@ -299,8 +305,9 @@ refill(cubeb_stream * stm, float * data, long frames_needed)
   if (should_upmix(stm)) {
     upmix(dest, got, data,
           stm->stream_params.channels, stm->mix_params.channels);
-  } else {
-    downmix_to_stereo(dest, got, data, stm->stream_params.channels);
+  } else if (should_downmix(stm)) {
+    downmix(dest, got, data,
+            stm->stream_params.channels, stm->mix_params.channels);
   }
 }
 
@@ -357,6 +364,7 @@ wasapi_stream_render_loop(LPVOID stream)
         is_playing = false;
         continue;
       }
+      assert(padding <= stm->buffer_frame_count);
 
       if (stm->draining) {
         if (padding == 0) {
@@ -391,7 +399,7 @@ wasapi_stream_render_loop(LPVOID stream)
     default:
       LOG("case %d not handled in render loop.", waitResult);
       abort();
-    };
+    }
   }
 
   if (FAILED(hr)) {
@@ -415,13 +423,38 @@ BOOL WINAPI revert_mm_thread_characteristics_noop(HANDLE mmcss_handle)
 {
   return true;
 }
+
+HRESULT get_default_endpoint(IMMDevice ** device)
+{
+  IMMDeviceEnumerator * enumerator;
+  HRESULT hr = CoCreateInstance(__uuidof(MMDeviceEnumerator),
+                                NULL, CLSCTX_INPROC_SERVER,
+                                IID_PPV_ARGS(&enumerator));
+  if (FAILED(hr)) {
+    LOG("Could not get device enumerator.");
+    return hr;
+  }
+  /* eMultimedia is okay for now ("Music, movies, narration, [...]").
+   * We will need to change this when we distinguish streams by use-case, other
+   * possible values being eConsole ("Games, system notification sounds [...]")
+   * and eCommunication ("Voice communication"). */
+  hr = enumerator->GetDefaultAudioEndpoint(eRender, eMultimedia, device);
+  if (FAILED(hr)) {
+    LOG("Could not get default audio endpoint.");
+    SafeRelease(enumerator);
+    return hr;
+  }
+
+  SafeRelease(enumerator);
+
+  return ERROR_SUCCESS;
+}
 } // namespace anonymous
 
 extern "C" {
 int wasapi_init(cubeb ** context, char const * context_name)
 {
   HRESULT hr;
-  IMMDeviceEnumerator * enumerator = NULL;
 
   hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
   if (FAILED(hr)) {
@@ -429,33 +462,21 @@ int wasapi_init(cubeb ** context, char const * context_name)
     return CUBEB_ERROR;
   }
 
+  /* We don't use the device yet, but need to make sure we can initialize one
+     so that this backend is not incorrectly enabled on platforms that don't
+     support WASAPI. */
+  IMMDevice * device;
+  hr = get_default_endpoint(&device);
+  if (FAILED(hr)) {
+    LOG("Could not get device.");
+    return CUBEB_ERROR;
+  }
+  SafeRelease(device);
+
   cubeb * ctx = (cubeb *)calloc(1, sizeof(cubeb));
 
   ctx->ops = &wasapi_ops;
 
-  hr = CoCreateInstance(__uuidof(MMDeviceEnumerator),
-                        NULL, CLSCTX_INPROC_SERVER,
-                        IID_PPV_ARGS(&enumerator));
-  if (FAILED(hr)) {
-    LOG("Could not get device enumerator.");
-    wasapi_destroy(ctx);
-    return CUBEB_ERROR;
-  }
-
-  /* eMultimedia is okay for now ("Music, movies, narration, [...]").
-   * We will need to change this when we distinguish streams by use-case, other
-   * possible values being eConsole ("Games, system notification sounds [...]")
-   * and eCommunication ("Voice communication"). */
-  hr = enumerator->GetDefaultAudioEndpoint(eRender,
-                                           eMultimedia,
-                                           &ctx->device);
-  if (FAILED(hr)) {
-    LOG("Could not get default audio endpoint.");
-    SafeRelease(enumerator);
-    wasapi_destroy(ctx);
-    return CUBEB_ERROR;
-  }
-
   ctx->mmcss_module = LoadLibraryA("Avrt.dll");
 
   if (ctx->mmcss_module) {
@@ -476,8 +497,6 @@ int wasapi_init(cubeb ** context, char const * context_name)
     ctx->set_mm_thread_characteristics = &set_mm_thread_characteristics_noop;
     ctx->revert_mm_thread_characteristics = &revert_mm_thread_characteristics_noop;
   }
-  
-  SafeRelease(enumerator);
 
   *context = ctx;
 
@@ -489,7 +508,6 @@ namespace {
 
 void wasapi_destroy(cubeb * context)
 {
-  SafeRelease(context->device);
   if (context->mmcss_module) {
     FreeLibrary(context->mmcss_module);
   }
@@ -504,21 +522,28 @@ char const* wasapi_get_backend_id(cubeb * context)
 int
 wasapi_get_max_channel_count(cubeb * ctx, uint32_t * max_channels)
 {
-  HRESULT hr;
   IAudioClient * client;
   WAVEFORMATEX * mix_format;
 
   assert(ctx && max_channels);
 
-  hr = ctx->device->Activate(__uuidof(IAudioClient),
-                             CLSCTX_INPROC_SERVER,
-                             NULL, (void **)&client);
+  IMMDevice * device;
+  HRESULT hr = get_default_endpoint(&device);
+  if (FAILED(hr)) {
+    return CUBEB_ERROR;
+  }
+
+  hr = device->Activate(__uuidof(IAudioClient),
+                        CLSCTX_INPROC_SERVER,
+                        NULL, (void **)&client);
+  SafeRelease(device);
   if (FAILED(hr)) {
     return CUBEB_ERROR;
   }
 
   hr = client->GetMixFormat(&mix_format);
   if (FAILED(hr)) {
+    SafeRelease(client);
     return CUBEB_ERROR;
   }
 
@@ -530,6 +555,78 @@ wasapi_get_max_channel_count(cubeb * ctx, uint32_t * max_channels)
   return CUBEB_OK;
 }
 
+int
+wasapi_get_min_latency(cubeb * ctx, cubeb_stream_params params, uint32_t * latency_ms)
+{
+  HRESULT hr;
+  IAudioClient * client;
+  REFERENCE_TIME default_period;
+
+  IMMDevice * device;
+  hr = get_default_endpoint(&device);
+  if (FAILED(hr)) {
+    return CUBEB_ERROR;
+  }
+
+  hr = device->Activate(__uuidof(IAudioClient),
+                        CLSCTX_INPROC_SERVER,
+                        NULL, (void **)&client);
+  SafeRelease(device);
+  if (FAILED(hr)) {
+    return CUBEB_ERROR;
+  }
+
+  /* The second parameter is for exclusive mode, that we don't use. */
+  hr = client->GetDevicePeriod(&default_period, NULL);
+  if (FAILED(hr)) {
+    SafeRelease(client);
+    return CUBEB_ERROR;
+  }
+
+  /* According to the docs, the best latency we can achieve is by synchronizing
+   * the stream and the engine.
+   * http://msdn.microsoft.com/en-us/library/windows/desktop/dd370871%28v=vs.85%29.aspx */
+  *latency_ms = hns_to_ms(default_period);
+
+  SafeRelease(client);
+
+  return CUBEB_OK;
+}
+
+int
+wasapi_get_preferred_sample_rate(cubeb * ctx, uint32_t * rate)
+{
+  HRESULT hr;
+  IAudioClient * client;
+  WAVEFORMATEX * mix_format;
+
+  IMMDevice * device;
+  hr = get_default_endpoint(&device);
+  if (FAILED(hr)) {
+    return CUBEB_ERROR;
+  }
+
+  hr = device->Activate(__uuidof(IAudioClient),
+                        CLSCTX_INPROC_SERVER,
+                        NULL, (void **)&client);
+  SafeRelease(device);
+  if (FAILED(hr)) {
+    return CUBEB_ERROR;
+  }
+
+  hr = client->GetMixFormat(&mix_format);
+  if (FAILED(hr)) {
+    SafeRelease(client);
+    return CUBEB_ERROR;
+  }
+
+  *rate = mix_format->nSamplesPerSec;
+
+  CoTaskMemFree(mix_format);
+  SafeRelease(client);
+
+  return CUBEB_OK;
+}
 
 void wasapi_stream_destroy(cubeb_stream * stm);
 
@@ -540,11 +637,12 @@ handle_channel_layout(cubeb_stream * stm,  WAVEFORMATEX ** mix_format, const cub
 {
   /* Common case: the hardware is stereo. Up-mixing and down-mixing will be
    * handled in the callback. */
-  if ((*mix_format)->nChannels == 2) {
+  if ((*mix_format)->nChannels <= 2) {
     return;
   }
 
   /* Otherwise, the hardware supports more than two channels. */
+  WAVEFORMATEX hw_mixformat = **mix_format;
 
   /* The docs say that GetMixFormat is always of type WAVEFORMATEXTENSIBLE [1],
    * so the reinterpret_cast below should be safe. In practice, this is not
@@ -584,17 +682,19 @@ handle_channel_layout(cubeb_stream * stm,  WAVEFORMATEX ** mix_format, const cub
 
   if (hr == S_FALSE) {
     /* Not supported, but WASAPI gives us a suggestion. Use it, and handle the
-     * eventual upmix/downmix ourselve */
+     * eventual upmix/downmix ourselves */
     LOG("Using WASAPI suggested format: channels: %d", closest->nChannels);
+    WAVEFORMATEXTENSIBLE * closest_pcm = reinterpret_cast<WAVEFORMATEXTENSIBLE *>(closest);
+    assert(closest_pcm->SubFormat == format_pcm->SubFormat);
     CoTaskMemFree(*mix_format);
     *mix_format = closest;
   } else if (hr == AUDCLNT_E_UNSUPPORTED_FORMAT) {
-    /* Not supported, no suggestion, there is a bug somewhere. */
-    assert(false && "Format not supported, and no suggestion from WASAPI.");
+    /* Not supported, no suggestion. This should not happen, but it does in the
+     * field with some sound cards. We restore the mix format, and let the rest
+     * of the code figure out the right conversion path. */
+    **mix_format = hw_mixformat;
   } else if (hr == S_OK) {
     LOG("Requested format accepted by WASAPI.");
-  } else {
-    assert(false && "Not reached.");
   }
 }
 
@@ -615,12 +715,6 @@ wasapi_stream_init(cubeb * context, cubeb_stream ** stream,
     return CUBEB_ERROR;
   }
 
-  /* 30ms in shared mode is the minimum we can get when using WASAPI */
-  if (latency < 30) {
-    LOG("Latency too low: got %u (30ms minimum)", latency);
-    return CUBEB_ERROR_INVALID_PARAMETER;
-  }
-
   cubeb_stream * stm = (cubeb_stream *)calloc(1, sizeof(cubeb_stream));
 
   assert(stm);
@@ -649,11 +743,19 @@ wasapi_stream_init(cubeb * context, cubeb_stream ** stream,
     return CUBEB_ERROR;
   }
 
+  IMMDevice * device;
+  hr = get_default_endpoint(&device);
+  if (FAILED(hr)) {
+    wasapi_stream_destroy(stm);
+    return CUBEB_ERROR;
+  }
+
   /* Get a client. We will get all other interfaces we need from
    * this pointer. */
-  hr = context->device->Activate(__uuidof(IAudioClient),
-                                 CLSCTX_INPROC_SERVER,
-                                 NULL, (void **)&stm->client);
+  hr = device->Activate(__uuidof(IAudioClient),
+                        CLSCTX_INPROC_SERVER,
+                        NULL, (void **)&stm->client);
+  SafeRelease(device);
   if (FAILED(hr)) {
     LOG("Could not activate the device to get an audio client.");
     wasapi_stream_destroy(stm);
@@ -662,10 +764,12 @@ wasapi_stream_init(cubeb * context, cubeb_stream ** stream,
 
   /* We have to distinguish between the format the mixer uses,
   * and the format the stream we want to play uses. */
-  stm->client->GetMixFormat(&mix_format);
-
-  /* this is the number of bytes per frame after the eventual upmix. */
-  stm->bytes_per_frame = static_cast<uint8_t>(mix_format->nBlockAlign);
+  hr = stm->client->GetMixFormat(&mix_format);
+  if (FAILED(hr)) {
+    LOG("Could not fetch current mix format from the audio client.");
+    wasapi_stream_destroy(stm);
+    return CUBEB_ERROR;
+  }
 
   handle_channel_layout(stm, &mix_format, &stream_params);
 
@@ -700,7 +804,7 @@ wasapi_stream_init(cubeb * context, cubeb_stream ** stream,
      * that is, the samples not consumed by the resampler that we will end up
      * using next time the render callback is called. */
     stm->leftover_frame_size = static_cast<uint32_t>(ceilf(1 / resampling_rate * 2) + 1);
-    stm->leftover_frames_buffer = (float *)malloc(frame_to_bytes_before_mix(stm, stm->leftover_frame_size));
+    stm->leftover_frames_buffer = (float *)malloc(frames_to_bytes_before_mix(stm, stm->leftover_frame_size));
 
     stm->refill_function = &refill_with_resampling;
   } else {
@@ -730,10 +834,8 @@ wasapi_stream_init(cubeb * context, cubeb_stream ** stream,
     return CUBEB_ERROR;
   }
 
-  assert(stm->mix_params.channels >= 2);
-
   if (should_upmix(stm) || should_downmix(stm)) {
-    stm->mix_buffer = (float *) malloc(frame_to_bytes_before_mix(stm, stm->buffer_frame_count));
+    stm->mix_buffer = (float *) malloc(frames_to_bytes_before_mix(stm, stm->buffer_frame_count));
   }
 
   /* If we are going to resample, we will end up needing a buffer
@@ -742,7 +844,7 @@ wasapi_stream_init(cubeb * context, cubeb_stream ** stream,
    * factor and the channel layout into account. */
   if (stm->resampler) {
     size_t frames_needed = static_cast<size_t>(frame_count_at_rate(stm->buffer_frame_count, resampling_rate));
-    stm->resampling_src_buffer = (float *)malloc(frame_to_bytes_before_mix(stm, frames_needed));
+    stm->resampling_src_buffer = (float *)malloc(frames_to_bytes_before_mix(stm, frames_needed));
   }
 
   hr = stm->client->SetEventHandle(stm->refill_event);
@@ -753,7 +855,7 @@ wasapi_stream_init(cubeb * context, cubeb_stream ** stream,
   }
 
   hr = stm->client->GetService(__uuidof(IAudioRenderClient),
-                                     (void **)&stm->render_client);
+                               (void **)&stm->render_client);
   if (FAILED(hr)) {
     LOG("Could not get the render client.");
     wasapi_stream_destroy(stm);
@@ -761,7 +863,7 @@ wasapi_stream_init(cubeb * context, cubeb_stream ** stream,
   }
 
   hr = stm->client->GetService(__uuidof(IAudioClock),
-                                     (void **)&stm->audio_clock);
+                               (void **)&stm->audio_clock);
   if (FAILED(hr)) {
     LOG("Could not get the IAudioClock.");
     wasapi_stream_destroy(stm);
@@ -892,14 +994,16 @@ int wasapi_stream_get_latency(cubeb_stream * stm, uint32_t * latency)
 cubeb_ops const wasapi_ops = {
   /*.init =*/ wasapi_init,
   /*.get_backend_id =*/ wasapi_get_backend_id,
-  /*.get_max_channel_count*/ wasapi_get_max_channel_count,
+  /*.get_max_channel_count =*/ wasapi_get_max_channel_count,
+  /*.get_min_latency =*/ wasapi_get_min_latency,
+  /*.get_preferred_sample_rate =*/ wasapi_get_preferred_sample_rate,
   /*.destroy =*/ wasapi_destroy,
   /*.stream_init =*/ wasapi_stream_init,
   /*.stream_destroy =*/ wasapi_stream_destroy,
   /*.stream_start =*/ wasapi_stream_start,
   /*.stream_stop =*/ wasapi_stream_stop,
-  /*.stream_get_position =*/ wasapi_stream_get_position
-  ///*.stream_get_latency =*/ wasapi_stream_get_latency
+  /*.stream_get_position =*/ wasapi_stream_get_position,
+  /*.stream_get_latency =*/ wasapi_stream_get_latency
  };
 } // namespace anonymous
 
