@@ -15,6 +15,7 @@
 #include "webrtc/common_video/libyuv/include/scaler.h"
 #include "webrtc/system_wrappers/interface/tick_util.h"
 #include "webrtc/test/testsupport/fileutils.h"
+#include "webrtc/test/testsupport/gtest_disable.h"
 
 namespace webrtc {
 
@@ -28,9 +29,6 @@ class TestScaler : public ::testing::Test {
                      FILE* source_file, std::string out_name,
                      int src_width, int src_height,
                      int dst_width, int dst_height);
-
-  // TODO(mikhal): add a sequence reader to libyuv.
-
   // Computes the sequence average PSNR between an input sequence in
   // |input_file| and an output sequence with filename |out_name|. |width| and
   // |height| are the frame sizes of both sequences.
@@ -117,7 +115,7 @@ TEST_F(TestScaler, ScaleSendingBufferTooSmall) {
 }
 
 //TODO (mikhal): Converge the test into one function that accepts the method.
-TEST_F(TestScaler, PointScaleTest) {
+TEST_F(TestScaler, DISABLED_ON_ANDROID(PointScaleTest)) {
   double avg_psnr;
   FILE* source_file2;
   ScaleMethod method = kScalePoint;
@@ -162,7 +160,7 @@ TEST_F(TestScaler, PointScaleTest) {
                 source_file_, out_name,
                 width_, height_,
                 400, 300);
-  // Dowsample to odd size frame and scale back up.
+  // Down-sample to odd size frame and scale back up.
   out_name = webrtc::test::OutputPath() + "LibYuvTest_PointScale_282_231.yuv";
   ScaleSequence(method,
                 source_file_, out_name,
@@ -182,7 +180,7 @@ TEST_F(TestScaler, PointScaleTest) {
   // average PSNR under same conditions.
   ASSERT_GT(avg_psnr, 25.8);
   ASSERT_EQ(0, fclose(source_file2));
-  // Upsample to odd size frame and scale back down.
+  // Up-sample to odd size frame and scale back down.
   out_name = webrtc::test::OutputPath() + "LibYuvTest_PointScale_699_531.yuv";
   ScaleSequence(method,
                 source_file_, out_name,
@@ -204,7 +202,7 @@ TEST_F(TestScaler, PointScaleTest) {
   ASSERT_EQ(0, fclose(source_file2));
 }
 
-TEST_F(TestScaler, BiLinearScaleTest) {
+TEST_F(TestScaler, DISABLED_ON_ANDROID(BiLinearScaleTest)) {
   double avg_psnr;
   FILE* source_file2;
   ScaleMethod method = kScaleBilinear;
@@ -214,7 +212,7 @@ TEST_F(TestScaler, BiLinearScaleTest) {
                 source_file_, out_name,
                 width_, height_,
                 width_ / 2, height_ / 2);
-  // Upsample back up and check PSNR.
+  // Up-sample back up and check PSNR.
   source_file2 = fopen(out_name.c_str(), "rb");
   out_name = webrtc::test::OutputPath() + "LibYuvTest_BilinearScale_352_288_"
       "upfrom_176_144.yuv";
@@ -254,7 +252,7 @@ TEST_F(TestScaler, BiLinearScaleTest) {
                 source_file_, out_name,
                 width_, height_,
                 400, 300);
-  // Downsample to odd size frame and scale back up.
+  // Down-sample to odd size frame and scale back up.
   out_name = webrtc::test::OutputPath() +
       "LibYuvTest_BilinearScale_282_231.yuv";
   ScaleSequence(method,
@@ -298,7 +296,7 @@ TEST_F(TestScaler, BiLinearScaleTest) {
   ASSERT_EQ(0, fclose(source_file2));
 }
 
-TEST_F(TestScaler, BoxScaleTest) {
+TEST_F(TestScaler, DISABLED_ON_ANDROID(BoxScaleTest)) {
   double avg_psnr;
   FILE* source_file2;
   ScaleMethod method = kScaleBox;
@@ -308,7 +306,7 @@ TEST_F(TestScaler, BoxScaleTest) {
                 source_file_, out_name,
                 width_, height_,
                 width_ / 2, height_ / 2);
-  // Upsample back up and check PSNR.
+  // Up-sample back up and check PSNR.
   source_file2 = fopen(out_name.c_str(), "rb");
   out_name = webrtc::test::OutputPath() + "LibYuvTest_BoxScale_352_288_"
       "upfrom_176_144.yuv";
@@ -343,7 +341,7 @@ TEST_F(TestScaler, BoxScaleTest) {
                 source_file_, out_name,
                 width_, height_,
                 400, 300);
-  // Downsample to odd size frame and scale back up.
+  // Down-sample to odd size frame and scale back up.
   out_name = webrtc::test::OutputPath() + "LibYuvTest_BoxScale_282_231.yuv";
   ScaleSequence(method,
                 source_file_, out_name,
@@ -363,7 +361,7 @@ TEST_F(TestScaler, BoxScaleTest) {
   // average PSNR under same conditions.
   ASSERT_GT(avg_psnr, 29.7);
   ASSERT_EQ(0, fclose(source_file2));
-  // Upsample to odd size frame and scale back down.
+  // Up-sample to odd size frame and scale back down.
   out_name = webrtc::test::OutputPath() + "LibYuvTest_BoxScale_699_531.yuv";
   ScaleSequence(method,
                 source_file_, out_name,
@@ -400,6 +398,7 @@ double TestScaler::ComputeAvgSequencePSNR(FILE* input_file,
 
   int frame_count = 0;
   double avg_psnr = 0;
+  I420VideoFrame in_frame, out_frame;
   while (feof(input_file) == 0) {
     if ((size_t)required_size !=
         fread(input_buffer, 1, required_size, input_file)) {
@@ -410,7 +409,9 @@ double TestScaler::ComputeAvgSequencePSNR(FILE* input_file,
       break;
     }
     frame_count++;
-    double psnr = I420PSNR(input_buffer, output_buffer, width, height);
+    ConvertFromI420(in_frame, kI420, 0, input_buffer);
+    ConvertFromI420(out_frame, kI420, 0, output_buffer);
+    double psnr = I420PSNR(&in_frame, &out_frame);
     avg_psnr += psnr;
   }
   avg_psnr = avg_psnr / frame_count;
