@@ -7,21 +7,24 @@ package org.mozilla.gecko;
 
 import org.mozilla.gecko.gfx.InputConnectionHandler;
 import org.mozilla.gecko.gfx.LayerView;
+import org.mozilla.gecko.util.ThreadUtils;
 
 import android.os.Build;
 import android.os.Handler;
 import android.os.Looper;
 import android.text.Editable;
 import android.text.InputFilter;
-import android.text.Spanned;
+import android.text.Selection;
 import android.text.Spannable;
 import android.text.SpannableString;
 import android.text.SpannableStringBuilder;
-import android.text.Selection;
+import android.text.Spanned;
 import android.text.TextPaint;
 import android.text.TextUtils;
 import android.text.style.CharacterStyle;
 import android.util.Log;
+import android.view.KeyCharacterMap;
+import android.view.KeyEvent;
 
 import java.lang.reflect.Field;
 import java.lang.reflect.InvocationHandler;
@@ -36,6 +39,7 @@ interface GeckoEditableClient {
     void sendEvent(GeckoEvent event);
     Editable getEditable();
     void setUpdateGecko(boolean update);
+    void setSuppressKeyUp(boolean suppress);
     Handler getInputConnectionHandler();
     boolean setInputConnectionHandler(Handler handler);
 }
@@ -43,22 +47,21 @@ interface GeckoEditableClient {
 /* interface for the Editable to listen to the Gecko thread
    and also for the IC thread to listen to the Editable */
 interface GeckoEditableListener {
-    // IME notification type for notifyIME()
-    final int NOTIFY_IME_RESETINPUTSTATE = 0;
-    final int NOTIFY_IME_REPLY_EVENT = 1;
-    final int NOTIFY_IME_CANCELCOMPOSITION = 2;
-    final int NOTIFY_IME_FOCUSCHANGE = 3;
-    // IME focus state for notifyIME(NOTIFY_IME_FOCUSCHANGE, ..)
-    final int IME_FOCUS_STATE_FOCUS = 1;
-    final int IME_FOCUS_STATE_BLUR = 0;
-    // IME enabled state for notifyIMEEnabled()
+    // IME notification type for notifyIME(), corresponding to NotificationToIME enum in Gecko
+    final int NOTIFY_IME_OPEN_VKB = -2;
+    final int NOTIFY_IME_REPLY_EVENT = -1;
+    final int NOTIFY_IME_OF_FOCUS = 1;
+    final int NOTIFY_IME_OF_BLUR = 2;
+    final int NOTIFY_IME_TO_COMMIT_COMPOSITION = 4;
+    final int NOTIFY_IME_TO_CANCEL_COMPOSITION = 5;
+    // IME enabled state for notifyIMEContext()
     final int IME_STATE_DISABLED = 0;
     final int IME_STATE_ENABLED = 1;
     final int IME_STATE_PASSWORD = 2;
     final int IME_STATE_PLUGIN = 3;
 
-    void notifyIME(int type, int state);
-    void notifyIMEEnabled(int state, String typeHint,
+    void notifyIME(int type);
+    void notifyIMEContext(int state, String typeHint,
                           String modeHint, String actionHint);
     void onSelectionChange(int start, int end);
     void onTextChange(String text, int start, int oldEnd, int newEnd);
@@ -97,6 +100,7 @@ final class GeckoEditable
     private int mLastIcUpdateSeqno;
     private boolean mUpdateGecko;
     private boolean mFocused;
+    private volatile boolean mSuppressKeyUp;
 
     /* An action that alters the Editable
 
@@ -138,7 +142,8 @@ final class GeckoEditable
 
         static Action newReplaceText(CharSequence text, int start, int end) {
             if (start < 0 || start > end) {
-                throw new IllegalArgumentException("invalid replace text offsets");
+                throw new IllegalArgumentException(
+                    "invalid replace text offsets: " + start + " to " + end);
             }
             final Action action = new Action(TYPE_REPLACE_TEXT);
             action.mSequence = text;
@@ -151,7 +156,8 @@ final class GeckoEditable
             // start == -1 when the start offset should remain the same
             // end == -1 when the end offset should remain the same
             if (start < -1 || end < -1) {
-                throw new IllegalArgumentException("invalid selection offsets");
+                throw new IllegalArgumentException(
+                    "invalid selection offsets: " + start + " to " + end);
             }
             final Action action = new Action(TYPE_SET_SELECTION);
             action.mStart = start;
@@ -161,7 +167,8 @@ final class GeckoEditable
 
         static Action newSetSpan(Object object, int start, int end, int flags) {
             if (start < 0 || start > end) {
-                throw new IllegalArgumentException("invalid span offsets");
+                throw new IllegalArgumentException(
+                    "invalid span offsets: " + start + " to " + end);
             }
             final Action action = new Action(TYPE_SET_SPAN);
             action.mSpanObject = object;
@@ -183,6 +190,7 @@ final class GeckoEditable
     private final class ActionQueue {
         private final ConcurrentLinkedQueue<Action> mActions;
         private final Semaphore mActionsActive;
+        private KeyCharacterMap mKeyMap;
 
         ActionQueue() {
             mActions = new ConcurrentLinkedQueue<Action>();
@@ -216,24 +224,73 @@ final class GeckoEditable
             case Action.TYPE_SET_SPAN:
             case Action.TYPE_REMOVE_SPAN:
             case Action.TYPE_SET_HANDLER:
-                GeckoAppShell.sendEventToGecko(
-                        GeckoEvent.createIMEEvent(GeckoEvent.IME_SYNCHRONIZE));
+                GeckoAppShell.sendEventToGecko(GeckoEvent.createIMEEvent(
+                        GeckoEvent.ImeAction.IME_SYNCHRONIZE));
                 break;
             case Action.TYPE_REPLACE_TEXT:
+                // try key events first
+                sendCharKeyEvents(action);
                 GeckoAppShell.sendEventToGecko(GeckoEvent.createIMEReplaceEvent(
                         action.mStart, action.mEnd, action.mSequence.toString()));
                 break;
             case Action.TYPE_ACKNOWLEDGE_FOCUS:
                 GeckoAppShell.sendEventToGecko(GeckoEvent.createIMEEvent(
-                        GeckoEvent.IME_ACKNOWLEDGE_FOCUS));
+                        GeckoEvent.ImeAction.IME_ACKNOWLEDGE_FOCUS));
                 break;
             }
             ++mIcUpdateSeqno;
         }
 
+        private KeyEvent [] synthesizeKeyEvents(CharSequence cs) {
+            try {
+                if (mKeyMap == null) {
+                    mKeyMap = KeyCharacterMap.load(
+                        Build.VERSION.SDK_INT < 11 ? KeyCharacterMap.ALPHA :
+                                                     KeyCharacterMap.VIRTUAL_KEYBOARD);
+                }
+            } catch (Exception e) {
+                // KeyCharacterMap.UnavailableExcepton is not found on Gingerbread;
+                // besides, it seems like HC and ICS will throw something other than
+                // KeyCharacterMap.UnavailableExcepton; so use a generic Exception here
+                return null;
+            }
+            KeyEvent [] keyEvents = mKeyMap.getEvents(cs.toString().toCharArray());
+            if (keyEvents == null || keyEvents.length == 0) {
+                return null;
+            }
+            return keyEvents;
+        }
+
+        private void sendCharKeyEvents(Action action) {
+            if (action.mSequence.length() == 0 ||
+                (action.mSequence instanceof Spannable &&
+                ((Spannable)action.mSequence).nextSpanTransition(
+                    -1, Integer.MAX_VALUE, null) < Integer.MAX_VALUE)) {
+                // Spans are not preserved when we use key events,
+                // so we need the sequence to not have any spans
+                return;
+            }
+            KeyEvent [] keyEvents = synthesizeKeyEvents(action.mSequence);
+            if (keyEvents == null) {
+                return;
+            }
+            for (KeyEvent event : keyEvents) {
+                if (KeyEvent.isModifierKey(event.getKeyCode())) {
+                    continue;
+                }
+                if (event.getAction() == KeyEvent.ACTION_UP && mSuppressKeyUp) {
+                    continue;
+                }
+                if (DEBUG) {
+                    Log.d(LOGTAG, "sending: " + event);
+                }
+                GeckoAppShell.sendEventToGecko(GeckoEvent.createIMEKeyEvent(event));
+            }
+        }
+
         void poll() {
             if (DEBUG) {
-                GeckoApp.assertOnGeckoThread();
+                ThreadUtils.assertOnGeckoThread();
             }
             if (mActions.isEmpty()) {
                 throw new IllegalStateException("empty actions queue");
@@ -251,7 +308,7 @@ final class GeckoEditable
 
         Action peek() {
             if (DEBUG) {
-                GeckoApp.assertOnGeckoThread();
+                ThreadUtils.assertOnGeckoThread();
             }
             if (mActions.isEmpty()) {
                 throw new IllegalStateException("empty actions queue");
@@ -293,10 +350,10 @@ final class GeckoEditable
                 Editable.class.getClassLoader(),
                 PROXY_INTERFACES, this);
 
-        LayerView v = GeckoApp.mAppContext.getLayerView();
+        LayerView v = GeckoAppShell.getLayerView();
         mListener = GeckoInputConnection.create(v, this);
 
-        mIcRunHandler = mIcPostHandler = GeckoApp.mAppContext.mMainHandler;
+        mIcRunHandler = mIcPostHandler = ThreadUtils.getUiHandler();
     }
 
     private boolean onIcThread() {
@@ -304,7 +361,7 @@ final class GeckoEditable
     }
 
     private void assertOnIcThread() {
-        GeckoApp.assertOnThread(mIcRunHandler.getLooper().getThread());
+        ThreadUtils.assertOnThread(mIcRunHandler.getLooper().getThread());
     }
 
     private void geckoPostToIc(Runnable runnable) {
@@ -318,6 +375,7 @@ final class GeckoEditable
         final int seqnoWhenPosted = mGeckoUpdateSeqno;
 
         geckoPostToIc(new Runnable() {
+            @Override
             public void run() {
                 mActionQueue.syncWithGecko();
                 if (seqnoWhenPosted == mGeckoUpdateSeqno) {
@@ -367,11 +425,12 @@ final class GeckoEditable
             Log.d(LOGTAG, " selection = " + selStart + "-" + selEnd);
         }
         if (composingStart >= composingEnd) {
-            GeckoAppShell.sendEventToGecko(GeckoEvent.createIMEEvent(
-                    GeckoEvent.IME_REMOVE_COMPOSITION));
             if (selStart >= 0 && selEnd >= 0) {
                 GeckoAppShell.sendEventToGecko(
                         GeckoEvent.createIMESelectEvent(selStart, selEnd));
+            } else {
+                GeckoAppShell.sendEventToGecko(GeckoEvent.createIMEEvent(
+                        GeckoEvent.ImeAction.IME_REMOVE_COMPOSITION));
             }
             return;
         }
@@ -473,19 +532,9 @@ final class GeckoEditable
     @Override
     public void sendEvent(final GeckoEvent event) {
         if (DEBUG) {
+            assertOnIcThread();
             Log.d(LOGTAG, "sendEvent(" + event + ")");
         }
-        if (!onIcThread()) {
-            // Events may get dispatched to the main thread;
-            // reroute to our IC thread instead
-            mIcRunHandler.post(new Runnable() {
-                @Override
-                public void run() {
-                    sendEvent(event);
-                }
-            });
-            return;
-        }
         /*
            We are actually sending two events to Gecko here,
            1. Event from the event parameter (key event, etc.)
@@ -526,10 +575,20 @@ final class GeckoEditable
     }
 
     @Override
-    public Handler getInputConnectionHandler() {
+    public void setSuppressKeyUp(boolean suppress) {
         if (DEBUG) {
-            assertOnIcThread();
+            // only used by key event handler
+            ThreadUtils.assertOnUiThread();
         }
+        // Suppress key up event generated as a result of
+        // translating characters to key events
+        mSuppressKeyUp = suppress;
+    }
+
+    @Override
+    public Handler getInputConnectionHandler() {
+        // Can be called from either UI thread or IC thread;
+        // care must be taken to avoid race conditions
         return mIcRunHandler;
     }
 
@@ -597,7 +656,7 @@ final class GeckoEditable
     private void geckoActionReply() {
         if (DEBUG) {
             // GeckoEditableListener methods should all be called from the Gecko thread
-            GeckoApp.assertOnGeckoThread();
+            ThreadUtils.assertOnGeckoThread();
         }
         final Action action = mActionQueue.peek();
 
@@ -620,6 +679,7 @@ final class GeckoEditable
             }
             Selection.setSelection(mText, selStart, selEnd);
             geckoPostToIc(new Runnable() {
+                @Override
                 public void run() {
                     mActionQueue.syncWithGecko();
                     final int start = Selection.getSelectionStart(mText);
@@ -645,15 +705,15 @@ final class GeckoEditable
     }
 
     @Override
-    public void notifyIME(final int type, final int state) {
+    public void notifyIME(final int type) {
         if (DEBUG) {
             // GeckoEditableListener methods should all be called from the Gecko thread
-            GeckoApp.assertOnGeckoThread();
+            ThreadUtils.assertOnGeckoThread();
             // NOTIFY_IME_REPLY_EVENT is logged separately, inside geckoActionReply()
             if (type != NOTIFY_IME_REPLY_EVENT) {
                 Log.d(LOGTAG, "notifyIME(" +
                               getConstantName(GeckoEditableListener.class, "NOTIFY_IME_", type) +
-                              ", " + state + ")");
+                              ")");
             }
         }
         if (type == NOTIFY_IME_REPLY_EVENT) {
@@ -673,47 +733,47 @@ final class GeckoEditable
             return;
         }
         geckoPostToIc(new Runnable() {
+            @Override
             public void run() {
-                if (type == NOTIFY_IME_FOCUSCHANGE) {
-                    if (state == IME_FOCUS_STATE_BLUR) {
-                        mFocused = false;
-                    } else {
-                        mFocused = true;
-                        // Unmask events on the Gecko side
-                        mActionQueue.offer(new Action(Action.TYPE_ACKNOWLEDGE_FOCUS));
-                    }
+                if (type == NOTIFY_IME_OF_BLUR) {
+                    mFocused = false;
+                } else if (type == NOTIFY_IME_OF_FOCUS) {
+                    mFocused = true;
+                    // Unmask events on the Gecko side
+                    mActionQueue.offer(new Action(Action.TYPE_ACKNOWLEDGE_FOCUS));
                 }
                 // Make sure there are no other things going on. If we sent
                 // GeckoEvent.IME_ACKNOWLEDGE_FOCUS, this line also makes us
                 // wait for Gecko to update us on the newly focused content
                 mActionQueue.syncWithGecko();
-                mListener.notifyIME(type, state);
+                mListener.notifyIME(type);
             }
         });
     }
 
     @Override
-    public void notifyIMEEnabled(final int state, final String typeHint,
+    public void notifyIMEContext(final int state, final String typeHint,
                           final String modeHint, final String actionHint) {
         // Because we want to be able to bind GeckoEditable to the newest LayerView instance,
         // this can be called from the Java IC thread in addition to the Gecko thread.
         if (DEBUG) {
-            Log.d(LOGTAG, "notifyIMEEnabled(" +
+            Log.d(LOGTAG, "notifyIMEContext(" +
                           getConstantName(GeckoEditableListener.class, "IME_STATE_", state) +
                           ", \"" + typeHint + "\", \"" + modeHint + "\", \"" + actionHint + "\")");
         }
         geckoPostToIc(new Runnable() {
+            @Override
             public void run() {
                 // Make sure there are no other things going on
                 mActionQueue.syncWithGecko();
-                // Set InputConnectionHandler in notifyIMEEnabled because
-                // GeckoInputConnection.notifyIMEEnabled calls restartInput() which will invoke
+                // Set InputConnectionHandler in notifyIMEContext because
+                // GeckoInputConnection.notifyIMEContext calls restartInput() which will invoke
                 // InputConnectionHandler.onCreateInputConnection
-                LayerView v = GeckoApp.mAppContext.getLayerView();
+                LayerView v = GeckoAppShell.getLayerView();
                 if (v != null) {
                     mListener = GeckoInputConnection.create(v, GeckoEditable.this);
                     v.setInputConnectionHandler((InputConnectionHandler)mListener);
-                    mListener.notifyIMEEnabled(state, typeHint, modeHint, actionHint);
+                    mListener.notifyIMEContext(state, typeHint, modeHint, actionHint);
                 }
             }
         });
@@ -723,11 +783,12 @@ final class GeckoEditable
     public void onSelectionChange(final int start, final int end) {
         if (DEBUG) {
             // GeckoEditableListener methods should all be called from the Gecko thread
-            GeckoApp.assertOnGeckoThread();
+            ThreadUtils.assertOnGeckoThread();
             Log.d(LOGTAG, "onSelectionChange(" + start + ", " + end + ")");
         }
         if (start < 0 || start > mText.length() || end < 0 || end > mText.length()) {
-            throw new IllegalArgumentException("invalid selection notification range");
+            throw new IllegalArgumentException("invalid selection notification range: " +
+                start + " to " + end + ", length: " + mText.length());
         }
         final int seqnoWhenPosted = ++mGeckoUpdateSeqno;
 
@@ -741,6 +802,7 @@ final class GeckoEditable
         }
 
         geckoPostToIc(new Runnable() {
+            @Override
             public void run() {
                 mActionQueue.syncWithGecko();
                 /* check to see there has not been another action that potentially changed the
@@ -772,19 +834,25 @@ final class GeckoEditable
                       final int unboundedOldEnd, final int unboundedNewEnd) {
         if (DEBUG) {
             // GeckoEditableListener methods should all be called from the Gecko thread
-            GeckoApp.assertOnGeckoThread();
-            Log.d(LOGTAG, "onTextChange(\"" + text + "\", " + start + ", " +
-                          unboundedOldEnd + ", " + unboundedNewEnd + ")");
+            ThreadUtils.assertOnGeckoThread();
+            StringBuilder sb = new StringBuilder("onTextChange(");
+            debugAppend(sb, text);
+            sb.append(", ").append(start).append(", ")
+                .append(unboundedOldEnd).append(", ")
+                .append(unboundedNewEnd).append(")");
+            Log.d(LOGTAG, sb.toString());
         }
         if (start < 0 || start > unboundedOldEnd) {
-            throw new IllegalArgumentException("invalid text notification range");
+            throw new IllegalArgumentException("invalid text notification range: " +
+                start + " to " + unboundedOldEnd);
         }
         /* For the "end" parameters, Gecko can pass in a large
            number to denote "end of the text". Fix that here */
         final int oldEnd = unboundedOldEnd > mText.length() ? mText.length() : unboundedOldEnd;
         // new end should always match text
         if (start != 0 && unboundedNewEnd != (start + text.length())) {
-            throw new IllegalArgumentException("newEnd does not match text");
+            throw new IllegalArgumentException("newEnd does not match text: " +
+                unboundedNewEnd + " vs " + (start + text.length()));
         }
         final int newEnd = start + text.length();
 
@@ -840,6 +908,7 @@ final class GeckoEditable
             geckoReplaceText(start, oldEnd, mChangedText);
         }
         geckoPostToIc(new Runnable() {
+            @Override
             public void run() {
                 mListener.onTextChange(text, start, oldEnd, newEnd);
             }
@@ -915,8 +984,10 @@ final class GeckoEditable
             }
             Log.w(LOGTAG, "Exception in GeckoEditable." + method.getName(), e.getCause());
             Class<?> retClass = method.getReturnType();
-            if (retClass != Void.TYPE && retClass.isPrimitive()) {
-                ret = retClass.newInstance();
+            if (retClass == Character.TYPE) {
+                ret = '\0';
+            } else if (retClass == Integer.TYPE) {
+                ret = 0;
             } else if (retClass == String.class) {
                 ret = "";
             } else {
@@ -1017,7 +1088,8 @@ final class GeckoEditable
 
         CharSequence text = source;
         if (start < 0 || start > end || end > text.length()) {
-            throw new IllegalArgumentException("invalid replace offsets");
+            throw new IllegalArgumentException("invalid replace offsets: " +
+                start + " to " + end + ", length: " + text.length());
         }
         if (start != 0 || end != text.length()) {
             text = text.subSequence(start, end);
