@@ -7,7 +7,6 @@ package org.mozilla.gecko;
 
 import org.mozilla.gecko.util.INIParser;
 import org.mozilla.gecko.util.INISection;
-import org.mozilla.gecko.util.ThreadUtils;
 
 import android.content.Context;
 import android.text.TextUtils;
@@ -27,14 +26,14 @@ public final class GeckoProfile {
     private static final String LOGTAG = "GeckoProfile";
     // Used to "lock" the guest profile, so that we'll always restart in it
     private static final String LOCK_FILE_NAME = ".active_lock";
+    public static final String DEFAULT_PROFILE = "default";
 
     private static HashMap<String, GeckoProfile> sProfileCache = new HashMap<String, GeckoProfile>();
     private static String sDefaultProfileName = null;
 
-    private final Context mContext;
     private final String mName;
-    private File mMozDir;
-    private File mDir;
+    private File mProfileDir;
+    public static boolean sIsUsingCustomProfile = false;
 
     // Constants to cache whether or not a profile is "locked".
     private enum LockState {
@@ -52,15 +51,22 @@ public final class GeckoProfile {
     private boolean mInGuestMode = false;
     private static GeckoProfile mGuestProfile = null;
 
-    static private INIParser getProfilesINI(Context context) {
-      File filesDir = context.getFilesDir();
-      File mozillaDir = new File(filesDir, "mozilla");
-      File profilesIni = new File(mozillaDir, "profiles.ini");
-      return new INIParser(profilesIni);
+    private static final String MOZILLA_DIR_NAME = "mozilla";
+    private static File sMozillaDir;
+
+    private static INIParser getProfilesINI(File mozillaDir) {
+        File profilesIni = new File(mozillaDir, "profiles.ini");
+        return new INIParser(profilesIni);
     }
 
     public static GeckoProfile get(Context context) {
-        if (context instanceof GeckoApp) {
+        boolean isGeckoApp = false;
+        try {
+            isGeckoApp = context instanceof GeckoApp;
+        } catch (NoClassDefFoundError ex) {}
+        
+
+        if (isGeckoApp) {
             // Check for a cached profile on this context already
             // TODO: We should not be caching profile information on the Activity context
             if (((GeckoApp)context).mProfile != null) {
@@ -74,7 +80,7 @@ public final class GeckoProfile {
             return guest;
         }
 
-        if (context instanceof GeckoApp) {
+        if (isGeckoApp) {
             // Otherwise, get the default profile for the Activity
             return get(context, ((GeckoApp)context).getDefaultProfileName());
         }
@@ -108,14 +114,15 @@ public final class GeckoProfile {
         if (TextUtils.isEmpty(profileName) && profileDir == null) {
             profileName = GeckoProfile.findDefaultProfile(context);
             if (profileName == null)
-                profileName = "default";
+                profileName = DEFAULT_PROFILE;
         }
 
         // actually try to look up the profile
         synchronized (sProfileCache) {
             GeckoProfile profile = sProfileCache.get(profileName);
             if (profile == null) {
-                profile = new GeckoProfile(context, profileName, profileDir);
+                profile = new GeckoProfile(context, profileName);
+                profile.setDir(profileDir);
                 sProfileCache.put(profileName, profile);
             } else {
                 profile.setDir(profileDir);
@@ -124,17 +131,18 @@ public final class GeckoProfile {
         }
     }
 
-    public static File ensureMozillaDirectory(Context context) throws IOException {
-        synchronized (context) {
-            File filesDir = context.getFilesDir();
-            File mozDir = new File(filesDir, "mozilla");
-            if (! mozDir.exists()) {
-                if (! mozDir.mkdirs()) {
-                    throw new IOException("Unable to create mozilla directory at " + mozDir.getAbsolutePath());
-                }
-            }
-            return mozDir;
+    private static File getMozillaDirectory(Context context) {
+        return new File(context.getFilesDir(), MOZILLA_DIR_NAME);
+    }
+
+    private synchronized File ensureMozillaDirectory() throws IOException {
+        if (sMozillaDir.exists() || sMozillaDir.mkdirs()) {
+            return sMozillaDir;
         }
+
+        // Although this leaks a path to the system log, the path is
+        // predictable (unlike a profile directory), so this is fine.
+        throw new IOException("Unable to create mozilla directory at " + sMozillaDir.getAbsolutePath());
     }
 
     public static boolean removeProfile(Context context, String profileName) {
@@ -234,8 +242,8 @@ public final class GeckoProfile {
         }
 
         // Don't use getDir() as it will create a dir if none exists
-        if (mDir != null && mDir.exists()) {
-            File lockFile = new File(mDir, LOCK_FILE_NAME);
+        if (mProfileDir != null && mProfileDir.exists()) {
+            File lockFile = new File(mProfileDir, LOCK_FILE_NAME);
             boolean res = lockFile.exists();
             mLocked = res ? LockState.LOCKED : LockState.UNLOCKED;
         } else {
@@ -265,12 +273,12 @@ public final class GeckoProfile {
 
     public boolean unlock() {
         // Don't use getDir() as it will create a dir
-        if (mDir == null || !mDir.exists()) {
+        if (mProfileDir == null || !mProfileDir.exists()) {
             return true;
         }
 
         try {
-            File lockFile = new File(mDir, LOCK_FILE_NAME);
+            File lockFile = new File(mProfileDir, LOCK_FILE_NAME);
             boolean result = delete(lockFile);
             if (result) {
                 mLocked = LockState.UNLOCKED;
@@ -286,14 +294,10 @@ public final class GeckoProfile {
     }
 
     private GeckoProfile(Context context, String profileName) {
-        mContext = context;
         mName = profileName;
-    }
-
-    private GeckoProfile(Context context, String profileName, File profileDir) {
-        mContext = context;
-        mName = profileName;
-        setDir(profileDir);
+        if (sMozillaDir == null) {
+            sMozillaDir = getMozillaDirectory(context);
+        }
     }
 
     public boolean inGuestMode() {
@@ -302,9 +306,9 @@ public final class GeckoProfile {
 
     private void setDir(File dir) {
         if (dir != null && dir.exists() && dir.isDirectory()) {
-            mDir = dir;
+            mProfileDir = dir;
         } else {
-            Log.w(LOGTAG, "requested profile directory missing: " + dir);
+            Log.w(LOGTAG, "Requested profile directory missing.");
         }
     }
 
@@ -313,24 +317,29 @@ public final class GeckoProfile {
     }
 
     public synchronized File getDir() {
-        if (mDir != null) {
-            return mDir;
+        forceCreate();
+        return mProfileDir;
+    }
+
+    public synchronized GeckoProfile forceCreate() {
+        if (mProfileDir != null) {
+            return this;
         }
 
         try {
             // Check if a profile with this name already exists.
-            File mozillaDir = ensureMozillaDirectory(mContext);
-            mDir = findProfileDir(mozillaDir);
-            if (mDir == null) {
+            File mozillaDir = ensureMozillaDirectory();
+            mProfileDir = findProfileDir(mozillaDir);
+            if (mProfileDir == null) {
                 // otherwise create it
-                mDir = createProfileDir(mozillaDir);
+                mProfileDir = createProfileDir(mozillaDir);
             } else {
-                Log.d(LOGTAG, "Found profile dir: " + mDir.getAbsolutePath());
+                Log.d(LOGTAG, "Found profile dir.");
             }
         } catch (IOException ioe) {
             Log.e(LOGTAG, "Error getting profile dir", ioe);
         }
-        return mDir;
+        return this;
     }
 
     public File getFile(String aFile) {
@@ -341,10 +350,6 @@ public final class GeckoProfile {
         return new File(f, aFile);
     }
 
-    public File getFilesDir() {
-        return mContext.getFilesDir();
-    }
-
     /**
      * Moves the session file to the backup session file.
      *
@@ -419,13 +424,13 @@ public final class GeckoProfile {
             if (dir.exists())
                 delete(dir);
 
-            File mozillaDir = ensureMozillaDirectory(mContext);
-            mDir = findProfileDir(mozillaDir);
-            if (mDir == null) {
+            File mozillaDir = ensureMozillaDirectory();
+            mProfileDir = findProfileDir(mozillaDir);
+            if (mProfileDir == null) {
                 return false;
             }
 
-            INIParser parser = getProfilesINI(mContext);
+            INIParser parser = getProfilesINI(mozillaDir);
 
             Hashtable<String, INISection> sections = parser.getSections();
             for (Enumeration<INISection> e = sections.elements(); e.hasMoreElements();) {
@@ -466,7 +471,7 @@ public final class GeckoProfile {
             parser.write();
             return true;
         } catch (IOException ex) {
-            Log.w(LOGTAG, "Failed to remove profile " + mName + ":\n" + ex);
+            Log.w(LOGTAG, "Failed to remove profile.", ex);
             return false;
         }
     }
@@ -480,7 +485,7 @@ public final class GeckoProfile {
         }
 
         // Open profiles.ini to find the correct path
-        INIParser parser = getProfilesINI(context);
+        INIParser parser = getProfilesINI(getMozillaDirectory(context));
 
         for (Enumeration<INISection> e = parser.getSections().elements(); e.hasMoreElements();) {
             INISection section = e.nextElement();
@@ -495,7 +500,7 @@ public final class GeckoProfile {
 
     private File findProfileDir(File mozillaDir) {
         // Open profiles.ini to find the correct path
-        INIParser parser = getProfilesINI(mContext);
+        INIParser parser = getProfilesINI(mozillaDir);
 
         for (Enumeration<INISection> e = parser.getSections().elements(); e.hasMoreElements();) {
             INISection section = e.nextElement();
@@ -523,7 +528,7 @@ public final class GeckoProfile {
     }
 
     private File createProfileDir(File mozillaDir) throws IOException {
-        INIParser parser = getProfilesINI(mContext);
+        INIParser parser = getProfilesINI(mozillaDir);
 
         // Salt the name of our requested profile
         String saltedName = saltProfileName(mName);
@@ -534,10 +539,10 @@ public final class GeckoProfile {
         }
 
         // Attempt to create the salted profile dir
-        if (! profileDir.mkdirs()) {
-            throw new IOException("Unable to create profile at " + profileDir.getAbsolutePath());
+        if (!profileDir.mkdirs()) {
+            throw new IOException("Unable to create profile.");
         }
-        Log.d(LOGTAG, "Created new profile dir at " + profileDir.getAbsolutePath());
+        Log.d(LOGTAG, "Created new profile dir.");
 
         // Now update profiles.ini
         // If this is the first time its created, we also add a General section
