@@ -9,9 +9,8 @@
 
 #include "nsHttpResponseHead.h"
 #include "nsPrintfCString.h"
-#include "prprf.h"
 #include "prtime.h"
-#include "nsCRT.h"
+#include "nsURLHelper.h"
 #include <algorithm>
 
 //-----------------------------------------------------------------------------
