@@ -19,25 +19,13 @@ nsScannerBufferList::Buffer*
 nsScannerBufferList::AllocBufferFromString( const nsAString& aString )
   {
     uint32_t len = aString.Length();
+    Buffer* buf = AllocBuffer(len);
 
-    if (len > MAX_CAPACITY)
-      return nullptr;
-
-    Buffer* buf = (Buffer*) malloc(sizeof(Buffer) + (len + 1) * sizeof(PRUnichar));
     if (buf)
       {
-        // leave PRCList members of Buffer uninitialized
-
-        buf->mUsageCount = 0;
-        buf->mDataEnd = buf->DataStart() + len;
-
         nsAString::const_iterator source;
         aString.BeginReading(source);
         nsCharTraits<PRUnichar>::copy(buf->DataStart(), source.get(), len);
-
-        // XXX null terminate.  this shouldn't be required, but we do it because
-        // nsScanner erroneously thinks it can dereference DataEnd :-(
-        *buf->mDataEnd = PRUnichar(0);
       }
     return buf;
   }
@@ -48,30 +36,29 @@ nsScannerBufferList::AllocBuffer( uint32_t capacity )
     if (capacity > MAX_CAPACITY)
       return nullptr;
 
-    Buffer* buf = (Buffer*) malloc(sizeof(Buffer) + (capacity + 1) * sizeof(PRUnichar));
-    if (buf)
-      {
-        // leave PRCList members of Buffer uninitialized
+    void* ptr = malloc(sizeof(Buffer) + (capacity + 1) * sizeof(PRUnichar));
+    if (!ptr)
+      return nullptr;
 
-        buf->mUsageCount = 0;
-        buf->mDataEnd = buf->DataStart() + capacity;
+    Buffer* buf = new (ptr) Buffer();
 
-        // XXX null terminate.  this shouldn't be required, but we do it because
-        // nsScanner erroneously thinks it can dereference DataEnd :-(
-        *buf->mDataEnd = PRUnichar(0);
-      }
+    buf->mUsageCount = 0;
+    buf->mDataEnd = buf->DataStart() + capacity;
+
+    // XXX null terminate.  this shouldn't be required, but we do it because
+    // nsScanner erroneously thinks it can dereference DataEnd :-(
+    *buf->mDataEnd = PRUnichar(0);
     return buf;
   }
 
 void
 nsScannerBufferList::ReleaseAll()
   {
-    while (!PR_CLIST_IS_EMPTY(&mBuffers))
+    while (!mBuffers.isEmpty())
       {
-        PRCList* node = PR_LIST_HEAD(&mBuffers);
-        PR_REMOVE_LINK(node);
+        Buffer* node = mBuffers.popFirst();
         //printf(">>> freeing buffer @%p\n", node);
-        free(static_cast<Buffer*>(node));
+        free(node);
       }
   }
 
@@ -106,10 +93,10 @@ nsScannerBufferList::DiscardUnreferencedPrefix( Buffer* aBuf )
   {
     if (aBuf == Head())
       {
-        while (!PR_CLIST_IS_EMPTY(&mBuffers) && !Head()->IsInUse())
+        while (!mBuffers.isEmpty() && !Head()->IsInUse())
           {
             Buffer* buffer = Head();
-            PR_REMOVE_LINK(buffer);
+            buffer->remove();
             free(buffer);
           }
       }
@@ -276,7 +263,7 @@ nsScannerSubstring::GetNextFragment( nsScannerFragment& frag ) const
     if (frag.mBuffer == mEnd.mBuffer)
       return false;
 
-    frag.mBuffer = static_cast<const Buffer*>(PR_NEXT_LINK(frag.mBuffer));
+    frag.mBuffer = frag.mBuffer->getNext();
 
     if (frag.mBuffer == mStart.mBuffer)
       frag.mFragmentStart = mStart.mPosition;
@@ -298,7 +285,7 @@ nsScannerSubstring::GetPrevFragment( nsScannerFragment& frag ) const
     if (frag.mBuffer == mStart.mBuffer)
       return false;
 
-    frag.mBuffer = static_cast<const Buffer*>(PR_PREV_LINK(frag.mBuffer));
+    frag.mBuffer = frag.mBuffer->getPrevious();
 
     if (frag.mBuffer == mStart.mBuffer)
       frag.mFragmentStart = mStart.mPosition;
@@ -485,7 +472,7 @@ CopyUnicodeTo( const nsScannerIterator& aSrcStart,
                nsAString& aDest )
   {
     nsAString::iterator writer;
-    if (!EnsureStringLength(aDest, Distance(aSrcStart, aSrcEnd))) {
+    if (!aDest.SetLength(Distance(aSrcStart, aSrcEnd), mozilla::fallible_t())) {
       aDest.Truncate();
       return; // out of memory
     }
@@ -518,7 +505,7 @@ AppendUnicodeTo( const nsScannerIterator& aSrcStart,
   {
     nsAString::iterator writer;
     uint32_t oldLength = aDest.Length();
-    if (!EnsureStringLength(aDest, oldLength + Distance(aSrcStart, aSrcEnd)))
+    if (!aDest.SetLength(oldLength + Distance(aSrcStart, aSrcEnd), mozilla::fallible_t()))
       return; // out of memory
     aDest.BeginWriting(writer).advance(oldLength);
     nsScannerIterator fromBegin(aSrcStart);
