@@ -30,6 +30,7 @@ import time
 import traceback
 import types
 
+from collections import OrderedDict
 from io import StringIO
 
 from mozbuild.util import (
@@ -39,6 +40,10 @@ from mozbuild.util import (
 
 from mozbuild.backend.configenvironment import ConfigEnvironment
 
+from .data import (
+    JavaJarData,
+)
+
 from .sandbox import (
     SandboxError,
     SandboxExecutionError,
@@ -109,7 +114,7 @@ class MozbuildSandbox(Sandbox):
     We expose a few useful functions and expose the set of variables defining
     Mozilla's build system.
     """
-    def __init__(self, config, path):
+    def __init__(self, config, path, metadata={}):
         """Create an empty mozbuild Sandbox.
 
         config is a ConfigStatus instance (the output of configure). path is
@@ -121,6 +126,7 @@ class MozbuildSandbox(Sandbox):
         self._log = logging.getLogger(__name__)
 
         self.config = config
+        self.metadata = dict(metadata)
 
         topobjdir = os.path.abspath(config.topobjdir)
         topsrcdir = config.topsrcdir
@@ -171,10 +177,18 @@ class MozbuildSandbox(Sandbox):
             d['CONFIG'] = ReadOnlyDefaultDict(self.config.substs_unicode,
                 global_default=None)
 
+            var = metadata.get('var', None)
+            if var and var in ['TOOL_DIRS', 'TEST_TOOL_DIRS']:
+                d['IS_TOOL_DIR'] = True
+
             # Register functions.
             for name, func in FUNCTIONS.items():
                 d[name] = getattr(self, func[0])
 
+            # Initialize the exports that we need in the global.
+            extra_vars = self.metadata.get('exports', dict())
+            self._globals.update(extra_vars)
+
     def exec_file(self, path, filesystem_absolute=False):
         """Override exec_file to normalize paths and restrict file loading.
 
@@ -212,6 +226,22 @@ class MozbuildSandbox(Sandbox):
 
         Sandbox.exec_file(self, path)
 
+    def _add_java_jar(self, name):
+        """Add a Java JAR build target."""
+        if not name:
+            raise Exception('Java JAR cannot be registered without a name')
+
+        if '/' in name or '\\' in name or '.jar' in name:
+            raise Exception('Java JAR names must not include slashes or'
+                ' .jar: %s' % name)
+
+        if self['JAVA_JAR_TARGETS'].has_key(name):
+            raise Exception('Java JAR has already been registered: %s' % name)
+
+        jar = JavaJarData(name)
+        self['JAVA_JAR_TARGETS'][name] = jar
+        return jar
+
     def _add_tier_directory(self, tier, reldir, static=False, external=False):
         """Register a tier directory with the build."""
         if isinstance(reldir, text_type):
@@ -236,6 +266,30 @@ class MozbuildSandbox(Sandbox):
 
             self['TIERS'][tier][key].append(path)
 
+    def _export(self, varname):
+        """Export the variable to all subdirectories of the current path."""
+
+        exports = self.metadata.setdefault('exports', dict())
+        if varname in exports:
+            raise Exception('Variable has already been exported: %s' % varname)
+
+        try:
+            # Doing a regular self._globals[varname] causes a set as a side
+            # effect. By calling the dict method instead, we don't have any
+            # side effects.
+            exports[varname] = dict.__getitem__(self._globals, varname)
+        except KeyError:
+            self.last_name_error = KeyError('global_ns', 'get_unknown', varname)
+            raise self.last_name_error
+
+    def recompute_exports(self):
+        """Recompute the variables to export to subdirectories with the current
+        values in the subdirectory."""
+
+        if 'exports' in self.metadata:
+            for key in self.metadata['exports']:
+                self.metadata['exports'][key] = self[key]
+
     def _include(self, path):
         """Include and exec another file within the context of this one."""
 
@@ -330,7 +384,8 @@ class BuildReaderError(Exception):
             s.write('The error occurred when validating the result of ')
             s.write('the execution. The reported error is:\n')
             s.write('\n')
-            s.write('    %s\n' % self.validation_error.message)
+            s.write(''.join('    %s\n' % l
+                            for l in self.validation_error.message.splitlines()))
             s.write('\n')
         else:
             s.write('The error appears to be part of the %s ' % __name__)
@@ -556,10 +611,10 @@ class BuildReader(object):
         """
         path = os.path.join(self.topsrcdir, 'moz.build')
         return self.read_mozbuild(path, read_tiers=True,
-            filesystem_absolute=True)
+            filesystem_absolute=True, metadata={'tier': None})
 
     def read_mozbuild(self, path, read_tiers=False, filesystem_absolute=False,
-            descend=True):
+            descend=True, metadata={}):
         """Read and process a mozbuild file, descending into children.
 
         This starts with a single mozbuild file, executes it, and descends into
@@ -577,12 +632,19 @@ class BuildReader(object):
         If descend is True (the default), we will descend into child
         directories and files per variable values.
 
+        Arbitrary metadata in the form of a dict can be passed into this
+        function. This metadata will be attached to the emitted output. This
+        feature is intended to facilitate the build reader injecting state and
+        annotations into moz.build files that is independent of the sandbox's
+        execution context.
+
         Traversal is performed depth first (for no particular reason).
         """
         self._execution_stack.append(path)
         try:
             for s in self._read_mozbuild(path, read_tiers=read_tiers,
-                filesystem_absolute=filesystem_absolute, descend=descend):
+                filesystem_absolute=filesystem_absolute, descend=descend,
+                metadata=metadata):
                 yield s
 
         except BuildReaderError as bre:
@@ -608,7 +670,8 @@ class BuildReader(object):
             raise BuildReaderError(list(self._execution_stack),
                 sys.exc_info()[2], other_error=e)
 
-    def _read_mozbuild(self, path, read_tiers, filesystem_absolute, descend):
+    def _read_mozbuild(self, path, read_tiers, filesystem_absolute, descend,
+            metadata):
         path = os.path.normpath(path)
         log(self._log, logging.DEBUG, 'read_mozbuild', {'path': path},
             'Reading file: {path}')
@@ -621,9 +684,23 @@ class BuildReader(object):
         self._read_files.add(path)
 
         time_start = time.time()
-        sandbox = MozbuildSandbox(self.config, path)
+        sandbox = MozbuildSandbox(self.config, path, metadata=metadata)
         sandbox.exec_file(path, filesystem_absolute=filesystem_absolute)
         sandbox.execution_time = time.time() - time_start
+        var = metadata.get('var', None)
+        forbidden = {
+            'TOOL_DIRS': ['DIRS', 'PARALLEL_DIRS', 'TEST_DIRS'],
+            'TEST_TOOL_DIRS': ['DIRS', 'PARALLEL_DIRS', 'TEST_DIRS', 'TOOL_DIRS'],
+        }
+        if var in forbidden:
+            matches = [v for v in forbidden[var] if sandbox[v]]
+            if matches:
+                raise SandboxValidationError('%s is registered as %s in %s/moz.build.\n'
+                    'The %s variable%s not allowed in such directories.'
+                    % (sandbox['RELATIVEDIR'], var, metadata['parent'],
+                       ' and '.join(', '.join(matches).rsplit(', ', 1)),
+                       's are' if len(matches) > 1 else ' is'))
+
         yield sandbox
 
         # Traverse into referenced files.
@@ -637,18 +714,23 @@ class BuildReader(object):
         # It's very tempting to use a set here. Unfortunately, the recursive
         # make backend needs order preserved. Once we autogenerate all backend
         # files, we should be able to convert this to a set.
-        dirs = []
+        recurse_info = OrderedDict()
         for var in dir_vars:
             if not var in sandbox:
                 continue
 
             for d in sandbox[var]:
-                if d in dirs:
+                if d in recurse_info:
                     raise SandboxValidationError(
                         'Directory (%s) registered multiple times in %s' % (
                             d, var))
 
-                dirs.append(d)
+                recurse_info[d] = {'tier': metadata.get('tier', None),
+                                   'parent': sandbox['RELATIVEDIR'],
+                                   'var': var}
+                if 'exports' in sandbox.metadata:
+                    sandbox.recompute_exports()
+                    recurse_info[d]['exports'] = dict(sandbox.metadata['exports'])
 
         # We also have tiers whose members are directories.
         if 'TIERS' in sandbox:
@@ -660,14 +742,16 @@ class BuildReader(object):
                 # We don't descend into static directories because static by
                 # definition is external to the build system.
                 for d in values['regular']:
-                    if d in dirs:
+                    if d in recurse_info:
                         raise SandboxValidationError(
                             'Tier directory (%s) registered multiple '
                             'times in %s' % (d, tier))
-                    dirs.append(d)
+                    recurse_info[d] = {'tier': tier,
+                                       'parent': sandbox['RELATIVEDIR'],
+                                       'var': 'DIRS'}
 
         curdir = os.path.dirname(path)
-        for relpath in dirs:
+        for relpath, child_metadata in recurse_info.items():
             child_path = os.path.join(curdir, relpath, 'moz.build')
 
             # Ensure we don't break out of the topsrcdir. We don't do realpath
@@ -684,8 +768,7 @@ class BuildReader(object):
                 continue
 
             for res in self.read_mozbuild(child_path, read_tiers=False,
-                filesystem_absolute=True):
+                filesystem_absolute=True, metadata=child_metadata):
                 yield res
 
         self._execution_stack.pop()
-
