@@ -5,9 +5,8 @@
 from gestures import *
 from by import By
 from marionette import Marionette, HTMLElement, Actions, MultiActions
-from marionette_test import MarionetteTestCase, CommonTestCase, expectedFailure, skip, SkipTest
+from marionette_test import MarionetteTestCase, MarionetteJSTestCase, CommonTestCase, expectedFailure, skip, SkipTest
 from emulator import Emulator
-from runtests import MarionetteTestResult
-from runtests import MarionetteTestRunner
-from runtests import MarionetteTestOptions
-from runtests import MarionetteTextTestRunner
+from errors import *
+from runner import *
+from wait import Wait
