@@ -9,13 +9,14 @@ import sys
 import time
 import traceback
 
-from client import MarionetteClient
 from application_cache import ApplicationCache
-from keys import Keys
-from errors import *
+from client import MarionetteClient
 from emulator import Emulator
-import geckoinstance
+from emulator_screen import EmulatorScreen
+from errors import *
+from keys import Keys
 
+import geckoinstance
 
 class HTMLElement(object):
     """
@@ -146,10 +147,17 @@ class HTMLElement(object):
 
     @property
     def location(self):
-        '''
-        A dictionary with the x and y location of an element
-        '''
-        return self.marionette._send_message('getElementPosition', 'value', id=self.id)
+        """Get an element's location on the page.
+
+        The returned point will contain the x and y coordinates of the
+        top left-hand corner of the given element.  The point (0,0)
+        refers to the upper-left corner of the document.
+
+        :returns: a dictionary containing x and y as entries
+
+        """
+
+        return self.marionette._send_message("getElementLocation", "value", id=self.id)
 
     def value_of_css_property(self, property_name):
         '''
@@ -160,6 +168,11 @@ class HTMLElement(object):
         return self.marionette._send_message('getElementValueOfCssProperty', 'value',
                                              id=self.id,
                                              propertyName=property_name)
+    def submit(self):
+        '''
+        Submits if the element is a form or is within a form
+        '''
+        return self.marionette._send_message('submitElement', 'ok', id=self.id)
 
 class Actions(object):
     '''
@@ -416,6 +429,12 @@ class Marionette(object):
     TIMEOUT_SEARCH = 'implicit'
     TIMEOUT_SCRIPT = 'script'
     TIMEOUT_PAGE = 'page load'
+    SCREEN_ORIENTATIONS = {"portrait": EmulatorScreen.SO_PORTRAIT_PRIMARY,
+                           "landscape": EmulatorScreen.SO_LANDSCAPE_PRIMARY,
+                           "portrait-primary": EmulatorScreen.SO_PORTRAIT_PRIMARY,
+                           "landscape-primary": EmulatorScreen.SO_LANDSCAPE_PRIMARY,
+                           "portrait-secondary": EmulatorScreen.SO_PORTRAIT_SECONDARY,
+                           "landscape-secondary": EmulatorScreen.SO_LANDSCAPE_SECONDARY}
 
     def __init__(self, host='localhost', port=2828, app=None, app_args=None, bin=None,
                  profile=None, emulator=None, sdcard=None, emulatorBinary=None,
@@ -437,7 +456,6 @@ class Marionette(object):
         self.noWindow = noWindow
         self.logcat_dir = logcat_dir
         self._test_name = None
-        self.symbols_path = symbols_path
         self.timeout = timeout
         self.device_serial = device_serial
 
@@ -466,6 +484,7 @@ class Marionette(object):
                                      logcat_dir=self.logcat_dir,
                                      arch=emulator,
                                      sdcard=sdcard,
+                                     symbols_path=symbols_path,
                                      emulatorBinary=emulatorBinary,
                                      userdata=emulatorImg,
                                      res=emulator_res)
@@ -527,7 +546,7 @@ class Marionette(object):
             # flagging the error.
             sys.exit()
 
-    def wait_for_port(self, timeout=30):
+    def wait_for_port(self, timeout=60):
         starttime = datetime.datetime.now()
         while datetime.datetime.now() - starttime < datetime.timedelta(seconds=timeout):
             try:
@@ -587,7 +606,7 @@ class Marionette(object):
             status = response['error'].get('status', 500)
             message = response['error'].get('message')
             stacktrace = response['error'].get('stacktrace')
-            # status numbers come from 
+            # status numbers come from
             # http://code.google.com/p/selenium/wiki/JsonWireProtocol#Response_Status_Codes
             if status == ErrorCodes.NO_SUCH_ELEMENT:
                 raise NoSuchElementException(message=message, status=status, stacktrace=stacktrace)
@@ -643,7 +662,7 @@ class Marionette(object):
                 name = 'emulator'
                 crashed = True
 
-            if self.symbols_path and self.emulator.check_for_minidumps(self.symbols_path):
+            if self.emulator.check_for_minidumps():
                 crashed = True
         elif self.instance:
             # In the future, a check for crashed Firefox processes
@@ -737,10 +756,18 @@ class Marionette(object):
 
     @property
     def current_window_handle(self):
-        '''
-        A reference to the current window.
-        '''
-        self.window = self._send_message('getWindow', 'value')
+        """Get the current window's handle.
+
+        Return an opaque server-assigned identifier to this window
+        that uniquely identifies it within this Marionette instance.
+        This can be used to switch to this window at a later point.
+
+        :returns: unique window handle
+        :rtype: string
+
+        """
+
+        self.window = self._send_message("getWindowHandle", "value")
         return self.window
 
     @property
@@ -753,13 +780,21 @@ class Marionette(object):
 
     @property
     def window_handles(self):
-        '''
-        A list of references to all available browser windows if called in
-        content context. If called while in the chrome context, it will list
-        all available windows, not just browser windows (ie: not just
-        'navigator:browser';).
-        '''
-        response = self._send_message('getWindows', 'value')
+        """Get list of windows in the current context.
+
+        If called in the content context it will return a list of
+        references to all available browser windows.  Called in the
+        chrome context, it will list all available windows, not just
+        browser windows (e.g. not just navigator.browser).
+
+        Each window handle is assigned by the server, and the list of
+        strings returned does not have a guaranteed ordering.
+
+        :returns: unordered list of unique window handles as strings
+
+        """
+
+        response = self._send_message("getWindowHandles", "value")
         return response
 
     @property
@@ -770,15 +805,15 @@ class Marionette(object):
         response = self._send_message('getPageSource', 'value')
         return response
 
-    def close(self, window_id=None):
-        '''
-        Closes the window that is in use by Marionette.
+    def close(self):
+        """Close the current window, ending the session if it's the last
+        window currently open.
 
-        :param window_id: id of the window you wish to closed
-        '''
-        if not window_id:
-            window_id = self.current_window_handle
-        response = self._send_message('closeWindow', 'ok', value=window_id)
+        On B2G this method is a noop and will return immediately.
+
+        """
+
+        response = self._send_message("close", "ok")
         return response
 
     def set_context(self, context):
@@ -816,6 +851,12 @@ class Marionette(object):
             return HTMLElement(self, response)
         return None
 
+    def switch_to_default_content(self):
+        '''
+        Switch the current context to page's default content.
+        '''
+        return self.switch_to_frame()
+
     def switch_to_frame(self, frame=None, focus=True):
         '''
         Switch the current context to the specified frame. Subsequent commands will operate in the context of the specified frame, if applicable.
@@ -830,10 +871,20 @@ class Marionette(object):
         return response
 
     def get_url(self):
-        '''
-        Returns the url of the active page in the browser.
-        '''
-        response = self._send_message('getUrl', 'value')
+        """Get a string representing the current URL.
+
+        On Desktop this returns a string representation of the URL of
+        the current top level browsing context.  This is equivalent to
+        document.location.href.
+
+        When in the context of the chrome, this returns the canonical
+        URL of the current resource.
+
+        :returns: string representation of URL
+
+        """
+
+        response = self._send_message("getCurrentUrl", "value")
         return response
 
     def get_window_type(self):
@@ -847,12 +898,34 @@ class Marionette(object):
         return response
 
     def navigate(self, url):
-        '''
-        Causes the browser to navigate to the specified url.
+        """Navigate to to given URL.
+
+        This will follow redirects issued by the server.  When the
+        method returns is based on the page load strategy that the
+        user has selected.
+
+        Documents that contain a META tag with the "http-equiv"
+        attribute set to "refresh" will return if the timeout is
+        greater than 1 second and the other criteria for determining
+        whether a page is loaded are met.  When the refresh period is
+        1 second or less and the page load strategy is "normal" or
+        "conservative", it will wait for the page to complete loading
+        before returning.
+
+        If any modal dialog box, such as those opened on
+        window.onbeforeunload or window.alert, is opened at any point
+        in the page load, it will return immediately.
+
+        If a 401 response is seen by the browser, it will return
+        immediately.  That is, if BASIC, DIGEST, NTLM or similar
+        authentication is required, the page load is assumed to be
+        complete.  This does not include FORM-based authentication.
 
         :param url: The url to navigate to.
-        '''
-        response = self._send_message('goUrl', 'ok', url=url)
+
+        """
+
+        response = self._send_message("get", "ok", url=url)
         return response
 
     def timeouts(self, timeout_type, ms):
@@ -1180,6 +1253,14 @@ class Marionette(object):
             js = f.read()
         return self._send_message('importScript', 'ok', script=js)
 
+    def clear_imported_scripts(self):
+        '''
+        Clears all imported scripts in this context, ie: calling clear_imported_scripts in chrome
+        context will clear only scripts you imported in chrome, and will leave the scripts
+        you imported in content context.
+        '''
+        return self._send_message('clearImportedScripts', 'ok')
+
     def add_cookie(self, cookie):
         """
         Adds a cookie to your current session.
@@ -1237,27 +1318,72 @@ class Marionette(object):
         return None
 
     def get_cookies(self):
-        '''
-        Gets all cookies in the scope of the current session.
-        '''
-        return self._send_message("getAllCookies", "value")
+        """Get all the cookies for the current domain.
+
+        This is the equivalent of calling `document.cookie` and
+        parsing the result.
+
+        :returns: A set of cookies for the current domain.
+
+        """
+
+        return self._send_message("getCookies", "value")
 
     @property
     def application_cache(self):
         return ApplicationCache(self)
 
     def screenshot(self, element=None, highlights=None):
-        '''
-        Creates a base64-encoded screenshot of the element, or the current frame if no element is specified.
+        """Takes a screenshot of a web element or the current frame.
 
-        :param element: The element to take a screenshot of. If None, will
-         take a screenshot of the current frame.
-        :param highlights: A list of HTMLElement objects to draw a red box around in the
-         returned screenshot.
-        '''
-        if element is not None:
+        The screen capture is returned as a lossless PNG image encoded
+        as a base 64 string.  If the `element` argument is defined the
+        capture area will be limited to the bounding box of that
+        element.  Otherwise, the capture area will be the bounding box
+        of the current frame.
+
+        :param element: The element to take a screenshot of.  If None, will
+            take a screenshot of the current frame.
+
+        :param highlights: A list of HTMLElement objects to draw a red
+            box around in the returned screenshot.
+
+        """
+
+        if element:
             element = element.id
         lights = None
-        if highlights is not None:
-            lights = [highlight.id for highlight in highlights if highlights]
-        return self._send_message("screenShot", 'value', id=element, highlights=lights)
+        if highlights:
+            lights = [highlight.id for highlight in highlights]
+        return self._send_message("takeScreenshot", "value",
+                                  id=element, highlights=lights)
+
+    @property
+    def orientation(self):
+        """Get the current browser orientation.
+
+        Will return one of the valid primary orientation values
+        portrait-primary, landscape-primary, portrait-secondary, or
+        landscape-secondary.
+
+        """
+        return self._send_message("getScreenOrientation", "value")
+
+    def set_orientation(self, orientation):
+        """Set the current browser orientation.
+
+        The supplied orientation should be given as one of the valid
+        orientation values.  If the orientation is unknown, an error
+        will be raised.
+
+        Valid orientations are "portrait" and "landscape", which fall
+        back to "portrait-primary" and "landscape-primary"
+        respectively, and "portrait-secondary" as well as
+        "landscape-secondary".
+
+        :param orientation: The orientation to lock the screen in.
+
+        """
+        self._send_message("setScreenOrientation", "ok", orientation=orientation)
+        if self.emulator:
+            self.emulator.screen.orientation = self.SCREEN_ORIENTATIONS[orientation.lower()]
