@@ -419,10 +419,13 @@ class DeviceManager(object):
 
 
     @abstractmethod
-    def killProcess(self, processName, forceKill=False):
+    def killProcess(self, processName, sig=None):
         """
-        Kills the process named processName. If forceKill is True, process is
-        killed regardless of state.
+        Kills the process named processName. If sig is not None, process is
+        killed with the specified signal.
+
+        :param processName: path or name of the process to kill
+        :param sig: signal to pass into the kill command (optional)
         """
 
     @abstractmethod
