@@ -142,7 +142,7 @@ class XPCShellRunner(MozbuildObject):
             'debugger': debugger,
             'debuggerArgs': debuggerArgs,
             'debuggerInteractive': debuggerInteractive,
-            'on_message': (lambda obj, msg: xpcshell.log.info(msg)) \
+            'on_message': (lambda obj, msg: xpcshell.log.info(msg.decode('utf-8', 'replace'))) \
                             if test_path is not None else None,
         }
 
@@ -226,11 +226,16 @@ class MachCommands(MachCommandBase):
     @CommandArgument('--rerun-failures', action='store_true',
         help='Reruns failures from last time.')
     def run_xpcshell_test(self, **params):
+        from mozbuild.controller.building import BuildDriver
+
         # We should probably have a utility function to ensure the tree is
         # ready to run tests. Until then, we just create the state dir (in
         # case the tree wasn't built with mach).
         self._ensure_state_subdir_exists('.')
 
+        driver = self._spawn(BuildDriver)
+        driver.install_tests(remove=False)
+
         xpcshell = self._spawn(XPCShellRunner)
 
         try:
