@@ -10,9 +10,11 @@ import subprocess
 import runxpcshelltests as xpcshell
 import tempfile
 from automationutils import replaceBackSlashes
-import devicemanagerADB, devicemanagerSUT, devicemanager
+from mozdevice import devicemanagerADB, devicemanagerSUT, devicemanager
 from zipfile import ZipFile
 import shutil
+import mozfile
+import mozinfo
 
 here = os.path.dirname(os.path.abspath(__file__))
 
@@ -132,6 +134,17 @@ class RemoteXPCShellTestThread(xpcshell.XPCShellTestThread):
         self.device.killProcess("xpcshell")
         return outputFile
 
+    def checkForCrashes(self,
+                        dump_directory,
+                        symbols_path,
+                        test_name=None):
+        with mozfile.TemporaryDirectory() as dumpDir:
+            self.device.getDirectory(self.remoteMinidumpDir, dumpDir)
+            crashed = xpcshell.XPCShellTestThread.checkForCrashes(self, dumpDir, symbols_path, test_name)
+            self.device.removeDir(self.remoteMinidumpDir)
+            self.device.mkDir(self.remoteMinidumpDir)
+        return crashed
+
     def communicate(self, proc):
         f = open(proc, "r")
         contents = f.read()
@@ -197,6 +210,7 @@ class XPCShellRemote(xpcshell.XPCShellTests, object):
         self.remoteScriptsDir = self.remoteTestRoot
         self.remoteComponentsDir = remoteJoin(self.remoteTestRoot, "c")
         self.remoteModulesDir = remoteJoin(self.remoteTestRoot, "m")
+        self.remoteMinidumpDir = remoteJoin(self.remoteTestRoot, "minidumps")
         self.profileDir = remoteJoin(self.remoteTestRoot, "p")
         self.remoteDebugger = options.debugger
         self.remoteDebuggerArgs = options.debuggerArgs
@@ -218,6 +232,7 @@ class XPCShellRemote(xpcshell.XPCShellTests, object):
             self.setupUtilities()
             self.setupModules()
             self.setupTestDir()
+        self.setupMinidumpDir()
         self.remoteAPK = None
         if options.localAPK:
             self.remoteAPK = remoteJoin(self.remoteBinDir, os.path.basename(options.localAPK))
@@ -235,6 +250,7 @@ class XPCShellRemote(xpcshell.XPCShellTests, object):
             'pathMapping': self.pathMapping,
             'profileDir': self.profileDir,
             'remoteTmpDir': self.remoteTmpDir,
+            'remoteMinidumpDir': self.remoteMinidumpDir,
         }
         if self.remoteAPK:
             self.mobileArgs['remoteAPK'] = self.remoteAPK
@@ -274,6 +290,7 @@ class XPCShellRemote(xpcshell.XPCShellTests, object):
         self.env["TMPDIR"] = self.remoteTmpDir
         self.env["HOME"] = self.profileDir
         self.env["XPCSHELL_TEST_TEMP_DIR"] = self.remoteTmpDir
+        self.env["XPCSHELL_MINIDUMP_DIR"] = self.remoteMinidumpDir
         if self.options.setup:
             self.pushWrapper()
 
@@ -400,6 +417,11 @@ class XPCShellRemote(xpcshell.XPCShellTests, object):
             # Foopies have an older mozdevice ver without retryLimit
             self.device.pushDir(self.xpcDir, self.remoteScriptsDir)
 
+    def setupMinidumpDir(self):
+        if self.device.dirExists(self.remoteMinidumpDir):
+            self.device.removeDir(self.remoteMinidumpDir)
+        self.device.mkDir(self.remoteMinidumpDir)
+
     def buildTestList(self):
         xpcshell.XPCShellTests.buildTestList(self)
         uniqueTestPaths = set([])
@@ -542,6 +564,11 @@ def main():
         print >>sys.stderr, "Error: You must specify a test filename in interactive mode!"
         sys.exit(1)
 
+    # Add Android version (SDK level) to mozinfo so that manifest entries
+    # can be conditional on android_version.
+    androidVersion = dm.shellCheckOutput(['getprop', 'ro.build.version.sdk'])
+    mozinfo.info['android_version'] = androidVersion
+
     xpcsh = XPCShellRemote(dm, options, args)
 
     # we don't run concurrent tests on mobile
