@@ -11,6 +11,7 @@
 
 #include <cctype>
 
+#include "mozilla/NullPtr.h"
 #include "crashreporter.h"
 #include "crashreporter_gtk_common.h"
 
@@ -30,9 +31,9 @@ static bool gEmailFieldHint = true;
 static bool gCommentFieldHint = true;
 
 // handle from dlopen'ing libgnome
-static void* gnomeLib = NULL;
+static void* gnomeLib = nullptr;
 // handle from dlopen'ing libgnomeui
-static void* gnomeuiLib = NULL;
+static void* gnomeuiLib = nullptr;
 
 static void LoadSettings()
 {
@@ -118,7 +119,7 @@ void SendReport()
 
   // and spawn a thread to do the sending
   GError* err;
-  gSendThreadID = g_thread_create(SendThread, NULL, TRUE, &err);
+  gSendThreadID = g_thread_create(SendThread, nullptr, TRUE, &err);
 }
 
 static void ShowReportInfo(GtkTextView* viewReportTextView)
@@ -178,7 +179,7 @@ static void ViewReportClicked(GtkButton* button,
                                            GTK_DIALOG_MODAL,
                                            GTK_STOCK_OK,
                                            GTK_RESPONSE_OK,
-                                           NULL));
+                                           nullptr));
 
   GtkWidget* scrolled = gtk_scrolled_window_new(0, 0);
   gtk_container_add(GTK_CONTAINER(dialog->vbox), scrolled);
@@ -235,26 +236,26 @@ static void CommentInsert(GtkTextBuffer* buffer,
 static void UpdateHintText(GtkWidget* widget, gboolean gainedFocus,
                            bool* hintShowing, const char* hintText)
 {
-  GtkTextBuffer* buffer = NULL;
+  GtkTextBuffer* buffer = nullptr;
   if (GTK_IS_TEXT_VIEW(widget))
     buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
 
   if (gainedFocus) {
     if (*hintShowing) {
-      if (buffer == NULL) { // sort of cheating
+      if (buffer == nullptr) { // sort of cheating
         gtk_entry_set_text(GTK_ENTRY(widget), "");
       }
       else { // GtkTextView
         gtk_text_buffer_set_text(buffer, "", 0);
       }
-      gtk_widget_modify_text(widget, GTK_STATE_NORMAL, NULL);
+      gtk_widget_modify_text(widget, GTK_STATE_NORMAL, nullptr);
       *hintShowing = false;
     }
   }
   else {
     // lost focus
-    const char* text = NULL;
-    if (buffer == NULL) {
+    const char* text = nullptr;
+    if (buffer == nullptr) {
       text = gtk_entry_get_text(GTK_ENTRY(widget));
     }
     else {
@@ -264,10 +265,10 @@ static void UpdateHintText(GtkWidget* widget, gboolean gainedFocus,
       text = gtk_text_buffer_get_text(buffer, &start, &end, TRUE);
     }
 
-    if (text == NULL || text[0] == '\0') {
+    if (text == nullptr || text[0] == '\0') {
       *hintShowing = true;
 
-      if (buffer == NULL) {
+      if (buffer == nullptr) {
         gtk_entry_set_text(GTK_ENTRY(widget), hintText);
       }
       else {
@@ -347,7 +348,7 @@ void TryInitGnome()
 
   if (gnome_program_init && libgnomeui_module_info_get) {
     gnome_program_init("crashreporter", "1.0", libgnomeui_module_info_get(),
-                       gArgc, gArgv, NULL);
+                       gArgc, gArgv, nullptr);
   }
 
 }
@@ -504,7 +505,7 @@ bool UIShowCrashUI(const string& dumpfile,
 
   // Get the throbber image from alongside the executable
   char* dir = g_path_get_dirname(gArgv[0]);
-  char* path = g_build_filename(dir, "Throbber-small.gif", NULL);
+  char* path = g_build_filename(dir, "Throbber-small.gif", nullptr);
   g_free(dir);
   gThrobber = gtk_image_new_from_file(path);
   gtk_box_pack_start(GTK_BOX(progressBox), gThrobber, FALSE, FALSE, 0);
