@@ -176,19 +176,21 @@ WebappsActor.prototype = {
         // Needed to evict manifest cache on content side
         // (has to be dispatched first, otherwise other messages like
         // Install:Return:OK are going to use old manifest version)
-        reg.broadcastMessage("Webapps:PackageEvent",
-                             { type: "installed",
-                               manifestURL: aApp.manifestURL,
-                               app: aApp,
-                               manifest: manifest
-                             });
-
+        reg.broadcastMessage("Webapps:UpdateState", {
+          app: aApp,
+          manifest: manifest,
+          manifestURL: aApp.manifestURL
+        });
+        reg.broadcastMessage("Webapps:FireEvent", {
+          eventType: ["downloadsuccess", "downloadapplied"],
+          manifestURL: aApp.manifestURL
+        });
         reg.broadcastMessage("Webapps:AddApp", { id: aId, app: aApp });
-        reg.broadcastMessage("Webapps:Install:Return:OK",
-                             { app: aApp,
-                               oid: "foo",
-                               requestID: "bar"
-                             });
+        reg.broadcastMessage("Webapps:Install:Return:OK", {
+          app: aApp,
+          oid: "foo",
+          requestID: "bar"
+        });
 
         Services.obs.notifyObservers(null, "webapps-installed",
           JSON.stringify({ manifestURL: aApp.manifestURL }));
@@ -416,12 +418,41 @@ WebappsActor.prototype = {
           zipFile.moveTo(installDir, "application.zip");
 
           let origin = "app://" + id;
+          let manifestURL = origin + "/manifest.webapp";
+
+          // Refresh application.zip content (e.g. reinstall app), as done here:
+          // http://hg.mozilla.org/mozilla-central/annotate/aaefec5d34f8/dom/apps/src/Webapps.jsm#l1125
+          // Do it in parent process for the simulator
+          let jar = installDir.clone();
+          jar.append("application.zip");
+          Services.obs.notifyObservers(jar, "flush-cache-entry", null);
+
+          // And then in app content process
+          // This function will be evaluated in the scope of the content process
+          // frame script. That will flush the jar cache for this app and allow
+          // loading fresh updated resources if we reload its document.
+          let FlushFrameScript = function (path) {
+            let jar = Components.classes["@mozilla.org/file/local;1"]
+                                .createInstance(Components.interfaces.nsILocalFile);
+            jar.initWithPath(path);
+            let obs = Components.classes["@mozilla.org/observer-service;1"]
+                                .getService(Components.interfaces.nsIObserverService);
+            obs.notifyObservers(jar, "flush-cache-entry", null);
+          };
+          for each (let frame in self._appFrames()) {
+            if (frame.getAttribute("mozapp") == manifestURL) {
+              let mm = frame.QueryInterface(Ci.nsIFrameLoaderOwner).frameLoader.messageManager;
+              mm.loadFrameScript("data:," +
+                encodeURIComponent("(" + FlushFrameScript.toString() + ")" +
+                                   "('" + jar.path + "')"), false);
+            }
+          }
 
           // Create a fake app object with the minimum set of properties we need.
           let app = {
             origin: origin,
             installOrigin: origin,
-            manifestURL: origin + "/manifest.webapp",
+            manifestURL: manifestURL,
             appStatus: appType,
             receipts: aReceipts,
           }
@@ -463,28 +494,26 @@ WebappsActor.prototype = {
 
     let appDir = FileUtils.getDir("TmpD", ["b2g", appId], false, false);
 
-    if (!appDir || !appDir.exists()) {
-      if (aRequest.upload) {
-        // Ensure creating the directory (recursively)
-        appDir = FileUtils.getDir("TmpD", ["b2g", appId], true, false);
-        let actor = this.conn.getActor(aRequest.upload);
-        if (!actor) {
-          return { error: "badParameter",
-                   message: "Unable to find upload actor '" + aRequest.upload
-                            + "'" };
-        }
-        let appFile = FileUtils.File(actor.getFilePath());
-        if (!appFile.exists()) {
-          return { error: "badParameter",
-                   message: "The uploaded file doesn't exist on device" };
-        }
-        appFile.moveTo(appDir, "application.zip");
+    if (aRequest.upload) {
+      // Ensure creating the directory (recursively)
+      appDir = FileUtils.getDir("TmpD", ["b2g", appId], true, false);
+      let actor = this.conn.getActor(aRequest.upload);
+      if (!actor) {
+        return { error: "badParameter",
+                 message: "Unable to find upload actor '" + aRequest.upload
+                          + "'" };
       }
-      else if (!aRequest.manifest && !aRequest.metadata) {
-        return { error: "badParameterType",
-                 message: "missing directory " + appDir.path
-               }
+      let appFile = FileUtils.File(actor.getFilePath());
+      if (!appFile.exists()) {
+        return { error: "badParameter",
+                 message: "The uploaded file doesn't exist on device" };
       }
+      appFile.moveTo(appDir, "application.zip");
+    } else if ((!appDir || !appDir.exists()) &&
+               !aRequest.manifest && !aRequest.metadata) {
+      return { error: "badParameterType",
+               message: "missing directory " + appDir.path
+             };
     }
 
     let testFile = appDir.clone();
@@ -737,6 +766,10 @@ WebappsActor.prototype = {
   },
 
   _appFrames: function () {
+    // For now, we only support app frames on b2g
+    if (Services.appinfo.ID != "{3c2e2abc-06d4-11e1-ac3b-374f68613e61}") {
+      return;
+    }
     // Register the system app
     let chromeWindow = Services.wm.getMostRecentWindow('navigator:browser');
     let systemAppFrame = chromeWindow.shell.contentBrowser;
@@ -820,7 +853,15 @@ WebappsActor.prototype = {
           // the actor.
           deferred.resolve(null);
         }
-        this._appActorsMap.delete(mm);
+        let actor = this._appActorsMap.get(mm);
+        if (actor) {
+          // The ContentAppActor within the child process doesn't necessary
+          // have to time to uninitialize itself when the app is closed/killed.
+          // So ensure telling the client that the related actor is detached.
+          this.conn.send({ from: actor.actor,
+                           type: "tabDetached" });
+          this._appActorsMap.delete(mm);
+        }
       }
     }).bind(this);
     Services.obs.addObserver(onMessageManagerDisconnect,
