@@ -7,6 +7,7 @@
 "use strict";
 
 let promise = Cu.import("resource://gre/modules/commonjs/sdk/core/promise.js", {}).Promise;
+XPCOMUtils.defineLazyModuleGetter(this, "Services", "resource://gre/modules/Services.jsm");
 XPCOMUtils.defineLazyModuleGetter(this, "AddonManager", "resource://gre/modules/AddonManager.jsm");
 
 /**
@@ -20,7 +21,7 @@ XPCOMUtils.defineLazyModuleGetter(this, "AddonManager", "resource://gre/modules/
  */
 function allAppShellDOMWindows(aWindowType)
 {
-  let e = windowMediator.getEnumerator(aWindowType);
+  let e = Services.wm.getEnumerator(aWindowType);
   while (e.hasMoreElements()) {
     yield e.getNext();
   }
@@ -67,9 +68,6 @@ function createRootActor(aConnection)
                        });
 }
 
-var windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"]
-                     .getService(Ci.nsIWindowMediator);
-
 /**
  * A live list of BrowserTabActors representing the current browser tabs,
  * to be provided to the root actor to answer 'listTabs' requests.
@@ -188,8 +186,18 @@ function BrowserTabList(aConnection)
 BrowserTabList.prototype.constructor = BrowserTabList;
 
 
+/**
+ * Get the selected browser for the given navigator:browser window.
+ * @private
+ * @param aWindow nsIChromeWindow
+ *        The navigator:browser window for which you want the selected browser.
+ * @return nsIDOMElement|null
+ *         The currently selected xul:browser element, if any. Note that the
+ *         browser window might not be loaded yet - the function will return
+ *         |null| in such cases.
+ */
 BrowserTabList.prototype._getSelectedBrowser = function(aWindow) {
-  return aWindow.gBrowser.selectedBrowser;
+  return aWindow.gBrowser ? aWindow.gBrowser.selectedBrowser : null;
 };
 
 BrowserTabList.prototype._getChildren = function(aWindow) {
@@ -197,7 +205,7 @@ BrowserTabList.prototype._getChildren = function(aWindow) {
 };
 
 BrowserTabList.prototype.getList = function() {
-  let topXULWindow = windowMediator.getMostRecentWindow(DebuggerServer.chromeWindowType);
+  let topXULWindow = Services.wm.getMostRecentWindow(DebuggerServer.chromeWindowType);
 
   // As a sanity check, make sure all the actors presently in our map get
   // picked up when we iterate over all windows' tabs.
@@ -212,6 +220,9 @@ BrowserTabList.prototype.getList = function() {
   // Iterate over all navigator:browser XUL windows.
   for (let win of allAppShellDOMWindows(DebuggerServer.chromeWindowType)) {
     let selectedBrowser = this._getSelectedBrowser(win);
+    if (!selectedBrowser) {
+      continue;
+    }
 
     // For each tab in this XUL window, ensure that we have an actor for
     // it, reusing existing actors where possible. We actually iterate
@@ -371,7 +382,7 @@ BrowserTabList.prototype.handleEvent = makeInfallible(function(aEvent) {
 BrowserTabList.prototype._listenToMediatorIf = function(aShouldListen) {
   if (!aShouldListen !== !this._listeningToMediator) {
     let op = aShouldListen ? "addListener" : "removeListener";
-    windowMediator[op](this);
+    Services.wm[op](this);
     this._listeningToMediator = aShouldListen;
   }
 };
@@ -470,9 +481,6 @@ function BrowserTabActor(aConnection, aBrowser, aTabBrowser)
   this._extraActors = {};
 
   this._onWindowCreated = this.onWindowCreated.bind(this);
-
-  // Number of event loops nested.
-  this._nestedEventLoopDepth = 0;
 }
 
 // XXX (bug 710213): BrowserTabActor attach/detach/exit/disconnect is a
@@ -492,28 +500,6 @@ BrowserTabActor.prototype = {
 
   _pendingNavigation: null,
 
-  /**
-   * Add the specified actor to the default actor pool connection, in order to
-   * keep it alive as long as the server is. This is used by breakpoints in the
-   * thread actor.
-   *
-   * @param actor aActor
-   *        The actor object.
-   */
-  addToParentPool: function BTA_addToParentPool(aActor) {
-    this.conn.addActor(aActor);
-  },
-
-  /**
-   * Remove the specified actor from the default actor pool.
-   *
-   * @param BreakpointActor aActor
-   *        The actor object.
-   */
-  removeFromParentPool: function BTA_removeFromParentPool(aActor) {
-    this.conn.removeActor(aActor);
-  },
-
   // A constant prefix that will be used to form the actor ID by the server.
   actorPrefix: "tab",
 
@@ -540,7 +526,12 @@ BrowserTabActor.prototype = {
    *         Tab URL.
    */
   get url() {
-    return this.browser.currentURI.spec;
+    if (this.browser.currentURI) {
+      return this.browser.currentURI.spec;
+    }
+    // Abrupt closing of the browser window may leave callbacks without a
+    // currentURI.
+    return null;
   },
 
   /**
@@ -621,9 +612,6 @@ BrowserTabActor.prototype = {
                        type: "tabDetached" });
     }
 
-    // Pop all nested event loops if we haven't already.
-    while (this._nestedEventLoopDepth > 0)
-      this.postNest();
     this._browser = null;
     this._tabbrowser = null;
   },
@@ -772,7 +760,6 @@ BrowserTabActor.prototype = {
                           .getInterface(Ci.nsIDOMWindowUtils);
     windowUtils.suppressEventHandling(true);
     windowUtils.suspendTimeouts();
-    this._nestedEventLoopDepth++;
   },
 
   /**
@@ -781,8 +768,6 @@ BrowserTabActor.prototype = {
   postNest: function BTA_postNest(aNestData) {
     if (!this.window) {
       // The tab is already closed.
-      dbg_assert(this._nestedEventLoopDepth === 0,
-                 "window shouldn't be closed before all nested event loops have been popped");
       return;
     }
     let windowUtils = this.window
@@ -794,7 +779,6 @@ BrowserTabActor.prototype = {
       this._pendingNavigation.resume();
       this._pendingNavigation = null;
     }
-    this._nestedEventLoopDepth--;
   },
 
   /**
