@@ -4,6 +4,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#ifndef mozilla_dom_ExternalHelperAppChild_h
+#define mozilla_dom_ExternalHelperAppChild_h
 
 #include "mozilla/dom/PExternalHelperAppChild.h"
 #include "nsIStreamListener.h"
@@ -34,3 +36,5 @@ private:
 
 } // namespace dom
 } // namespace mozilla
+
+#endif // mozilla_dom_ExternalHelperAppChild_h
