@@ -4,13 +4,16 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "GfxInfo.h"
+#include "GLContext.h"
+#include "GLContextProvider.h"
 #include "nsUnicharUtils.h"
 #include "prenv.h"
 #include "prprf.h"
 #include "nsHashKeys.h"
 #include "nsVersionComparator.h"
-
 #include "AndroidBridge.h"
+#include "nsIWindowWatcher.h"
+#include "nsServiceManagerUtils.h"
 
 #if defined(MOZ_CRASHREPORTER)
 #include "nsExceptionHandler.h"
@@ -18,14 +21,97 @@
 #define NS_CRASHREPORTER_CONTRACTID "@mozilla.org/toolkit/crash-reporter;1"
 #endif
 
-using namespace mozilla::widget;
+namespace mozilla {
+namespace widget {
+
+class GfxInfo::GLStrings
+{
+  nsCString mVendor;
+  nsCString mRenderer;
+  nsCString mVersion;
+  bool mReady;
+
+public:
+  GLStrings()
+    : mReady(false)
+  {}
+
+  const nsCString& Vendor() {
+    EnsureInitialized();
+    return mVendor;
+  }
+
+  void SpoofVendor(const nsCString& s) {
+    EnsureInitialized();
+    mVendor = s;
+  }
+
+  const nsCString& Renderer() {
+    EnsureInitialized();
+    return mRenderer;
+  }
+
+  void SpoofRenderer(const nsCString& s) {
+    EnsureInitialized();
+    mRenderer = s;
+  }
+
+  const nsCString& Version() {
+    EnsureInitialized();
+    return mVersion;
+  }
+
+  void SpoofVersion(const nsCString& s) {
+    EnsureInitialized();
+    mVersion = s;
+  }
+
+  void EnsureInitialized() {
+    if (mReady) {
+      return;
+    }
+
+    nsRefPtr<gl::GLContext> gl = gl::GLContextProvider::CreateOffscreen(
+      gfxIntSize(16, 16),
+      gfx::SurfaceCaps::ForRGB());
+
+    if (!gl) {
+      // Setting mReady to true here means that we won't retry. Everything will
+      // remain blacklisted forever. Ideally, we would like to update that once
+      // any GLContext is successfully created, like the compositor's GLContext.
+      mReady = true;
+      return;
+    }
+
+    gl->MakeCurrent();
+
+    const char *spoofedVendor = PR_GetEnv("MOZ_GFX_SPOOF_GL_VENDOR");
+    if (spoofedVendor)
+        mVendor.Assign(spoofedVendor);
+    else
+        mVendor.Assign((const char*)gl->fGetString(LOCAL_GL_VENDOR));
+    const char *spoofedRenderer = PR_GetEnv("MOZ_GFX_SPOOF_GL_RENDERER");
+    if (spoofedRenderer)
+        mRenderer.Assign(spoofedRenderer);
+    else
+        mRenderer.Assign((const char*)gl->fGetString(LOCAL_GL_RENDERER));
+    const char *spoofedVersion = PR_GetEnv("MOZ_GFX_SPOOF_GL_VERSION");
+    if (spoofedVersion)
+        mVersion.Assign(spoofedVersion);
+    else
+        mVersion.Assign((const char*)gl->fGetString(LOCAL_GL_VERSION));
+
+    mReady = true;
+  }
+};
 
 #ifdef DEBUG
 NS_IMPL_ISUPPORTS_INHERITED1(GfxInfo, GfxInfoBase, nsIGfxInfoDebug)
 #endif
 
 GfxInfo::GfxInfo()
-  : mInitializedFromJavaData(false)
+  : mInitialized(false)
+  , mGLStrings(new GLStrings)
 {
 }
 
@@ -58,117 +144,68 @@ GfxInfo::GetCleartypeParameters(nsAString & aCleartypeParams)
 }
 
 void
-GfxInfo::EnsureInitializedFromGfxInfoData()
+GfxInfo::EnsureInitialized()
 {
-  if (mInitializedFromJavaData)
+  if (mInitialized)
     return;
-  mInitializedFromJavaData = true;
 
-  {
-    nsCString gfxInfoData;
-    mozilla::AndroidBridge::Bridge()->GetGfxInfoData(gfxInfoData);
+  mGLStrings->EnsureInitialized();
 
-    // the code here is a mini-parser for the text that GfxInfoThread.java produces.
-    // Here, |stringToFill| is the parser state. If it's null, we are expecting
-    // the next line to tell us what is the next string we'll read, e.g. "VENDOR"
-    // means that the next string we'll read is |mVendor|. We record that knowledge
-    // in the |stringToFill| pointer. So when it's not null, we just copy the next
-    // input line into the string pointed to by |stringToFill|.
-    nsCString *stringToFill = nullptr;
-    char *bufptr = gfxInfoData.BeginWriting();
+  MOZ_ASSERT(mozilla::AndroidBridge::Bridge());
 
-    while(true) {
-      char *line = NS_strtok("\n", &bufptr);
-      if (!line)
-        break;
-      if (stringToFill) {
-        stringToFill->Assign(line);
-        stringToFill = nullptr;
-      } else if(!strcmp(line, "VENDOR")) {
-        stringToFill = &mVendor;
-      } else if(!strcmp(line, "RENDERER")) {
-        stringToFill = &mRenderer;
-      } else if(!strcmp(line, "VERSION")) {
-        stringToFill = &mVersion;
-      } else if(!strcmp(line, "ERROR")) {
-        stringToFill = &mError;
-      }
-    }
+  if (mozilla::AndroidBridge::Bridge()->GetStaticStringField("android/os/Build", "MODEL", mModel)) {
+    mAdapterDescription.AppendPrintf("Model: %s",  NS_LossyConvertUTF16toASCII(mModel).get());
   }
 
-
-  if (!mError.IsEmpty()) {
-    mAdapterDescription.AppendPrintf("An error occurred earlier while querying gfx info: %s. ",
-                                     mError.get());
-    printf_stderr("%s\n", mAdapterDescription.get());
+  if (mozilla::AndroidBridge::Bridge()->GetStaticStringField("android/os/Build", "PRODUCT", mProduct)) {
+    mAdapterDescription.AppendPrintf(", Product: %s", NS_LossyConvertUTF16toASCII(mProduct).get());
   }
 
-  const char *spoofedVendor = PR_GetEnv("MOZ_GFX_SPOOF_GL_VENDOR");
-  if (spoofedVendor)
-      mVendor.Assign(spoofedVendor);
-  const char *spoofedRenderer = PR_GetEnv("MOZ_GFX_SPOOF_GL_RENDERER");
-  if (spoofedRenderer)
-      mRenderer.Assign(spoofedRenderer);
-  const char *spoofedVersion = PR_GetEnv("MOZ_GFX_SPOOF_GL_VERSION");
-  if (spoofedVersion)
-      mVersion.Assign(spoofedVersion);
-
-  mAdapterDescription.AppendPrintf("%s -- %s -- %s",
-                                   mVendor.get(),
-                                   mRenderer.get(),
-                                   mVersion.get());
-
-  // Now we append general (non-gfx) device information. The only reason why this code is still here
-  // is that this used to be all we had in GfxInfo on Android, and we can't trivially remove it
-  // as it's useful information that isn't given anywhere else in about:support of in crash reports.
-  // But we should really move this out of GfxInfo.
-  if (mozilla::AndroidBridge::Bridge()) {
-    if (mozilla::AndroidBridge::Bridge()->GetStaticStringField("android/os/Build", "MODEL", mModel)) {
-      mAdapterDescription.AppendPrintf(" -- Model: %s",  NS_LossyConvertUTF16toASCII(mModel).get());
-    }
-
-    if (mozilla::AndroidBridge::Bridge()->GetStaticStringField("android/os/Build", "PRODUCT", mProduct)) {
-      mAdapterDescription.AppendPrintf(", Product: %s", NS_LossyConvertUTF16toASCII(mProduct).get());
-    }
-
-    if (mozilla::AndroidBridge::Bridge()->GetStaticStringField("android/os/Build", "MANUFACTURER", mManufacturer)) {
-      mAdapterDescription.AppendPrintf(", Manufacturer: %s", NS_LossyConvertUTF16toASCII(mManufacturer).get());
-    }
-
-    int32_t sdkVersion;
-    if (!mozilla::AndroidBridge::Bridge()->GetStaticIntField("android/os/Build$VERSION", "SDK_INT", &sdkVersion))
-      sdkVersion = 0;
-
-    // the HARDWARE field isn't available on Android SDK < 8
-    if (sdkVersion >= 8 && mozilla::AndroidBridge::Bridge()->GetStaticStringField("android/os/Build", "HARDWARE", mHardware)) {
-      mAdapterDescription.AppendPrintf(", Hardware: %s", NS_LossyConvertUTF16toASCII(mHardware).get());
-    }
-
-    nsString release;
-    mozilla::AndroidBridge::Bridge()->GetStaticStringField("android/os/Build$VERSION", "RELEASE", release);
-    mOSVersion = NS_LossyConvertUTF16toASCII(release);
-
-    mOSVersionInteger = 0;
-    char a[5], b[5], c[5], d[5];
-    SplitDriverVersion(mOSVersion.get(), a, b, c, d);
-    uint8_t na = atoi(a);
-    uint8_t nb = atoi(b);
-    uint8_t nc = atoi(c);
-    uint8_t nd = atoi(d);
-
-    mOSVersionInteger = (uint32_t(na) << 24) |
-                        (uint32_t(nb) << 16) |
-                        (uint32_t(nc) << 8)  |
-                        uint32_t(nd);
+  if (mozilla::AndroidBridge::Bridge()->GetStaticStringField("android/os/Build", "MANUFACTURER", mManufacturer)) {
+    mAdapterDescription.AppendPrintf(", Manufacturer: %s", NS_LossyConvertUTF16toASCII(mManufacturer).get());
   }
 
+  int32_t sdkVersion;
+  if (!mozilla::AndroidBridge::Bridge()->GetStaticIntField("android/os/Build$VERSION", "SDK_INT", &sdkVersion))
+    sdkVersion = 0;
+
+  // the HARDWARE field isn't available on Android SDK < 8
+  if (sdkVersion >= 8 && mozilla::AndroidBridge::Bridge()->GetStaticStringField("android/os/Build", "HARDWARE", mHardware)) {
+    mAdapterDescription.AppendPrintf(", Hardware: %s", NS_LossyConvertUTF16toASCII(mHardware).get());
+  }
+
+  nsString release;
+  mozilla::AndroidBridge::Bridge()->GetStaticStringField("android/os/Build$VERSION", "RELEASE", release);
+  mOSVersion = NS_LossyConvertUTF16toASCII(release);
+
+  mOSVersionInteger = 0;
+  char a[5], b[5], c[5], d[5];
+  SplitDriverVersion(mOSVersion.get(), a, b, c, d);
+  uint8_t na = atoi(a);
+  uint8_t nb = atoi(b);
+  uint8_t nc = atoi(c);
+  uint8_t nd = atoi(d);
+
+  mOSVersionInteger = (uint32_t(na) << 24) |
+                      (uint32_t(nb) << 16) |
+                      (uint32_t(nc) << 8)  |
+                      uint32_t(nd);
+
+  mAdapterDescription.AppendPrintf(", OpenGL: %s -- %s -- %s",
+                                   mGLStrings->Vendor().get(),
+                                   mGLStrings->Renderer().get(),
+                                   mGLStrings->Version().get());
+
   AddCrashReportAnnotations();
+
+  mInitialized = true;
 }
 
 /* readonly attribute DOMString adapterDescription; */
 NS_IMETHODIMP
 GfxInfo::GetAdapterDescription(nsAString & aAdapterDescription)
 {
+  EnsureInitialized();
   aAdapterDescription = NS_ConvertASCIItoUTF16(mAdapterDescription);
   return NS_OK;
 }
@@ -177,6 +214,7 @@ GfxInfo::GetAdapterDescription(nsAString & aAdapterDescription)
 NS_IMETHODIMP
 GfxInfo::GetAdapterDescription2(nsAString & aAdapterDescription)
 {
+  EnsureInitialized();
   return NS_ERROR_FAILURE;
 }
 
@@ -184,6 +222,7 @@ GfxInfo::GetAdapterDescription2(nsAString & aAdapterDescription)
 NS_IMETHODIMP
 GfxInfo::GetAdapterRAM(nsAString & aAdapterRAM)
 {
+  EnsureInitialized();
   aAdapterRAM.AssignLiteral("");
   return NS_OK;
 }
@@ -192,6 +231,7 @@ GfxInfo::GetAdapterRAM(nsAString & aAdapterRAM)
 NS_IMETHODIMP
 GfxInfo::GetAdapterRAM2(nsAString & aAdapterRAM)
 {
+  EnsureInitialized();
   return NS_ERROR_FAILURE;
 }
 
@@ -199,6 +239,7 @@ GfxInfo::GetAdapterRAM2(nsAString & aAdapterRAM)
 NS_IMETHODIMP
 GfxInfo::GetAdapterDriver(nsAString & aAdapterDriver)
 {
+  EnsureInitialized();
   aAdapterDriver.AssignLiteral("");
   return NS_OK;
 }
@@ -207,6 +248,7 @@ GfxInfo::GetAdapterDriver(nsAString & aAdapterDriver)
 NS_IMETHODIMP
 GfxInfo::GetAdapterDriver2(nsAString & aAdapterDriver)
 {
+  EnsureInitialized();
   return NS_ERROR_FAILURE;
 }
 
@@ -214,7 +256,8 @@ GfxInfo::GetAdapterDriver2(nsAString & aAdapterDriver)
 NS_IMETHODIMP
 GfxInfo::GetAdapterDriverVersion(nsAString & aAdapterDriverVersion)
 {
-  aAdapterDriverVersion = NS_ConvertASCIItoUTF16(mVersion);
+  EnsureInitialized();
+  aAdapterDriverVersion = NS_ConvertASCIItoUTF16(mGLStrings->Version());
   return NS_OK;
 }
 
@@ -222,6 +265,7 @@ GfxInfo::GetAdapterDriverVersion(nsAString & aAdapterDriverVersion)
 NS_IMETHODIMP
 GfxInfo::GetAdapterDriverVersion2(nsAString & aAdapterDriverVersion)
 {
+  EnsureInitialized();
   return NS_ERROR_FAILURE;
 }
 
@@ -229,6 +273,7 @@ GfxInfo::GetAdapterDriverVersion2(nsAString & aAdapterDriverVersion)
 NS_IMETHODIMP
 GfxInfo::GetAdapterDriverDate(nsAString & aAdapterDriverDate)
 {
+  EnsureInitialized();
   aAdapterDriverDate.AssignLiteral("");
   return NS_OK;
 }
@@ -237,6 +282,7 @@ GfxInfo::GetAdapterDriverDate(nsAString & aAdapterDriverDate)
 NS_IMETHODIMP
 GfxInfo::GetAdapterDriverDate2(nsAString & aAdapterDriverDate)
 {
+  EnsureInitialized();
   return NS_ERROR_FAILURE;
 }
 
@@ -244,7 +290,8 @@ GfxInfo::GetAdapterDriverDate2(nsAString & aAdapterDriverDate)
 NS_IMETHODIMP
 GfxInfo::GetAdapterVendorID(nsAString & aAdapterVendorID)
 {
-  aAdapterVendorID = NS_ConvertASCIItoUTF16(mVendor);
+  EnsureInitialized();
+  aAdapterVendorID = NS_ConvertASCIItoUTF16(mGLStrings->Vendor());
   return NS_OK;
 }
 
@@ -252,6 +299,7 @@ GfxInfo::GetAdapterVendorID(nsAString & aAdapterVendorID)
 NS_IMETHODIMP
 GfxInfo::GetAdapterVendorID2(nsAString & aAdapterVendorID)
 {
+  EnsureInitialized();
   return NS_ERROR_FAILURE;
 }
 
@@ -259,7 +307,8 @@ GfxInfo::GetAdapterVendorID2(nsAString & aAdapterVendorID)
 NS_IMETHODIMP
 GfxInfo::GetAdapterDeviceID(nsAString & aAdapterDeviceID)
 {
-  aAdapterDeviceID = NS_ConvertASCIItoUTF16(mRenderer);
+  EnsureInitialized();
+  aAdapterDeviceID = NS_ConvertASCIItoUTF16(mGLStrings->Renderer());
   return NS_OK;
 }
 
@@ -267,6 +316,7 @@ GfxInfo::GetAdapterDeviceID(nsAString & aAdapterDeviceID)
 NS_IMETHODIMP
 GfxInfo::GetAdapterDeviceID2(nsAString & aAdapterDeviceID)
 {
+  EnsureInitialized();
   return NS_ERROR_FAILURE;
 }
 
@@ -274,6 +324,7 @@ GfxInfo::GetAdapterDeviceID2(nsAString & aAdapterDeviceID)
 NS_IMETHODIMP
 GfxInfo::GetIsGPU2Active(bool* aIsGPU2Active)
 {
+  EnsureInitialized();
   return NS_ERROR_FAILURE;
 }
 
@@ -282,9 +333,9 @@ GfxInfo::AddCrashReportAnnotations()
 {
 #if defined(MOZ_CRASHREPORTER)
   CrashReporter::AnnotateCrashReport(NS_LITERAL_CSTRING("AdapterVendorID"),
-                                     mVendor);
+                                     mGLStrings->Vendor());
   CrashReporter::AnnotateCrashReport(NS_LITERAL_CSTRING("AdapterDeviceID"),
-                                     mRenderer);
+                                     mGLStrings->Renderer());
 
   /* Add an App Note for now so that we get the data immediately. These
    * can go away after we store the above in the socorro db */
@@ -299,17 +350,10 @@ const nsTArray<GfxDriverInfo>&
 GfxInfo::GetGfxDriverInfo()
 {
   if (mDriverInfo->IsEmpty()) {
-#ifdef MOZ_ANDROID_OMTC
     APPEND_TO_DRIVER_BLOCKLIST2( DRIVER_OS_ALL,
       (nsAString&) GfxDriverInfo::GetDeviceVendor(VendorAll), GfxDriverInfo::allDevices,
       nsIGfxInfo::FEATURE_OPENGL_LAYERS, nsIGfxInfo::FEATURE_NO_INFO,
       DRIVER_COMPARISON_IGNORED, GfxDriverInfo::allDriverVersions );
-#else
-    APPEND_TO_DRIVER_BLOCKLIST2( DRIVER_OS_ALL,
-      (nsAString&) GfxDriverInfo::GetDeviceVendor(VendorAll), GfxDriverInfo::allDevices,
-      nsIGfxInfo::FEATURE_OPENGL_LAYERS, nsIGfxInfo::FEATURE_BLOCKED_DEVICE,
-      DRIVER_COMPARISON_IGNORED, GfxDriverInfo::allDriverVersions );
-#endif
   }
 
   return *mDriverInfo;
@@ -329,9 +373,17 @@ GfxInfo::GetFeatureStatusImpl(int32_t aFeature,
   if (aOS)
     *aOS = os;
 
-  EnsureInitializedFromGfxInfoData();
+  // OpenGL layers are never blacklisted on Android.
+  // This early return is so we avoid potentially slow
+  // GLStrings initialization on startup when we initialize GL layers.
+  if (aFeature == nsIGfxInfo::FEATURE_OPENGL_LAYERS) {
+    *aStatus = nsIGfxInfo::FEATURE_NO_INFO;
+    return NS_OK;
+  }
 
-  if (!mError.IsEmpty()) {
+  EnsureInitialized();
+
+  if (mGLStrings->Vendor().IsEmpty() || mGLStrings->Renderer().IsEmpty()) {
     *aStatus = nsIGfxInfo::FEATURE_BLOCKED_DEVICE;
     return NS_OK;
   }
@@ -339,8 +391,8 @@ GfxInfo::GetFeatureStatusImpl(int32_t aFeature,
   // Don't evaluate special cases when evaluating the downloaded blocklist.
   if (aDriverInfo.IsEmpty()) {
     if (aFeature == FEATURE_WEBGL_OPENGL) {
-      if (mRenderer.Find("Adreno 200") != -1 ||
-          mRenderer.Find("Adreno 205") != -1)
+      if (mGLStrings->Renderer().Find("Adreno 200") != -1 ||
+          mGLStrings->Renderer().Find("Adreno 205") != -1)
       {
         *aStatus = nsIGfxInfo::FEATURE_BLOCKED_DEVICE;
         return NS_OK;
@@ -358,7 +410,6 @@ GfxInfo::GetFeatureStatusImpl(int32_t aFeature,
       NS_LossyConvertUTF16toASCII cHardware(mHardware);
 
       if (cHardware.Equals("antares") ||
-          cHardware.Equals("endeavoru") ||
           cHardware.Equals("harmony") ||
           cHardware.Equals("picasso") ||
           cHardware.Equals("picasso_e") ||
@@ -388,9 +439,11 @@ GfxInfo::GetFeatureStatusImpl(int32_t aFeature,
         // Gingerbread HTC devices are whitelisted.
         // Gingerbread Samsung devices are whitelisted except for:
         //   Samsung devices identified in Bug 847837
+        // Gingerbread Sony devices are whitelisted.
         // All other Gingerbread devices are blacklisted.
         bool isWhitelisted =
           cManufacturer.Equals("htc", nsCaseInsensitiveCStringComparator()) ||
+          (cManufacturer.Find("sony", true) != -1) ||
           cManufacturer.Equals("samsung", nsCaseInsensitiveCStringComparator());
 
         if (cModel.Equals("GT-I8160", nsCaseInsensitiveCStringComparator()) ||
@@ -447,16 +500,16 @@ GfxInfo::GetFeatureStatusImpl(int32_t aFeature,
         //     Samsung SGH-I717 (Bug 845729)
         //     Samsung SGH-I727 (Bug 845729)
         //     Samsung SGH-I757 (Bug 845729)
-        //     Samsung SGH-T989 (Bug 845729)
         //   All Galaxy nexus ICS devices
+        //   Sony Xperia Ion (LT28) ICS devices
         bool isWhitelisted =
+          cModel.Equals("LT28h", nsCaseInsensitiveCStringComparator()) ||
           cManufacturer.Equals("samsung", nsCaseInsensitiveCStringComparator()) ||
           cModel.Equals("galaxy nexus", nsCaseInsensitiveCStringComparator()); // some Galaxy Nexus have manufacturer=amazon
 
         if (cModel.Find("SGH-I717", true) != -1 ||
             cModel.Find("SGH-I727", true) != -1 ||
-            cModel.Find("SGH-I757", true) != -1 ||
-            cModel.Find("SGH-T989", true) != -1)
+            cModel.Find("SGH-I757", true) != -1)
         {
           isWhitelisted = false;
         }
@@ -468,20 +521,33 @@ GfxInfo::GetFeatureStatusImpl(int32_t aFeature,
       }
       else if (CompareVersions(mOSVersion.get(), "4.2.0") < 0)
       {
+        // Whitelist:
+        //   All JB phones except for those in blocklist below
+        // Blocklist:
+        //   Samsung devices from bug 812881 and 853522.
+        //   Motorola XT890 from bug 882342.
         bool isBlocklisted =
-          cModel.Find("GT-P3100", true) ||
-          cModel.Find("GT-P3110", true) ||
-          cModel.Find("GT-P3113", true) ||
-          cModel.Find("GT-P5100", true) ||
-          cModel.Find("GT-P5110", true) ||
-          cModel.Find("GT-P5113", true) ||
-          cManufacturer.Equals("Sony", nsCaseInsensitiveCStringComparator());
+          cModel.Find("GT-P3100", true) != -1 ||
+          cModel.Find("GT-P3110", true) != -1 ||
+          cModel.Find("GT-P3113", true) != -1 ||
+          cModel.Find("GT-P5100", true) != -1 ||
+          cModel.Find("GT-P5110", true) != -1 ||
+          cModel.Find("GT-P5113", true) != -1 ||
+          cModel.Find("XT890", true) != -1;
 
         if (isBlocklisted) {
           *aStatus = nsIGfxInfo::FEATURE_BLOCKED_DEVICE;
           return NS_OK;
         }
       }
+      else if (CompareVersions(mOSVersion.get(), "4.3.0") < 0)
+      {
+        // Blocklist all Sony devices
+        if (cManufacturer.Find("Sony", true) != -1) {
+          *aStatus = nsIGfxInfo::FEATURE_BLOCKED_DEVICE;
+          return NS_OK;
+        }
+      }
     }
   }
 
@@ -495,58 +561,66 @@ GfxInfo::GetFeatureStatusImpl(int32_t aFeature,
 /* void spoofVendorID (in DOMString aVendorID); */
 NS_IMETHODIMP GfxInfo::SpoofVendorID(const nsAString & aVendorID)
 {
-  EnsureInitializedFromGfxInfoData(); // initialization from GfxInfo data overwrites mVendor
-  mVendor = NS_LossyConvertUTF16toASCII(aVendorID);
+  EnsureInitialized();
+  mGLStrings->SpoofVendor(NS_LossyConvertUTF16toASCII(aVendorID));
   return NS_OK;
 }
 
 /* void spoofDeviceID (in unsigned long aDeviceID); */
 NS_IMETHODIMP GfxInfo::SpoofDeviceID(const nsAString & aDeviceID)
 {
-  EnsureInitializedFromGfxInfoData(); // initialization from GfxInfo data overwrites mRenderer
-  mRenderer = NS_LossyConvertUTF16toASCII(aDeviceID);
+  EnsureInitialized();
+  mGLStrings->SpoofRenderer(NS_LossyConvertUTF16toASCII(aDeviceID));
   return NS_OK;
 }
 
 /* void spoofDriverVersion (in DOMString aDriverVersion); */
 NS_IMETHODIMP GfxInfo::SpoofDriverVersion(const nsAString & aDriverVersion)
 {
-  EnsureInitializedFromGfxInfoData(); // initialization from GfxInfo data overwrites mVersion
-  mVersion = NS_LossyConvertUTF16toASCII(aDriverVersion);
+  EnsureInitialized();
+  mGLStrings->SpoofVersion(NS_LossyConvertUTF16toASCII(aDriverVersion));
   return NS_OK;
 }
 
 /* void spoofOSVersion (in unsigned long aVersion); */
 NS_IMETHODIMP GfxInfo::SpoofOSVersion(uint32_t aVersion)
 {
-  EnsureInitializedFromGfxInfoData(); // initialization from GfxInfo data overwrites mOSVersion
+  EnsureInitialized();
   mOSVersion = aVersion;
   return NS_OK;
 }
 
 #endif
 
-nsString GfxInfo::Model() const
+nsString GfxInfo::Model()
 {
+  EnsureInitialized();
   return mModel;
 }
 
-nsString GfxInfo::Hardware() const
+nsString GfxInfo::Hardware()
 {
+  EnsureInitialized();
   return mHardware;
 }
 
-nsString GfxInfo::Product() const
+nsString GfxInfo::Product()
 {
+  EnsureInitialized();
   return mProduct;
 }
 
-nsString GfxInfo::Manufacturer() const
+nsString GfxInfo::Manufacturer()
 {
+  EnsureInitialized();
   return mManufacturer;
 }
 
-uint32_t GfxInfo::OperatingSystemVersion() const
+uint32_t GfxInfo::OperatingSystemVersion()
 {
+  EnsureInitialized();
   return mOSVersionInteger;
 }
+
+}
+}
\ No newline at end of file
