@@ -16,7 +16,7 @@ public:
   NS_DECL_NSIOBSERVER
 
   MetroAppShell() :
-    mEventWnd(NULL),
+    mEventWnd(nullptr),
     mPowerRequestCount(0)
   {
   }
@@ -28,6 +28,7 @@ public:
   static LRESULT CALLBACK EventWindowProc(HWND, UINT, WPARAM, LPARAM);
   static bool ProcessOneNativeEventIfPresent();
   static void MarkEventQueueForPurge();
+  static void InputEventsDispatched();
 
 protected:
   NS_IMETHOD Run();
