@@ -1,22 +1,23 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim:set ts=2 sw=2 sts=2 ci et: */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/AvailableMemoryTracker.h"
 
+#if defined(XP_WIN)
 #include "prinrval.h"
-#include "pratom.h"
 #include "prenv.h"
-
 #include "nsIMemoryReporter.h"
+#include "nsMemoryPressure.h"
+#endif
+
 #include "nsIObserver.h"
 #include "nsIObserverService.h"
 #include "nsIRunnable.h"
 #include "nsISupports.h"
-#include "nsPrintfCString.h"
-#include "nsThread.h"
+#include "nsThreadUtils.h"
 
 #include "mozilla/Preferences.h"
 #include "mozilla/Services.h"
@@ -127,9 +128,9 @@ uint32_t sLowCommitSpaceThreshold = 0;
 uint32_t sLowPhysicalMemoryThreshold = 0;
 uint32_t sLowMemoryNotificationIntervalMS = 0;
 
-uint32_t sNumLowVirtualMemEvents = 0;
-uint32_t sNumLowCommitSpaceEvents = 0;
-uint32_t sNumLowPhysicalMemEvents = 0;
+Atomic<uint32_t> sNumLowVirtualMemEvents;
+Atomic<uint32_t> sNumLowCommitSpaceEvents;
+Atomic<uint32_t> sNumLowPhysicalMemEvents;
 
 WindowsDllInterceptor sKernel32Intercept;
 WindowsDllInterceptor sGdi32Intercept;
@@ -187,7 +188,7 @@ bool MaybeScheduleMemoryPressureEvent()
   sLastLowMemoryNotificationTime = PR_IntervalNow();
 
   LOG("Scheduling memory pressure notification.");
-  ScheduleMemoryPressureEvent();
+  NS_DispatchEventualMemoryPressure(MemPressure_New);
   return true;
 }
 
@@ -211,19 +212,19 @@ void CheckMemAvailable()
       // notification.  We'll probably crash if we run out of virtual memory,
       // so don't worry about firing this notification too often.
       LOG("Detected low virtual memory.");
-      PR_ATOMIC_INCREMENT(&sNumLowVirtualMemEvents);
-      ScheduleMemoryPressureEvent();
+      ++sNumLowVirtualMemEvents;
+      NS_DispatchEventualMemoryPressure(MemPressure_New);
     }
     else if (stat.ullAvailPageFile < sLowCommitSpaceThreshold * 1024 * 1024) {
       LOG("Detected low available page file space.");
       if (MaybeScheduleMemoryPressureEvent()) {
-        PR_ATOMIC_INCREMENT(&sNumLowCommitSpaceEvents);
+        ++sNumLowCommitSpaceEvents;
       }
     }
     else if (stat.ullAvailPhys < sLowPhysicalMemoryThreshold * 1024 * 1024) {
       LOG("Detected low physical memory.");
       if (MaybeScheduleMemoryPressureEvent()) {
-        PR_ATOMIC_INCREMENT(&sNumLowPhysicalMemEvents);
+        ++sNumLowPhysicalMemEvents;
       }
     }
   }
@@ -324,158 +325,76 @@ CreateDIBSectionHook(HDC aDC,
   return result;
 }
 
-class NumLowMemoryEventsReporter : public nsIMemoryReporter
+static int64_t
+LowMemoryEventsVirtualDistinguishedAmount()
 {
-  NS_IMETHOD GetProcess(nsACString &aProcess)
-  {
-    aProcess.Truncate();
-    return NS_OK;
-  }
+  return sNumLowVirtualMemEvents;
+}
 
-  NS_IMETHOD GetKind(int *aKind)
-  {
-    *aKind = KIND_OTHER;
-    return NS_OK;
-  }
-
-  NS_IMETHOD GetUnits(int *aUnits)
-  {
-    *aUnits = UNITS_COUNT_CUMULATIVE;
-    return NS_OK;
-  }
-};
-
-class NumLowVirtualMemoryEventsMemoryReporter MOZ_FINAL : public NumLowMemoryEventsReporter
+class LowMemoryEventsVirtualReporter MOZ_FINAL : public MemoryUniReporter
 {
 public:
-  NS_DECL_ISUPPORTS
+  LowMemoryEventsVirtualReporter()
+    : MemoryUniReporter("low-memory-events/virtual",
+                         KIND_OTHER, UNITS_COUNT_CUMULATIVE,
+"Number of low-virtual-memory events fired since startup. We fire such an "
+"event if we notice there is less than memory.low_virtual_mem_threshold_mb of "
+"virtual address space available (if zero, this behavior is disabled).  The "
+"process will probably crash if it runs out of virtual address space, so "
+"this event is dire.")
+  {}
 
-  NS_IMETHOD GetPath(nsACString &aPath)
-  {
-    aPath.AssignLiteral("low-memory-events/virtual");
-    return NS_OK;
-  }
-
-  NS_IMETHOD GetAmount(int64_t *aAmount)
+private:
+  int64_t Amount() MOZ_OVERRIDE
   {
     // This memory reporter shouldn't be installed on 64-bit machines, since we
     // force-disable virtual-memory tracking there.
     MOZ_ASSERT(sizeof(void*) == 4);
 
-    *aAmount = sNumLowVirtualMemEvents;
-    return NS_OK;
-  }
-
-  NS_IMETHOD GetDescription(nsACString &aDescription)
-  {
-    aDescription.AssignLiteral(
-      "Number of low-virtual-memory events fired since startup. ");
-
-    if (sLowVirtualMemoryThreshold == 0) {
-      aDescription.AppendLiteral(
-        "Tracking low-virtual-memory events is disabled, but you can enable it "
-        "by giving the memory.low_virtual_mem_threshold_mb pref a non-zero "
-        "value.");
-    }
-    else {
-      aDescription.Append(nsPrintfCString(
-        "We fire such an event if we notice there is less than %d MB of virtual "
-        "address space available (controlled by the "
-        "'memory.low_virtual_mem_threshold_mb' pref).  We'll likely crash if "
-        "we run out of virtual address space, so this event is somewhat dire.",
-        sLowVirtualMemoryThreshold));
-    }
-    return NS_OK;
+    return LowMemoryEventsVirtualDistinguishedAmount();
   }
 };
 
-NS_IMPL_ISUPPORTS1(NumLowVirtualMemoryEventsMemoryReporter, nsIMemoryReporter)
-
-class NumLowCommitSpaceEventsMemoryReporter MOZ_FINAL : public NumLowMemoryEventsReporter
+class LowCommitSpaceEventsReporter MOZ_FINAL : public MemoryUniReporter
 {
 public:
-  NS_DECL_ISUPPORTS
+  LowCommitSpaceEventsReporter()
+    : MemoryUniReporter("low-commit-space-events",
+                         KIND_OTHER, UNITS_COUNT_CUMULATIVE,
+"Number of low-commit-space events fired since startup. We fire such an "
+"event if we notice there is less than memory.low_commit_space_threshold_mb of "
+"commit space available (if zero, this behavior is disabled).  Windows will "
+"likely kill the process if it runs out of commit space, so this event is "
+"dire.")
+  {}
 
-  NS_IMETHOD GetPath(nsACString &aPath)
-  {
-    aPath.AssignLiteral("low-commit-space-events");
-    return NS_OK;
-  }
-
-  NS_IMETHOD GetAmount(int64_t *aAmount)
-  {
-    *aAmount = sNumLowCommitSpaceEvents;
-    return NS_OK;
-  }
-
-  NS_IMETHOD GetDescription(nsACString &aDescription)
-  {
-    aDescription.AssignLiteral(
-      "Number of low-commit-space events fired since startup. ");
-
-    if (sLowCommitSpaceThreshold == 0) {
-      aDescription.Append(
-        "Tracking low-commit-space events is disabled, but you can enable it "
-        "by giving the memory.low_commit_space_threshold_mb pref a non-zero "
-        "value.");
-    }
-    else {
-      aDescription.Append(nsPrintfCString(
-        "We fire such an event if we notice there is less than %d MB of "
-        "available commit space (controlled by the "
-        "'memory.low_commit_space_threshold_mb' pref).  Windows will likely "
-        "kill us if we run out of commit space, so this event is somewhat dire.",
-        sLowCommitSpaceThreshold));
-    }
-    return NS_OK;
-  }
+private:
+  int64_t Amount() MOZ_OVERRIDE { return sNumLowCommitSpaceEvents; }
 };
 
-NS_IMPL_ISUPPORTS1(NumLowCommitSpaceEventsMemoryReporter, nsIMemoryReporter)
+static int64_t
+LowMemoryEventsPhysicalDistinguishedAmount()
+{
+  return sNumLowPhysicalMemEvents;
+}
 
-class NumLowPhysicalMemoryEventsMemoryReporter MOZ_FINAL : public NumLowMemoryEventsReporter
+class LowMemoryEventsPhysicalReporter MOZ_FINAL : public MemoryUniReporter
 {
 public:
-  NS_DECL_ISUPPORTS
+  LowMemoryEventsPhysicalReporter()
+    : MemoryUniReporter("low-memory-events/physical",
+                         KIND_OTHER, UNITS_COUNT_CUMULATIVE,
+"Number of low-physical-memory events fired since startup. We fire such an "
+"event if we notice there is less than memory.low_physical_memory_threshold_mb "
+"of physical memory available (if zero, this behavior is disabled).  The "
+"machine will start to page if it runs out of physical memory.  This may "
+"cause it to run slowly, but it shouldn't cause it to crash.")
+  {}
 
-  NS_IMETHOD GetPath(nsACString &aPath)
-  {
-    aPath.AssignLiteral("low-memory-events/physical");
-    return NS_OK;
-  }
-
-  NS_IMETHOD GetAmount(int64_t *aAmount)
-  {
-    *aAmount = sNumLowPhysicalMemEvents;
-    return NS_OK;
-  }
-
-  NS_IMETHOD GetDescription(nsACString &aDescription)
-  {
-    aDescription.AssignLiteral(
-      "Number of low-physical-memory events fired since startup. ");
-
-    if (sLowPhysicalMemoryThreshold == 0) {
-      aDescription.Append(
-        "Tracking low-physical-memory events is disabled, but you can enable it "
-        "by giving the memory.low_physical_memory_threshold_mb pref a non-zero "
-        "value.");
-    }
-    else {
-      aDescription.Append(nsPrintfCString(
-        "We fire such an event if we notice there is less than %d MB of "
-        "available physical memory (controlled by the "
-        "'memory.low_physical_memory_threshold_mb' pref).  The machine will start "
-        "to page if it runs out of physical memory; this may cause it to run "
-        "slowly, but it shouldn't cause us to crash.",
-        sLowPhysicalMemoryThreshold));
-    }
-    return NS_OK;
-  }
+private:
+  int64_t Amount() MOZ_OVERRIDE { return LowMemoryEventsPhysicalDistinguishedAmount(); }
 };
 
-NS_IMPL_ISUPPORTS1(NumLowPhysicalMemoryEventsMemoryReporter, nsIMemoryReporter)
-
 #endif // defined(XP_WIN)
 
 /**
@@ -517,10 +436,15 @@ public:
   NS_DECL_NSIOBSERVER
 
   void Init();
+
+private:
+  static bool sFreeDirtyPages;
 };
 
 NS_IMPL_ISUPPORTS1(nsMemoryPressureWatcher, nsIObserver)
 
+bool nsMemoryPressureWatcher::sFreeDirtyPages = false;
+
 /**
  * Initialize and subscribe to the memory-pressure events. We subscribe to the
  * observer service in this method and not in the constructor because we need
@@ -534,12 +458,14 @@ nsMemoryPressureWatcher::Init()
   if (os) {
     os->AddObserver(this, "memory-pressure", /* ownsWeak */ false);
   }
+
+  Preferences::AddBoolVarCache(&sFreeDirtyPages, "memory.free_dirty_pages",
+                               false);
 }
 
 /**
  * Reacts to all types of memory-pressure events, launches a runnable to
  * free dirty pages held by jemalloc.
- * @see nsMemoryPressureWatcher::FreeDirtyPages
  */
 NS_IMETHODIMP
 nsMemoryPressureWatcher::Observe(nsISupports *subject, const char *topic,
@@ -547,9 +473,11 @@ nsMemoryPressureWatcher::Observe(nsISupports *subject, const char *topic,
 {
   MOZ_ASSERT(!strcmp(topic, "memory-pressure"), "Unknown topic");
 
-  nsRefPtr<nsIRunnable> runnable = new nsJemallocFreeDirtyPagesRunnable();
+  if (sFreeDirtyPages) {
+    nsRefPtr<nsIRunnable> runnable = new nsJemallocFreeDirtyPagesRunnable();
 
-  NS_DispatchToMainThread(runnable);
+    NS_DispatchToMainThread(runnable);
+  }
 
   return NS_OK;
 }
@@ -582,19 +510,19 @@ void Activate()
   Preferences::AddUintVarCache(&sLowMemoryNotificationIntervalMS,
       "memory.low_memory_notification_interval_ms", 10000);
 
-  NS_RegisterMemoryReporter(new NumLowCommitSpaceEventsMemoryReporter());
-  NS_RegisterMemoryReporter(new NumLowPhysicalMemoryEventsMemoryReporter());
+  RegisterStrongMemoryReporter(new LowCommitSpaceEventsReporter());
+  RegisterStrongMemoryReporter(new LowMemoryEventsPhysicalReporter());
   if (sizeof(void*) == 4) {
-    NS_RegisterMemoryReporter(new NumLowVirtualMemoryEventsMemoryReporter());
+    RegisterStrongMemoryReporter(new LowMemoryEventsVirtualReporter());
   }
+  RegisterLowMemoryEventsVirtualDistinguishedAmount(LowMemoryEventsVirtualDistinguishedAmount);
+  RegisterLowMemoryEventsPhysicalDistinguishedAmount(LowMemoryEventsPhysicalDistinguishedAmount);
   sHooksActive = true;
 #endif
 
-  if (Preferences::GetBool("memory.free_dirty_pages", false)) {
-    // This object is held alive by the observer service.
-    nsRefPtr<nsMemoryPressureWatcher> watcher = new nsMemoryPressureWatcher();
-    watcher->Init();
-  }
+  // This object is held alive by the observer service.
+  nsRefPtr<nsMemoryPressureWatcher> watcher = new nsMemoryPressureWatcher();
+  watcher->Init();
 }
 
 void Init()
