@@ -43,13 +43,13 @@ ToSupports(nsISupports* p)
 inline nsISupports*
 ToCanonicalSupports(nsISupports* p)
 {
-    return NULL;
+    return nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Macros to help detect thread-safety:
 
-#if (defined(DEBUG) || defined(NIGHTLY_BUILD)) && !defined(XPCOM_GLUE_AVOID_NSPR)
+#if (defined(DEBUG) || (defined(NIGHTLY_BUILD) && !defined(MOZ_PROFILING))) && !defined(XPCOM_GLUE_AVOID_NSPR)
 
 class nsAutoOwningThread {
 public:
@@ -64,27 +64,16 @@ private:
 #define NS_ASSERT_OWNINGTHREAD_AGGREGATE(agg, _class) \
   NS_CheckThreadSafe(agg->_mOwningThread.GetThread(), #_class " not thread-safe")
 #define NS_ASSERT_OWNINGTHREAD(_class) NS_ASSERT_OWNINGTHREAD_AGGREGATE(this, _class)
-#else // !DEBUG && !NIGHTLY_BUILD
+#else // !DEBUG && !(NIGHTLY_BUILD && !MOZ_PROFILING)
 
 #define NS_DECL_OWNINGTHREAD            /* nothing */
 #define NS_ASSERT_OWNINGTHREAD_AGGREGATE(agg, _class) ((void)0)
 #define NS_ASSERT_OWNINGTHREAD(_class)  ((void)0)
 
-#endif // DEBUG || NIGHTLY_BUILD
+#endif // DEBUG || (NIGHTLY_BUILD && !MOZ_PROFILING)
 
 // Support for ISupports classes which interact with cycle collector.
 
-struct nsPurpleBufferEntry {
-  union {
-    void *mObject;                        // when low bit unset
-    nsPurpleBufferEntry *mNextInFreeList; // when low bit set
-  };
-
-  nsCycleCollectingAutoRefCnt *mRefCnt;
-
-  nsCycleCollectionParticipant *mParticipant; // NULL for nsISupports
-};
-
 #define NS_NUMBER_OF_FLAGS_IN_REFCNT 2
 #define NS_IN_PURPLE_BUFFER (1 << 0)
 #define NS_IS_PURPLE (1 << 1)
