@@ -6,12 +6,16 @@
 #include "mozilla/Mutex.h"
 #include "mozilla/Attributes.h"
 #include "nsStreamUtils.h"
+#include "nsAutoPtr.h"
 #include "nsCOMPtr.h"
 #include "nsIPipe.h"
 #include "nsIEventTarget.h"
 #include "nsIRunnable.h"
 #include "nsISafeOutputStream.h"
 #include "nsString.h"
+#include "nsIAsyncInputStream.h"
+#include "nsIAsyncOutputStream.h"
+#include "nsIBufferedStreams.h"
 
 using namespace mozilla;
 
@@ -21,7 +25,7 @@ class nsInputStreamReadyEvent MOZ_FINAL : public nsIRunnable
                                         , public nsIInputStreamCallback
 {
 public:
-    NS_DECL_ISUPPORTS
+    NS_DECL_THREADSAFE_ISUPPORTS
 
     nsInputStreamReadyEvent(nsIInputStreamCallback *callback,
                             nsIEventTarget *target)
@@ -45,10 +49,9 @@ private:
         bool val;
         nsresult rv = mTarget->IsOnCurrentThread(&val);
         if (NS_FAILED(rv) || !val) {
-            nsCOMPtr<nsIInputStreamCallback> event;
-            NS_NewInputStreamReadyEvent(getter_AddRefs(event), mCallback,
-                                        mTarget);
-            mCallback = 0;
+            nsCOMPtr<nsIInputStreamCallback> event =
+                NS_NewInputStreamReadyEvent(mCallback, mTarget);
+            mCallback = nullptr;
             if (event) {
                 rv = event->OnInputStreamReady(nullptr);
                 if (NS_FAILED(rv)) {
@@ -91,8 +94,8 @@ private:
     nsCOMPtr<nsIEventTarget>         mTarget;
 };
 
-NS_IMPL_THREADSAFE_ISUPPORTS2(nsInputStreamReadyEvent, nsIRunnable,
-                              nsIInputStreamCallback)
+NS_IMPL_ISUPPORTS2(nsInputStreamReadyEvent, nsIRunnable,
+                   nsIInputStreamCallback)
 
 //-----------------------------------------------------------------------------
 
@@ -100,7 +103,7 @@ class nsOutputStreamReadyEvent MOZ_FINAL : public nsIRunnable
                                          , public nsIOutputStreamCallback
 {
 public:
-    NS_DECL_ISUPPORTS
+    NS_DECL_THREADSAFE_ISUPPORTS
 
     nsOutputStreamReadyEvent(nsIOutputStreamCallback *callback,
                              nsIEventTarget *target)
@@ -124,10 +127,9 @@ private:
         bool val;
         nsresult rv = mTarget->IsOnCurrentThread(&val);
         if (NS_FAILED(rv) || !val) {
-            nsCOMPtr<nsIOutputStreamCallback> event;
-            NS_NewOutputStreamReadyEvent(getter_AddRefs(event), mCallback,
-                                         mTarget);
-            mCallback = 0;
+            nsCOMPtr<nsIOutputStreamCallback> event =
+                NS_NewOutputStreamReadyEvent(mCallback, mTarget);
+            mCallback = nullptr;
             if (event) {
                 rv = event->OnOutputStreamReady(nullptr);
                 if (NS_FAILED(rv)) {
@@ -170,37 +172,31 @@ private:
     nsCOMPtr<nsIEventTarget>          mTarget;
 };
 
-NS_IMPL_THREADSAFE_ISUPPORTS2(nsOutputStreamReadyEvent, nsIRunnable,
-                              nsIOutputStreamCallback)
+NS_IMPL_ISUPPORTS2(nsOutputStreamReadyEvent, nsIRunnable,
+                   nsIOutputStreamCallback)
 
 //-----------------------------------------------------------------------------
 
-nsresult
-NS_NewInputStreamReadyEvent(nsIInputStreamCallback **event,
-                            nsIInputStreamCallback *callback,
+already_AddRefed<nsIInputStreamCallback>
+NS_NewInputStreamReadyEvent(nsIInputStreamCallback *callback,
                             nsIEventTarget *target)
 {
     NS_ASSERTION(callback, "null callback");
     NS_ASSERTION(target, "null target");
-    nsInputStreamReadyEvent *ev = new nsInputStreamReadyEvent(callback, target);
-    if (!ev)
-        return NS_ERROR_OUT_OF_MEMORY;
-    NS_ADDREF(*event = ev);
-    return NS_OK;
+    nsRefPtr<nsInputStreamReadyEvent> ev =
+        new nsInputStreamReadyEvent(callback, target);
+    return ev.forget();
 }
 
-nsresult
-NS_NewOutputStreamReadyEvent(nsIOutputStreamCallback **event,
-                             nsIOutputStreamCallback *callback,
+already_AddRefed<nsIOutputStreamCallback>
+NS_NewOutputStreamReadyEvent(nsIOutputStreamCallback *callback,
                              nsIEventTarget *target)
 {
     NS_ASSERTION(callback, "null callback");
     NS_ASSERTION(target, "null target");
-    nsOutputStreamReadyEvent *ev = new nsOutputStreamReadyEvent(callback, target);
-    if (!ev)
-        return NS_ERROR_OUT_OF_MEMORY;
-    NS_ADDREF(*event = ev);
-    return NS_OK;
+    nsRefPtr<nsOutputStreamReadyEvent> ev =
+        new nsOutputStreamReadyEvent(callback, target);
+    return ev.forget();
 }
 
 //-----------------------------------------------------------------------------
@@ -212,7 +208,7 @@ class nsAStreamCopier : public nsIInputStreamCallback
                       , public nsIRunnable
 {
 public:
-    NS_DECL_ISUPPORTS
+    NS_DECL_THREADSAFE_ISUPPORTS
 
     nsAStreamCopier()
         : mLock("nsAStreamCopier.mLock")
@@ -466,10 +462,10 @@ protected:
     nsresult                       mCancelStatus;
 };
 
-NS_IMPL_THREADSAFE_ISUPPORTS3(nsAStreamCopier,
-                              nsIInputStreamCallback,
-                              nsIOutputStreamCallback,
-                              nsIRunnable)
+NS_IMPL_ISUPPORTS3(nsAStreamCopier,
+                   nsIInputStreamCallback,
+                   nsIOutputStreamCallback,
+                   nsIRunnable)
 
 class nsStreamCopierIB MOZ_FINAL : public nsAStreamCopier
 {
@@ -673,6 +669,11 @@ TestInputStream(nsIInputStream *inStr,
 bool
 NS_InputStreamIsBuffered(nsIInputStream *stream)
 {
+    nsCOMPtr<nsIBufferedInputStream> bufferedIn = do_QueryInterface(stream);
+    if (bufferedIn) {
+        return true;
+    }
+
     bool result = false;
     uint32_t n;
     nsresult rv = stream->ReadSegments(TestInputStream,
@@ -696,6 +697,11 @@ TestOutputStream(nsIOutputStream *outStr,
 bool
 NS_OutputStreamIsBuffered(nsIOutputStream *stream)
 {
+    nsCOMPtr<nsIBufferedOutputStream> bufferedOut = do_QueryInterface(stream);
+    if (bufferedOut) {
+        return true;
+    }
+
     bool result = false;
     uint32_t n;
     stream->WriteSegments(TestOutputStream, &result, 1, &n);
@@ -780,3 +786,30 @@ NS_WriteSegmentThunk(nsIInputStream *inStr,
     return thunk->mFun(thunk->mStream, thunk->mClosure, buffer, offset, count,
                        countWritten);
 }
+
+NS_METHOD
+NS_FillArray(FallibleTArray<char>& aDest, nsIInputStream *aInput,
+             uint32_t aKeep, uint32_t *aNewBytes)
+{
+  MOZ_ASSERT(aInput, "null stream");
+  MOZ_ASSERT(aKeep <= aDest.Length(), "illegal keep count");
+
+  char* aBuffer = aDest.Elements();
+  int64_t keepOffset = int64_t(aDest.Length()) - aKeep;
+  if (0 != aKeep && keepOffset > 0) {
+    memmove(aBuffer, aBuffer + keepOffset, aKeep);
+  }
+
+  nsresult rv =
+    aInput->Read(aBuffer + aKeep, aDest.Capacity() - aKeep, aNewBytes);
+  if (NS_FAILED(rv)) {
+    *aNewBytes = 0;
+  }
+  // NOTE: we rely on the fact that the new slots are NOT initialized by
+  // SetLengthAndRetainStorage here, see nsTArrayElementTraits::Construct()
+  // in nsTArray.h:
+  aDest.SetLengthAndRetainStorage(aKeep + *aNewBytes);
+
+  MOZ_ASSERT(aDest.Length() <= aDest.Capacity(), "buffer overflow");
+  return rv;
+}
