@@ -4,6 +4,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "mozilla/MathAlgorithms.h"
+
 // Local includes
 #include "nsXULWindow.h"
 #include <algorithm>
@@ -23,11 +25,9 @@
 #include "nsIServiceManager.h"
 #include "nsIContentViewer.h"
 #include "nsIDocument.h"
-#include "nsIDOMBarProp.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMXULDocument.h"
 #include "nsIDOMElement.h"
-#include "nsIDOMEventTarget.h"
 #include "nsIDOMXULElement.h"
 #include "nsPIDOMWindow.h"
 #include "nsIDOMScreen.h"
@@ -35,7 +35,6 @@
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIIOService.h"
-#include "nsIJSContextStack.h"
 #include "nsIMarkupDocumentViewer.h"
 #include "nsIObserverService.h"
 #include "nsIWindowMediator.h"
@@ -51,13 +50,16 @@
 #include "nsStyleConsts.h"
 #include "nsPresContext.h"
 #include "nsContentUtils.h"
+#include "nsCxPusher.h"
 #include "nsWebShellWindow.h" // get rid of this one, too...
+#include "nsDOMEvent.h"
+#include "nsGlobalWindow.h"
 
 #include "prenv.h"
+#include "mozilla/AutoRestore.h"
 #include "mozilla/Preferences.h"
+#include "mozilla/dom/BarProps.h"
 #include "mozilla/dom/Element.h"
-#include <cstdlib> // for std::abs(int/long)
-#include <cmath> // for std::abs(float/double)
 
 using namespace mozilla;
 
@@ -115,8 +117,8 @@ nsXULWindow::~nsXULWindow()
 // nsXULWindow::nsISupports
 //*****************************************************************************
 
-NS_IMPL_THREADSAFE_ADDREF(nsXULWindow)
-NS_IMPL_THREADSAFE_RELEASE(nsXULWindow)
+NS_IMPL_ADDREF(nsXULWindow)
+NS_IMPL_RELEASE(nsXULWindow)
 
 NS_INTERFACE_MAP_BEGIN(nsXULWindow)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIXULWindow)
@@ -209,12 +211,10 @@ NS_IMETHODIMP nsXULWindow::SetZLevel(uint32_t aLevel)
 
   /* refuse to raise a maximized window above the normal browser level,
      for fear it could hide newly opened browser windows */
-  if (aLevel > nsIXULWindow::normalZ) {
-    int32_t sizeMode;
-    if (mWindow) {
-      mWindow->GetSizeMode(&sizeMode);
-      if (sizeMode == nsSizeMode_Maximized || sizeMode == nsSizeMode_Fullscreen)
-        return NS_ERROR_FAILURE;
+  if (aLevel > nsIXULWindow::normalZ && mWindow) {
+    int32_t sizeMode = mWindow->SizeMode();
+    if (sizeMode == nsSizeMode_Maximized || sizeMode == nsSizeMode_Fullscreen) {
+      return NS_ERROR_FAILURE;
     }
   }
 
@@ -226,20 +226,17 @@ NS_IMETHODIMP nsXULWindow::SetZLevel(uint32_t aLevel)
   nsCOMPtr<nsIContentViewer> cv;
   mDocShell->GetContentViewer(getter_AddRefs(cv));
   if (cv) {
-    nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(cv->GetDocument());
-    if (domDoc) {
-      nsCOMPtr<nsIDOMEvent> event;
-      domDoc->CreateEvent(NS_LITERAL_STRING("Events"), getter_AddRefs(event));
+    nsCOMPtr<nsIDocument> doc = cv->GetDocument();
+    if (doc) {
+      ErrorResult rv;
+      nsRefPtr<nsDOMEvent> event = doc->CreateEvent(NS_LITERAL_STRING("Events"),rv);
       if (event) {
         event->InitEvent(NS_LITERAL_STRING("windowZLevel"), true, false);
 
         event->SetTrusted(true);
 
-        nsCOMPtr<nsIDOMEventTarget> targ = do_QueryInterface(domDoc);
-        if (targ) {
-          bool defaultActionEnabled;
-          targ->DispatchEvent(event, &defaultActionEnabled);
-        }
+        bool defaultActionEnabled;
+        doc->DispatchEvent(event, &defaultActionEnabled);
       }
     }
   }
@@ -360,16 +357,14 @@ NS_IMETHODIMP nsXULWindow::ShowModal()
   mContinueModalLoop = true;
   EnableParent(false);
 
-  nsCOMPtr<nsIJSContextStack> stack(do_GetService("@mozilla.org/js/xpc/ContextStack;1"));
-  if (stack && NS_SUCCEEDED(stack->Push(nullptr))) {
+  {
+    nsCxPusher pusher;
+    pusher.PushNull();
     nsIThread *thread = NS_GetCurrentThread();
     while (mContinueModalLoop) {
       if (!NS_ProcessNextEvent(thread))
         break;
     }
-    JSContext* cx;
-    stack->Pop(&cx);
-    NS_ASSERTION(cx == nullptr, "JSContextStack mismatch");
   }
 
   mContinueModalLoop = false;
@@ -527,7 +522,7 @@ NS_IMETHODIMP nsXULWindow::Destroy()
 
 NS_IMETHODIMP nsXULWindow::GetUnscaledDevicePixelsPerCSSPixel(double *aScale)
 {
-  *aScale = mWindow ? mWindow->GetDefaultScale() : 1.0;
+  *aScale = mWindow ? mWindow->GetDefaultScale().scale : 1.0;
   return NS_OK;
 }
 
@@ -535,7 +530,8 @@ NS_IMETHODIMP nsXULWindow::SetPosition(int32_t aX, int32_t aY)
 {
   // Don't reset the window's size mode here - platforms that don't want to move
   // maximized windows should reset it in their respective Move implementation.
-  double invScale = 1.0 / mWindow->GetDefaultScale();
+  CSSToLayoutDeviceScale scale = mWindow->GetDefaultScale();
+  double invScale = 1.0 / scale.scale;
   nsresult rv = mWindow->Move(aX * invScale, aY * invScale);
   NS_ENSURE_SUCCESS(rv, NS_ERROR_FAILURE);
   if (!mChromeLoaded) {
@@ -563,7 +559,8 @@ NS_IMETHODIMP nsXULWindow::SetSize(int32_t aCX, int32_t aCY, bool aRepaint)
 
   mIntrinsicallySized = false;
 
-  double invScale = 1.0 / mWindow->GetDefaultScale();
+  CSSToLayoutDeviceScale scale = mWindow->GetDefaultScale();
+  double invScale = 1.0 / scale.scale;
   nsresult rv = mWindow->Resize(aCX * invScale, aCY * invScale, aRepaint);
   NS_ENSURE_SUCCESS(rv, NS_ERROR_FAILURE);
   if (!mChromeLoaded) {
@@ -595,7 +592,8 @@ NS_IMETHODIMP nsXULWindow::SetPositionAndSize(int32_t aX, int32_t aY,
 
   mIntrinsicallySized = false;
 
-  double invScale = 1.0 / mWindow->GetDefaultScale();
+  CSSToLayoutDeviceScale scale = mWindow->GetDefaultScale();
+  double invScale = 1.0 / scale.scale;
   nsresult rv = mWindow->Resize(aX * invScale, aY * invScale,
                                 aCX * invScale, aCY * invScale,
                                 aRepaint);
@@ -703,16 +701,16 @@ NS_IMETHODIMP nsXULWindow::Center(nsIXULWindow *aRelative, bool aScreen, bool aA
 
   if (screenCoordinates || windowCoordinates) {
     NS_ASSERTION(mWindow, "what, no window?");
-    double scale = mWindow->GetDefaultScale();
+    CSSToLayoutDeviceScale scale = mWindow->GetDefaultScale();
     GetSize(&ourWidth, &ourHeight);
-    ourWidth = NSToIntRound(ourWidth / scale);
-    ourHeight = NSToIntRound(ourHeight / scale);
+    ourWidth = NSToIntRound(ourWidth / scale.scale);
+    ourHeight = NSToIntRound(ourHeight / scale.scale);
     left += (width - ourWidth) / 2;
     top += (height - ourHeight) / (aAlert ? 3 : 2);
     if (windowCoordinates) {
       mWindow->ConstrainPosition(false, &left, &top);
     }
-    SetPosition(left * scale, top * scale);
+    SetPosition(left * scale.scale, top * scale.scale);
     return NS_OK;
   }
   return NS_ERROR_FAILURE;
@@ -1026,15 +1024,14 @@ void nsXULWindow::OnChromeLoaded()
 
 bool nsXULWindow::LoadPositionFromXUL()
 {
-  nsresult rv;
   bool     gotPosition = false;
-  
+
   // if we're the hidden window, don't try to validate our size/position. We're
   // special.
   if (mIsHiddenWindow)
     return false;
 
-  nsCOMPtr<nsIDOMElement> windowElement = GetWindowDOMElement();
+  nsCOMPtr<dom::Element> windowElement = GetWindowDOMElement();
   NS_ENSURE_TRUE(windowElement, false);
 
   int32_t currX = 0;
@@ -1048,34 +1045,30 @@ bool nsXULWindow::LoadPositionFromXUL()
 
   // Convert to global display pixels for consistent window management across
   // screens with diverse resolutions
-  double scale = mWindow->GetDefaultScale();
-  currX = NSToIntRound(currX / scale);
-  currY = NSToIntRound(currY / scale);
-  currWidth = NSToIntRound(currWidth / scale);
-  currHeight = NSToIntRound(currHeight / scale);
+  CSSToLayoutDeviceScale scale = mWindow->GetDefaultScale();
+  currX = NSToIntRound(currX / scale.scale);
+  currY = NSToIntRound(currY / scale.scale);
+  currWidth = NSToIntRound(currWidth / scale.scale);
+  currHeight = NSToIntRound(currHeight / scale.scale);
 
   // Obtain the position information from the <xul:window> element.
   int32_t specX = currX;
   int32_t specY = currY;
   nsAutoString posString;
 
-  rv = windowElement->GetAttribute(SCREENX_ATTRIBUTE, posString);
-  if (NS_SUCCEEDED(rv)) {
-    temp = posString.ToInteger(&errorCode);
-    if (NS_SUCCEEDED(errorCode)) {
-      specX = temp;
-      gotPosition = true;
-    }
+  windowElement->GetAttribute(SCREENX_ATTRIBUTE, posString);
+  temp = posString.ToInteger(&errorCode);
+  if (NS_SUCCEEDED(errorCode)) {
+    specX = temp;
+    gotPosition = true;
   }
-  rv = windowElement->GetAttribute(SCREENY_ATTRIBUTE, posString);
-  if (NS_SUCCEEDED(rv)) {
-    temp = posString.ToInteger(&errorCode);
-    if (NS_SUCCEEDED(errorCode)) {
-      specY = temp;
-      gotPosition = true;
-    }
+  windowElement->GetAttribute(SCREENY_ATTRIBUTE, posString);
+  temp = posString.ToInteger(&errorCode);
+  if (NS_SUCCEEDED(errorCode)) {
+    specY = temp;
+    gotPosition = true;
   }
-    
+
   if (gotPosition) {
     // our position will be relative to our parent, if any
     nsCOMPtr<nsIBaseWindow> parent(do_QueryReferent(mParentWindow));
@@ -1097,8 +1090,8 @@ bool nsXULWindow::LoadPositionFromXUL()
   }
   mWindow->ConstrainPosition(false, &specX, &specY);
   if (specX != currX || specY != currY) {
-    double scale = mWindow->GetDefaultScale();
-    SetPosition(specX * scale, specY * scale);
+    CSSToLayoutDeviceScale scale = mWindow->GetDefaultScale();
+    SetPosition(specX * scale.scale, specY * scale.scale);
   }
 
   return gotPosition;
@@ -1106,15 +1099,14 @@ bool nsXULWindow::LoadPositionFromXUL()
 
 bool nsXULWindow::LoadSizeFromXUL()
 {
-  nsresult rv;
   bool     gotSize = false;
-  
+
   // if we're the hidden window, don't try to validate our size/position. We're
   // special.
   if (mIsHiddenWindow)
     return false;
 
-  nsCOMPtr<nsIDOMElement> windowElement = GetWindowDOMElement();
+  nsCOMPtr<dom::Element> windowElement = GetWindowDOMElement();
   NS_ENSURE_TRUE(windowElement, false);
 
   int32_t currWidth = 0;
@@ -1124,31 +1116,28 @@ bool nsXULWindow::LoadSizeFromXUL()
 
   NS_ASSERTION(mWindow, "we expected to have a window already");
 
-  double scale = mWindow ? mWindow->GetDefaultScale() : 1.0;
+  CSSToLayoutDeviceScale scale = mWindow ? mWindow->GetDefaultScale()
+                                         : CSSToLayoutDeviceScale(1.0);
   GetSize(&currWidth, &currHeight);
-  currWidth = NSToIntRound(currWidth / scale);
-  currHeight = NSToIntRound(currHeight / scale);
+  currWidth = NSToIntRound(currWidth / scale.scale);
+  currHeight = NSToIntRound(currHeight / scale.scale);
 
   // Obtain the position and sizing information from the <xul:window> element.
   int32_t specWidth = currWidth;
   int32_t specHeight = currHeight;
   nsAutoString sizeString;
 
-  rv = windowElement->GetAttribute(WIDTH_ATTRIBUTE, sizeString);
-  if (NS_SUCCEEDED(rv)) {
-    temp = sizeString.ToInteger(&errorCode);
-    if (NS_SUCCEEDED(errorCode) && temp > 0) {
-      specWidth = std::max(temp, 100);
-      gotSize = true;
-    }
+  windowElement->GetAttribute(WIDTH_ATTRIBUTE, sizeString);
+  temp = sizeString.ToInteger(&errorCode);
+  if (NS_SUCCEEDED(errorCode) && temp > 0) {
+    specWidth = std::max(temp, 100);
+    gotSize = true;
   }
-  rv = windowElement->GetAttribute(HEIGHT_ATTRIBUTE, sizeString);
-  if (NS_SUCCEEDED(rv)) {
-    temp = sizeString.ToInteger(&errorCode);
-    if (NS_SUCCEEDED(errorCode) && temp > 0) {
-      specHeight = std::max(temp, 100);
-      gotSize = true;
-    }
+  windowElement->GetAttribute(HEIGHT_ATTRIBUTE, sizeString);
+  temp = sizeString.ToInteger(&errorCode);
+  if (NS_SUCCEEDED(errorCode) && temp > 0) {
+    specHeight = std::max(temp, 100);
+    gotSize = true;
   }
 
   if (gotSize) {
@@ -1172,8 +1161,8 @@ bool nsXULWindow::LoadSizeFromXUL()
 
     mIntrinsicallySized = false;
     if (specWidth != currWidth || specHeight != currHeight) {
-      double scale = mWindow->GetDefaultScale();
-      SetSize(specWidth * scale, specHeight * scale, false);
+      CSSToLayoutDeviceScale scale = mWindow->GetDefaultScale();
+      SetSize(specWidth * scale.scale, specHeight * scale.scale, false);
     }
   }
 
@@ -1186,9 +1175,8 @@ bool nsXULWindow::LoadSizeFromXUL()
    attributes (sizemode) and they require extra processing. */
 bool nsXULWindow::LoadMiscPersistentAttributesFromXUL()
 {
-  nsresult rv;
   bool     gotState = false;
-  
+
   /* There are no misc attributes of interest to the hidden window.
      It's especially important not to try to validate that window's
      size or position, because some platforms (Mac OS X) need to
@@ -1196,64 +1184,63 @@ bool nsXULWindow::LoadMiscPersistentAttributesFromXUL()
   if (mIsHiddenWindow)
     return false;
 
-  nsCOMPtr<nsIDOMElement> windowElement = GetWindowDOMElement();
+  nsCOMPtr<dom::Element> windowElement = GetWindowDOMElement();
   NS_ENSURE_TRUE(windowElement, false);
 
   nsAutoString stateString;
 
   // sizemode
-  rv = windowElement->GetAttribute(MODE_ATTRIBUTE, stateString);
-  if (NS_SUCCEEDED(rv)) {
-    int32_t sizeMode = nsSizeMode_Normal;
-    /* ignore request to minimize, to not confuse novices
-    if (stateString.Equals(SIZEMODE_MINIMIZED))
-      sizeMode = nsSizeMode_Minimized;
-    */
-    if (!mIgnoreXULSizeMode &&
-        (stateString.Equals(SIZEMODE_MAXIMIZED) || stateString.Equals(SIZEMODE_FULLSCREEN))) {
-      /* Honor request to maximize only if the window is sizable.
-         An unsizable, unmaximizable, yet maximized window confuses
-         Windows OS and is something of a travesty, anyway. */
-      if (mChromeFlags & nsIWebBrowserChrome::CHROME_WINDOW_RESIZE) {
-        mIntrinsicallySized = false;
-        
-        if (stateString.Equals(SIZEMODE_MAXIMIZED))
-          sizeMode = nsSizeMode_Maximized;
-        else
-          sizeMode = nsSizeMode_Fullscreen;
-      }
-    }
+  windowElement->GetAttribute(MODE_ATTRIBUTE, stateString);
+  int32_t sizeMode = nsSizeMode_Normal;
+  /* ignore request to minimize, to not confuse novices
+  if (stateString.Equals(SIZEMODE_MINIMIZED))
+    sizeMode = nsSizeMode_Minimized;
+  */
+  if (!mIgnoreXULSizeMode &&
+      (stateString.Equals(SIZEMODE_MAXIMIZED) || stateString.Equals(SIZEMODE_FULLSCREEN))) {
+    /* Honor request to maximize only if the window is sizable.
+       An unsizable, unmaximizable, yet maximized window confuses
+       Windows OS and is something of a travesty, anyway. */
+    if (mChromeFlags & nsIWebBrowserChrome::CHROME_WINDOW_RESIZE) {
+      mIntrinsicallySized = false;
 
-    // If we are told to ignore the size mode attribute update the
-    // document so the attribute and window are in sync.
-    if (mIgnoreXULSizeMode) {
-      nsAutoString sizeString;
-      if (sizeMode == nsSizeMode_Maximized)
-        sizeString.Assign(SIZEMODE_MAXIMIZED);
-      else if (sizeMode == nsSizeMode_Fullscreen)
-        sizeString.Assign(SIZEMODE_FULLSCREEN);
-      else if (sizeMode == nsSizeMode_Normal)
-        sizeString.Assign(SIZEMODE_NORMAL);
-      if (!sizeString.IsEmpty()) {
-        windowElement->SetAttribute(MODE_ATTRIBUTE, sizeString);
-      }
+      if (stateString.Equals(SIZEMODE_MAXIMIZED))
+        sizeMode = nsSizeMode_Maximized;
+      else
+        sizeMode = nsSizeMode_Fullscreen;
     }
-
-    if (sizeMode == nsSizeMode_Fullscreen) {
-      nsCOMPtr<nsIDOMWindow> ourWindow;
-      GetWindowDOMWindow(getter_AddRefs(ourWindow));
-      ourWindow->SetFullScreen(true);
-    } else {
-      mWindow->SetSizeMode(sizeMode);
-    }
-    gotState = true;
   }
 
+  // If we are told to ignore the size mode attribute update the
+  // document so the attribute and window are in sync.
+  if (mIgnoreXULSizeMode) {
+    nsAutoString sizeString;
+    if (sizeMode == nsSizeMode_Maximized)
+      sizeString.Assign(SIZEMODE_MAXIMIZED);
+    else if (sizeMode == nsSizeMode_Fullscreen)
+      sizeString.Assign(SIZEMODE_FULLSCREEN);
+    else if (sizeMode == nsSizeMode_Normal)
+      sizeString.Assign(SIZEMODE_NORMAL);
+    if (!sizeString.IsEmpty()) {
+      ErrorResult rv;
+      windowElement->SetAttribute(MODE_ATTRIBUTE, sizeString, rv);
+    }
+  }
+
+  if (sizeMode == nsSizeMode_Fullscreen) {
+    nsCOMPtr<nsIDOMWindow> ourWindow;
+    GetWindowDOMWindow(getter_AddRefs(ourWindow));
+    ourWindow->SetFullScreen(true);
+  } else {
+    mWindow->SetSizeMode(sizeMode);
+  }
+  gotState = true;
+
   // zlevel
-  rv = windowElement->GetAttribute(ZLEVEL_ATTRIBUTE, stateString);
-  if (NS_SUCCEEDED(rv) && stateString.Length() > 0) {
+  windowElement->GetAttribute(ZLEVEL_ATTRIBUTE, stateString);
+  if (!stateString.IsEmpty()) {
     nsresult errorCode;
-    uint32_t zLevel = stateString.ToInteger(&errorCode);
+    int32_t zLevel = stateString.ToInteger(&errorCode);
     if (NS_SUCCEEDED(errorCode) && zLevel >= lowestZ && zLevel <= highestZ)
       SetZLevel(zLevel);
   }
@@ -1270,9 +1257,8 @@ void nsXULWindow::StaggerPosition(int32_t &aRequestedX, int32_t &aRequestedY,
                                   int32_t aSpecWidth, int32_t aSpecHeight)
 {
   const int32_t kOffset = 22;
-  const int32_t kSlop   = 4;
+  const uint32_t kSlop  = 4;
 
-  nsresult rv;
   bool     keepTrying;
   int      bouncedX = 0, // bounced off vertical edge of screen
            bouncedY = 0; // bounced off horizontal edge
@@ -1282,16 +1268,14 @@ void nsXULWindow::StaggerPosition(int32_t &aRequestedX, int32_t &aRequestedY,
   if (!wm)
     return;
 
-  nsCOMPtr<nsIDOMElement> windowElement = GetWindowDOMElement();
+  nsCOMPtr<dom::Element> windowElement = GetWindowDOMElement();
   if (!windowElement)
     return;
 
   nsCOMPtr<nsIXULWindow> ourXULWindow(this);
 
   nsAutoString windowType;
-  rv = windowElement->GetAttribute(WINDOWTYPE_ATTRIBUTE, windowType);
-  if (NS_FAILED(rv))
-    return;
+  windowElement->GetAttribute(WINDOWTYPE_ATTRIBUTE, windowType);
 
   int32_t screenTop = 0,    // it's pointless to initialize these ...
           screenRight = 0,  // ... but to prevent oversalubrious and ...
@@ -1349,8 +1333,7 @@ void nsXULWindow::StaggerPosition(int32_t &aRequestedX, int32_t &aRequestedY,
           listY = NSToIntRound(listY / scale);
         }
 
-        if (std::abs(listX - aRequestedX) <= kSlop &&
-            std::abs(listY - aRequestedY) <= kSlop) {
+        if (Abs(listX - aRequestedX) <= kSlop && Abs(listY - aRequestedY) <= kSlop) {
           // collision! offset and start over
           if (bouncedX & 0x1)
             aRequestedX -= kOffset;
@@ -1391,60 +1374,53 @@ void nsXULWindow::StaggerPosition(int32_t &aRequestedX, int32_t &aRequestedY,
 
 void nsXULWindow::SyncAttributesToWidget()
 {
-  nsCOMPtr<nsIDOMElement> windowElement = GetWindowDOMElement();
+  nsCOMPtr<dom::Element> windowElement = GetWindowDOMElement();
   if (!windowElement)
     return;
 
   nsAutoString attr;
 
   // "hidechrome" attribute
-  nsresult rv = windowElement->GetAttribute(NS_LITERAL_STRING("hidechrome"), attr);
-  if (NS_SUCCEEDED(rv) && attr.EqualsLiteral("true")) {
+  if (windowElement->AttrValueIs(kNameSpaceID_None, nsGkAtoms::hidechrome,
+                                 nsGkAtoms::_true, eCaseMatters)) {
     mWindow->HideWindowChrome(true);
   }
 
   // "chromemargin" attribute
   nsIntMargin margins;
-  rv = windowElement->GetAttribute(NS_LITERAL_STRING("chromemargin"), attr);
-  if (NS_SUCCEEDED(rv) && nsContentUtils::ParseIntMarginValue(attr, margins)) {
+  windowElement->GetAttribute(NS_LITERAL_STRING("chromemargin"), attr);
+  if (nsContentUtils::ParseIntMarginValue(attr, margins)) {
     mWindow->SetNonClientMargins(margins);
   }
 
   // "accelerated" attribute
-  bool isAccelerated;
-  rv = windowElement->HasAttribute(NS_LITERAL_STRING("accelerated"), &isAccelerated);
-  if (NS_SUCCEEDED(rv)) {
-    mWindow->SetLayersAcceleration(isAccelerated);
-  }
+  bool isAccelerated = windowElement->HasAttribute(NS_LITERAL_STRING("accelerated"));
+  mWindow->SetLayersAcceleration(isAccelerated);
 
   // "windowtype" attribute
-  rv = windowElement->GetAttribute(WINDOWTYPE_ATTRIBUTE, attr);
-  if (NS_SUCCEEDED(rv) && !attr.IsEmpty()) {
+  windowElement->GetAttribute(WINDOWTYPE_ATTRIBUTE, attr);
+  if (!attr.IsEmpty()) {
     mWindow->SetWindowClass(attr);
   }
 
   // "id" attribute for icon
-  rv = windowElement->GetAttribute(NS_LITERAL_STRING("id"), attr);
-  if (NS_FAILED(rv) || attr.IsEmpty()) {
+  windowElement->GetAttribute(NS_LITERAL_STRING("id"), attr);
+  if (attr.IsEmpty()) {
     attr.AssignLiteral("default");
   }
   mWindow->SetIcon(attr);
 
   // "toggletoolbar" attribute
-  rv = windowElement->GetAttribute(NS_LITERAL_STRING("toggletoolbar"), attr);
-  if (NS_SUCCEEDED(rv)) {
-    mWindow->SetShowsToolbarButton(attr.LowerCaseEqualsLiteral("true"));
-  }
+  windowElement->GetAttribute(NS_LITERAL_STRING("toggletoolbar"), attr);
+  mWindow->SetShowsToolbarButton(attr.LowerCaseEqualsLiteral("true"));
 
   // "fullscreenbutton" attribute
-  rv = windowElement->GetAttribute(NS_LITERAL_STRING("fullscreenbutton"), attr);
-  if (NS_SUCCEEDED(rv)) {
-    mWindow->SetShowsFullScreenButton(attr.LowerCaseEqualsLiteral("true"));
-  }
+  windowElement->GetAttribute(NS_LITERAL_STRING("fullscreenbutton"), attr);
+  mWindow->SetShowsFullScreenButton(attr.LowerCaseEqualsLiteral("true"));
 
   // "macanimationtype" attribute
-  rv = windowElement->GetAttribute(NS_LITERAL_STRING("macanimationtype"), attr);
-  if (NS_SUCCEEDED(rv) && attr.EqualsLiteral("document")) {
+  windowElement->GetAttribute(NS_LITERAL_STRING("macanimationtype"), attr);
+  if (attr.EqualsLiteral("document")) {
     mWindow->SetWindowAnimationType(nsIWidget::eDocumentWindowAnimation);
   }
 }
@@ -1456,7 +1432,7 @@ NS_IMETHODIMP nsXULWindow::SavePersistentAttributes()
   if (!mDocShell)
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIDOMElement> docShellElement = GetWindowDOMElement();
+  nsCOMPtr<dom::Element> docShellElement = GetWindowDOMElement();
   if (!docShellElement)
     return NS_ERROR_FAILURE;
 
@@ -1467,13 +1443,12 @@ NS_IMETHODIMP nsXULWindow::SavePersistentAttributes()
     return NS_OK;
   }
 
-  int32_t x, y, cx, cy;
-  int32_t sizeMode;
-
   // get our size, position and mode to persist
+  int32_t x, y, cx, cy;
   NS_ENSURE_SUCCESS(GetPositionAndSize(&x, &y, &cx, &cy), NS_ERROR_FAILURE);
-  mWindow->GetSizeMode(&sizeMode);
-  double scale = mWindow->GetDefaultScale();
+
+  int32_t sizeMode = mWindow->SizeMode();
+  CSSToLayoutDeviceScale scale = mWindow->GetDefaultScale();
 
   // make our position relative to our parent, if any
   nsCOMPtr<nsIBaseWindow> parent(do_QueryReferent(mParentWindow));
@@ -1490,29 +1465,27 @@ NS_IMETHODIMP nsXULWindow::SavePersistentAttributes()
   nsAutoString                windowElementId;
   nsCOMPtr<nsIDOMXULDocument> ownerXULDoc;
 
-  { // fetch docShellElement's ID and XUL owner document
-    nsCOMPtr<nsIDOMDocument> ownerDoc;
-    docShellElement->GetOwnerDocument(getter_AddRefs(ownerDoc));
-    ownerXULDoc = do_QueryInterface(ownerDoc);
-    nsCOMPtr<mozilla::dom::Element> XULElement(do_QueryInterface(docShellElement));
-    if (XULElement && XULElement->IsXUL())
-      XULElement->GetId(windowElementId);
+  // fetch docShellElement's ID and XUL owner document
+  ownerXULDoc = do_QueryInterface(docShellElement->OwnerDoc());
+  if (docShellElement->IsXUL()) {
+    docShellElement->GetId(windowElementId);
   }
 
+  ErrorResult rv;
   // (only for size elements which are persisted)
   if ((mPersistentAttributesDirty & PAD_POSITION) &&
       sizeMode == nsSizeMode_Normal) {
     if (persistString.Find("screenX") >= 0) {
-      PR_snprintf(sizeBuf, sizeof(sizeBuf), "%d", NSToIntRound(x / scale));
+      PR_snprintf(sizeBuf, sizeof(sizeBuf), "%d", NSToIntRound(x / scale.scale));
       sizeString.AssignWithConversion(sizeBuf);
-      docShellElement->SetAttribute(SCREENX_ATTRIBUTE, sizeString);
+      docShellElement->SetAttribute(SCREENX_ATTRIBUTE, sizeString, rv);
       if (ownerXULDoc) // force persistence in case the value didn't change
         ownerXULDoc->Persist(windowElementId, SCREENX_ATTRIBUTE);
     }
     if (persistString.Find("screenY") >= 0) {
-      PR_snprintf(sizeBuf, sizeof(sizeBuf), "%d", NSToIntRound(y / scale));
+      PR_snprintf(sizeBuf, sizeof(sizeBuf), "%d", NSToIntRound(y / scale.scale));
       sizeString.AssignWithConversion(sizeBuf);
-      docShellElement->SetAttribute(SCREENY_ATTRIBUTE, sizeString);
+      docShellElement->SetAttribute(SCREENY_ATTRIBUTE, sizeString, rv);
       if (ownerXULDoc)
         ownerXULDoc->Persist(windowElementId, SCREENY_ATTRIBUTE);
     }
@@ -1521,16 +1494,16 @@ NS_IMETHODIMP nsXULWindow::SavePersistentAttributes()
   if ((mPersistentAttributesDirty & PAD_SIZE) &&
       sizeMode == nsSizeMode_Normal) {
     if (persistString.Find("width") >= 0) {
-      PR_snprintf(sizeBuf, sizeof(sizeBuf), "%d", NSToIntRound(cx / scale));
+      PR_snprintf(sizeBuf, sizeof(sizeBuf), "%d", NSToIntRound(cx / scale.scale));
       sizeString.AssignWithConversion(sizeBuf);
-      docShellElement->SetAttribute(WIDTH_ATTRIBUTE, sizeString);
+      docShellElement->SetAttribute(WIDTH_ATTRIBUTE, sizeString, rv);
       if (ownerXULDoc)
         ownerXULDoc->Persist(windowElementId, WIDTH_ATTRIBUTE);
     }
     if (persistString.Find("height") >= 0) {
-      PR_snprintf(sizeBuf, sizeof(sizeBuf), "%d", NSToIntRound(cy / scale));
+      PR_snprintf(sizeBuf, sizeof(sizeBuf), "%d", NSToIntRound(cy / scale.scale));
       sizeString.AssignWithConversion(sizeBuf);
-      docShellElement->SetAttribute(HEIGHT_ATTRIBUTE, sizeString);
+      docShellElement->SetAttribute(HEIGHT_ATTRIBUTE, sizeString, rv);
       if (ownerXULDoc)
         ownerXULDoc->Persist(windowElementId, HEIGHT_ATTRIBUTE);
     }
@@ -1544,7 +1517,7 @@ NS_IMETHODIMP nsXULWindow::SavePersistentAttributes()
         sizeString.Assign(SIZEMODE_FULLSCREEN);
       else
         sizeString.Assign(SIZEMODE_NORMAL);
-      docShellElement->SetAttribute(MODE_ATTRIBUTE, sizeString);
+      docShellElement->SetAttribute(MODE_ATTRIBUTE, sizeString, rv);
       if (ownerXULDoc && persistString.Find("sizemode") >= 0)
         ownerXULDoc->Persist(windowElementId, MODE_ATTRIBUTE);
     }
@@ -1555,7 +1528,7 @@ NS_IMETHODIMP nsXULWindow::SavePersistentAttributes()
         mediator->GetZLevel(this, &zLevel);
         PR_snprintf(sizeBuf, sizeof(sizeBuf), "%lu", (unsigned long)zLevel);
         sizeString.AssignWithConversion(sizeBuf);
-        docShellElement->SetAttribute(ZLEVEL_ATTRIBUTE, sizeString);
+        docShellElement->SetAttribute(ZLEVEL_ATTRIBUTE, sizeString, rv);
         ownerXULDoc->Persist(windowElementId, ZLEVEL_ATTRIBUTE);
       }
     }
@@ -1578,7 +1551,7 @@ NS_IMETHODIMP nsXULWindow::GetWindowDOMWindow(nsIDOMWindow** aDOMWindow)
   return NS_OK;
 }
 
-nsIDOMElement*
+dom::Element*
 nsXULWindow::GetWindowDOMElement() const
 {
   NS_ENSURE_TRUE(mDocShell, nullptr);
@@ -1590,10 +1563,7 @@ nsXULWindow::GetWindowDOMElement() const
   const nsIDocument* document = cv->GetDocument();
   NS_ENSURE_TRUE(document, nullptr);
 
-  dom::Element* element = document->GetRootElement();
-  NS_ENSURE_TRUE(element, nullptr);
-
-  return static_cast<nsIDOMElement*>(element->AsDOMNode());
+  return document->GetRootElement();
 }
 
 nsresult nsXULWindow::ContentShellAdded(nsIDocShellTreeItem* aContentShell,
@@ -1800,8 +1770,7 @@ NS_IMETHODIMP nsXULWindow::CreateNewContentWindow(int32_t aChromeFlags,
   // it to make things work right, so push a null cx. See bug 799348 comment 13
   // for a description of what happens when we don't.
   nsCxPusher pusher;
-  if (!pusher.PushNull())
-    return NS_ERROR_FAILURE;
+  pusher.PushNull();
   nsCOMPtr<nsIXULWindow> newWindow;
   appShell->CreateTopLevelWindow(this, uri,
                                  aChromeFlags, 615, 480,
@@ -1817,17 +1786,15 @@ NS_IMETHODIMP nsXULWindow::CreateNewContentWindow(int32_t aChromeFlags,
   xulWin->LockUntilChromeLoad();
 
   // Push nullptr onto the JSContext stack before we dispatch a native event.
-  nsCOMPtr<nsIJSContextStack> stack(do_GetService("@mozilla.org/js/xpc/ContextStack;1"));
-  if (stack && NS_SUCCEEDED(stack->Push(nullptr))) {
+  {
+    nsCxPusher pusher;
+    pusher.PushNull();
     nsIThread *thread = NS_GetCurrentThread();
     while (xulWin->IsLocked()) {
       if (!NS_ProcessNextEvent(thread))
         break;
     }
-    JSContext *cx;
-    stack->Pop(&cx);
-    NS_ASSERTION(cx == nullptr, "JSContextStack mismatch");
-  }
+ }
 
   NS_ENSURE_STATE(xulWin->mPrimaryContentShell);
 
@@ -1878,7 +1845,7 @@ bool nsXULWindow::ConstrainToZLevel(bool        aImmediate,
   altered = false;
   mediator->GetZLevel(this, &zLevel);
 
-  // translate from nsGUIEvent to nsIWindowMediator constants
+  // translate from WidgetGUIEvent to nsIWindowMediator constants
   position = nsIWindowMediator::zLevelTop;
   if (*aPlacement == nsWindowZBottom || zLevel == nsIXULWindow::lowestZ)
     position = nsIWindowMediator::zLevelBottom;
@@ -1993,12 +1960,14 @@ void nsXULWindow::PlaceWindowLayersBehind(uint32_t aLowLevel,
 
 void nsXULWindow::SetContentScrollbarVisibility(bool aVisible)
 {
-  nsCOMPtr<nsIDOMWindow> contentWin(do_GetInterface(mPrimaryContentShell));
+  nsCOMPtr<nsPIDOMWindow> contentWin(do_GetInterface(mPrimaryContentShell));
   if (contentWin) {
-    nsCOMPtr<nsIDOMBarProp> scrollbars;
-    contentWin->GetScrollbars(getter_AddRefs(scrollbars));
-    if (scrollbars)
-      scrollbars->SetVisible(aVisible);
+    mozilla::ErrorResult rv;
+    nsRefPtr<nsGlobalWindow> window = static_cast<nsGlobalWindow*>(contentWin.get());
+    nsRefPtr<mozilla::dom::BarProp> scrollbars = window->GetScrollbars(rv);
+    if (scrollbars) {
+      scrollbars->SetVisible(aVisible, rv);
+    }
   }
 }
 
@@ -2033,7 +2002,7 @@ void nsXULWindow::PersistentAttributesDirty(uint32_t aDirtyFlags)
 
 NS_IMETHODIMP nsXULWindow::ApplyChromeFlags()
 {
-  nsCOMPtr<nsIDOMElement> window = GetWindowDOMElement();
+  nsCOMPtr<dom::Element> window = GetWindowDOMElement();
   NS_ENSURE_TRUE(window, NS_ERROR_FAILURE);
 
   if (mChromeLoaded) {
@@ -2041,7 +2010,7 @@ NS_IMETHODIMP nsXULWindow::ApplyChromeFlags()
     // don't cause a global restyle on the document.  Not only that, but the
     // scrollbar stuff needs a content area to toggle the scrollbars on anyway.
     // So just don't do these until mChromeLoaded is true.
-    
+
     // Scrollbars have their own special treatment.
     SetContentScrollbarVisibility(mChromeFlags &
                                   nsIWebBrowserChrome::CHROME_SCROLLBARS ?
@@ -2073,7 +2042,8 @@ NS_IMETHODIMP nsXULWindow::ApplyChromeFlags()
 
   // Note that if we're not actually changing the value this will be a no-op,
   // so no need to compare to the old value.
-  window->SetAttribute(NS_LITERAL_STRING("chromehidden"), newvalue);
+  ErrorResult rv;
+  window->SetAttribute(NS_LITERAL_STRING("chromehidden"), newvalue, rv);
 
   return NS_OK;
 }
